/**
  *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.dweezil.*;
import uk.ac.reload.dweezil.util.*;

/**
 * This is a JFrame Window that can be shown or hidden from a menu bar.
 * The idea is that all status messages, debug output and error messages are diverted
 * here so that the user can see them in a nice way and save them to file if need be.
 * Setting redirect to true will redirect all System.out and
 * System.err messages to this window.
 *
 * @author Phillip Beauvoir
 * @version 2003-11-03
 */
public class ReloadStatusWindow
extends JFrame
implements DweezilConstants
{
    /**
     * A kludge for Carriage returns on the Macintosh
     */
    protected boolean _macKludge;

    /**
     * The Text area to display to
     */
    protected JTextArea _editor;

    /**
     * The Application name
     */
    protected String _appName;

    /**
     * The Application version
     */
    protected String _appVersion;

    /**
     * The Application build date
     */
    protected String _buildDate;

    /**
     * Constructor
     * @param appName the application name
     * @param appVersion the application version
     * @param buildDate the build date
     * @param icon the image icon for the window
     * @param redirect if true then re-direct system output to the status window
     */
    public ReloadStatusWindow(String appName, String appVersion, String buildDate, ImageIcon icon, boolean redirect) {
        // Set Title bar
        super(appName + " - Status Window");

        _appName = appName;
        _appVersion = appVersion;
        _buildDate = buildDate;

        // Set application Icon
        if(icon != null) setIconImage(icon.getImage());

        // BorderLayout
        getContentPane().setLayout(new BorderLayout());

        // Set up JTextArea, read-only, text cursor
        _editor = new JTextArea();
        _editor.setEditable(false);
        _editor.setCursor(ReloadUIManager.TEXT_CURSOR);
        getContentPane().add(new JScrollPane(_editor), BorderLayout.CENTER);

        // Add a Menu bar
        JMenuBar menuBar = constructMenuBar();
        setJMenuBar(menuBar);

        // Centre the Window
        ReloadUtils.centreWindow(this, 450, 400);

        // Only hide the Window if the user closes it
        setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);

        // Do we have a Mac?
        _macKludge = (ReloadUtils.getOS() == ReloadUtils.MACINTOSH);

        // Re-direct system output to the Status window
        if(redirect) {
            try {
                PrintStream output = new PrintStream(new DumpStream(), true);
                PrintStream err = new PrintStream(new ErrorStream(), true);
                System.setOut(output);
                System.setErr(err);
            }
            catch (Exception e) { }
        }

        // Now print some useful stuff
        if(DEBUG) System.out.println(appName + " version:\t" + appVersion + " (" + buildDate + ")");
        if(DEBUG) System.out.println("Java version:\t\t" + System.getProperties().getProperty("java.version"));
        if(DEBUG) System.out.println("Started at:\t\t" + ReloadUtils.parseDate(ReloadUtils.getNow()));
        if(DEBUG) System.out.println("User Home:\t\t" + System.getProperties().getProperty("user.home"));
        if(DEBUG) System.out.println("User Name:\t\t" + System.getProperties().getProperty("user.name"));
        if(DEBUG) System.out.println();
    }

    /**
     * An OutputStream that redirects all System output to the Status Window
     */
    private class DumpStream extends OutputStream {
        String s;

		/**
		 * Default constructor
		 */
        public DumpStream() {
        }

		/**
		 * Writes a byte to the status window
		 *
		 * @param b the byte
		 */
        public void write(int b) {
            s = String.valueOf((char)(b & 255));
            // Kludge time!
            if(_macKludge && (s.equals(CR) || s.equals("\r"))) {
                _editor.append("\r\n");
            }
            else _editor.append(s);
            _editor.setCaretPosition(_editor.getText().length());
        }
    }

    /**
     * An OutputStream that redirects all Error output to the Error log file
     */
    private class ErrorStream extends DumpStream {
        File file;
        FileWriter writer;
        boolean dateWritten;
        boolean canWrite = true;

		/**
		 * Default constructor
		 */
        public ErrorStream() {
            // Open the error log file
            openErrorFile();
        }

        /**
         * Write a byte to the log file.
         * @param b The byte
         */
        public void write(int b) {
           if(canWrite) {
                try {
                    if(!dateWritten) {
                        dateWritten = true;
                        writeDate();
                    }
                    writer.write(b);
                    writer.flush();
                }
                catch(IOException ex) {
                    canWrite = false;
                    //if(DEBUG) System.err.println("Could not write to error log: " + ex);
                }
            }

            super.write(b);
        }

        /**
         * Open the Error Log File.
         */
        private void openErrorFile() {
            String folder = System.getProperty("user.dir");
            String logName = _appName + ".log";
            file = new File(folder, logName);
            try {
                // If file size > 128k, rename it and create a new one
                if(file.length() > (1024*128)) {
                    Date date = new Date();
                    File tmp = new File(folder + _appName + date.getTime() + ".log");
                    file.renameTo(tmp);
                    file = new File(folder + logName);
                }
                writer = new FileWriter(file.getPath(), true);
            }
            catch(IOException ex) {
                canWrite = false;
                //if(DEBUG) System.err.println("Could not open error log: " + ex);
            }
        }

        /**
         * Write the Date and other bits to the log file.
         * @throws IOException
         */
        private void writeDate() throws IOException {
            writer.write("-----------------------------------------------------------------------------" + CR);
            writer.write(_appName + " version:\t" + _appVersion + " (" + _buildDate + ")" + CR);
            writer.write("Java version:\t\t" + System.getProperties().getProperty("java.version") + CR);
            writer.write("Started at:\t\t" + ReloadUtils.parseDate(ReloadUtils.getNow()) + CR);
            writer.write("-----------------------------------------------------------------------------" + CR);
            writer.write(CR);
            writer.flush();
        }
    }

    /**
     * Show the Status Window
     */
    public void showWindow() {
        setVisible(true);
    }

    /**
     * Print a Debug Trace message to System.out, and hence the StatusWindow.
     * @param message The message to output
     */
    public static void printTrace(String message) {
        //boolean trace = UserPrefs.getUserPrefs().getBooleanProperty(UserPrefs.STATUS_MESSAGES);
        boolean trace = true;
        if(trace) System.out.println(message);
    }

    /**
     * Construct the Status Window Menu Bar.
     * @return The Menu bar
     */
    private JMenuBar constructMenuBar() {
     	JMenuBar mb = new JMenuBar();
      	JMenuItem menuItem;

      	JMenu fileMenu = mb.add(new JMenu("File"));
        // Clear
        menuItem = fileMenu.add(new JMenuItem("Clear"));
        menuItem.addActionListener(new Action_Clear());
        // Save text to file
        menuItem = fileMenu.add(new JMenuItem("Save to File..."));
        menuItem.addActionListener(new Action_Save());

        // Garbage Collect
        menuItem = fileMenu.add(new JMenuItem("Garbage Collect"));
        menuItem.addActionListener(new Action_GC());

        return mb;
    }

    /**
     * Clear The Status Window
     */
    private class Action_Clear extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            _editor.setText("");
        }
    }

	/**
	 * Garbage collection to free memory
	 */
    private class Action_GC extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            Runtime rt = Runtime.getRuntime();
            rt.gc();
            float freeMem = (float)rt.freeMemory() /  1024 / 1024;
            if(DEBUG) System.out.println("Free Mem:\t\t" + freeMem + " Mb");
            float totalMem = (float)rt.totalMemory() /  1024 / 1024;
            if(DEBUG) System.out.println("Total Mem:\t\t" + totalMem + " Mb");
            if(DEBUG) System.out.println();
        }
    }

    /**
     * Save the contents of the Status Window
     */
    private class Action_Save extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            // Ask for a file name
            JFileChooser chooser = new JFileChooser();
            int returnVal = chooser.showSaveDialog(ReloadStatusWindow.this);
            // User cancelled
            if(returnVal != JFileChooser.APPROVE_OPTION) return;

            try {
                File file = chooser.getSelectedFile();
                BufferedWriter out = new BufferedWriter(new FileWriter(file));
                out.write(_editor.getText());
                out.flush();
                out.close();
            }
            catch(IOException ex) {
                if(DEBUG) System.out.println("Status log save error " + ex);
            }
        }
    }
}


