/**
  *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui.tree;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.*;

/**
 * A Tree based on JTree with convenience methods
 *
 * @author Phillip Beauvoir
 * @version 2003-10-07
 */
public abstract class ReloadTree
extends JTree implements DweezilConstants {

    /**
     * This flag is set when we expand or contract a tree node.  Then we can save
     * The tree state if needed
     */
    protected boolean dirtyTree;

    /**
     * Default Constructor
     */
    public ReloadTree() {
        super();
    }

    // ============================= Node management ===========================

    /**
     * Determine root path
     * @param path the tree path concerned
     * @return true if the tree path is a root path, false otherwise
     */
    protected boolean isRootPath(TreePath path) {
        return isRootVisible() && getRowForPath(path) == 0;
    }

    /**
     * Select a node on the Tree given the contained object
     * @param anObject an object concerned
     * @return the ReloadTreeNode that contains this object
     */
    public synchronized ReloadTreeNode selectNodeByObject(Object anObject) {
        if(anObject == null) return null;
        ReloadTreeNode node = getNode(anObject);
        selectNode(node);
        return node;
    }

    /**
     * Select a node on the tree and scroll to it
     * @param node the DefaultMutableTreeNode
     */
    public synchronized void selectNode(final DefaultMutableTreeNode node) {
        if(node != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    TreePath path = new TreePath(node.getPath());
                    scrollPathToVisible(path);     // This first
                    setSelectionPath(path);        // Then this
                }
            });
        }
    }

    /**
     * Returns the currently selected node
     * @return the currently selected node
     */
    public ReloadTreeNode getSelectedNode() {
        TreePath selPath = getSelectionPath();
        return selPath == null ? null : (ReloadTreeNode)selPath.getLastPathComponent();
    }

    /**
     * Gets a node on the tree given its contained object.
     * This assumes that all nodes are unique.
     * @param object a contained object
     * @return a ReloadTreeNode that contains this object
     */
    public ReloadTreeNode getNode(Object object) {
        if(object == null) return null;

        Enumeration nodes = ((DefaultMutableTreeNode) treeModel.getRoot()).preorderEnumeration();
        // Drill into and find it
        while(nodes.hasMoreElements()) {
            ReloadTreeNode node = (ReloadTreeNode) nodes.nextElement();
            Object o = node.getUserObject();
            if(o == object) return node;
        }

        return null;
    }

    /**
     * Expand/Collapse a node on the tree. It only expands the top nodes.
     * expand = true or false
     * @param anObject a contained object
     * @param expand true to expand a node on the tree, false to collapse the node
     */
    public void expandNode(Object anObject, boolean expand) {
        ReloadTreeNode node = getNode(anObject);
        expandNode(node, expand);
    }

    /**
     * Expand/Collapse a node on the tree. It only expands the top nodes.
     * expand = true or false
     * @param node a DefaultMutableTreeNode concerned
     * @param expand true to expand a node on the tree, false to collapse the node
     */
    public void expandNode(DefaultMutableTreeNode node, boolean expand) {
        if(node != null) {
            TreePath path = new TreePath(node.getPath());
            // This works! - expandPath() doesn't always work
            int row = getRowForPath(path);
            if(row >= 0) {
                if(expand) expandRow(row);
                else collapseRow(row);
            }
            // Failsafe
            else {
                if(expand) expandPath(path);
                else collapsePath(path);
            }
        }
    }

    /**
     * Scroll to a node on the tree
     * @param node a DefaultMutableTreeNode
     */
    protected void scrollNodeToVisible(DefaultMutableTreeNode node) {
        if(node != null) {
            TreePath path = new TreePath(node.getPath());
            scrollPathToVisible(path);
        }
    }

    /**
     * Update (repaint) the node that is currently selected
     */
    public void updateSelectedNode() {
        ReloadTreeNode node = (ReloadTreeNode)getSelectedNode();
        if(node != null) ((ReloadTreeModel)getModel()).nodeChanged(node);
    }

    /**
     * Update all nodes on the tree
     */
    public void updateNodes() {
        ReloadTreeModel treeModel = (ReloadTreeModel)getModel();
        Enumeration nodes = ((ReloadTreeModel)getModel()).getRootNode().preorderEnumeration();
        while(nodes.hasMoreElements()) {
            ReloadTreeNode node = (ReloadTreeNode)nodes.nextElement();
            treeModel.nodeChanged(node);
        }
    }

    /**
     * Get all the expanded node on the Tree as a String of node numbers
     * @return a string of node numbers representing all the expanded nodes on the tree
     */
    public String getExpandedNodes() {
        int count = 0;
        StringBuffer sb = new StringBuffer();
        Enumeration nodes = ((ReloadTreeModel)getModel()).getRootNode().preorderEnumeration();
        while(nodes.hasMoreElements()) {
            ReloadTreeNode node = (ReloadTreeNode)nodes.nextElement();
            if(node.isExpanded) {
                sb.append(":");
                sb.append(count);
            }
            count++;
        }
        sb.append(":");
        return sb.toString();
    }

    /**
     * Expand node on a tree given an expansion String
     * @param nodeSet the given expansion string
     */
    public void expandedNodes(String nodeSet) {
        if(nodeSet != null) {
            int count = 0;
            Enumeration nodes = ((ReloadTreeModel)getModel()).getRootNode().preorderEnumeration();
            while(nodes.hasMoreElements()) {
                ReloadTreeNode node = (ReloadTreeNode)nodes.nextElement();
                if(nodeSet.indexOf(":" + count + ":") != -1) expandNode(node, true);
                count++;
            }
        }
    }

    // ============================= Tree events ===============================

    /**
     * Tree Expansion Listener
     * @param e the TreeExpansionEvent
     */
    public void treeExpanded(TreeExpansionEvent e) {
        TreePath path = e.getPath();
        ReloadTreeNode node = (ReloadTreeNode)path.getLastPathComponent();
        node.isExpanded = true;
        dirtyTree = true;
    }

    /**
     * Tree Collapsed Listener
     * @param e the TreeExpansionEvent
     */
    public void treeCollapsed(TreeExpansionEvent e) {
        TreePath path = e.getPath();
        ReloadTreeNode node = (ReloadTreeNode)path.getLastPathComponent();
        node.isExpanded = false;
        dirtyTree = true;
    }

    /**
     * Whether or not the Tree State has changed
     * @return true if the tree state has changed, false otherwise
     */
    public boolean isDirtyTree() {
        return dirtyTree;
    }

    /**
     * Expand/Contract the whole Tree.
     * @param expand if true will expand the tree, if false will contract the tree
     */
    public void expandTree(boolean expand) {
        DefaultMutableTreeNode rootNode = ((ReloadTreeModel)getModel()).getRootNode();
        DefaultMutableTreeNode theNode;
        Enumeration nodes;
        // If expanding, work forwards, else backwards.
        if(expand) nodes = rootNode.preorderEnumeration();   // Forwards
        else nodes = rootNode.postorderEnumeration();        // Backwards
        while(nodes.hasMoreElements()) {
            theNode = (DefaultMutableTreeNode)nodes.nextElement();
            expandNode(theNode, expand);
        }
    }

    // ============================= TREE STATE =======================

    /**
     * Write the expanded node tree state to File
     * @param file the output file
     */
    public void writeState(File file) {
        Element docRoot = new Element("tree_state");
        Document doc = new Document(docRoot);
        Element element = new Element("expanded_nodes");
        element.setText(getExpandedNodes());
        docRoot.addContent(element);
        try {
            XMLUtils.write2XMLFile(doc, file);
            dirtyTree = false;
        }
        catch(DweezilXMLException ex) {
            if(DEBUG) System.out.println("writeState Error: " + ex);
        }
    }

    /**
     * Load the expanded node tree state from File
     * @param file the input file
     */
    public void loadState(File file) {
        String nodeSet = null;

        if(file.exists()) {
            try {
                Document source = XMLUtils.readXMLFile(file);
                Element element = source.getRootElement().getChild("expanded_nodes");
                if(element != null) nodeSet = element.getText();
                expandedNodes(nodeSet);
            }
            catch(DweezilXMLException ex) {
                if(DEBUG) System.out.println("loadState Error: " + ex);
            }
        }
        // Reset this since listeners will have set it true
        dirtyTree = false;
    }


}