/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.gui.widgets;

import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;

/**
 * A Combination of JTextField and File Chooser for selecting files
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public class ReloadFileTextField
extends JPanel
implements ReloadWidget
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * A Textfield to display the value
     */
    protected JTextField _textField;

    /**
     * A button to open the File Chooser
     */
    protected JButton _button;

    /**
     * The chooser
     */
    protected ReloadFileChooser _chooser;


    /**
     * The text to display
     */
    protected String _text;

    /**
     * Type of chooser - file or folder
     */
    protected int _type;

    /**
     * Whether the file path should be relative or absolute
     */
    protected File _relativePath;

    /**
     * Type of chooser - FILE_TYPE
     */
    public static int FILE_TYPE = 1;

    /**
     * Type of chooser - FOLDER_TYPE
     */
    public static int FOLDER_TYPE = 2;

    /**
     * Constructor
     * @param type the type of choose file/folder
     * @param text the choose dialog title
     * @param icon the open button icon
     */
    public ReloadFileTextField(int type, String text, Icon icon) {
        super(new BorderLayout());

        _type = type;
        _text = text;

        // Add text field
        _textField = new JTextField();
        _textField.setFont(ReloadUIManager.plainFont11);
        add(_textField, BorderLayout.CENTER);

        // Add Button
        _button = new JButton(icon);
        add(_button, BorderLayout.EAST);
        setupAction();
    }

    /**
     * Set up the chooser button action
     */
    protected void setupAction() {
        _button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if(_chooser == null) {
                    if(_type == FILE_TYPE) _chooser = new ReloadFileChooser();
                    else _chooser = new ReloadFolderChooser();
                    _chooser.setDialogTitle(_text);
                    _chooser.setApproveButtonText("Select");
                }

                int returnVal = _chooser.showOpenDialog(null);
                if(returnVal == ReloadFileChooser.APPROVE_OPTION) {
                    // Get the chosen File
                    File file = _chooser.getSelectedFileAndStore();
                    String path;
                    // relative path?
                    if(_relativePath != null && _relativePath.exists()) {
                        path = ReloadUtils.getRelativePath(_relativePath, file);
                    }
                    else path = file.getPath();
                    setTextValue(path);
                }
            }
        });
    }

    /**
     * Get the Text Field
     * @return the TextField
     */
    public JTextField getTextField() {
        return _textField;
    }

    /**
     * Set the Value in the Text Field
     * @param value the value to be set in the text field
     */
    public void setTextValue(String value) {
        _textField.setText(value);
    }

    /**
     * Insert some text into the text field
     * @param text the text to be inserted into the text field
     */
    public void insertTextValue(String text) {

    }

    /**
     * Get the Value from the text field
     * @return the text in the text field
     */
    public String getTextValue() {
        return _textField.getText();
    }

    /**
     * Set the maximum allowed number of characters
     * @param maxLength allowed number of characters
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * Determine whether this widget only needs a single line to display itself
     * @return true if this widget needs only a single line to display itself, false otherwise
     */
    public boolean isSingleLine() {
        return true;
    }

    /**
     * Get the relative file path
     * @return the relative file path or null
     */
    public File getRelativePath() {
        return _relativePath;
    }

    /**
     * Set the relative file path. If null, it will not be relative
     * @param relativePath the relative file path
     */
    public void setRelativePath(File relativePath) {
        _relativePath = relativePath;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl the DocumentListener to add
     */
    public void addDocumentListener(DocumentListener dl) {
        _textField.getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl the DocumentListener to remove
     */
    public void removeDocumentListener(DocumentListener dl) {
        _textField.getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the components enabled/disabled
     * @param enabled if true enables the component, and if false disables the component
     */
    public void setEnabled(boolean enabled) {
        _textField.setEnabled(enabled);
        _button.setEnabled(enabled);
    }
}