/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.gui.widgets;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.dweezil.gui.*;

// Tina Manoharan, 2003-12-05

import uk.ac.reload.moonunit.schema.*;

/**
 * A Combination of JTextField and JList
 *
 * @author Phillip Beauvoir
 * @version 2003-07-30
 */
public class ReloadListBox
extends JPanel
implements ReloadWidget, ListSelectionListener, ReloadVocabularyListener
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * A Textfield to display the value
     */
    protected JTextField textField;

    /**
     * A list box to display the list
     */
    protected JList listBox;


	// Tina Manoharan, 2003-12-05

	/**
	 * The Reload Vocabulary List
	 */
    protected ReloadVocabularyList _rvList;


    /**
     * Constructor
     */
    public ReloadListBox() {
        super(new BorderLayout());

        // Add text field
        textField = new JTextField();
        textField.setFont(ReloadUIManager.plainFont11);
        add(textField, BorderLayout.NORTH);

        // Add List
        listBox = new JList();
        listBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        add(new JScrollPane(listBox), BorderLayout.CENTER);

        // Listen to List Selections
        listBox.addListSelectionListener(this);
    }

    /**
     * Get the Text Field
     * @return the TextField
     */
    public JTextField getTextField() {
        return textField;
    }

    /**
     * Get the List Box
     * @return the ListBox
     */
    public JList getListBox() {
        return listBox;
    }

    /**
     * Set the Lists Data
     * @param values the list items
     */
    public void setListData(Object[] values) {
        listBox.setListData(values);
    }


    /**
     * Set this value to the TextField and set this value as the current selected item in the ListBox
     * @param value the value to be set
     */
    public void setTextValue(String value) {
        listBox.setSelectedValue(value, true);
        textField.setText(value);
        //textField.requestFocus();
    }

    /**
     * Insert some text
     * @param text the text to insert
     */
    public void insertTextValue(String text) {

    }

    /**
     * Get the Value from the TextField
     * @return the text from the TextField
     */
    public String getTextValue() {
        return textField.getText();
    }

    /**
     * Set the maximum allowed of digits
     * @param maxLength the maximum allowed digits
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * Determine whether this widget only needs a single line to display itself
     * @return true if this widget needs only a single line to display itself, false otherwise
     */
    public boolean isSingleLine() {
        return false;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl the DocumentListener to add
     */
    public void addDocumentListener(DocumentListener dl) {
        textField.getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl the DocumentListener to remove
     */
    public void removeDocumentListener(DocumentListener dl) {
        textField.getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the components enabled/disabled
     * @param enabled true or false depending on whether you want to enable or disable the component
     */
    public void setEnabled(boolean enabled) {
        textField.setEnabled(enabled);
        listBox.setEnabled(enabled);
    }

    /**
     * ListSelection Listener for the Vocabulary List and update the Text Field
     * @param e the ListSelectionEvent
     */
    public void valueChanged(ListSelectionEvent e) {
        if(e.getSource() == listBox) {
            Object object = listBox.getSelectedValue();
            if(object != null) textField.setText(object.toString());
        }
    }

	// Tina Manoharan, 2003-11-10

    /**
     * Set the VocabularyList
     * @param rvList the vocabulary list
     */
    public void setReloadVocabularyList(ReloadVocabularyList rvList) {

        _rvList = rvList;

        // We listen to VocabularyList changes
        _rvList.addReloadVocabularyListener(this);
    }

	// Tina Manoharan, 2003-11-10

	/**
	 * Destroy, remove the ReloadVocabularyListener
	 */
    public void destroy() {
        _rvList.removeReloadVocabularyListener(this);
    }



	// Tina Manoharan, 2003-11-10

// ===================================================================================
// ===================== These events are received FROM the ReloadVocabularyList =====
// ===================================================================================

	/**
	 * Update the ListBox items since the Vocabulary List has changed
	 * @param event the ReloadVocabularyListenerEvent
	 */
    public void vocabularyListChanged(ReloadVocabularyListenerEvent event) {

		System.out.println("ReloadListBox: Vocabulary List Changed");

		// list changed update comboBox

		setListData(event.getList());

		listBox.clearSelection();
    }
}