/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.menu;

import java.util.*;

import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;

/**
 * The Main Menu Action for each Menu Item in the Application.
 * It extends AbstractAction and can be added to a Menu Bar, Toolbar or Menu Popup.
 * It is an abstract parent class for each menu item and button in the Application.
 *
 * It also has a JButton created for it.
 *
 * @author Phillip Beauvoir
 * @version 2003-09-17
 */
public abstract class MenuAction
extends AbstractAction
{
    /**
     * The main button Icon
     */
    private ImageIcon icon;

    /**
     * The text for the menu action
     */
    private String text;

    /**
     * The Button for this action.  This can be null for no button.
     */
    private JButton button;

    /**
     * Default Constructor
     */
    public MenuAction() {
        this("", null);
    }

    /**
     * Constructor for new MenuAction with no icon
     * @param text The text to display
     */
    public MenuAction(String text) {
        this(text, null);
    }

    /**
     * Constructor for new MenuAction
     * @param text The text to display
     * @param iconPath The full path of the icon.  If this is null no icon will be displayed
     */
    public MenuAction(String text, String iconPath) {
        this.text = text;

        // If we have an icon path, make an icon
        if(iconPath != null) {
            icon = ReloadUIManager.getIcon(iconPath);
        }

        if(icon != null) setMenuIcon(icon);

        // Menu
        putValue(Action.NAME, text);

        // Button
        button = createButton();
    }

    /**
     * Set the Icon on the Menu
     * @param icon The Icon to set
     */
    public void setMenuIcon(ImageIcon icon) {
        putValue(Action.SMALL_ICON, icon);
    }

    /**
     * Set the Icon for the Button
     * @param icon the Button Icon to set
     */
    public void setButtonIcon(ImageIcon icon) {
        if(button != null) button.setIcon(icon);
    }

    /**
     * Sets the text for the menu item
     * @param text The Text to set
     */
    public void setText(String text) {
        putValue(Action.NAME, text);
        if(button != null) button.setToolTipText(text);
    }

	// Tina Manoharan 2003-11-10

    /**
     * Sets the text for the menu item
     * @param text The Text to set
     */
    public void setButtonText(String text) {
        if(button != null) button.setText(text);
    }

    /**
     * Set the menu item / button enabled or disabled (greyed out)
     * @param newValue enabled or disabled
     */
    public void setEnabled(boolean newValue) {
        button.setEnabled(newValue);
        super.setEnabled(newValue);
    }

    /**
     * Get the JButton for this Menu Item
     * @return The Button
     */
    public JButton getButton() {
        return button;
    }

    /**
     * Create a JButton for this Menu Ite,
     * @return The button
     */
    private JButton createButton() {
        JButton button = new JButton();

        if(icon != null) button.setIcon(icon);

        button.setEnabled(isEnabled());
        button.addActionListener(this);
        button.setToolTipText(text);
        //button.setContentAreaFilled(false);
        button.setFocusPainted(false);

        // Set it thus so our windows don't lose the focus and therefore their menus
        button.setFocusable(false);

        return button;
    }

    //============================== LISTENER EVENTS  ==========================

    /**
     * List of Listeners
     */
    private Vector listeners = new Vector();

    /**
     * Add a MenuActionListener Listener
     * @param listener The MenuActionListener Listener
     */
    public synchronized void addMenuActionListener(MenuActionListener listener) {
        if(!listeners.contains(listener)) listeners.addElement(listener);
    }

    /**
     * Remove a MenuActionListener Listener
     * @param listener The MenuActionListener Listener
     */
    public synchronized void removeMenuActionListener(MenuActionListener listener) {
        listeners.removeElement(listener);
    }

    /**
     * Clear all listeners
     */
    public synchronized void removeAllMenuActionListeners() {
        listeners.clear();
    }

    public void actionPerformed(ActionEvent e) {
        ReloadMenuEvent menuEvent = new ReloadMenuEvent(this);
        for(int i = listeners.size() - 1; i >= 0; i--) {
            MenuActionListener listener = (MenuActionListener)listeners.elementAt(i);
            listener.menuActionPerformed(menuEvent);
        }
    }

}


