/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.util;

import java.io.*;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.transform.*;

import org.xml.sax.*;

import uk.ac.reload.moonunit.dom.*;


/**
 * Some useful XML Utilities that leverage the JDOM Package<br>
 *
 * @author Phillip Beauvoir
 * @version 2003-11-04
 */
public final class XMLUtils {

    /**
     * The default Sax Parser that JDOM will use.  This can be replaced by something else.
     */
    public static String DEFAULT_SAX_DRIVER_CLASS = "org.apache.xerces.parsers.SAXParser";


    /**
     * Writes a JDOM Document to file
     * @param doc The JDOM Document to write
     * @param file The file to write to
     * @throws DweezilXMLException If there is any problem
     */
    public static void write2XMLFile(Document doc, File file) throws DweezilXMLException{
        try {
            XMLOutputter outputter = new XMLOutputter("  ", true);

            // This gets rid of junk characters
            outputter.setTextNormalize(true);

            // Don't write this - it probably isn't UTF-8
            outputter.setOmitEncoding(true);

            FileOutputStream out = new FileOutputStream(file);
            outputter.output(doc, out);
            out.close();
        }
        catch(Exception ex) {
            throw new DweezilXMLException(ex);
        }
    }

    /**
     * Reads and returns a JDOM Document from file with Schema validation
     * @param file The XML File
     * @param schemaNamespace The Schema Target Namespace
     * @param schemaLocation The Schema Location
     * @return The JDOM Document or null if not found
     * @throws DweezilXMLException If there is any problem
     */
    public static Document readXMLFile(File file, String schemaNamespace, String schemaLocation) throws DweezilXMLException{
        Document doc = null;

        try {
            SAXBuilder builder = new SAXBuilder(DEFAULT_SAX_DRIVER_CLASS, true);
            builder.setFeature("http://apache.org/xml/features/validation/schema", true);
            builder.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation",
                                schemaNamespace + " " + schemaLocation);
            // changed - this allows UNC mapped locations to load
            // doc = builder.build(file);
            doc = builder.build(new FileInputStream(file));
        }
        catch(Exception ex) {
            throw new DweezilXMLException(ex);
        }

        return doc;
    }

    /**
     * Reads and returns a JDOM Document from file without Schema Validation
     * @param file The XML File
     * @return The JDOM Document or null if not found
     * @throws DweezilXMLException If there is any problem
     */
    public static Document readXMLFile(File file) throws DweezilXMLException{
        Document doc = null;

        try {
            SAXBuilder builder = new SAXBuilder(DEFAULT_SAX_DRIVER_CLASS);
            // changed - this allows UNC mapped locations to load
            // doc = builder.build(file);
            doc = builder.build(new FileInputStream(file));
        }
        catch(Exception ex) {
            throw new DweezilXMLException(ex);
        }

        return doc;
    }


	// Tina Manoharan, 2003-09-09

    //============================== VDEX VOCABULARY UTILS ==========================

    /**
     * Gets the value of an element from its name
     * @param rootName the name of the documents root element
     * @param domPath the domPath of the element concerned
     * @param xmlFile the XML file of the document concerned
     * @return the string value of element or null if element not found
     * @throws DweezilXMLException
     */
    public static String getElementValueByName(String rootName, DomPath domPath, File xmlFile) throws DweezilXMLException{

		Document doc;

		Element element;

		try {

			doc = readXMLFile(xmlFile);

			element = doc.getRootElement();

			StringTokenizer st = domPath.getElements();

			// Now work thru the path
			while(st.hasMoreTokens()) {

				// Get next Element name
				String name = st.nextToken();

				// igonore Root element
				if(name.equals(rootName)) continue;

				element = element.getChild(name, element.getNamespace());

				// Not found
				if(element == null) break;
			}

			if(element != null) return element.getTextTrim();
		}
		catch(Exception ex) {

			throw new DweezilXMLException(ex);
		}

		return null;
    }

    /**
     * Gets the an element using domPath and its value
     * @param rootName the name of the documents root element
     * @param domPath the domPath of the element concerned
     * @param xmlFile the XML file of the document concerned
     * @param value the value of the element we are searching for
     * @return true or false depending on whether element was found or not
     * @throws DweezilXMLException
     */
    public static boolean getElementByNameValuePair(String rootName, DomPath domPath, File xmlFile, String value) throws DweezilXMLException{

		Document doc;

		Element element;

		try {

			doc = readXMLFile(xmlFile);

			element = doc.getRootElement();

			StringTokenizer st = domPath.getElements();

			// Now work thru the path
			while(st.hasMoreTokens()) {

				// Get next Element name
				String name = st.nextToken();

				// igonore Root element
				if(name.equals(rootName)) continue;

				element = element.getChild(name, element.getNamespace());

				// Not found
				if(element == null) break;
			}

			if(element != null && element.getText().equals(value)) return true;
		}
		catch(Exception ex) {

			throw new DweezilXMLException(ex);
		}

		return false;
    }


    /**
     * Identifies the file using domPath and its value
     * @param rootName the name of the documents root element
     * @param domPath the domPath of the element concerned
     * @param xmlFile the XML file of the document concerned
     * @param value the value of the element we are searching for
     * @return the true/false depending on whether element was found or not
     * @throws DweezilXMLException
     */
    public static boolean getFileByNameValuePair(String rootName, DomPath domPath, File xmlFile, String value) throws DweezilXMLException{

		Document doc;

		Element element;

		try {

			doc = readXMLFile(xmlFile);

			element = doc.getRootElement();

			StringTokenizer st = domPath.getElements();

			// Now work thru the path
			while(st.hasMoreTokens()) {

				// Get next Element name
				String name = st.nextToken();

				// ignore Root element
				if(name.equals(rootName)) continue;

				element = element.getChild(name, element.getNamespace());

				// Not found
				if(element == null) break;
			}

			if(element != null && element.getText().equals(value)) return true;
		}
		catch(Exception ex) {

			throw new DweezilXMLException(ex);
		}

		return false;
    }


    /**
     * Gets all values of a particular element using its name from the file
     * @param source the XML file
     * @param stylesheet the XSLT stylesheet to be used for transformations and obtaining data
     * @return all values of the element
     * @throws DweezilXMLException
     */
    public static ArrayList getElementValuesFromFileByName(File source, File stylesheet) throws DweezilXMLException{

		ArrayList result = new ArrayList();

		try {

			Document sourceDoc = readXMLFile(source);

	    	// Now do a transformation

			Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(stylesheet));


			// create result by performing transformation using stylesheet

			JDOMSource sourceJdom = new JDOMSource(sourceDoc);

			JDOMResult resultJdom = new JDOMResult();

			transformer.transform(sourceJdom, resultJdom);

			Document resultDoc = resultJdom.getDocument();


    		if(resultDoc != null) {

				Element root = resultDoc.getRootElement();

				List children = root.getChildren("item");

				for(int i = 0; i < children.size(); i++) {

					Element element = (Element) children.get(i);

					String item = element.getTextTrim();

					if(item != null && !item.equals("") && !result.contains(item)) result.add(item);
				}
			}

		}
        catch(TransformerConfigurationException tex) {
            throw new DweezilXMLException(tex);
        }
        catch(TransformerException tex2) {
            throw new DweezilXMLException(tex2);
        }
        catch(Exception ex) {
			throw new DweezilXMLException(ex);
        }

		return result;
    }


    /**
     * Gets the metadata file
     * @param source the XML file
     * @param stylesheet the XSLT stylesheet to be used for transformations and obtaining data
     * @param filename the filename to be used for the metadata file
     * @return the metadata file created
     * @throws DweezilXMLException
     */
    public static File getMetadataFile(File source, File stylesheet, String filename) throws DweezilXMLException{

		File folderTemp = new File(System.getProperty("user.dir"), "temp");

		if(!folderTemp.exists()) {

			folderTemp.mkdir();
		}

		File result = new File(folderTemp, filename);

		try {

			Document sourceDoc = readXMLFile(source);

	    	// Now do a transformation

			Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(stylesheet));


			// create result by performing transformation using stylesheet

			JDOMSource sourceJdom = new JDOMSource(sourceDoc);

			transformer.transform(sourceJdom, new StreamResult(result));
		}
        catch(TransformerConfigurationException tex) {
            throw new DweezilXMLException(tex);
        }
        catch(TransformerException tex2) {
            throw new DweezilXMLException(tex2);
        }
        catch(Exception ex) {
			throw new DweezilXMLException(ex);
        }

		return result;

	}

}