/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor;

import java.io.*;

import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.contentpackaging.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.simplesequencing.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.editor.vocabulary.*;



/**
 * The Reload Editor Schema Controller
 *
 * @author Phillip Beauvoir
 * @version 2003-11-04
 */
public abstract class ReloadEditorSchemaController
extends ReloadSchemaController
{
    /**
     * The Helper Profile
     */
    protected ReloadHelperProfile _helperProfile;

    /**
     * Constructor
     */
    public ReloadEditorSchemaController() {
        super();
    }

    /**
     * Constructor
     */
    public ReloadEditorSchemaController(File fileSchema, File fileHelper, File fileVocab) {
        loadSchema(fileSchema);
        loadSchemaHelper(fileHelper);
        loadVocab(fileVocab);
    }

    /**
     * Set new Profile but keep original Schema
     */
    public void setHelperProfile(ReloadHelperProfile profile) {
        loadSchemaHelper(profile.getSchemaHelperFile());
        loadVocab(profile.getVocabFile());
        _helperProfile = profile;
    }

    /**
     * Load the Schema and warn if need be
     */
    public void loadSchema(File fileSchema) {
        try {
            super.loadSchema(fileSchema);
        }
        catch(ReloadSchemaException ex) {
            ErrorDialogBox.showWarning("Could not load Schema: ", "Load Schema", ex);
        }
    }

    /**
     * Load the Schema Helper and warn if need be
     */
    public void loadSchemaHelper(File fileHelper) {
        try {
            super.loadSchemaHelper(fileHelper);
        }
        catch(ReloadSchemaException ex) {
            ErrorDialogBox.showWarning("Could not load Schema Helper: ", "Load SchemaHelper", ex);
        }
    }

    /**
     * Load the Vocab and warn if need be
     */
    public void loadVocab(File fileVocab) {
        try {
            super.loadVocab(fileVocab);
        }
        catch(ReloadSchemaException ex) {
            ErrorDialogBox.showWarning("Could not load Vocabulary: ", "Load Schema", ex);
        }
    }

    /**
     * Get an appropriate ReloadSchemaController for a given Namespace in an Instance ReloadDocument
     * @param doc the ReloadDocument instance
     *	 	  ns the Namespace in the document
     * @return an appropriate ReloadSchemaController for a given Namespace in an Instance ReloadDocument
     */
    public static ReloadSchemaController getReloadSchemaController(Document doc, Namespace ns) {
        // Try to correct the Namespace
        ns = IMSDocumentHandler.correctAdditionalNamespace(doc, ns);

        String version = IMSDocumentHandler.getSchemaVersionFromNamespace(ns);
        return getReloadSchemaController(version);
    }

    /**
     * Get a Schema Controller depending on a Schema version
     * @param schemaVersion the schema version
     * @return a Schema Controller depending on a Schema version
     */
    public static ReloadSchemaController getReloadSchemaController(String schemaVersion) {
        ReloadSchemaController schemaController = null;

        if(schemaVersion != null) {
            // Do the versions that we don't support first

            // METADATA 1.1 we don't support
            if(schemaVersion.equals(IMSDocumentHandler.IMSMD_110)) {
                showWarningMessage("This is Metadata version 1.1.  Will use version 1.2.2 Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSMD_122);
            }

            // METADATA 1.2 we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSMD_120)) {
                showWarningMessage("This is Metadata version 1.2.  Will use version 1.2.2 Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSMD_122);
            }


		 	// Tina Manoharan, 2003-08-13

            // VDEX 0.0.0, ProfileType LAX we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_LAX_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 LAX.  Will use version 1.0.0 LAX Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSVX_LAX_000);
            }

            // VDEX 0.0.0, ProfileType FLAT we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_FLAT_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 FLAT.  Will use version 1.0.0 FLAT Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSVX_FLAT_000);
            }

            // VDEX 0.0.0, ProfileType HIERARCHICAL we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_HIERARCHICAL_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 HIERARCHICAL.  Will use version 1.0.0 HIERARCHICAL Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSVX_HIERARCHICAL_000);
            }

            // VDEX 0.0.0, ProfileType GLOSSARY we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_GLOSSARY_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 GLOSSARY.  Will use version 1.0.0 GLOSSARY Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSVX_GLOSSARY_000);
            }

            // VDEX 0.0.0, ProfileType THESAURUS we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_THESAURUS_000)) {
                showWarningMessage("This is Vocabulary version 0.0.0 THESAURUS.  Will use version 1.0.0 THESAURUS Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSVX_THESAURUS_000);
            }


            // CP 1.1.1 we don't support
            else if(schemaVersion.equals(IMSDocumentHandler.IMSCP_111)) {
                showWarningMessage("This is Content Packaging version 1.1.  Will use version 1.1.3 Schema model.");
                return getReloadSchemaController(IMSDocumentHandler.IMSCP_113);
            }

            // Now do the versions that we do support

            // METADATA 1.2.1
            else if(schemaVersion.equals(IMSDocumentHandler.IMSMD_121)) {
                schemaController = new MetadataSchemaController(ReloadEditorDocumentHandler.fileSchemaMD1_2_1, schemaVersion);
            }

            // METADATA 1.2.2
            else if(schemaVersion.equals(IMSDocumentHandler.IMSMD_122)) {
                schemaController = new MetadataSchemaController(ReloadEditorDocumentHandler.fileSchemaMD1_2_2, schemaVersion);
            }

		 	// Tina Manoharan, 2003-08-13

            // VDEX 1.0, ProfileType LAX
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_LAX_100)) {
				schemaController = new VocabularySchemaController(ReloadEditorDocumentHandler.fileSchemaVX_LAX_1_0_0, schemaVersion);
            }

            // VDEX 1.0, ProfileType FLAT
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_FLAT_100)) {
				schemaController = new VocabularySchemaController(ReloadEditorDocumentHandler.fileSchemaVX_FLAT_1_0_0, schemaVersion);
            }

            // VDEX 1.0, ProfileType HIERARCHICAL
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_HIERARCHICAL_100)) {
                schemaController = new VocabularySchemaController(ReloadEditorDocumentHandler.fileSchemaVX_HIERARCHICAL_1_0_0, schemaVersion);
            }

            // VDEX 1.0, ProfileType GLOSSARY
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_GLOSSARY_100)) {
                schemaController = new VocabularySchemaController(ReloadEditorDocumentHandler.fileSchemaVX_GLOSSARY_1_0_0, schemaVersion);
            }

            // VDEX 1.0, ProfileType THESAURUS
            else if(schemaVersion.equals(IMSDocumentHandler.IMSVX_THESAURUS_100)) {
                schemaController = new VocabularySchemaController(ReloadEditorDocumentHandler.fileSchemaVX_THESAURUS_1_0_0, schemaVersion);
            }

            // CP 1.1.2
            else if(schemaVersion.equals(IMSDocumentHandler.IMSCP_112)) {
                schemaController = new ContentPackageSchemaController(ReloadEditorDocumentHandler.fileSchemaCP1_1_2, schemaVersion);
            }

            // CP 1.1.3
            else if(schemaVersion.equals(IMSDocumentHandler.IMSCP_113)) {
                schemaController = new ContentPackageSchemaController(ReloadEditorDocumentHandler.fileSchemaCP1_1_3, schemaVersion);
            }

            // SCORM 1.2 IMS CP 1.1.2
            else if(schemaVersion.equals(IMSDocumentHandler.ADLCP_120_112)) {
                schemaController = new SCORM1_2SchemaController(ReloadEditorDocumentHandler.fileSchemaSCORM1_2_IMCP1_1_2, schemaVersion);
            }

            // SCORM 1.2 IMS CP 1.1.3
            else if(schemaVersion.equals(IMSDocumentHandler.ADLCP_120_113)) {
                schemaController = new SCORM1_2SchemaController(ReloadEditorDocumentHandler.fileSchemaSCORM1_2_IMCP1_1_3, schemaVersion);
            }

            // SS 1.0 - TO DO
            else if(schemaVersion.equals(IMSDocumentHandler.IMSSS_100)) {
                schemaController = new SimpleSequencingSchemaController(ReloadEditorDocumentHandler.fileSchemaSS1_0, ReloadEditorDocumentHandler.fileHelperSS, ReloadEditorDocumentHandler.fileVocabSS);
            }
        }

        return schemaController;
    }

    /**
     * Warn the user that we can't cope with an old version
     */
    protected static void showWarningMessage(String msg) {
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg,
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Get the Reload Helper profile
     * @return the Helper Profile
     */
    public ReloadHelperProfile getReloadHelperProfile() {
        return _helperProfile;
    }

    /**
     * Get The Leaf Icon (for the Full View Tree)
     * @param element the leaf element
     * @return the leaf icon
     */
    public abstract Icon getLeafIcon(Element element);

    /**
     * Get The Closed Icon (for the Full View Tree)
     * @param element the closed element
     * @return the close icon
     */
    public abstract Icon getClosedIcon(Element element);

    /**
     * Get The Open Icon (for the Full View Tree)
     * @param element the open element
     * @return the open icon
     */
    public abstract Icon getOpenIcon(Element element);

    /**
     * Get The Leaf Icon (for the Full View Tree)
     * @param schemaElement the leaf schema element
     * @return the leaf icon
     */
    public abstract Icon getLeafIcon(ReloadSchemaElement schemaElement);

    /**
     * Get The Closed Icon (for the Full View Tree)
     * @param schemaElement the closed schemaElement
     * @return the close icon
     */
    public abstract Icon getClosedIcon(ReloadSchemaElement schemaElement);

    /**
     * Get The Open Icon (for the Full View Tree)
     * @param schemaElement the open schema element
     * @return the open icon
     */
    public abstract Icon getOpenIcon(ReloadSchemaElement schemaElement);


 }