/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;
import java.util.zip.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;
import uk.ac.reload.dweezil.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.manifestview.*;
import uk.ac.reload.editor.contentpackaging.resourceview.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.*;


/**
 * The Content Package Editor Frame
 *
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-11-04
 */
public class ContentPackageEditor
extends ReloadInternalFrame
implements ApplicationListener, ReloadDocumentListener
{
    /**
     * The ContentPackage Document
     */
    protected ContentPackage _contentPackage;

    /**
     * The Resources Panel
     */
    protected CPResourcesPanel _resourcesPanel;

    /**
     * The CP Manifest Tree Panel
     */
    protected ManifestPanel _manifestPanel;

    /**
     * The SplitPane
     */
    protected JSplitPane _splitPane;

    /**
     * Handles Save and SaveAs Events
     */
    protected ProxySaveHandler _saveHandler, _saveAsHandler;

    /**
     * Our shared zip handler from the main menu
     */
    protected ProxyZipHandler _zipHandler;

    /**
     * Our shared view CP handler from the main menu
     */
    protected ProxyViewCPHandler _viewCPHandler;

    /**
     * The Undo Menu Manager for this Window
     */
    protected ReloadUndoMenuManager _undoMenuManager;

    /**
     * The Prefs panel
     */
    protected CPPrefsPanel _prefsPanel;

    /**
     * Default Constructor
     */
    public ContentPackageEditor() {
        setFrameIcon(ReloadEditor.ICON_CP);
        setTitle("Content Package");

        // Resources Panel
        _resourcesPanel = new CPResourcesPanel();

        // Manifest Tree Panel
        _manifestPanel = new ManifestPanel(this);

        // Get a Prefs panel
        _prefsPanel = new CPPrefsPanel();

        // Add Prefs Panel and Manifest View to another Panel
        JPanel p = new JPanel(new BorderLayout());
        p.add(_prefsPanel, BorderLayout.NORTH);
        p.add(_manifestPanel, BorderLayout.CENTER);

        // SplitPane
        _splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        _splitPane.setOneTouchExpandable(true);
        getContentPane().add(_splitPane);
        _splitPane.setLeftComponent(_resourcesPanel);
        _splitPane.setRightComponent(p);

        // Ensure we handle closing
        setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);

        // Tell the application that we want to know if it's closing
        ReloadEditor.getInstance().addApplicationListener(this);

        // We listen for what this frame is doing
        addInternalFrameListener(new InternalFrameAdapter() {
            // The Frame is closing
            public void internalFrameClosing(InternalFrameEvent e) {
                // Make sure button stuff is completed
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        boolean ok = checkNeedsSaving();
                        if(ok) dispose();
                    }
                });
            }

            // The Frame is activated
            public void internalFrameActivated(InternalFrameEvent e) {
                setFocusGained();
            }

            // The Frame is deactivated
            public void internalFrameDeactivated(InternalFrameEvent e) {
                setFocusLost();
            }
        });
    }

    /**
     * Over-ride this so we can set view stuff
     */
    public void show() {
        super.show();
        _splitPane.setDividerLocation(0.25);
        _manifestPanel.initView();
    }

    /**
     * Load an existing Content Package imsmanifest.xml
     * @throws ReloadEditorException
     */
    public void setDocument(File file, Document doc, ContentPackageSchemaController schemaController) throws ReloadEditorException {
        _contentPackage = new ContentPackage(file, doc, schemaController);
        setContentPackage(_contentPackage);
    }

    /**
     * Set up some stuff when a new CP is loaded
     */
    protected void setContentPackage(ContentPackage contentPackage) {
        // Put it in the opened list
        ReloadInternalFrameManager.addReloadInternalFrameToWindowMap(_contentPackage.getFile(), this);

        setTitle("Content Package - " + contentPackage.getProjectName());

        _resourcesPanel.setFileView(contentPackage.getProjectFolder());

        _manifestPanel.setContentPackage(contentPackage);

        // Proxy Save Handlers
        _saveHandler = new ProxySaveHandler(MainMenu.sharedInstance.actionSave);
        _saveAsHandler = new ProxySaveHandler(MainMenu.sharedInstance.actionSaveAs);

        // Proxy Zip handler
        _zipHandler = new ProxyZipHandler();

        // Proxy View CP handler
        _viewCPHandler = new ProxyViewCPHandler();

        // We'll listen to edits
        contentPackage.addReloadDocumentListener(this);

        // New Undo Manager
        _undoMenuManager = new ReloadUndoMenuManager(MainMenu.sharedInstance.editMenu.actionUndo, MainMenu.sharedInstance.editMenu.actionRedo);
        contentPackage.setUndoHandler(_undoMenuManager.getUndoHandler());

        _prefsPanel.init(contentPackage);
    }

    /**
     * Start a new CP Project with a new imsmanifest.xml
     */
    public void newContentPackage(File projectFolder) throws ReloadEditorException {
        _contentPackage = new ContentPackage();

        // Default CP Schema Controller
        ContentPackageSchemaController cpController = ContentPackageSchemaController.getDefaultSchemaController();

        // Default MD Schema Controller
        MetadataSchemaController mdController = ContentPackageSchemaController.getDefaultMetadataSchemaController();

        _contentPackage.newContentPackage(projectFolder, cpController, mdController);

        setContentPackage(_contentPackage);
    }


//==============================================================================
// EXPORT MANIFEST
//==============================================================================

    /**
     * Export (Disaggregate) a sub-Manifest to a new folder
     */
    public void exportManifest(Element manifestElement) {

        try {
            // Ask for Folder
            File targetFolder = ReloadEditorDocumentHandler.askNewContentPackageFolder("Select Folder for Disaggregated Content Package");
            // User cancelled or Folder not valid
            if(targetFolder == null || !targetFolder.exists()) return;

            // Do it
            boolean result = _contentPackage.exportManifest(manifestElement, targetFolder);

            // Show message
            if(result) {
                JOptionPane.showMessageDialog(this, "Sub-Manifest disaggregated successfully.", "Disaggregate Package", JOptionPane.INFORMATION_MESSAGE, null);
            }
            else {
                JOptionPane.showMessageDialog(this, "Error in disaggregation.", "Disaggregate Package", JOptionPane.ERROR_MESSAGE, null);
            }
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning(this, "Error Disaggregating", "Could not disaggregate Content Package", ex);
        }
    }


//==============================================================================
// IMPORT MANIFEST
//==============================================================================

    /**
     * Import a Package as a sub-Manifest.
     * manifestElement is where we are going to graft it
     */
    public void importManifest(Element manifestElement) {
        ReloadProgressMonitor progressMonitor = null;

        try {
            // Ask for Manifest/Zip to Import
            File manifestFile = ReloadEditorDocumentHandler.askManifestFileOpen(this);

            // User cancelled or Manifest file not valid
            if(manifestFile == null || !manifestFile.exists()) return;

            // Make sure it's not the target!
            if(manifestFile.equals(_contentPackage.getFile())) {
                JOptionPane.showMessageDialog(this, "Cannot import the same Manifest!", "Aggregate Package", JOptionPane.WARNING_MESSAGE);
                return;
            }

            // File base
            File fileBase = null;

            // Do we already have a manifest base folder?
            String base = _contentPackage.getElementBase(manifestElement);
            if(base != null) {
                fileBase = new File(_contentPackage.getProjectFolder(), base);
            }
            else fileBase = _contentPackage.getProjectFolder();

            // Ascertain sub-Folder
            File targetFolder = null;
            int i = 1;
            do {
                targetFolder = new File(fileBase, "submanifest" + i);
                i++;
            }
            while(targetFolder.exists());

            // If it's a zip file, unzip contents to new folder
            if(manifestFile.getName().toLowerCase().endsWith(".zip")){
                // Check that archive contains a manifest file
                if(!IMSDocumentHandler.containsManifest(manifestFile)) {
                    // no manifest found in archive
                    JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                              "Manifest file " + IMSDocumentHandler.MANIFEST_NAME +
                              " cannot be located in archive.", "Archive error", JOptionPane.WARNING_MESSAGE);
                    return;
                }

                // Create the sub-folder
                targetFolder.mkdirs();

                progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Unzipping Content Package",
                    "Unzipping...", "Searching...", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                // Unzip and get imsmanifest
                manifestFile = ReloadEditorDocumentHandler.unzipContentPackage(manifestFile, targetFolder, progressMonitor);
                if(manifestFile == null) return;
            }

            // Not a zip, so copy files to new folder
            else {
                progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Copying Content Package",
                    "Copying...", "Searching...", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                // Create the sub-folder
                targetFolder.mkdirs();

                // Copy files
                ReloadUtils.copyFolder(manifestFile.getParentFile(), targetFolder, progressMonitor);
            }

            // Graft Manifest file to manifestElement
            _contentPackage.importManifest(manifestFile, manifestElement, targetFolder.getName());
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning(this, "Error importing Content Package", "Could not import Content Package", ex);
        }
        finally {
            if(progressMonitor != null) progressMonitor.close();
            // Refresh Resources Panel
            _resourcesPanel.refreshResourceTree();
        }

    }

//==============================================================================
//==============================================================================
//==============================================================================

    /**
     * Zip up the Content Package
     */
    protected void zipIt() {
        if(_contentPackage != null) {
            // Check whether to save
            boolean doSave = checkNeedsSaving();

            if(doSave == false) return;

            ReloadProgressMonitor progressMonitor = null;

            try {
                // Ask for a file name
                ReloadFileFilter filter = new ReloadFileFilter(new String[] {"zip", "jar"}, "Archive files");
                File file = ReloadUtils.askFileNameSave(this, "Save as Zip File", filter, "zip");
                if(file != null) {
                    _zipHandler.setEnabled(false);

                    progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Zip Content Package",
                        "Zipping...", "", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                    BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(file));
                    ZipOutputStream zOut = new ZipOutputStream(out);
                    ZipUtils.addFolderToZip(_contentPackage.getProjectFolder(), _contentPackage.getProjectFolder(), zOut, file);
                    zOut.flush();
                    zOut.close();
                }
            }
            catch(Exception ex) {
                if(ReloadEditor.DEBUG) ex.printStackTrace();
                ErrorDialogBox.showWarning(this, "Zip Content Package", "Error zipping Package", ex);
            }
            finally {
                if(progressMonitor != null) progressMonitor.close();
                _zipHandler.setEnabled(true);
            }
        }
    }


    /**
     * Add all .xsd (Schema) files to zip file
     */
/*
    private static void addSchemaFilesToZip(File rootFolder, File folder, ZipOutputStream zOut) throws ReloadDomException {
        File[] files = folder.listFiles();
        for(int i = 0; i < files.length; i++) {
            if(files[i].isDirectory()) addSchemaFilesToZip(rootFolder, files[i], zOut);
            else if(ReloadUtils.getFileExtension(files[i]).equals("xsd")) {
                String entryName = ReloadUtils.getRelativePath(rootFolder, files[i]);
                ZipUtils.addFileToZip(files[i], entryName, zOut);
            }
        }
    }
*/

    /**
     * @return The ContentPackage in this editor
     */
    public ContentPackage getContentPackage() {
        return _contentPackage;
    }


    /**
     * Dynamically load a CP Profile
     */
    public void loadProfile(ContentPackageHelperProfile cpProfile) {
        // New Schema Controller for this Profile
        ((ContentPackageSchemaController)getContentPackage().getReloadSchemaController()).setHelperProfile(cpProfile);
    }

    /**
     * @return all available profiles
     */
    public ContentPackageHelperProfile[] getHelperProfiles() {
        return ContentPackageHelperProfile.getContentPackageHelperProfiles();
    }

// =============================================================================
// ReloadDocument Listener Events so we know when to set Save Menu
// =============================================================================

    public void elementAdded(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_contentPackage.isDirty());
    }

    public void elementChanged(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_contentPackage.isDirty());
    }

    public void elementRemoved(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_contentPackage.isDirty());
    }

    public void documentSaved(ReloadSchemaDocument doc) {
        if(isSelected()) _saveHandler.setEnabled(_contentPackage.isDirty());
    }


// =============================================================================
// Focus Handler
// =============================================================================

    /**
     * We got the focus
     */
    private void setFocusGained() {
        if(_contentPackage != null) {
            _resourcesPanel.setFocusGained();

            _undoMenuManager.setFocusGained();

            _zipHandler.setEnabled(true);
            _zipHandler.addListener();

            _viewCPHandler.setEnabled(true);
            _viewCPHandler.addListener();

            _saveHandler.setEnabled(_contentPackage.isDirty());
            _saveHandler.addListener();

            _saveAsHandler.setEnabled(false);  // Need to think about what Save As... does!
            _saveAsHandler.addListener();
        }
    }

    /**
     * We lost the focus
     */
    private void setFocusLost() {
        _undoMenuManager.setFocusLost();

        _resourcesPanel.setFocusLost();

        _zipHandler.setEnabled(false);
        _zipHandler.removeListener();

        _viewCPHandler.setEnabled(false);
        _viewCPHandler.removeListener();

        _saveHandler.setEnabled(false);
        _saveHandler.removeListener();

        _saveAsHandler.setEnabled(false);
        _saveAsHandler.removeListener();
    }


// =============================================================================
// Window Closing, Cleanup and Saving Handlers
// =============================================================================

    /**
     * We have been told by the Application that it is closing.
     * Here we can save if need be.
     * @return true if all is OK, false if not.
     */
    public boolean applicationClosing() {
        return checkNeedsSaving();
    }

    /**
     * We can check if the File needs saving
     * @return true if all is OK (YES or NO), false if not (CANCEL).
     */
    protected boolean checkNeedsSaving() {
        boolean isOK = true;

        if(_contentPackage != null && _contentPackage.isDirty()) {
            int doSave = JOptionPane.showConfirmDialog(this,
                "The Manifest has been modified. Do you wish to save these changes?",
                "Save Content Package Manifest",
                JOptionPane.YES_NO_CANCEL_OPTION);

            if(doSave == JOptionPane.YES_OPTION) isOK = saveDocument();
            else if(doSave == JOptionPane.CANCEL_OPTION) isOK = false;
        }

        return isOK;
    }

    /**
    * Dispose of this Window and clean up
    */
    public void dispose() {
        try {
            ReloadEditor.getInstance().removeApplicationListener(this);

            // Unregister Proxy Handlers
            _saveHandler.removeListener();
            _saveAsHandler.removeListener();
            _zipHandler.removeListener();
            _viewCPHandler.removeListener();

            // Destroy Resources Panel
            _resourcesPanel.destroy();

            // Destroy Manifest Editor and Manifest
            _contentPackage.removeReloadDocumentListener(this);
            _manifestPanel.destroy();
            _contentPackage.destroy();

            // Undo Manager
            _undoMenuManager.destroy();

            super.dispose();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }
    }


    /**
     * If the manifest has been edited, ask whether we should save it
     * @return true if OK, false if not
     */
    protected boolean saveDocument() {
        // If we don't have a file name, ask for one
        if(_contentPackage.getFile() == null) return saveAsDocument();

        try {
            _contentPackage.saveDocument();
        }
        catch(ReloadDomException ex) {
            ErrorDialogBox.showWarning(this, "Could not save file: ", "Save Manifest", ex);
            return false;
        }
        return true;
    }

    /**
     * Save As...
     * @return true if file saved successfully, false otherwise
     */
    protected boolean saveAsDocument() {
        // Ask for name
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "xml files");
        File file = ReloadUtils.askFileNameSave(this, "Save Manifest", filter, "xml");
        // User cancelled
        if(file == null) return false;
        // Set Title bar
        setTitle("Content Package - " + file.getPath());

        try {
            _contentPackage.saveAsDocument(file);

            // Put it in the opened list
            ReloadInternalFrameManager.addReloadInternalFrameToWindowMap(file, this);
        }
        catch(ReloadDomException ex) {
            ErrorDialogBox.showWarning(this, "Could not save file: ", "Save Manifest", ex);
            return false;
        }
        return true;
    }

//==============================================================================
//======================PROXY MENU HANDLERS=====================================
//==============================================================================


    /**
     * Deals with Save Menu Events
     */
    class ProxySaveHandler extends ProxyAction {
        public ProxySaveHandler(MenuAction menuAction) {
            super(menuAction);
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            if(isSelected() && event.getSource() == MainMenu.sharedInstance.actionSave) saveDocument();
            if(isSelected() && event.getSource() == MainMenu.sharedInstance.actionSaveAs) saveAsDocument();
        }
    }

    /**
     * Deals with shared Zip Menu Item from the main menu
     */
    class ProxyZipHandler extends ProxyAction {
        public ProxyZipHandler() {
            super(MainMenu.sharedInstance.actionZipIt);
        }

        // When user clicks 'Zip' from the main menu...
        public void menuActionPerformed(ReloadMenuEvent event) {
            Thread thread = new Thread() {
                public void run() {
                    zipIt();
                }
            };

            if(isSelected()) thread.start();
        }
    }

    /**
     * Deals with shared View Menu Item from the main menu
     */
    class ProxyViewCPHandler extends ProxyAction {
        public ProxyViewCPHandler() {
            super(MainMenu.sharedInstance.actionViewCP);
        }

        // When user clicks 'View CP' from the main menu...
        public void menuActionPerformed(ReloadMenuEvent event) {
            Thread thread = new Thread() {
                public void run() {
                    ContentPackageViewer cpViewer = new ContentPackageViewer(_contentPackage);
                    cpViewer.launch();
                }
            };

            if(isSelected()) thread.start();
        }
    }


    /**
     * The Preferences Panel
     */
    class CPPrefsPanel extends JPanel {

        /**
         * Profile Combo box
         */
        ReloadComboBox profileCombobox;

        /**
         * Constructor
         */
        CPPrefsPanel() {
            setLayout(new FlowLayout(FlowLayout.LEFT));

            // PROFILE COMBO BOX
            add(new JLabel("Profile: "));
            profileCombobox = new ReloadComboBox(false);
            add(profileCombobox);

            // Populate it
            ContentPackageHelperProfile[] profiles = getHelperProfiles();
            profileCombobox.setItems(profiles);
        }

        /**
         * Initialise
         */
        public void init(ContentPackage cp) {
            ContentPackageHelperProfile cpProfile = (ContentPackageHelperProfile)((ContentPackageSchemaController)cp.getReloadSchemaController()).getReloadHelperProfile();
            profileCombobox.setSelectedItem(cpProfile);

            // Listen to changes on the Profile Combo
            profileCombobox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    ContentPackageHelperProfile profile = (ContentPackageHelperProfile)profileCombobox.getSelectedItem();
                    loadProfile(profile);
                }
            });
        }
    }

}
