/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.moonunit.dom.contentpackaging.*;


 /**
  * The Content Package Editor Viewer
  *
  * @author Phillip Beauvoir
  * @author Paul Sharples
  * @version 2003-09-25
  */
public class ContentPackageViewer {
    /**
     * The Content Package to Display
     */
    protected ContentPackage _contentPackage;

    /**
     * Constructor
     */
    public ContentPackageViewer(ContentPackage cp) {
        _contentPackage = cp;
    }

    /**
     * Launch the Package in the Browser
     */
    public void launch() {
        File navPage = createNavPage();
        File mainPage = createMainPage(navPage);
        ReloadUtils.launchFile(mainPage);
    }

    /**
     * Generate an HTML Navigation Frame page
     * @return the created File
     */
    protected File createNavPage() {
        File file = null;
        BufferedWriter out = null;

        try {
            // Create a new temporary file
            file = File.createTempFile("~rld-", ".html");
            if(file != null) {
                // Make sure we delete it on exit
                file.deleteOnExit();

                // New Writer
                out = new BufferedWriter(new FileWriter(file));

                // Head
                out.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
                out.write("<html>");
                out.write("<head>");
                out.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">");

                // Styles
                out.write("<style type=\"text/css\">");
                out.write("<!-- ");
                out.write("body { font-family:Verdana, Arial, Helvetica, sans-serif; font-size:13px; font-weight:bold; }");
                out.write("A { font-weight:bold; text-decoration:none; }");
                out.write("A:link { color:#0033CC; }");
                out.write("A:visited { color:#025999; }");
                out.write("A:hover { color:#ff9900; }");
                out.write(" -->");
                out.write("</style>");

                // End Head
                out.write("</head>");

                // Body
                out.write("<body bgcolor=\"#9eb0b5\">");

                // Project Name
                out.write("<strong><font color=\"#0000CC\">");
                out.write(_contentPackage.getProjectName());
                out.write("</font></strong>");
                out.write("<hr WIDTH=\"100%\">");

                // Package Links - use a clone so we can add referenced elements
                addLinks((Element)_contentPackage.getRootElement().clone(), out, 0);

                // End Body
                out.write("</body>");

                // End HTML
                out.write("</html>");

                // Close File
                out.flush();
                out.close();
            }

            return file;
        }
        catch(Exception ex) {
            try {
                if(out != null) {
                    out.flush();
                    out.close();
                }
            }
            catch(IOException ex1) {
            }
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }

        return file;
    }


    /**
     * Add the links to the Nav Page for Organizations and Items
     */
    private void addLinks(Element element, BufferedWriter out, int indent) throws IOException {
        String name = element.getName();

        // ORGANIZATION
        if(name.equals(ContentPackageCore.ORGANIZATION) && _contentPackage.isDocumentNamespace(element)) {
            String title = "Organization";
            // Display Title if there is one
            Element titleElement = element.getChild(ContentPackageCore.TITLE, element.getNamespace());
            if(titleElement != null) {
               if(!titleElement.getText().equals("")) title = titleElement.getText();
            }
            writeItem(out, title, indent);
        }

        // ITEM
        else if(name.equals(ContentPackageCore.ITEM) && _contentPackage.isDocumentNamespace(element)) {
            StringBuffer sb = new StringBuffer();

            // Display Title if there is one
            String title = "Item";
            Element titleElement = element.getChild(ContentPackageCore.TITLE, element.getNamespace());
            if(titleElement != null) {
                if(!titleElement.getText().equals("")) title = titleElement.getText();
            }

            // What does this Item reference?
            Element ref_element = _contentPackage.getReferencedElement(element);
            if(ref_element != null) {
                String ref_name = ref_element.getName();

                // A RESOURCE
                if(ref_name.equals(ContentPackageCore.RESOURCE)) {
                    String url = _contentPackage.getURL(element);
                    if(url != null) {
                        sb.append("<a href=\"");
                        sb.append(url);
                        sb.append("\" target=\"main\">");
                        sb.append(title);
                        sb.append("</a>");
                    }
                    else sb.append(title);
                }

                // A sub-MANIFEST
                else if(ref_name.equals(ContentPackageCore.MANIFEST)) {
                    sb.append(title);

                    // Get ORGANIZATIONS Element
                    Element orgsElement = ref_element.getChild(ContentPackageCore.ORGANIZATIONS, ref_element.getNamespace());
                    // Now we have to get the default ORGANIZATION
                    if(orgsElement != null) ref_element = _contentPackage.getDefaultOrganization(orgsElement);

                    // Get the children of the referenced <organization> element and graft clones
                    if(ref_element != null) {
                        Iterator it = ref_element.getChildren().iterator();
                        while(it.hasNext()) {
                            Element ref_child = (Element) it.next();
                            element.addContent((Element) ref_child.clone());
                        }
                    }
                }

            }

            // No referenced Element
            else sb.append(title);

            writeItem(out, sb.toString(), indent);
        }

        // Sub-MANIFESTs
        else if(name.equals(ContentPackageCore.MANIFEST) && _contentPackage.isDocumentNamespace(element)) {
            if(element.getParent() != null) {
                writeItem(out, "<font color=\"#006600\">Manifest</font>", indent);
                indent++;
            }
        }

        boolean indented = false;

        // CHILDREN
        Iterator it = element.getChildren().iterator();
        while(it.hasNext()) {
            Element child = (Element) it.next();
            String child_name = child.getName();

            // Indent
            if(!indented && child_name.equals(ContentPackageCore.ITEM)) {
                indent++;
                indented = true;
            }

            addLinks(child, out, indent);
        }
    }

    /**
     * Write an Item to the Nav Page
     */
    private void writeItem(BufferedWriter out, String s, int indent) throws IOException {
        out.write("<p>");
        for(int i = 0; i < indent; i++) out.write("&nbsp;&nbsp;&nbsp;");
        out.write(s);
        out.write("</p>");
    }

    /**
     * Create the Main Viewing Frame HTML File
     * <h3>Amendment History</h3>
     * <p>Line 294-297 - MacOS X bug fix.  Each time the user wants to
     * 'quick view' a CP a temporary frameset is created in the systems default
     * <b>tmp</b> folder.  On MacOS X this folder is <i>/tmp/</i>.  In fact
     * on the Mac this is actually a symbolic link to the folder <i>/private/tmp/</i>.
     * When the user launches the frameset the symlink is resolved for the
     * frameset and the browser loads the starting page, however it cannot resolve
     * the symlinks for any &lt;frame src=""&gt; attributes and so fails.   The fix (kludge)
     * below ascertains OS type and then if MacOS is detected forces the &lt;frame src&gt;
     * to point the the resolved path on the Mac.</p>
     * @return The File
     */
    private File createMainPage(File navPage) {
        File file = null;
        BufferedWriter out = null;

        try {
            // Create a new temporary file
            file = File.createTempFile("~rld-", ".html");
            if(file != null) {
                // Make sure we delete it on exit
                file.deleteOnExit();

                // New Writer
                out = new BufferedWriter(new FileWriter(file));

                // Head
                out.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
                out.write("<html>");
                out.write("<head>");
                out.write("<title>");
                out.write("Content Package - " + _contentPackage.getProjectName());
                out.write("</title>");
                out.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">");
                out.write("<frameset framespacing=\"0\" border=\"0\" cols=\"220,*\">");
                out.write("<frame scrolling=\"auto\" name=\"navbar\" src=\"");
                // Depending on OS
                if(ReloadUtils.getOS() == ReloadUtils.MACINTOSH) out.write("file:///private" + navPage.toString());
                else out.write("file:///" + navPage.toString());
                //
                out.write("\">");
                out.write("<frame name=\"main\" scrolling=\"auto\">");
                out.write("</frameset>");
                out.write("</head>");
                out.write("<body></body>");

                // Close
                out.write("</html>");

                // Close File
                out.flush();
                out.close();
            }

            return file;
        }
        catch(Exception ex) {
            try {
                if(out != null) {
                    out.flush();
                    out.close();
                }
            }
            catch(IOException ex1) {
            }
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }

        return file;
    }

}