/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;
import java.util.*;

import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.IMSDocumentHandler;
import uk.ac.reload.moonunit.dom.contentpackaging.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The SCORM 1.2 CP ReloadSchema Controller
 *
 * @author Phillip Beauvoir
 * @version 2003-11-04
 */
public class SCORM1_2SchemaController extends ContentPackageSchemaController {

     /**
      * Default Constructor with Default Profile
      */
     public SCORM1_2SchemaController(File fileSchema, String version) {
         try {
             setVersion(version);

             loadSchema(fileSchema);

             // Get the Default Profile
             SCORM1_2HelperProfile defProfile = SCORM1_2HelperProfile.getDefaultSCORM1_2HelperProfile();
             setHelperProfile(defProfile);
         }
         catch(ReloadEditorException ex) {
             ErrorDialogBox.showWarning("Could not load Default Content Package Profile", "Load Profile", ex);
         }
     }

     /**
      * @return the Default SCORM1_2SchemaController - 1.1.3
      */
     public static SCORM1_2SchemaController getDefaultSCORMSchemaController() {
         return (SCORM1_2SchemaController)getReloadSchemaController(IMSDocumentHandler.ADLCP_120_113);
     }

     /**
      * @return the Default MD SchemaController - 1.2.2
      */
     public static MetadataSchemaController getDefaultMetadataSchemaController() {
         return (MetadataSchemaController)getReloadSchemaController(IMSDocumentHandler.IMSMD_122);
     }

     /**
      * Over-ride to get The Leaf Icon
      */
     public Icon getLeafIcon(Element element) {
         String elementName = element.getName();

         // Manifest
         if(elementName.equals(ContentPackageCore.MANIFEST)) {
             return ReloadUIManager.getIcon(ReloadEditor.ICON_CPSCORM);
         }

         // See if Item has SCORM elements
         else if(elementName.equals(ContentPackageCore.ITEM)) {
             if(hasScormElements(element)) return ReloadUIManager.getIcon(ReloadEditor.ICON_ITEMSCORM);
             else return super.getLeafIcon(element);
         }

         // ADL Namespace
         if(IMSDocumentHandler.ADLCP_NAMESPACE_12.equals(element.getNamespace())) {
             return ReloadUIManager.getIcon(ReloadEditor.ICON_ADL);
         }

         return super.getLeafIcon(element);
     }

     /**
      * Over-ride to get The Leaf Icon
      */
     public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
         String elementName = schemaElement.getName();

         if(elementName.equals(ContentPackageCore.MANIFEST)) {
             return ReloadUIManager.getIcon(ReloadEditor.ICON_CPSCORM);
         }

         // ADL Namespace
         if(IMSDocumentHandler.ADLCP_NAMESPACE_12.equals(schemaElement.getNamespace())) {
             return ReloadUIManager.getIcon(ReloadEditor.ICON_ADL);
         }

         return super.getLeafIcon(schemaElement);
     }

     /**
      * Copy the Schema Files to the Project Folder
      */
     public void copySchemaFilesToFolder(File projectFolder) throws ReloadEditorException {
         try {
             String version = getVersion();

             // ADL SCORM CP 113
             if(version.equals(ReloadEditorDocumentHandler.ADLCP_120_113)) {
                // Copy Metadata Schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.IMSMD_122), projectFolder, null);
                // Copy ADL schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.ADLCP_120_113), projectFolder, null);
             }

             // ADL SCORM CP 112
             else if(version.equals(ReloadEditorDocumentHandler.ADLCP_120_112)) {
                // Copy Metadata Schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.IMSMD_121), projectFolder, null);
                // Copy ADL schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.ADLCP_120_112), projectFolder, null);
             }
         }
         catch(IOException ex) {
             throw new ReloadEditorException(ex);
         }
     }

     /**
      * @return true if element has SCORM sub-elements
      */
     public boolean hasScormElements(Element element) {
         Iterator it = element.getChildren().iterator();
         while(it.hasNext()) {
             Element child = (Element)it.next();
             if(child.getNamespace().equals(IMSDocumentHandler.ADLCP_NAMESPACE_12)) return true;
         }
         return false;
     }
}