/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging.manifestview;

import java.util.*;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.tree.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.*;
import uk.ac.reload.editor.gui.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.contentpackaging.*;
import uk.ac.reload.moonunit.schema.*;


/**
 * A Viewer Tree that will display the CP Manifest.
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-11-02
 */
public class ManifestTree
extends ReloadDragTree
implements TreeSelectionListener, ReloadDocumentListener, FocusListener
{
    /**
     * The Editor
     */
    protected ContentPackageEditor _cpEditor;

    /**
     * The Element Value/Attribute Edit Panel
     */
    protected ManifestInfoPanel _infoPanel;

    /**
     * The Tree Model
     */
    protected ManifestTreeModel _treeModel;

    /**
     * Popup Menu on Right-clicks
     */
    protected JPopupMenu _popupMenu;

    /**
     * Keeping note of the current selected tree node means we don't have to do
     * unneccesary updating
     */
    protected TreePath _currentTreePath;

    /**
     * The Schema Controller
     */
    protected ReloadEditorSchemaController _schemaController;

    /**
     * The Edit menu which we will use
     */
    protected Menu_Edit _editMenu;

    /**
     * The Proxy Delete Handler
     */
    protected ProxyDeleteHandler _deleteHandler;

    /**
     * The Proxy Cut Handler
     */
    protected ProxyCutHandler _cutHandler;

    /**
     * The Proxy Copy Handler
     */
    protected ProxyCopyHandler _copyHandler;

    /**
     * The Proxy Paste Handler
     */
    protected ProxyPasteHandler _pasteHandler;

    /**
     * The Proxy Move Up Handler
     */
    protected ProxyMoveUpHandler _moveUpHandler;

    /**
     * The Proxy Move Down Handler
     */
    protected ProxyMoveDownHandler _moveDownHandler;

    /**
     * The Proxy View File Handler
     */
    protected ProxyLaunchFileHandler _launchFileHandler;

    /**
     * The Proxy Edit Metadata Handler
     */
    protected ProxyEditMetadataHandler _editMetadataHandler;

    /**
     * The Proxy Edit SCORM Handler
     */
    protected ProxyEditSCORMHandler _editSCORMHandler;

    /**
     * The CP Document
     */
    protected ContentPackage _contentPackage;


    /**
     * Constructor
     */
    public ManifestTree(ContentPackageEditor cpEditor, ManifestInfoPanel infoPanel) {
        super();

        _treeModel = new ManifestTreeModel();
        setModel(_treeModel);

        _cpEditor = cpEditor;
        _infoPanel = infoPanel;

        addTreeSelectionListener(this);

        // Focus Listener
        addFocusListener(this);

        // Cell renderer
        setCellRenderer(new ManifestTreeRenderer());

        // Tooltips
        ToolTipManager.sharedInstance().registerComponent(this);

        // Popup menu
        _popupMenu = new JPopupMenu();
        // Workaround - when a popupmenu is invoked, focus (temporary type) is lost
        // to the popup.  If user then moves to another window, focus is not lost
        // properly and therefore menu items are not cleared
        //popupMenu.setFocusable(false);

        // Listen to mouse-clicks for the Popup Menu
        addMouseListener(new MouseAdapter() {
           // PC
           public void mouseReleased(MouseEvent e) {
               checkPopupTrigger(e);
           }

           // Mac
           public void mousePressed(MouseEvent e) {
               checkPopupTrigger(e);
           }
        });

        // Get the Edit menu which we will use
        _editMenu = MainMenu.sharedInstance.editMenu;

        // Remap key bindings
        _editMenu.remapKeyStrokes(this);

        // We are a Drag and Drop Source and Target
        DropTarget dropTarget = new DropTarget(this, this);
        DragSource dragSource = DragSource.getDefaultDragSource();
        dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY_OR_MOVE, this);
    }

    /**
     * Set a new Content Package to be displayed
     * @param cp
     */
    public void setContentPackage(ContentPackage cp) {
        _contentPackage = cp;
        // Get the Schema Controller first!
        _schemaController = (ReloadEditorSchemaController)_contentPackage.getReloadSchemaController();
        // Tell Tree Model
        _treeModel.setContentPackage(_contentPackage);
        // Select Root Node
        selectNode(_treeModel.getRootNode());
        // Add ourselves as Element Listener
        _contentPackage.addReloadDocumentListener(this);
    }

    /**
     * We got the focus
     */
    public void focusGained(FocusEvent e) {
        //if(ReloadEditor.DEBUG) System.out.println(e.paramString());
        if(!e.isTemporary()) {
            // Update Menus
            updateMenus(getSelectionPath());
        }
    }

    /**
     * We lost the focus
     */
    public void focusLost(FocusEvent e) {
        //if(ReloadEditor.DEBUG) System.out.println(e.paramString());
        // If moving to an Internal Frame
        if(!e.isTemporary()) {
            // Clear our temporary menus
            clearMenus();
        }
    }

    /**
     * Clean up
     */
    public void destroy() {
        _contentPackage.removeReloadDocumentListener(this);
        clearMenus();
    }

    /**
     * A node has been selected on the tree.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {
        TreePath selPath = event.getPath();
        if(selPath != _currentTreePath) {
            updateMenus(selPath);
            updateInfoPanel(selPath);
            _currentTreePath = selPath;
        }
    }

    /**
     * Update the Info Panel according to the selected node on the tree.
     * @param selPath the path of the selected node on the tree
     */
    protected synchronized void updateInfoPanel(TreePath selPath) {
        if(_infoPanel != null) {
            if(selPath == null) {
                _infoPanel.clear();
                return;
            }

            Object selectedNode = selPath.getLastPathComponent();
            if(selectedNode instanceof ManifestTreeNode) {
                // Cast it
                ManifestTreeNode treeNode = (ManifestTreeNode) selectedNode;

                // And get the ReloadElementBinding
                ReloadElementBinding eb = treeNode.createReloadElementBinding();

                // Tell the panel
                _infoPanel.setElement(eb);
            }
        }
    }

    /**
     * Update the menus according to the selected node on the tree.
     * This has to be synchronized so different threads don't access it at the same time
     * @param selPath the path of the selected node on the tree
     */
    protected synchronized void updateMenus(TreePath selPath) {
        // Clear the deck regardless
        clearMenus();

        if(selPath == null) return;

        Object selectedNode = selPath.getLastPathComponent();
        if(selectedNode instanceof ManifestTreeNode) {
            // Cast it
            ManifestTreeNode treeNode = (ManifestTreeNode) selectedNode;

            // And get our bits
            Element element = treeNode.getElement();
            ReloadSchemaElement schemaElement = treeNode.getSchemaElement();
            String name = element.getName();

            // New Cut Handler
            _cutHandler = new ProxyCutHandler(_editMenu.actionCut, element, schemaElement);
            // New Copy Handler
            _copyHandler = new ProxyCopyHandler(_editMenu.actionCopy, element);
            // New Paste Handler
            _pasteHandler = new ProxyPasteHandler(_editMenu.actionPaste, element);
            // New Delete Handler
            _deleteHandler = new ProxyDeleteHandler(_editMenu.actionDelete, element, schemaElement);

            // Add to popup
            _popupMenu.add(_cutHandler.getMenuAction());
            _popupMenu.add(_copyHandler.getMenuAction());
            _popupMenu.add(_pasteHandler.getMenuAction());
            _popupMenu.add(_deleteHandler.getMenuAction());

            _popupMenu.addSeparator();

            // New Move Up Handler
            _moveUpHandler = new ProxyMoveUpHandler(_editMenu.actionMoveUp, element);
            _popupMenu.add(_moveUpHandler.getMenuAction());

            // New Move Down Handler
            _moveDownHandler = new ProxyMoveDownHandler(_editMenu.actionMoveDown, element);
            _popupMenu.add(_moveDownHandler.getMenuAction());

            // If the Schema Element has Children display them in the menu
            if(schemaElement != null && schemaElement.hasChildren()) {
                // Separator
                _editMenu.addAdditionalItem(new JPopupMenu.Separator());
                _popupMenu.addSeparator();

                ReloadSchemaElement[] children = schemaElement.getChildren();

                for(int i = 0; i < children.length; i++) {
                    ReloadSchemaElement childSchemaElement = children[i];
                    // Only certain nodes
                    if(_treeModel.canAddNode(childSchemaElement)) {
                        Action_AddChildElement actionAddElement = new Action_AddChildElement(element, childSchemaElement);
                        _popupMenu.add(actionAddElement);
                        _editMenu.addAdditionalItem(actionAddElement);
                    }
                }
            }

            // If we selected the "metadata" node show an "Edit Metadata" Menu item
            if(_contentPackage.isMetadataElement(name)) {
                // Separator
                _editMenu.addAdditionalItem(new JPopupMenu.Separator());
                _popupMenu.addSeparator();

                Action_EditMetadata actionEditMD = new Action_EditMetadata(element);
                _popupMenu.add(actionEditMD);
                _editMenu.addAdditionalItem(actionEditMD);

                // Menu Item Proxy
                _editMetadataHandler = new ProxyEditMetadataHandler(element);
                _editMetadataHandler.setEnabled(true);
            }

            // SCORM Item
            else if(name.equals(ContentPackageCore.ITEM) && _contentPackage instanceof SCORM1_2Package) {
                if(schemaElement != null) {
                    // Separator
                    _editMenu.addAdditionalItem(new JPopupMenu.Separator());
                    _popupMenu.addSeparator();

                    Action_EditSCORM actionEditSCORM = new Action_EditSCORM(element, schemaElement);
                    _popupMenu.add(actionEditSCORM);
                    _editMenu.addAdditionalItem(actionEditSCORM);

                    // Menu Item Proxy
                    _editSCORMHandler = new ProxyEditSCORMHandler(element, schemaElement);
                    _editSCORMHandler.setEnabled(true);
                }
            }

            // If we selected a sub-Manifest node show a Menu item
            else if(name.equals(ContentPackageCore.MANIFEST) && _contentPackage.isDocumentNamespace(element)) {
                _editMenu.addAdditionalItem(new JPopupMenu.Separator());
                _popupMenu.addSeparator();

                // Import (Aggregate)
                Action_ImportManifest actionImportManifest = new Action_ImportManifest(element);
                _popupMenu.add(actionImportManifest);
                _editMenu.addAdditionalItem(actionImportManifest);

                // Export (Disaggregate)
                if(!element.isRootElement()) {
                    Action_ExportManifest actionExportManifest = new Action_ExportManifest(element);
                    _popupMenu.add(actionExportManifest);
                    _editMenu.addAdditionalItem(actionExportManifest);
                }
            }

            // Find Referenced Node
            if(_contentPackage.isReferencingElement(element)) {
                _editMenu.addAdditionalItem(new JPopupMenu.Separator());
                _popupMenu.addSeparator();

                Action_JumpToRefNode actionJumpNode = new Action_JumpToRefNode(element);
                _editMenu.addAdditionalItem(actionJumpNode);
                _popupMenu.add(actionJumpNode);
            }

            // View file
            _launchFileHandler = new ProxyLaunchFileHandler(element);
        }
    }

    /**
     * Clear any popup or other menus
     */
    protected synchronized void clearMenus() {
        // We can remove all from the popup menu
        _popupMenu.removeAll();

        // Remove any menu items we created
        _editMenu.removeAdditionalItems();

        // Clear the Handlers
        if(_cutHandler != null) _cutHandler.clearMenu();
        if(_copyHandler != null) _copyHandler.clearMenu();
        if(_pasteHandler != null) _pasteHandler.clearMenu();
        if(_deleteHandler != null) _deleteHandler.clearMenu();
        if(_moveUpHandler != null) _moveUpHandler.clearMenu();
        if(_moveDownHandler != null) _moveDownHandler.clearMenu();
        if(_launchFileHandler != null) _launchFileHandler.clearMenu();
        if(_editMetadataHandler != null) _editMetadataHandler.clearMenu();
        if(_editSCORMHandler != null) _editSCORMHandler.clearMenu();
    }

    /**
     * Check to see if we have triggered the popup menu.
     * @param e The MouseEvent that has been triggered.
     */
    protected void checkPopupTrigger(MouseEvent e) {
       if(e.isPopupTrigger()) {
           TreePath selPath = getPathForLocation(e.getX(), e.getY());
           if(selPath == null) return;
           // Select the node
           setSelectionPath(selPath);
           // Show the popup menu
           _popupMenu.show(this, e.getX(), e.getY());
       }
    }


    /**
     * Edit the Metadata in the Metadata dialog Editor
     */
    protected void editMetadata(final Element mdElement) {
        // Loading of Document on Thread
        Thread thread = new Thread() {
            MetadataEditorDialog mdDialog;

            public void run() {
                try {
                    if(_editMetadataHandler != null) _editMetadataHandler.setEnabled(false);  // block multiple key presses
                    ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);
                    mdDialog = new MetadataEditorDialog(mdElement, _contentPackage);
                    mdDialog.show();
                }
                catch(Exception ex) {
                    if(ReloadEditor.DEBUG) ex.printStackTrace();
                    if(mdDialog != null) mdDialog.dispose();
                    ErrorDialogBox.showWarning("Error in Metadata", "Could not open Metadata Editor", ex);
                }
                finally {
                    ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                }
            }
        };

        thread.start();
    }

    /**
     * Edit the SCORM Elements in the SCORM dialog Editor
     */
    protected void editSCORM(final Element itemElement, final ReloadSchemaElement schemaElement) {
        Thread thread = new Thread() {
            SCORM1_2EditorDialog editor;

            public void run() {
                if(_editSCORMHandler != null) _editSCORMHandler.setEnabled(false);  // block multiple key presses
                editor = new SCORM1_2EditorDialog(_contentPackage, itemElement, schemaElement);
                // Show the Window later in the Event queue
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        editor.show();
                    }
                });
            }
        };

        thread.start();
    }

//////////////////////// ELEMENT LISTENER //////////////////////////////////////

    /**
     * An Element was added to the Document so we must add a new Tree node
     * @param event the ReloadDocumentListenerEvent
     */
    public void elementAdded(ReloadDocumentListenerEvent event) {
        Element element = event.getElement();

        // Get Parent node
        Element parent = element.getParent();
        DefaultMutableTreeNode parentNode = getNode(parent);

        if(parentNode != null) {
            ManifestTreeNode newNode = addNode(parentNode, element);
            if(newNode != null && event.doSelect()) selectNode(newNode);
        }
    }

    /**
     * Add an Element
     */
    private ManifestTreeNode addNode(DefaultMutableTreeNode parentNode, Element element) {
        ManifestTreeNode newNode = null;

        // Ascertain correct position to insert
        int index = getInsertNodePosition(parentNode, element);

        // If not allowed to insert, don't bother
        if(index != -1) {
            // New node
            newNode = new ManifestTreeNode(element);
            // Insert into model
            _treeModel.insertNodeInto(newNode, parentNode, index);

            // Children
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                addNode(newNode, child);
            }
        }

        return newNode;
    }


    /**
     * @return where to correctly insert the node for an Element.
     * Some nodes might not be visible and therefore the node to Elements count is wrong
     * Return -1 if we shouldn't insert it
     */
    protected int getInsertNodePosition(DefaultMutableTreeNode parentNode, Element element) {
        // Don't display certain nodes
        if(_treeModel.doShowNode(element) == false) return -1;

        int index = 0;

        // Because we might not display certain nodes we have to compensate
        java.util.List children = element.getParent().getChildren();
        for(int i = 0; i < children.size(); i++) {
            Element child = (Element)children.get(i);
            if(child == element) break;
            if(_treeModel.doShowNode(child)) index++;
        }

        // Make sure we don't exceed the child count
        if(index > parentNode.getChildCount()) index = parentNode.getChildCount();
        if(index < 0) index = 0;

        return index;
    }

    /**
     * An Element was removed from the Document so we must find its tree node and
     * remove that from the Tree.
     */
    public void elementRemoved(ReloadDocumentListenerEvent event) {
        // Find the node
        Element element = event.getElement();

        DefaultMutableTreeNode nodeToDelete = getNode(element);
        if(nodeToDelete != null) {
            // The currently selected node is a descendent of/same as the node being deleted
            if(nodeToDelete.isNodeDescendant(getSelectedNode()))  {
                // Save sibling/parent
                DefaultMutableTreeNode prevNode = nodeToDelete.getPreviousSibling();
                if(prevNode == null) prevNode = (DefaultMutableTreeNode)nodeToDelete.getParent();

                // Delete node
                _treeModel.removeNodeFromParent(nodeToDelete);
                // Select previous node this will take care of updating menus
                selectNode(prevNode);
            }

            // Else we were sat on another node and need to update menus
            else {
                // Delete node
                _treeModel.removeNodeFromParent(nodeToDelete);
                // Update Menus
                updateMenus(getSelectionPath());
            }
        }
    }

    /**
     * An Element was changed in some way (maybe the text or Attribute) so we
     * inform the Tree Table Model
     * @param event the ReloadDocumentListenerEvent
     */
    public void elementChanged(ReloadDocumentListenerEvent event) {
        final DefaultMutableTreeNode node = getNode(event.getElement());
        if(node != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    _treeModel.nodeChanged(node);
                }
            });
        }
    }

    public void documentSaved(ReloadSchemaDocument doc) {}

//==============================================================================
//       ACTION MENU HANDLERS
//==============================================================================

    /**
     * An Action Class that allows us to add a new Child Element to a Parent Element
     * There will be some validity checks.
     */
    class Action_AddChildElement extends AbstractAction {
        Element parentElement;
        ReloadSchemaElement childSchemaElement;
        String name;

        public Action_AddChildElement(Element parentElement, ReloadSchemaElement childSchemaElement) {
            this.parentElement = parentElement;
            this.childSchemaElement = childSchemaElement;

            // Set up name that will be displayed on the Menu
            name = _schemaController.getElementFriendlyName(childSchemaElement.getDomPath());
            if(name == null) name = childSchemaElement.getName();
            putValue(Action.NAME, "Add " + name);

            putValue(Action.SMALL_ICON, _schemaController.getLeafIcon(childSchemaElement));

            // Set Enabled according to whether we are allowed to add this child
            setEnabled(_contentPackage.canAddElement(parentElement, childSchemaElement));
        }

        public void actionPerformed(ActionEvent e) {
            // Add new Element
            Element element = _contentPackage.addElementBySchemaUndoable(ManifestTree.this, parentElement, childSchemaElement, true);
            // If Metadata Element, launch Editor
            //if(element != null) {
            //    if(_contentPackage.isMetadataElement(element.getName())) editMetadata(element);
            //}
        }
    }


    /**
     * An Action Class that allows us to edit the Metadata
     */
    class Action_EditMetadata extends AbstractAction {
        Element _element;

        public Action_EditMetadata(Element element) {
            super("Edit Metadata...", ReloadUIManager.getIcon(ReloadEditor.ICON_MD));
            _element = element;
        }

        public void actionPerformed(ActionEvent e) {
            editMetadata(_element);
        }
    }

    /**
     * An Action Class that allows us to edit the SCORM Elements
     */
    class Action_EditSCORM extends AbstractAction {
        Element _element;
        ReloadSchemaElement _schemaElement;

        public Action_EditSCORM(Element element, ReloadSchemaElement schemaElement) {
            super("Edit SCORM...", ReloadUIManager.getIcon(ReloadEditor.ICON_ADL));
            _element = element;
            _schemaElement = schemaElement;
        }

        public void actionPerformed(ActionEvent e) {
            editSCORM(_element, _schemaElement);
        }
    }

    /**
     * An Action Class that allows us to Import (aggregate) a Manifest
     */
    class Action_ImportManifest extends AbstractAction {
        Element element;

        public Action_ImportManifest(Element element) {
            super("Aggregate...", _schemaController.getLeafIcon(element));
            this.element = element;
        }

        public void actionPerformed(ActionEvent e) {
            // Loading of Document on Thread
            Thread thread = new Thread() {
                public void run() {
                    _cpEditor.importManifest(element);
                }
            };

            thread.start();
        }
    }

    /**
     * An Action Class that allows us to Export (disaggregate) a Manifest
     */
    class Action_ExportManifest extends AbstractAction {
        Element element;

        public Action_ExportManifest(Element element) {
            super("Disaggregate...", _schemaController.getLeafIcon(element));
            this.element = element;
        }

        public void actionPerformed(ActionEvent e) {
            // Loading of Document on Thread
            Thread thread = new Thread() {
                public void run() {
                    _cpEditor.exportManifest(element);
                }
            };

            thread.start();
        }
    }

    /**
     * An Action Class that allows us to jump to a referenced Node
     */
    class Action_JumpToRefNode extends AbstractAction {
        Element element;

        public Action_JumpToRefNode(Element element) {
            super("Find reference", ReloadUIManager.getIcon(ReloadEditor.ICON_JUMP));
            this.element = element;
        }

        public void actionPerformed(ActionEvent e) {
            Element ref_element = _contentPackage.getReferencedElement(element);
            if(ref_element != null) selectNodeByObject(ref_element);
        }
    }

//==============================================================================
//       PROXY ACTION HANDLERS
//==============================================================================

    /**
     * Move Up Element Proxy Action
     */
    class ProxyMoveUpHandler extends ProxyAction {
        Element element;

        public ProxyMoveUpHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to move this up
            setEnabled(_contentPackage.canMoveElementUp(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _contentPackage.moveElementUp(ManifestTree.this, element, true);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Move Down Element Proxy Action
     */
    class ProxyMoveDownHandler extends ProxyAction {
        Element element;

        public ProxyMoveDownHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to move this down
            setEnabled(_contentPackage.canMoveElementDown(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _contentPackage.moveElementDown(ManifestTree.this, element, true);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Delete Element Proxy Action
     */
    class ProxyDeleteHandler extends ProxyAction {
        Element element;

        public ProxyDeleteHandler(MenuAction proxyMenuAction, Element element, ReloadSchemaElement schemaElement) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to delete this element
            setEnabled(_contentPackage.canDeleteElement(element, schemaElement));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _contentPackage.deleteElementUndoable(ManifestTree.this, element);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Cut Element Proxy Action
     */
    class ProxyCutHandler extends ProxyAction {
        Element element;

        public ProxyCutHandler(MenuAction proxyMenuAction, Element element, ReloadSchemaElement schemaElement) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to cut this element
            setEnabled(_contentPackage.canCutElement(element, schemaElement));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _contentPackage.cutElementUndoable(ManifestTree.this, element);
            ReloadDocumentClipboard.addCutElement(element);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Copy Element Proxy Action
     */
    class ProxyCopyHandler extends ProxyAction {
        Element element;

        public ProxyCopyHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to copy this element
            setEnabled(_contentPackage.canCopyElement(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            ReloadDocumentClipboard.addCopiedElement(element);
            // Update Menus
            updateMenus(getSelectionPath());
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Paste Element Proxy Action
     */
    class ProxyPasteHandler extends ProxyAction {
        Element element;

        public ProxyPasteHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to paste this element
            setEnabled(_contentPackage.canPasteFromClipboard(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            if(_contentPackage.canPasteFromClipboard(element)) {
                Element clipBoardElement = ReloadDocumentClipboard.getElement();
                _contentPackage.pasteElementUndoable(ManifestTree.this, clipBoardElement, element, false);
                //_contentPackage.shiftElement(clipBoardElement, element, ReloadDocumentClipboard.ACTION_COPY);
                // Update Menus
                updateMenus(getSelectionPath());
            }
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Edit Metadata Proxy Action
     */
    class ProxyEditMetadataHandler extends ProxyAction {
        Element _element;

        public ProxyEditMetadataHandler(Element element) {
            super(MainMenu.sharedInstance.actionEditMetadata);
            _element = element;
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            editMetadata(_element);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Edit SCORM Proxy Action
     */
    class ProxyEditSCORMHandler extends ProxyAction {
        Element _element;
        ReloadSchemaElement _schemaElement;

        public ProxyEditSCORMHandler(Element element, ReloadSchemaElement schemaElement) {
            super(MainMenu.sharedInstance.actionEditSCORM);
            _element = element;
            _schemaElement = schemaElement;
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            editSCORM(_element, _schemaElement);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }


    /**
     * Launch File Proxy Action
     */
    class ProxyLaunchFileHandler extends ProxyAction {
        String url;

        public ProxyLaunchFileHandler(Element element) {
            super(MainMenu.sharedInstance.actionViewFile);

            // Set Enabled according to whether there is a file to view
            url = _contentPackage.getURL(element);
            setEnabled(url != null);
        }

        /**
         * Launch the file
         * @param event
         */
        public void menuActionPerformed(ReloadMenuEvent event) {
            if(url != null) {
                // Start the loader thread
                Thread thread = new Thread() {
                    public void run() {
                        ReloadUtils.launchFile(url);
                    }
                };

                thread.start();
            }
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

//==============================================================================
//       DRAG AND DROP HANDLERS
//==============================================================================

    // =================== Drag and Drop Source events =========================

    /**
     * A drag gesture has been initiated from this tree
     */
    public void dragGestureRecognized(DragGestureEvent event) {
        int action = event.getDragAction();
        if((action & DnDConstants.ACTION_COPY_OR_MOVE) == 0) return;

        ReloadElementTreeNode node = (ReloadElementTreeNode)getDragSourceTreeNode(event);
        if(node != null) {
            ReloadElementBinding eb = node.createReloadElementBinding();
            if(_contentPackage.canDragElement(eb.getElement(), action)) {
                DragTreeObject dragObject = new DragTreeObject(eb, this);

                Point ptDragOrigin = event.getDragOrigin();
                TreePath path = getPathForLocation(ptDragOrigin.x, ptDragOrigin.y);
                if(path == null) return;

                // Get the Drag Image
                BufferedImage imgGhost = getDragImage(path, ptDragOrigin);

                event.startDrag(DragSource.DefaultCopyNoDrop, imgGhost, new Point(5, 5), dragObject, this);
            }
        }
    }

    // =================== Drag and Drop Target events =========================

    /**
     * We are dragging and want to know whether we can drop
     */
    public boolean isDropOK(DropTargetDragEvent event) {
        // Get the node we're dragging over
        ReloadElementTreeNode targetNode = (ReloadElementTreeNode)getDragOverTreeNode(event);
        if(targetNode == null) return false;

        // Workaround for Java bug ID #4357494 and ID #4248542 and ID #4378091
        DropTargetDropEvent tempDTDropEvent = new  DropTargetDropEvent(event.getDropTargetContext(),
                        event.getLocation(), DnDConstants.ACTION_COPY, 0);
        Transferable transferable = tempDTDropEvent.getTransferable();

        // Find out whether we are the correct object
        if(transferable.isDataFlavorSupported(DragTreeObject.flavor)){
            // Check for DnD Action
            int action = getCorrectDropContext(event);
            // Copy & Move is OK
            if((action & DnDConstants.ACTION_COPY) != 0 || (action & DnDConstants.ACTION_MOVE) != 0) {
                // Get the User Object
                try {
                    Object userObject = transferable.getTransferData(DragTreeObject.flavor);

                    // A ReloadResource array
                    if(userObject instanceof ReloadResource[]) {
                        // Return whether the CP can accept this
                        return _contentPackage.acceptsReloadResources(targetNode.getElement());
                    }

                    // An internal DnD
                    else if(userObject instanceof ReloadElementBinding) {
                        // Return whether the CP can accept this
                        return _contentPackage.acceptElement(((ReloadElementBinding)userObject).getElement(), targetNode.getElement(), action);
                    }
                }
                catch(Exception ex) {
                    if(ReloadEditor.DEBUG) ex.printStackTrace();
                }
            }
            return false;
        }

        else return false;
    }

    /**
     * We just Dropped something!
     */
    public void drop(DropTargetDropEvent event) {
        // Where did we drop it?
        Point location = event.getLocation();
        TreePath treePath = getPathForLocation(location.x, location.y);
        if(treePath == null) return;
        ReloadElementTreeNode targetNode = (ReloadElementTreeNode)treePath.getLastPathComponent();

        // Get what we dropped
        Transferable transferable = event.getTransferable();

        // If we are the correct drag object
        if(transferable.isDataFlavorSupported(DragTreeObject.flavor)){
            // Accept the Drop
            event.acceptDrop(getCorrectDropContext(event));

            // Get the User Object and do something
            try {
                Object userObject = transferable.getTransferData(DragTreeObject.flavor);

                // Get Action - Copy or Move
                int action = getCorrectDropContext(event);

                setCursor(ReloadUIManager.WAIT_CURSOR);

                // Dropped an array of ReloadResource(s)
                if(userObject instanceof ReloadResource[]) {
                    Element newElement = _contentPackage.addReloadResources((ReloadResource[]) userObject, targetNode.getElement());
                    selectNodeByObject(newElement);
                }

                // Internal Drop
                else if(userObject instanceof ReloadElementBinding) {
                    _contentPackage.shiftElement(((ReloadElementBinding) userObject).getElement(), targetNode.getElement(), action);
                }
            }
            catch(Exception ex) {
                if(ReloadEditor.DEBUG) ex.printStackTrace();
            }
            finally {
                setCursor(ReloadUIManager.DEFAULT_CURSOR);
            }

            hiliteNode(prevHilitedNode, false);
            event.getDropTargetContext().dropComplete(true);
        }

        // Else we are not the right object
        else event.rejectDrop();
    }

    /**
     * <h3>Amendment History</h3>
     * <p>Due to Mac specific issues, this method has been added to ensure
     * that the correct drop target event is recorded.  On MacOS X
     * "event.getDropAction()" does not return 1 for move and 2 for copy as
     * it should do (like it does on PC).  This method checks to see
     * if the OS is Mac and then makes sure the correct int value is returned. It
     * ask the superclass method - getCorrectDropContext(event) for this value</p>
     *
     * @param event a DropTargetDragEvent
     * @return int representing the type of action - move or copy
     */
    public int getCorrectDropContext(DropTargetDragEvent event) {
        return super.getCorrectDropContext(event);
    }

    /**
     * <h3>Amendment History</h3>
     * <p>Due to Mac specific issues, this method has been added to ensure
     * that the correct drop target event is recorded.  On MacOS X
     * "event.getDropAction()" does not return 1 for move and 2 for copy as
     * it should do (like it does on PC).  This method checks to see
     * if the OS is Mac and then makes sure the correct int value is returned. It
     * ask the superclass method - getCorrectDropContext(event) for this value</p>
     *
     * @param event a DropTargetDropEvent
     * @return int representing the type of action - move or copy
     */
    public int getCorrectDropContext(DropTargetDropEvent event){
        return super.getCorrectDropContext(event);
    }
}
