/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging.resourceview;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.tree.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.*;



/**
 * A Content Package Resources Tree for the CP UI
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-10-07
 */
public class CPResourcesTree
extends ReloadDragTree
implements TreeSelectionListener
{
    /**
     * The Tree Model
     */
    protected CPResourcesTreeModel _treeModel;

    /**
     * Our parent Panel with Menu Actions on
     */
    protected CPResourcesPanel _resourcesPanel;

    /**
     * Popup Menu on Right-clicks
     */
    protected JPopupMenu _popupMenu;


    /**
     * Constructor
     */
    public CPResourcesTree(CPResourcesPanel resourcesPanel) {
        _resourcesPanel = resourcesPanel;

        _treeModel = new CPResourcesTreeModel();
        setModel(_treeModel);

        // Tree Selection Listener
        addTreeSelectionListener(this);

        setCellRenderer(new CPResourcesTreeRenderer());

        // Tooltips
        ToolTipManager.sharedInstance().registerComponent(this);

        // Popup menu
        _popupMenu = createPopupMenu();

        // Listen to mouse-clicks for the Popup Menu
        addMouseListener(new MouseAdapter() {
           // PC
           public void mouseReleased(MouseEvent e) {
               checkPopupTrigger(e);
           }

           // Mac
           public void mousePressed(MouseEvent e) {
               checkPopupTrigger(e);
           }
        });

        // Drag Source
        DragSource dragSource = DragSource.getDefaultDragSource();
        dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY_OR_MOVE, this);

        // Drag Target
        DropTarget dropTarget = new DropTarget(this, this);
    }

    /**
     * Set a new File View to be displayed
     */
    public void setFileView(File rootFolder) {
        // Tell Model
        _treeModel.setFileView(rootFolder);

        // Select Root Node
        selectNode(_treeModel.getRootNode());
    }

    /**
     * We got the focus
     */
    public void setFocusGained() {
        // Update any Menus
        updateMenus(getSelectionPath());
    }

    /**
     * We lost the focus
     */
    public void setFocusLost() {

    }

    /**
     * Refresh the tree and model
     */
    public void refresh() {
        refresh((CPResourcesTreeNode)getSelectedNode());
    }

    /**
     * Refresh the tree and model and reselect the chosen node
     */
    protected void refresh(CPResourcesTreeNode targetNode) {
        // No selected node so just refresh
        if(targetNode == null) {
            _treeModel.refresh();
            return;
        }

        // Save the current selected file
        ReloadResource currentFile = targetNode.getReloadResource();

        // Build the Tree Model again
        _treeModel.refresh();

        // Find selected node (which will be a new one by now)
        CPResourcesTreeNode selectednode = null;
        Enumeration nodes = ((DefaultMutableTreeNode) treeModel.getRoot()).preorderEnumeration();
        // Drill into and find it
        while(nodes.hasMoreElements()) {
            selectednode = (CPResourcesTreeNode)nodes.nextElement();
            ReloadResource res = selectednode.getReloadResource();
            if(res.equals(currentFile)) break;
            selectednode = null;
        }

        // Select Node
        if(selectednode != null) {
            expandNode(selectednode, true);
            selectNode(selectednode);
        }
        else selectNode(_treeModel.getRootNode());
    }

    /**
     * Accessor Method to return which nodes the user has selected
     * @return an array containing the nodes as ReloadResources
     */
    public ReloadResource[] getSelectedResourcesToDelete(){
        Vector v = new Vector();
        TreePath[] paths = getSelectionPaths();
        for (int i = 0; i < paths.length; i++) {
             CPResourcesTreeNode fileSysEntity = (CPResourcesTreeNode)paths[i].getLastPathComponent();
             // Need to make sure this does not pass the root folder back!
             if(!fileSysEntity.isRoot()){
                 ReloadResource aFileToDelete = fileSysEntity.getReloadResource();
                 v.add(aFileToDelete);
             }
        }
        ReloadResource[] resources = new ReloadResource[v.size()];
        v.copyInto(resources);
        return resources;
    }

    /**
     * Remove Resources From Tree
     * @param object
     */
    public void removeTreeResource(Object object) {
        _treeModel.removeNodeFromParent(getNode(object));
    }

    /**
     * A node has been selected on the tree.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {
        // Update any Menus
        updateMenus(event.getPath());
    }

    /**
     * Update the menus according to the selected node on the tree.
     * This has to be synchronized so different threads don't access it at the same time
     */
    protected synchronized void updateMenus(TreePath selPath) {
        if(selPath == null) return;

        Object selectedNode = selPath.getLastPathComponent();
        if(selectedNode instanceof CPResourcesTreeNode) {
            // Cast it
            CPResourcesTreeNode treeNode = (CPResourcesTreeNode) selectedNode;
            // Get Reload Resource
            ReloadResource resource = treeNode.getReloadResource();
            // Tell Button Panel
            _resourcesPanel.updateMenus(resource);
        }
    }

    /**
     * Check to see if we have triggered the popup menu.
     * @param e The MouseEvent that has been triggered.
     */
    protected void checkPopupTrigger(MouseEvent e) {
       if(e.isPopupTrigger()) {
           TreePath selPath = getPathForLocation(e.getX(), e.getY());
           if(selPath == null) return;
           // Select the node
           setSelectionPath(selPath);
           // Show the popup menu
           _popupMenu.show(this, e.getX(), e.getY());
       }
    }

    /**
     * @return a JPopupMenu with suitable menu items borrowed from the Resources
     * Menu Panel
     */
    protected JPopupMenu createPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();

        // Import Pool
        popupMenu.add(_resourcesPanel._proxyImportHandlerLocal.getMenuAction());
        // New Folder
        popupMenu.add(_resourcesPanel._proxyNewFolderHandler.getMenuAction());
        // Delete from Pool
        popupMenu.add(_resourcesPanel._proxyRemoveResourceHandlerLocal.getMenuAction());
        // View File
        popupMenu.add(_resourcesPanel._proxyViewFileHandlerLocal.getMenuAction());

        //popupMenu.addSeparator();

        return popupMenu;
    }

    /**
     * Drop some files on a node
     * @param files
     */
    protected void addFiles(final File[] files, final CPResourcesTreeNode targetNode) {
        final File targetFolder = targetNode.getReloadResource();

        // Sanity check
        if(!targetFolder.isDirectory()) return;

        Thread thread = new Thread() {
            public void run() {
                ReloadProgressMonitor progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Import Resources",
                    "Copying...", "", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                // Note - should we do dependencies?
                ResourceImporter importRes = new ResourceImporter(files, targetFolder, false, progressMonitor);
                progressMonitor.close();

                refresh(targetNode);
            }
        };

        thread.start();

    }

    // =================== Drag and Drop Source events =========================

    /**
     * A drag gesture has been initiated from this tree
     * <h3>Amendment History</h3>
     * <p>Basic outline code for handling a ghosted drag image.  This works
     * on the Mac, but not on PC.  Without this code on the Mac, a drag event
     * shows the whole JPanel as a ghosted image.<p/>
     * @param event the DragGestureEvent
     */
    public void dragGestureRecognized(DragGestureEvent event) {
        if((event.getDragAction() & DnDConstants.ACTION_COPY_OR_MOVE) == 0) return;

        TreePath[] paths = getSelectionPaths();
        if(paths != null) {
            Vector v = new Vector();

            for(int i = 0; i < paths.length; i++) {
                CPResourcesTreeNode node = (CPResourcesTreeNode) paths[i].getLastPathComponent();
                v.add(node.getReloadResource());
            }

            if(v.isEmpty() == false) {
                ReloadResource[] resources = new ReloadResource[v.size()];
                v.copyInto(resources);

                DragTreeObject dragObject = new DragTreeObject(resources, this);

                Point ptDragOrigin = event.getDragOrigin();
                TreePath path = getPathForLocation(ptDragOrigin.x, ptDragOrigin.y);
                if(path == null) return;

                // Get the Drag Image
                BufferedImage imgGhost = getDragImage(path, ptDragOrigin);

                event.startDrag(DragSource.DefaultCopyNoDrop, imgGhost, new Point(5, 5), dragObject, this);
            }
        }
    }

    /**
     * Old deprecated method
     */
    public void _dragGestureRecognized(DragGestureEvent event) {
        if((event.getDragAction() & DnDConstants.ACTION_COPY_OR_MOVE) == 0) return;

        CPResourcesTreeNode node = (CPResourcesTreeNode)getDragSourceTreeNode(event);
        if(node != null) {
            ReloadResource resource = node.getReloadResource();
            DragTreeObject dragObject = new DragTreeObject(resource, this);

            Point ptDragOrigin = event.getDragOrigin();
            TreePath path = getPathForLocation(ptDragOrigin.x, ptDragOrigin.y);
            if(path == null) return;

            // Get the Drag Image
            BufferedImage imgGhost = getDragImage(path, ptDragOrigin);

            event.startDrag(DragSource.DefaultCopyNoDrop, imgGhost, new Point(5, 5), dragObject, this);
        }
    }

    // =================== Drag and Drop Target events =========================

    /**
     * We accept Native Drops
     */
    public boolean isDropOK(DropTargetDragEvent event) {
        // Get the node we're dragging over
        CPResourcesTreeNode targetNode = (CPResourcesTreeNode)getDragOverTreeNode(event);
        if(targetNode == null) return false;

        // Only allow to drop on a folder
        ReloadResource file = targetNode.getReloadResource();
        if(!file.isDirectory()) return false;

        // Only support javaFileListFlavor
        DataFlavor[] flavors = event.getCurrentDataFlavors();
        for(int i = 0; i < flavors.length; i++) {
            if(flavors[i].equals(DataFlavor.javaFileListFlavor)) {
                return true;
            }
        }

        return false;
    }

    /**
     * We just Dropped some files
     */
    public void drop(DropTargetDropEvent event) {
        // Where did we drop it?
        Point location = event.getLocation();
        TreePath treePath = getPathForLocation(location.x, location.y);
        if(treePath == null) return;

        CPResourcesTreeNode targetNode = (CPResourcesTreeNode)treePath.getLastPathComponent();

        // Get what we dropped
        Transferable transferable = event.getTransferable();

        // If we are the correct drag object
        if(transferable.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
            // Accept the Drop
            event.acceptDrop(getCorrectDropContext(event));

            // Get the User Object and do something
            try {
                Object userObject = transferable.getTransferData(DataFlavor.javaFileListFlavor);
                if(userObject instanceof AbstractList) {
                    AbstractList list = (AbstractList)userObject;
                    Object[] files = list.toArray();
                    addFiles((File[])files, targetNode);
                }
            }
            catch(Exception ex) {
                if(ReloadEditor.DEBUG) ex.printStackTrace();
            }
            finally {

            }

            hiliteNode(prevHilitedNode, false);
            event.getDropTargetContext().dropComplete(true);
        }

        // Else we are not the right object
        else event.rejectDrop();
    }

}