/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.IMSDocumentHandler;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.moonunit.dom.*;


/**
 * The Metadata Editor Dialog used for Vdex vocabulary
 * It edits MD from the metadata node of a Vdex vocabulary
 *
 * @author Tina Manoharan
 * @version 2003-11-14
 */
public class MetadataEditorDialog_Vocabulary
extends JDialog
{
    /**
     * The Metadata Editor
     */
    protected MetadataEditor _mdEditor;

    /**
     * The parent "metadata" element containing the lom
     */
    protected Element _mdElement;

    /**
     * An existing MD Lom Node
     */
    protected Element _oldLom;


    /**
     * The Vdex Vocabulary to which this MD Element belongs
     */
    protected VocabularyMetadata _vocabularyMetadata;

    /**
     * Constructor
     * @param vocabularyMetadata the vdex vocabulary to which this metadata element belongs
     * @throws ReloadEditorException
     */
    public MetadataEditorDialog_Vocabulary(VocabularyMetadata vocabularyMetadata) throws ReloadEditorException {
        super(ReloadEditor.getInstance(), "Metadata - " + vocabularyMetadata.getTargetRootElementName(), false);

        _vocabularyMetadata = vocabularyMetadata;
        _mdElement = _vocabularyMetadata.getMDElement();

        // Trap window closing event for our Window listener
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // Add a listener to Close our window down on exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });

        // Menu Bar
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        Menu_Edit editMenu = new Menu_Edit();
        menuBar.add(editMenu);

        // ToolBar
        ReloadToolBar toolBar = createToolBar(editMenu);
        getContentPane().add(toolBar, BorderLayout.NORTH);

        // Add MD Editor
        _mdEditor = new MetadataEditor(editMenu);
        getContentPane().add(_mdEditor, BorderLayout.CENTER);

        // We'll put the Prefs Panel here, actually!
        toolBar.add(_mdEditor._prefsPanel);

        // Button Panel
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

        // OK Button
        JButton btnOK = new JButton("OK");
        btnOK.addActionListener(new OKClick());
        getRootPane().setDefaultButton(btnOK);
        buttonPanel.add(btnOK);

        // Cancel Buttton
        JButton btnCancel = new JButton("Cancel");
        btnCancel.addActionListener(new CancelClick());
        buttonPanel.add(btnCancel);

        // Left Panel
        JPanel leftPanel = new JPanel();
        leftPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        // Import Button
        JButton btnImport = new JButton("Import...");
        btnImport.setToolTipText("Import a standalone Metadata file");
        btnImport.addActionListener(new ImportMetadata());
        leftPanel.add(btnImport);

        // Export Button
        JButton btnExport = new JButton("Export...");
        btnExport.setToolTipText("Export as a standalone Metadata file");
        btnExport.addActionListener(new ExportMetadata());
        leftPanel.add(btnExport);

        // Add bottom Panel
        JPanel bottomPanel = new JPanel(new BorderLayout());
        bottomPanel.add(leftPanel, BorderLayout.WEST);
        bottomPanel.add(buttonPanel, BorderLayout.EAST);
        getContentPane().add(bottomPanel, BorderLayout.SOUTH);

        // Dialog Size
        setSize(ReloadEditor.getInstance().getWidth() * 3 / 4, ReloadEditor.getInstance().getHeight() * 3 / 4);
        setLocationRelativeTo(ReloadEditor.getInstance());

        // Create the MD JDOM Document
        createDocument();
    }

    /**
     * Create a Metadata Document for Editing
     * @throws ReloadEditorException
     */
    protected void createDocument() throws ReloadEditorException {

        MetadataSchemaController mdSchemaController = _vocabularyMetadata.getMetadataSchemaController();

        _oldLom = _vocabularyMetadata.getLOMElement();

        if(_oldLom != null && mdSchemaController != null) _mdEditor.setMetadataElement((Element)_oldLom.clone(), mdSchemaController);
        // No luck, so create a new blank MD document
        else {
            // Determine which Schema Controller to use
            mdSchemaController = _vocabularyMetadata.getMetadataSchemaController();

            _mdEditor.newDocument(false, mdSchemaController);
        }

        // Tell Undo Manager we got the Focus - this wakes it up
        _mdEditor.getUndoManager().setFocusGained();
    }


    /**
     * Import a standalone MD File into a Vdex Vocabulary
     * @throws ReloadEditorException
     */
    protected void importMetadata() throws ReloadEditorException {
        // Get the chosen File
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "xml files");
        File file = ReloadUtils.askFileNameOpen(this, "Import Metadata File", filter);

        if(file != null) {

			// Load it
			Document doc = null;

            try {

				doc = XMLUtils.readXMLFile(file);
			}
			catch(DweezilXMLException ex) {
				throw new ReloadEditorException(ex);
			}

            // Check version
            String schemaVersion = IMSDocumentHandler.getSchemaVersion(doc);
            // Valid??
            if(schemaVersion == null || !schemaVersion.startsWith("imsmd")) {
                throw new ReloadEditorException("Invalid Metadata file");
            }

            // For now we will use existing SchemaController
            // TO DO - determine SchemaController for imported file and change (and Profile)
            MetadataSchemaController schemaController = (MetadataSchemaController)_mdEditor.getMetadata().getReloadSchemaController();

            _mdEditor.importMetadata(doc, schemaController);

            // Tell Undo Manager we got the Focus
            _mdEditor.getUndoManager().setFocusGained();
        }
    }

    /**
     * Export a standalone MD File
     */
    protected void exportMetadata() {
        try {
            // Ask for the File Name
            ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "xml files");
            File file = ReloadUtils.askFileNameSave(this, "Save Metadata", filter, "xml");
            if(file != null) _mdEditor.exportMetadata(file);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error in Metadata", "Could not export Metadata File", ex);
        }
    }

    /**
     * Show the Dialog
     */
    public void show() {
        // Do this later because a modal dialog will block
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                _mdEditor.initView();
            }
        });
        super.show();
    }


    /**
     * User pressed OK
     */
    protected void finish() {

        // Get the Metadata from the Editor
        Metadata md = _mdEditor.getMetadata();

        Element parent = null, mdElement = null;

        int index = 0;

		Vocabulary vocabulary = _vocabularyMetadata.getVocabulary();

        // If user edited the MD
        if(md.isDirty()) {

            // If we had an old node, remove it
            if(_oldLom != null) _mdElement.removeContent(_oldLom);

			parent = _mdElement.getParent();

			index = ReloadSchemaDocument.getIndexOfElement(_mdElement);

			mdElement = (Element) _mdElement.clone();

			if(mdElement != null) vocabulary.removeElement(this, _mdElement);


            // Get new lom from MD
            Element newLom = md.getRootElement();
            newLom.detach();

            // Add it
            mdElement.addContent(newLom);

            // Tell listeners
            if(parent != null && mdElement != null) vocabulary.addElementAtIndex(this, parent, mdElement, index, true);
        }

		// Added this code to allow MD editor launch from Form Panel
		// When this editor is launched it creates a new MD element if
		// there is none. But we dont want it if we dont create any MD,
		// so we have to delete it.
		if(mdElement != null) {

			java.util.List list = mdElement.getContent();

			if(list.isEmpty()) vocabulary.removeElement(this, mdElement);
		}

        // Finished
        dispose();
    }


    /**
     * Clean up
     */
    public void dispose() {
        try {
            _mdEditor.destroy();
            super.dispose();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }


    /**
     * Create the Toolbar.
     * @return The Toolbar
     */
    private ReloadToolBar createToolBar(Menu_Edit editMenu) {
        ReloadToolBar tBar = new ReloadToolBar();

        // Undo
        tBar.add(editMenu.actionUndo);

        // Redo
        tBar.add(editMenu.actionRedo);

        // Add a separator
        tBar.addSeparator();

        // Cut
        tBar.add(editMenu.actionCut);

        // Copy
        tBar.add(editMenu.actionCopy);

        // Paste
        tBar.add(editMenu.actionPaste);

        // Delete
        tBar.add(editMenu.actionDelete);

        // Add a separator
        tBar.addSeparator();

        // Move Up
        tBar.add(editMenu.actionMoveUp);

        // Move Down
        tBar.add(editMenu.actionMoveDown);

        // Add a separator
        tBar.addSeparator();

        return tBar;
    }

    /**
     * Export a Metadata file
     */
    private class ExportMetadata extends AbstractAction {

	   /**
     	* The Export button was pressed
     	* @param e ActionEvent
     	*/
        public void actionPerformed(ActionEvent e) {
            exportMetadata();
        }
    }

    /**
     * Import a Metadata file
     */
    private class ImportMetadata extends AbstractAction {

	   /**
     	* The Import button was pressed
     	* @param e ActionEvent
     	*/
        public void actionPerformed(ActionEvent e) {
            try {
                importMetadata();
            }
            catch(Exception ex) {
                if(ReloadEditor.DEBUG) ex.printStackTrace();
                ErrorDialogBox.showWarning("Error in Metadata", "Could not import Metadata File", ex);
            }
        }
    }

    /**
     * OK handler
     */
    private class OKClick extends AbstractAction {
	   /**
     	* The OK button was pressed
     	* @param e ActionEvent
     	*/
        public void actionPerformed(ActionEvent e) {
            finish();
        }
    }

    /**
     * Cancel handler
     */
    private class CancelClick extends AbstractAction {
	   /**
     	* The Cancel button was pressed
     	* @param e ActionEvent
     	*/
        public void actionPerformed(ActionEvent e) {

			// Added this code to allow MD editor launch from form panel
			// When this editor is launched it creates a new MD element if
			// there is none. But we dont want it if we dont create any MD,
			// so we have to delete it.
			if(_mdElement != null) {

				Vocabulary vocabulary = _vocabularyMetadata.getVocabulary();

				java.util.List list = _mdElement.getContent();

				if(list.isEmpty()) vocabulary.removeElement(this, _mdElement);
			}

			dispose();
        }
    }

}
