/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata.formview;

import java.util.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

import uk.ac.reload.editor.metadata.*;


/**
 * A Metadata Form Panel for the Metadata UI
 *
 * @author Phillip Beauvoir
 * @version 2003-10-22
 */
public class MetadataFormPanel
extends JPanel
{
    /**
     * The Form Model
     */
    protected MetadataFormModel _mdFormModel;

    /**
     * Keep track of fields so we can release listeners and clean up
     */
    protected Vector _fields = new Vector();

    /**
     * Default constructor
     */
    public MetadataFormPanel() {
        setLayout(new FlowLayout(FlowLayout.LEFT));

        // New form model
        _mdFormModel = new MetadataFormModel();
    }

    /**
     * Set the view up to display the Metadata Document
     * @param metadata the metadata document
     */
    public void setDocument(Metadata metadata) {
        // Clean up
        destroy();
        removeAll();

        // Update Model
        _mdFormModel.setDocument(metadata);

        // Get Root
        MetadataProfileElement rootElement = _mdFormModel.getRootElement();
        // Add Root Panel
        add(new GroupPanel(rootElement));
        revalidate();
    }

    /**
     * Clean up
     */
    public void destroy() {
        for(int i = 0; i < _fields.size(); i++) {
            MetadataField mdField = (MetadataField)_fields.get(i);
            mdField.destroy();
        }
        _fields.clear();
    }

    /**
     * A Panel that uses GridBagLayout to set out its components
     */
    class GroupPanel extends JPanel {
        /**
         * The Constraints
         */
        protected GridBagConstraints constraints;

        /**
         * Left Indent factor
         */
        int INDENT = 15;

        public GroupPanel(MetadataProfileElement profileElement) {
            // Set up the constraints
            setLayout(new GridBagLayout());
            constraints = new GridBagConstraints();
            constraints.insets = new Insets(0, INDENT, 5, 0);
            constraints.gridx = 0;
            constraints.gridy = 0;
            constraints.weightx = 1.0;
            constraints.weighty = 0.0;
            constraints.fill = GridBagConstraints.HORIZONTAL;
            constraints.anchor = GridBagConstraints.WEST;

            // Add the Group Label
            addGroupLabel(profileElement);

            // Get the Children
            MetadataProfileElement[] children = profileElement.getChildren();
            for(int i = 0; i < children.length; i++) {
                MetadataProfileElement child = children[i];
                // If it's a container type. add new sub-Panel
                if(child.isGroup()) {
                    constraints.gridx = 0;
                    constraints.gridwidth = 2;
                    constraints.weightx = 1.0;
                    constraints.fill = GridBagConstraints.HORIZONTAL;
                    add(new GroupPanel(child), constraints);
                    constraints.gridy++;
                }
                // Else add a field
                else addField(child);
            }
        }

        /**
         * Add a single field for an Element
         * @param profileElement
         */
        protected void addField(MetadataProfileElement profileElement) {
            constraints.fill = GridBagConstraints.NONE;
            // Add a Label
            JLabel l = new JLabel(profileElement.getName());
            l.setBorder(new EmptyBorder(0, INDENT, 0, 0));
            constraints.gridx = 0;
            constraints.gridwidth = 1;
            constraints.weightx = 0;
            add(l, constraints);

            // Ask the Model for the Widget to use
            MetadataField mdField = _mdFormModel.createWidget(profileElement);
            if(mdField != null) _fields.add(mdField);

            // Add it
            constraints.gridx = 1;
            if(mdField != null) add(mdField.getComponent(), constraints);
            constraints.gridy++;
        }


        /**
         * Add a JLabel to mark a Group
         * @param profileElement the MetadataProfileElement
         */
        protected void addGroupLabel(MetadataProfileElement profileElement) {
            constraints.fill = GridBagConstraints.HORIZONTAL;
            constraints.gridwidth = 2;
            constraints.gridx = 0;

            // Set Label Color
            // Default
            Color color = Color.black;
            // Profile Name
            if(profileElement.getParent() == null) color = Color.blue;
            // Top Group
            else if(profileElement.getParent().getParent() == null) color = Color.blue;

            JLabel label = new JLabel(profileElement.getName());
            label.setOpaque(true);
            label.setBackground(Color.lightGray);
            label.setForeground(color);
            label.setBorder(new EmptyBorder(0, 3, 0, 0));

            add(label, constraints);
            constraints.gridy++;
        }
    }


}