/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.metadata.vdexBrowser;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.tree.*;


/**
 * A Browser Tree that will display the hierarchical term contents of an Vdex XML File. Each node of the tree displays
 * the term (id:caption).
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class VdexTree
extends ReloadTree
implements TreeSelectionListener
{
	/**
	 * The Vdex Term Panel that displays this Vdex Tree (left side) and other term information (right side)
	 */
	private VdexTermPanel termPanel;

	/**
	 * The node name, for example of the form (term_id:term_caption)
	 */
	private String nodeName;

	/**
	 * The list of all node names of the tree
	 */
	private ArrayList nodeNames;

    /**
     * Default constructor
     */
    public VdexTree() {
        super();
        addTreeSelectionListener(this);

        setCellRenderer(new ReloadTreeRenderer());

        nodeName = "";

        nodeNames = new ArrayList();
    }

	/**
	 * Set the term panel
	 * @param termPanel the VdexTermPanel
	 */
    public void setTermPanel(VdexTermPanel termPanel) {

		this.termPanel = termPanel;
	}

    /**
     * A node selection has been made on the tree.  This will call setElement(element)
     * on the editor and then display the node's information values.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {

        if(event == null) return;
        TreePath selPath = event.getPath();
        if(selPath == null) return;

        // Get selected node
        VdexTreeNode selectedNode = (VdexTreeNode)selPath.getLastPathComponent();
        if(selectedNode == null) return;

		// get the current nodeName
		nodeName = (String)selectedNode.getUserObject();
        Element element = selectedNode.getElement();
        termPanel.setElement(element);

        // get all node names from the current one up to the root
        Object[] selectedNodes = selPath.getPath();

		// clear nodeNames first before adding things to it
		nodeNames.clear();

        for(int i = 0; i < selectedNodes.length; i++) {

			if(selectedNodes[i] instanceof VdexTreeNode) {

				selectedNode = (VdexTreeNode) selectedNodes[i];

				String temp = (String) selectedNode.getUserObject();

				if(temp != null && !nodeNames.contains(temp) && !temp.equals("Root")) nodeNames.add(temp);
			}
		}
    }

	/**
	 * Get the name of the selected node on the tree
	 * @return the selected node name
	 */
    public String getNodeName() {

		return nodeName;
	}

	/**
	 * Get all node names of the tree
	 * @return all node names of the tree
	 */
	public String[] getAllNodeNames() {

		return (String[]) nodeNames.toArray(new String[nodeNames.size()]);
	}

	/**
	 * This function is called when a hyperlink is selected from the term relationship information.
	 * This causes a jump to a particular node on the tree and hence the node has to be selected.
	 * @param search the node name of the node that has to be selected
	 */
	public void highlightNode(String search) {


		TreeModel treeModel = this.getModel();
		VdexTreeNode rootNode = (VdexTreeNode) treeModel.getRoot();

		Enumeration enum = rootNode.depthFirstEnumeration();

		VdexTreeNode node = null;
		String name;
		int index;
		boolean found = false;


		while(enum.hasMoreElements()) {

			node = (VdexTreeNode) enum.nextElement();

			name = node.getNodeName();

			index = name.indexOf(":");

			if(index != -1)	{

				name = name.substring(0,index);

				name.trim();

				if(name.startsWith(search)) {

					found = true;
					break;
				}
			}
			else {

				if(name.startsWith(search)) {

					found = true;
					break;
				}
			}
		}

		if(found && node!=null) {

			TreePath treePath = new TreePath(node.getPath());
			this.setSelectionPath(treePath);
		}

	}

}

