/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.metadata.vdexBrowser;

import java.util.*;
import java.io.*;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.tree.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.vocabulary.*;


/**
 * The Vdex Tree Model.
 * This reads the vdex file and builds the nodes of the tree and creates the VdexVocabNode.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class VdexTreeModel
extends ReloadTreeModel
{

	/**
	 * The Vdex Vocabulary Node that maintains information about the vocabulary, for example, name, id, default language, metadata
	 */
	private VdexVocabNode vdexVocabNode;

	private String tempTermID, tempTermCaption, tempTermVal;

	/**
     * Constructor
     * @param vocabulary the vocabulary document
     */
    public VdexTreeModel(Vocabulary vocabulary) {

		vdexVocabNode = new VdexVocabNode();

		root = new VdexTreeNode("Root", vocabulary.getRootElement());

        buildChildren((VdexTreeNode)root);
    }


    /**
     * Build up child nodes for creating the VdexTree
     * @param node The Parent VdexTreeNode
     */
    public void buildChildren(VdexTreeNode node) {
        Iterator children = node.getElement().getChildren().iterator();

        while(children.hasNext()) {
            Element child = (Element)children.next();

            if(doShowNode(child)) {

				if(child.getName().equals(VocabularyCore.TERM)) {

	                VdexTreeNode newNode = new VdexTreeNode(child.getName(), child);
	                node.add(newNode);
	                buildChildren(newNode);
				}
				else if(child.getName().equals(VocabularyCore.TERM_IDENTIFIER)) {

					tempTermID = child.getText();
				}
				else if(child.getName().equals(VocabularyCore.TERM_CAPTION)) {

					tempTermCaption = child.getChildTextTrim(VocabularyCore.LANGSTRING, child.getNamespace());

					if(tempTermID != null && tempTermCaption != null) {

						tempTermVal = tempTermID + " : " + tempTermCaption;
					}
					else if(tempTermID != null) {

						tempTermVal = tempTermID + " : " + "-";
					}
					else if(tempTermCaption != null) {

						tempTermVal = "-" + " : " + tempTermCaption;
					}
					else tempTermVal = "- : -";

	                //VdexTreeNode newNode = new VdexTreeNode(tempTermVal, child.getParent());
	                //node.add(newNode);

	                node.setNodeName(tempTermVal);
				}
            }
        }
    }


    /**
     * Decide whether to display an Element on the Tree
     * @param element the element concerned
     * @return true if the element can be displayed, false otherwise
     */
    protected boolean doShowNode(Element element) {

		String name = element.getName();

		if(name.equals(VocabularyCore.TERM) || name.equals(VocabularyCore.TERM_IDENTIFIER) || name.equals(VocabularyCore.TERM_CAPTION)) {

			return true;
		}
		else if(name.equals(VocabularyCore.VOCABULARY_NAME) || name.equals(VocabularyCore.VOCABULARY_IDENTIFIER) || (name.equals(VocabularyCore.VOCABULARY_METADATA) && element.getParent().isRootElement())) {

			buildVdexVocabNode(element);

			return false;
		}

        return false;
    }


    /**
     * Build the Vdex Vocab Node
     * @param element vocabulary elements such as name, id, metadata
     */
    public void buildVdexVocabNode(Element element) {

		String name = element.getName();

		File folderTemp = new File(System.getProperty("user.dir"), "temp");


		if(name.equals(VocabularyCore.VOCABULARY_NAME)) {

			vdexVocabNode.setVocabName(element.getChildTextTrim(VocabularyCore.LANGSTRING, element.getNamespace()));

			Element attribElem = element.getChild(VocabularyCore.LANGSTRING,element.getNamespace());

			if(attribElem != null) {

				vdexVocabNode.setVocabLang(attribElem.getAttributeValue(VocabularyCore.LANGUAGE));
			}
		}
		else if(name.equals(VocabularyCore.VOCABULARY_IDENTIFIER)) {

			vdexVocabNode.setVocabID(element.getText());
		}
		else if(name.equals(VocabularyCore.VOCABULARY_METADATA) && element.getParent().isRootElement()) {

			File stylesheet =  ReloadEditorDocumentHandler.VDEX_METADATA_STYLESHEET;

			if(MetadataVdexBrowser._file != null && stylesheet != null) {

				String filename = "vocabMD_" + MetadataVdexBrowser._file.getName();

				File mdFile = null;

				if(folderTemp.exists()) {

					File[] files = folderTemp.listFiles();

					for(int i = 0; i < files.length; i++) {

						mdFile = files[i];

						if(mdFile.getName().equals(filename)) break;
						else mdFile = null;
					}
				}

				if(mdFile == null) {

					try {

						mdFile = XMLUtils.getMetadataFile(MetadataVdexBrowser._file, stylesheet, filename);
					}
					catch(DweezilXMLException ex) {}
				}

				if(mdFile != null) vdexVocabNode.setVocabMDFile(mdFile);
			}
		}
	}


    /**
     * Get the Vdex Vocab Node
     * @return the VdexVocabNode
     */
	public VdexVocabNode getVdexVocabNode() {

		return vdexVocabNode;
	}


}