/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.formview;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.widgets.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.schema.*;


/**
 * A Text Area widget.
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
public class VocabularyTextField
extends VocabularyField
implements DocumentListener
{
    /**
     * An internal flag to notify listeners of element changes
     */
    boolean _allowNotification = true;

    /**
     * The suitable Component we shall use
     */
    protected ReloadWidget _reloadWidget;

    /**
     * Constructor
     * @param vocabulary the vocabulary JDOM document
     * 		  schemaElement the ReloadSchemaElement bound to this VocabularyTextField
     */
    public VocabularyTextField(Vocabulary vocabulary, ReloadSchemaElement schemaElement) {
        super(vocabulary, schemaElement);

        // Determine type of text field
        _reloadWidget = getVocabularyTextFieldWidget(schemaElement);
        ((JComponent)_reloadWidget).setBorder(null);

        _element = vocabulary.getElement(_schemaElement);
        if(_element != null) setValue(_element.getText());

        // We listen to Text Changes
        _reloadWidget.addDocumentListener(this);

        // We listen to Element changes
        vocabulary.addReloadDocumentListener(this);
    }

    /**
     * Get a Text Widget suitable for the VX Form display
     * @param schemaElement the schema element bound to this widget
     * @return A Text Widget suitable for the VX Form display
     */
    protected ReloadWidget getVocabularyTextFieldWidget(ReloadSchemaElement schemaElement) {

        if(schemaElement == null) return getDefaultWidget();

        ReloadWidget component = null;

        ReloadSchemaController schemaController = _reloadDocument.getReloadSchemaController();

        String widget_type = schemaController.getWidgetType(schemaElement.getDomPath());

        if(widget_type != null) {
            if(widget_type.equals("textpane")) {
                component = new ReloadTextArea();
                ((ReloadTextArea)component).setColumns(50);
            }
            else if(widget_type.equals("numberfield")) {
                component = new ReloadNumberField();
                ((ReloadNumberField)component).setColumns(30);
            }

            // Tina Manoharan 2003-12-15

			else if(widget_type.equals("buttontextpane")) { // for vdex/metadata & term/metadata
				component = getReloadButtonTextArea();

				((ReloadButtonTextArea)component).setColumns(50);

				// Set Schema Element
				((ReloadButtonTextArea)component).setSchemaElement(schemaElement);

				// Set Element
				((ReloadButtonTextArea)component).setElement(_element);

				// Set Form Field, this is required only for the Form View not the Tree View
				((ReloadButtonTextArea)component).setFormField(VocabularyTextField.this);
			}

            else component = getDefaultWidget();
        }
        else component = getDefaultWidget();

        // Set Max String length
        int maxLength = -1;
        String value = schemaController.getFacetValue(schemaElement, "maxLength");
        if(value != null) maxLength = Integer.parseInt(value);
        component.setMaxLength(maxLength);

        return component;
    }

    /**
     * Get the default widget to be used
     * @return the default widget
     */
    private ReloadWidget getDefaultWidget() {
        ReloadTextField textField = new ReloadTextField();
        textField.setColumns(50);
        return textField;
    }

    // Tina Manoharan 2003-12-15

    /**
     * Get a TextArea and Button
     * @return a TextArea and Button
     */
    public ReloadButtonTextArea getReloadButtonTextArea() {
        return new ReloadButtonTextArea((Vocabulary)_reloadDocument, ReloadButtonTextArea.VDEX_METADATA_TYPE, "Launch Metadata Editor");
    }

    /**
     * Clean up
     */
    public void destroy() {
        _reloadWidget.removeDocumentListener(this);
        _reloadDocument.removeReloadDocumentListener(this);
    }

    /**
     * Get the value of this Widget
     * @return The value contained in the Widget
     */
    public String getValue() {
        return _reloadWidget.getTextValue();
    }

    /**
     * Set the value of the Widget
     * @param value The value to be set to the widget
     */
    public void setValue(String value) {
        _allowNotification = false;
        _reloadWidget.setTextValue(value);
        _allowNotification = true;
    }

    /**
     * Get the component that we're using
     * @return the Component that we're using
     */
    public Component getComponent() {

        return (Component)_reloadWidget;
    }

    public Element getElement() {
		return _element;
	}

// =============================================================================
// === These events are fired from the JTextArea when the textfield changes ====
// =============================================================================

	/**
	 * Notify listeners that the textfield value is removed
	 */
    public void removeUpdate(DocumentEvent evt) {
        if(_allowNotification && _reloadWidget != null) fireElementChanged(getValue());
    }

	/**
	 * Notify listeners that the textfield value is changed
	 */
    public void changedUpdate(DocumentEvent evt) {
        if(_allowNotification && _reloadWidget != null) fireElementChanged(getValue());
    }

	/**
	 * Notify listeners that the textfield value is inserted
	 */
    public void insertUpdate(DocumentEvent evt) {
        if(_allowNotification && _reloadWidget != null) fireElementChanged(getValue());
    }

}