/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import java.io.*;
import java.util.*;

import org.exolab.castor.xml.schema.*;
import org.exolab.castor.xml.schema.reader.*;


/**
 * The ReloadSchema Class - a Java representation of an XML Schema. Together with
 * certain XML Schema helper files an XML Schema is read in, parsed and constructed
 * in memory as a hierarchical linked list of ReloadSchemaNode objects - ReloadSchemaElement
 * and ReloadSchemaAttribute.
 * <p>
 * <p>It relies on Castor (www.castor.org) and JDOM (www.jdom.org) to do the donkey work.<p>
 *
 * @author Phillip Beauvoir
 * @version 2003-09-23
 */
public class ReloadSchema {

    /**
     * The Castor Schema that we will query.
     */
    protected Schema _castorSchema;

    /**
     * The root ReloadSchemaElement
     */
    private ReloadSchemaElement _root;

    /**
     * The name of the Root Element
     */
    private String _rootName;

    /**
     * The following are static instantiations of the various specs.  We could
     * make ReloadSchema an abstract class and implement a class for each spec
     * but this would mean doing it for every new spec version.
     *
     * We only want to create these just once for each spec and re-use them
     */
    private static Hashtable _reloadSchemaTable = new Hashtable();

    /**
     * Factory method for getting a static re-usable ReloadSchema
     * @param fileSpec The file of the Specification Schema XSD to get
     * @param rootName We need to know the root name element.  This is found in
     * the Helper file.
     * @return The ReloadSchema or null if not found
     * @throws ReloadSchemaException if something goes wrong
     */
    public static ReloadSchema getReloadSchema(File fileSpec, String rootName) throws ReloadSchemaException {
        // Do we have it already?
        ReloadSchema reloadSchema = (ReloadSchema)_reloadSchemaTable.get(fileSpec);
        // No, first time creation, create and store in table
        if(reloadSchema == null) {
            reloadSchema = new ReloadSchema(fileSpec, rootName);
            _reloadSchemaTable.put(fileSpec, reloadSchema);
        }
        return reloadSchema;
    }


    /**
     * Constructor, make it private so that user has to use the factory.
     * @param schemaFile The XML Schema File to read
     * @param rootName we need to know the root name element
     * @throws ReloadSchemaException If something goes wrong
     */
    private ReloadSchema(File schemaFile, String rootName) throws ReloadSchemaException {
        // Get the schema document
        try {
            _castorSchema = loadCastorSchema(schemaFile);
        }
        catch(IOException ex) {
            throw new ReloadSchemaException(ex);
        }

        _rootName = rootName;

        // Get a Root Element
        ElementDecl rootElementDecl = _castorSchema.getElementDecl(_rootName);
        _root = new ReloadSchemaElement(rootElementDecl);
        _root.setReloadSchema(this);
    }

    /**
     * Get the root element
     * @return The root ReloadSchemaElement
     */
    public ReloadSchemaElement getRootElement() {
        return _root;
    }

    /**
     * Get the name of the Root Element in this Schema
     * @return The Root Element Name
     */
    public String getRootElementName() {
        return _rootName;
    }

    ////////////////////////////////////////////////////////////////////////////
    // SCHEMA ACCESSORS (delegate to Castor)
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Returns the target namespace for this Schema, or null if no
     * namespace has been defined.
     * @return the target namespace for this Schema, or null if no
     * namespace has been defined
     */
    public String getTargetNamespace() {
        return _castorSchema.getTargetNamespace();
    }

    /**
     * Returns the namespace of the XML Schema
     * Note: This is not the same as targetNamespace. This is
     * the namespace of "XML Schema" itself and not the namespace of the
     * schema that is represented by this object model
     * @return the namespace of the XML Schema
     */
    public String getSchemaNamespace() {
        return _castorSchema.getSchemaNamespace();
    }

    /**
     * Returns the full absolute path schemaLocation hint provided for this schema
     * @return the schemaLocation hint provided of this schema
     */
    public String getSchemaLocation() {
        return _castorSchema.getSchemaLocation();
    }

    /**
     * Get the schemaLocation for a given Namespace URI
     * @return the schemaLocation for a given Namespace URI
     */
    public String getSchemaLocation(String namespaceURI) {
        Schema schema = _castorSchema.getImportedSchema(namespaceURI);
        return schema == null ? null : schema.getSchemaLocation();
    }

    /**
     * Returns the short schemaLocation hint provided of this schema
     * @return the schemaLocation hint provided of this schema
     */
    public String getSchemaName() {
        // String path = getSchemaLocation();
        // return new File(path).getName();

		// Tina Manoharan, 2003-09-10

		// Changed this because the resulting XML file, was not validating
		// against the schema.

        String path = getTargetNamespace();
        int index = path.lastIndexOf('/');
        path = path.substring(0,index+1);

        String path2 = getSchemaLocation();
        path2 = new File(path2).getName();
        path += path2;

        return path;
    }

    /**
     * Get the short schemaLocation for a given Namespace URI
     * @return the short schemaLocation for a given Namespace URI,
     * or null if no assoficate namespace URI exists.
     */
    public String getSchemaName(String namespaceURI) {
        String path = getSchemaLocation(namespaceURI);
        return path == null ? null : new File(path).getName();
    }

    /**
     * Returns the namespace URI associated with the given prefix.
     *
     * @return the namespace associated with the given prefix,
     * or null if no associated namespace exists.
     */
    public String getNamespaceURI(String prefix) {
        return _castorSchema.getNamespace(prefix);
    }

    ////////////////////////////////////////////////////////////////////////////
    // HELPERS
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Get the raw Castor Schema
     * @return The Castor Schema
     */
    protected Schema getCastorSchema() {
        return _castorSchema;
    }

    /**
     * Load the Castor Schema
     * @param schemaFile The XML Schema File
     * @return The Castor Schema
     * @throws IOException
     */
    protected Schema loadCastorSchema(File schemaFile) throws IOException {
        // We need the URL String, not a file path to work on Mac
        String url = schemaFile.toURL().toString();
        SchemaReader reader = new SchemaReader(url);
        return reader.read();
    }
}