/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import java.io.*;
import java.util.*;

import uk.ac.reload.moonunit.dom.*;


/**
 * The Schema Controller - this controls access to helper and vocabulary files
 * as well as various default values.
 *
 * @author Phillip Beauvoir
 * @version 2003-11-04
 */
public abstract class ReloadSchemaController {
    /**
     * The version String
     */
    protected String _version;

    /**
     * The Schema that will be our template
     */
    protected ReloadSchema _reloadSchema;

    /**
     * The Schema File
     */
    protected File _schemaFile;

    /**
     * The Schema Helper that we will query
     */
    protected ReloadSchemaHelper _schemaHelper;

    /**
     * The Reload Schema Vocabulary
     */
    protected ReloadVocabulary _vocab;

    /**
     * Default Constructor
     */
    public ReloadSchemaController() { }

    /**
     * Constructor
     * @param fileSchema the schema file
     * @param fileHelper the schema helper file that we will query
     * @param fileVocab the reload vocabulary file
     * @throws ReloadSchemaException
     */
    public ReloadSchemaController(File fileSchema, File fileHelper, File fileVocab) throws ReloadSchemaException {
        loadSchema(fileSchema);
        loadSchemaHelper(fileHelper);
        loadVocab(fileVocab);
    }

    /**
     * Load the Schema File
     * @param schemaFile the schema file
     * @throws ReloadSchemaException
     */
    public void loadSchema(File schemaFile) throws ReloadSchemaException {
        _schemaFile = schemaFile;
        _reloadSchema = ReloadSchema.getReloadSchema(schemaFile, getRootElementName());
    }

    /**
     * Load the Schema Helper File
     * @param fileHelper the schema helper file
     * @throws ReloadSchemaException
     */
    public void loadSchemaHelper(File fileHelper) throws ReloadSchemaException {
        _schemaHelper = ReloadSchemaHelper.getReloadSchemaHelper(fileHelper);
    }

    /**
     * Load the Vocabulary Helper File
     * @param fileVocab the vocabulary helper file
     * @throws ReloadSchemaException
     */
    public void loadVocab(File fileVocab) throws ReloadSchemaException {
        _vocab = ReloadVocabulary.getReloadVocabulary(fileVocab);
    }

    /**
     * Set the Default Language for the Language vocabulary
     */
    public void setDefaultVocabLanguage(String lang) {
        if(_vocab != null) _vocab.setDefaultLanguage(lang);
    }

    /**
     * Get the schema version
     * @return the Version String of the Schema.  This will be one of the constants
     * in IMSDocumentHandler
     */
    public String getVersion() {
        return _version;
    }

    /**
     * Set the Version String of the Schema.  This will be one of the constants
     * in IMSDocumentHandler
     * @param version the schema version
     */
    public void setVersion(String version) {
        _version = version;
    }

    /**
     * For each JDOM Path there is a corresponding
     * ReloadSchemaNode.  This method finds that corresponding
     * ReloadSchemaNode in this Schema and returns it or null if not found.
     * @param domPath the Path to find the ReloadSchemaElement or ReloadSchemaAttribute
     * @return The ReloadSchemaNode or null if not found
     */
    public ReloadSchemaNode getReloadSchemaNode(DomPath domPath) {
        String attName = null;

        // Get Attribute & Elements Path if this is an Attribute
        if(domPath.isAttribute()) {
            attName = domPath.getAttributePart();
            domPath = new DomPath(domPath.getElementsPart());
        }

        ReloadSchemaElement schemaElement = _reloadSchema.getRootElement();
        // Now work thru the path so we can drill into the Schema to find the correct point in the Schema
        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();
            // If schemaElement is not root because we are at the top level
            if(!name.equals(_reloadSchema.getRootElementName())) {
                schemaElement = schemaElement.getChild(name);
                if(schemaElement == null) break; // Oops!
            }
        }

        // We do have an Attribute
        if(attName != null) {
            ReloadSchemaAttribute schemaAttribute = schemaElement.getAttribute(attName);
            if(schemaAttribute != null) return schemaAttribute;
        }

        return schemaElement;
    }

    /**
     * Get a Helper value for a given key
     * @param domPath the path to find the ReloadSchemaNode
     * @param helperKey the helper key to get a value from the schema helper file
     * @return a Helper value for a given key
     */
    public String getElementHelperValue(DomPath domPath, String helperKey) {
        return _schemaHelper == null ? null : _schemaHelper.getHelperValue(domPath, helperKey);
    }

    /**
     * Get a friendly name for a schema element
     * @param domPath the path to find the ReloadSchemaNode
     * @return a friendly name
     */
    public String getElementFriendlyName(DomPath domPath) {
        return getElementHelperValue(domPath, ReloadSchemaHelper.FNAME);
    }

    /**
     * Get an element tip
     * @param domPath the path to find the ReloadSchemaNode
     * @return a tip for an element
     */
    public String getElementTip(DomPath domPath) {
        return getElementHelperValue(domPath, ReloadSchemaHelper.TIP);
    }

    /**
     * Get the widget type of the element from the helper file or null if not set
     * @param domPath the path to find the ReloadSchemaElement
     * @return the widget type or null if not set
     */
    public String getWidgetType(DomPath domPath) {
        return getElementHelperValue(domPath, ReloadSchemaHelper.WIDGET);
    }

    /**
     * Get the root element name
     * @return The root element name
     */
    public abstract String getRootElementName();

    /**
     * Get the Vocabulary list for a given schema node
     * We first of all look to see if we have a Helper Vocabulary defined.
     * If not, look for one as defined in the Schema.
     * @param schemaNode The Element or Attribute for which to get the vocab list
     * @return The Vocabulary List or null if no list present
     */
    public ReloadVocabularyList getVocabularyList(ReloadSchemaNode schemaNode) {
        if(schemaNode == null) return null;

        ReloadVocabularyList rvList = null;

        // Check for Helper list first
        if(_vocab != null) rvList = _vocab.getVocabularyList(schemaNode);

        // Now check for internal Schema
        if(rvList == null) rvList = schemaNode.getVocabularyList();

        return rvList;
    }


    // Tina Manoharan, 2003-11-10

    /**
     * Get the Vocabulary list for a given element dompath
     * We first of all look to see if we have a Helper Vocabulary defined.
     * If not, look for one as defined in the Schema.
     * @param domPath the path of the Element or Attribute for which to get the vocab list
     * @return The Vocabulary List or null if no list present
     */
    public ReloadVocabularyList getVocabularyList(DomPath domPath) {
        if(domPath == null) return null;

        ReloadVocabularyList rvList = null;

        // Check for Helper list first
        if(_vocab != null) rvList = _vocab.getVocabularyList(domPath);

        return rvList;
    }

	// Tina Manoharan, 2003-11-10

    /**
     * Get the Vocabulary list for a given schema node and reload document
     * We first of all look to see if we have a Helper Vocabulary defined.
     * If not, look for one as defined in the Schema.
     * @param schemaNode The Element or Attribute for which to get the vocab list
     * @param doc the ReloadDocument instance
     * @return The Vocabulary List or null if no list present
     */
    public ReloadVocabularyList getVocabularyList(ReloadSchemaNode schemaNode, ReloadDocument doc) {
        if(schemaNode == null) return null;

        ReloadVocabularyList rvList = null;

        // Check for Helper list first
        if(_vocab != null) rvList = _vocab.getVocabularyList(schemaNode);

        // Now check for internal Schema
        if(rvList == null) rvList = schemaNode.getVocabularyList();

        return rvList;
    }



    /**
     * Get the default value of a ReloadSchemaNode
     * @param schemaNode the ReloadSchemaNode concerned
     * @return The default value or null
     */
    public String getDefaultValue(ReloadSchemaNode schemaNode) {
        String def = null;

        // Check for helper value first
        ReloadVocabularyList rvList = getVocabularyList(schemaNode);
        if(rvList != null) def = rvList.getDefaultValue();

        // Not found, so check for Schema
        if(def == null) def = schemaNode.getDefaultValue();

        return def;
    }

    /**
     * Get a Facet value of an Element or attribute.  Will look in the Helper file
     * first and then the Schema. The Facet has to be a valid Schema type Facet.
     * @param schemaNode the ReloadSchema Element or Attribute concerned
     * @param facetName the name of the facet for which we need a value
     * @return a Facet value of an Element or attribute.
     */
    public String getFacetValue(ReloadSchemaNode schemaNode, String facetName) {
        // Try Helper file first
        String value = getElementHelperValue(schemaNode.getDomPath(), facetName);
        // Try Schema
        if(value == null) value = schemaNode.getFacetValue(facetName);
        return value;
    }

    /**
     * Return the ReloadSchema connected with this Controller
     * @return the ReloadSchema connected with this controller
     */
    public ReloadSchema getReloadSchema() {
        return _reloadSchema;
    }

    /**
     * Get the Schema File for this Schema
     * @return the Schema File for this Schema
     */
    public File getSchemaFile() {
        return _schemaFile;
    }

    /**
     * Get the Schema Helper
     * @return The Schema Helper
     */
    public ReloadSchemaHelper getSchemaHelper() {
        return _schemaHelper;
    }
}