/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.moonunit.dom.*;

/**
 * The Reload Schema Helper - this provides a bunch of helper methods to the
 * ReloadSchema class, like friendly names etc.  It's a middleman class.  It reads
 * in the Helper XML File.
 *
 * @author Phillip Beauvoir
 * @version 2003-09-19
 */
public class ReloadSchemaHelper {

    public static final String ELEMENT = "element";
    public static final String SCHEMA = "schema";
    public static final String PATH = "path";
    public static final String FNAME = "fname";
    public static final String TIP = "tip";
    public static final String WIDGET = "widget";
    public static final String MAXLENGTH = "maxLength";

    /**
     * The JDOM Document of the XML Helper File
     */
    protected Document _docHelper;

    /**
     * The File of the XML Helper File
     */
    protected File _fileHelper;


    /**
     * Static SchemaHelpers - we only want to create these just once for each spec and re-use them
     */
    private static Hashtable _reloadSchemaHelperTable = new Hashtable();

    /**
     * Factory method for getting a static re-usable ReloadSchemaHelper
     * @param fileHelper The file of the Helper file to get
     * @return The ReloadSchemaHelper or null if not found
     * @throws ReloadSchemaException
     */
    public static ReloadSchemaHelper getReloadSchemaHelper(File fileHelper) throws ReloadSchemaException {
        // Do we have it already?
        ReloadSchemaHelper reloadSchemaHelper = (ReloadSchemaHelper) _reloadSchemaHelperTable.get(fileHelper);
        // No, first time creation, create and store in table
        if(reloadSchemaHelper == null) {
            reloadSchemaHelper = new ReloadSchemaHelper(fileHelper);
            _reloadSchemaHelperTable.put(fileHelper, reloadSchemaHelper);
        }
        return reloadSchemaHelper;
    }

    /**
     * Constructor.
     *
     * @param fileHelper The XML Helper File to read in
     * @throws ReloadSchemaException If anything weird happens
     */
    private ReloadSchemaHelper(File fileHelper) throws ReloadSchemaException {
        if(fileHelper == null) throw new ReloadSchemaException("Helper file cannot be null");
        _fileHelper = fileHelper;

        // Get the helper JDOM Document
        try {
            _docHelper = loadSchemaHelperFile(fileHelper);
        }
        catch(DweezilXMLException ex) {
            throw new ReloadSchemaException(ex);
        }
    }

    /**
     * Load the XML Schema File into a JDOM Document
     *
     * @param helperFile The XML Helper File to read in
     * @return The JDOM Document
     * @throws DweezilXMLException
     */
    protected Document loadSchemaHelperFile(File helperFile) throws DweezilXMLException {
        return XMLUtils.readXMLFile(helperFile);
    }

    /**
     * Get the JDOM Helper Document.
     * @return The JDOM Helper Document
     */
    protected Document getSchemaHelperDocument() {
        return _docHelper;
    }

    /**
     * Get the name of the IMS XML Schema we are helping.
     * @return The name of the IMS XML Schema
     */
    public String getSchemaName() {
        Element root = _docHelper.getRootElement();
        Attribute att = root.getAttribute(SCHEMA);
        return att == null ? null : att.getValue();
    }

    /**
     * Get a helper value for an element
     * @param domPath the path of the element concerned
     * @param helperKey the helper key to be used for getting the helper value
     * @return a helper value for an Element
     */
    public String getHelperValue(DomPath domPath, String helperKey) {
         if(_docHelper != null && _docHelper.hasRootElement()) {
            String path = domPath.getPath();
            Element root = _docHelper.getRootElement();
            Iterator i = root.getChildren(ELEMENT).iterator();
            while(i.hasNext()) {
                Element childElement = (Element)i.next();
                String name = childElement.getAttributeValue(PATH);
                if(name != null) {
                    if(path.equals(name) || path.endsWith(name)) {
                        // Try the attribute first
                        String value = childElement.getAttributeValue(helperKey);
                        // Try a child element
                        if(value == null) {
                            Element child = childElement.getChild(helperKey);
                            if(child != null) return child.getText();
                        }
                        else return value;
                    }
                }
            }
        }

        return null;
    }
}