/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import org.exolab.castor.xml.schema.*;
import org.jdom.*;
import uk.ac.reload.moonunit.dom.*;

/**
 * An abstract node that represents either an Element or an Attribute of the Schema.
 * These are hierarchical and contained by a ReloadSchema class.
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public abstract class ReloadSchemaNode {
    /**
     * A Castor Simple types factory that we shall re-use
     */
    static SimpleTypesFactory SIMPLETYPESFACTORY = new SimpleTypesFactory();

    /**
     * The parent of this node.  Either a ReloadSchemaElement or the topmost ReloadSchema
     */
    protected Object _parent;

    /**
     * Default Constructor
     */
    public ReloadSchemaNode() {

    }

    /**
     * Set the parent of this node.
     * @param parent the parent node
     * @return this node
     */
    public ReloadSchemaNode setParent(ReloadSchemaElement parent) {
        _parent = parent;
        return this;
    }

    /**
     * Get the parent node
     * @return the parent node or null if there isn't one or if the parent is the
     * topmost ReloadSchema
     */
    public ReloadSchemaNode getParent() {
        if(_parent instanceof ReloadSchemaNode) return (ReloadSchemaNode)_parent;
        return null;
    }

    /**
     * Get the owning ReloadSchema that this node is a member of
     * @return the owning ReloadSchema that this node is a member of or null if
     * there isn't one
     */
    public ReloadSchema getReloadSchema() {
        if(_parent instanceof ReloadSchema) return (ReloadSchema)_parent;
        if(_parent instanceof ReloadSchemaElement) return (ReloadSchema) ((ReloadSchemaElement)_parent).getReloadSchema();
        return null;
    }

    /**
     * Determine whether this node belongs to another Namespace
     * @return true if this node belongs to another Namespace, false otherwise
     */
    public boolean isExternalNamespace() {
        return !getNamespacePrefix().equals("");
    }

    /**
     * Get the name of this node - implementations should provide this
     * @return the name of this node
     */
    public abstract String getName();

    /**
     * Get the path to this node - implementations should provide this
     * @return the path to this node
     */
    public abstract DomPath getDomPath();

    /**
     * Get the vocabulary list for this node - implementations should provide this
     * @return the vocabulary list for this node
     */
    abstract ReloadVocabularyList getVocabularyList();

    /**
     * Get the type name of this node - implementations should provide this
     * @return the type name of this node
     */
    public abstract String getTypeName();

    /**
     * Get the base type name of this node - implementations should provide this
     * @return the base type name of this node
     */
    public abstract String getBaseTypeName();

    /**
     * Get the default value of this node.
     * @return the default value of this node or null if no default was specified
     */
    public abstract String getDefaultValue();

    /**
     * Get a Facet Value
     * @param facetName the name of the facet
     * @return a Facet Value or null
     */
    public abstract String getFacetValue(String facetName);

    /**
     * Get the Prefix of this Node
     * @return the Prefix of this Node, or "" if no prefix
     */
    public abstract String getNamespacePrefix();

    /**
     * Get the Namespace of this Attribute
     * @return the Namespace of this Attribute
     */
    public abstract Namespace getNamespace();
}