/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.moonunit.dom.*;


/**
 * The Reload Vocabulary - Defines a Vocabulary mapping between Elements/Attributes
 * and a Vocabulary List.  The actual Vocabulary is an XML File that defines the mappings.
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-10-01
 */
public class ReloadVocabulary {
    /**
     * Static ReloadVocabularies - we only want to create these just once and re-use them
     */
    private static Hashtable reloadVocabulariesTable = new Hashtable();

    /**
     * Factory method for getting a static re-usable ReloadVocabulary
     * @param fileVocab The file of the Vocabulary file to get
     * @return The ReloadVocabulary or null if not found
     * @throws ReloadSchemaException
     */
    public static ReloadVocabulary getReloadVocabulary(File fileVocab) throws ReloadSchemaException {
        if(fileVocab == null) return null;
        // Do we have it already?
        ReloadVocabulary reloadVocabulary = (ReloadVocabulary)reloadVocabulariesTable.get(fileVocab);
        // No, first time creation, create and store in table
        if(reloadVocabulary == null) {
            // Does it exist?
            if(fileVocab.exists()) {
                reloadVocabulary = new ReloadVocabulary(fileVocab);
                reloadVocabulariesTable.put(fileVocab, reloadVocabulary);
            }
        }
        return reloadVocabulary;
    }

    /**
     * The JDOM Document that is created from the XML File
     */
    protected Document _docVocab;

    /**
     * The XML Vocabulary File.
     */
    protected File _fileVocab;


    /**
     * The Vocabulary lists
     */
    protected Vector _lists;


    /**
     * Constructor
     *
     * @param fileVocab The Vocabulary XML File
     * @throws ReloadSchemaException
     */
    private ReloadVocabulary(File fileVocab) throws ReloadSchemaException {
        if(fileVocab == null) throw new ReloadSchemaException("Vocabulary file cannot be null");
        _fileVocab = fileVocab;

        // Get the vocab document
        try {
            _docVocab = loadVocabularyFile(fileVocab);
        }
        catch(DweezilXMLException ex) {
            throw new ReloadSchemaException(ex);
        }

        // Map the lists
        _lists = mapLists();

        // Set the default language
        String lang = getRootAttributeValue("lang");
        setDefaultLanguage(lang);
    }

    /**
     * Load the XML Vocabualry file into a JDOM Document.
     *
     * @param fileVocab the vocabulary XML file
     * @return The JDOM Document
     * @throws DweezilXMLException if it cannot create Document
     */
    private Document loadVocabularyFile(File fileVocab) throws DweezilXMLException {
        return XMLUtils.readXMLFile(fileVocab);
    }

    /**
     * Get the Vocabulary JDOM Document
     *
     * @return The Vocabulary JDOM Document
     */
    public Document getVocabularyDocument() {
        return _docVocab;
    }

    /**
     * Get the name of the vocabulary. e.g. "IMS-LRM"
     * @return The name of the vocabulary.
     */
    public String getVocabularyName() {
        return _docVocab.getRootElement().getAttributeValue("name");
    }


    /**
     * Get a Vocabulary List for a Given Element or Attribute.
     * @param schemaNode The Element or Attribute
     * @return The Vocabulary List or null if there isn't one
     */
    public ReloadVocabularyList getVocabularyList(ReloadSchemaNode schemaNode) {
        if(schemaNode == null) return null;

        // Get the path of this node
        DomPath nodePath = schemaNode.getDomPath();

        // Enumerate through the lists
        Enumeration lists = _lists.elements();
        while(lists.hasMoreElements()) {
            ReloadVocabularyList rvList = (ReloadVocabularyList)lists.nextElement();
            DomPath listPath = rvList.getDomPath();
            // Exact match or Partial match
            if(nodePath.equals(listPath) || nodePath.endsWith(listPath)) {
                // An over-riding none list
                if(rvList.getListName().equals("none")) return null;
                return rvList;
            }
        }

        return null;
    }

    // Tina Manoharan, 2003-11-01

    /**
     * Get a Vocabulary List for a Given Element or Attribute.
     * @param nodePath the domPath of the element
     * @return the Vocabulary List or null if there isn't one
     */
    public ReloadVocabularyList getVocabularyList(DomPath nodePath) {
        if(nodePath == null) return null;

        // Enumerate through the lists
        Enumeration lists = _lists.elements();
        while(lists.hasMoreElements()) {
            ReloadVocabularyList rvList = (ReloadVocabularyList)lists.nextElement();

            DomPath listPath = rvList.getDomPath();

            // Exact match or Partial match
            // if(nodePath.equals(listPath) || nodePath.endsWith(listPath)) {
            if(nodePath.equals(listPath)) {
                // An over-riding none list
                if(rvList.getListName().equals("none")) return null;
                return rvList;
            }
        }

        return null;
    }

    /**
     * Set the default language of format "en", "fr" etc
     * @param lang the default language
     */
    public void setDefaultLanguage(String lang) {
        // If null, set to default
        if(lang == null) lang = Locale.getDefault().getLanguage();

        Enumeration lists = _lists.elements();
        while(lists.hasMoreElements()) {
            ReloadVocabularyList rvList = (ReloadVocabularyList)lists.nextElement();
            if(rvList.isLangList()) rvList.setDefaultValue(lang);
        }
    }

    /**
     * Map the Vocabulary Lists in a Vector such that we can get a List from its path
     * @return a Vector of vocabulary lists
     */
    private Vector mapLists() {
        Vector map = new Vector();

        // Get all "element" children
        Iterator elementlist = _docVocab.getRootElement().getChildren("element").iterator();

        // Enumerate through them
        while(elementlist.hasNext()) {
            // Get the Element
            Element childElement = (Element) elementlist.next();

            // Get the Path and List name
            String listPath = childElement.getAttributeValue("path");
            String listName = childElement.getAttributeValue("list");

            // Tina Manoharan, 2003-11-01

            String defaultListName = childElement.getAttributeValue("defaultList");


            if(listPath == null || listName == null) continue;

            // New ReloadVocabularyList
            ReloadVocabularyList reloadvocabList = new ReloadVocabularyList(new DomPath(listPath), listName);
            map.addElement(reloadvocabList);

            // Tina Manoharan, 2003-11-01

            reloadvocabList.setDefaultListName(defaultListName);

            // Iterate through the vocab lists
            Iterator vocablist = _docVocab.getRootElement().getChildren("vocabList").iterator();
            while(vocablist.hasNext()) {
                Element vocabElement = (Element)vocablist.next();

                // Get the name of the vocab list
                String vocabName = vocabElement.getAttributeValue("name");
                // Oops!
                if(vocabName == null) break;

                // Do a match
                if(listName.equals(vocabName)) {
                    // Get each "item" in the list
                    List items = vocabElement.getChildren("item");

                    // Create an appropriate String array
                    String[] s = new String[items.size()];
                    for(int i = 0; i < items.size(); i++) {
                        Element e = (Element) items.get(i);

                        // Add item to String List
                        s[i] = e.getText();

                        // If default, save it
                        Attribute att = e.getAttribute("default");
                        if(att != null) reloadvocabList.setDefaultValue(e.getText());
                    }
                    reloadvocabList.setList(s);

					// Tina Manoharan, 2003-11-01

                    reloadvocabList.setDefaultList(s);


                    break;
                }
            }
        }

        return map;
    }

    /**
     * Get the value of a root attribute
     * @return the value of a root attribute or null
     */
    protected String getRootAttributeValue(String attName) {
        if(_docVocab != null && _docVocab.hasRootElement()) {
            Element root = _docVocab.getRootElement();
            Attribute att = root.getAttribute(attName);
            return att == null ? null : att.getValue();
        }
        return null;
    }
}