/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer;

import org.jdom.*;
import java.io.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;

import uk.ac.reload.moonunit.IMSDocumentHandler;

import uk.ac.reload.scormplayer.client.contentpackaging.*;
import uk.ac.reload.scormplayer.client.gui.*;
import uk.ac.reload.scormplayer.tomcat.*;


/**
 * Control station.  All of the methods required to import/export
 * start/reset/stop packages and also to start/stop Tomcat either via
 * the gui interfaces or via calls from another java app.
 * @author Paul Sharples
 * @version 2003-09-17
 */

public class ControlStation {

    // a var to decide whether or not to show the tree widget
    public static boolean showTreeWidget = true;

    // a var to decide whether or not to show the navigation widget
    public static boolean showNavigation = true;

    // a manifest handler reference, to parse the package
    protected static ReloadManifestHandler _manifest;

    /*
     ***************************************************************************
     *   Methods for importing, validation and manipulation of content packages.
     *   These methods are wrapped in this class, so they can be used by both
     *   the GUI in MainWindow and also called directly here, so the same things
     *   can be done without having to use the GUI interface.
     ***************************************************************************
     */


    /**
     * Method to ascertain if manifest is valid
     * @return true or false denoting if manifest is valid
     */
    public static boolean isValidManifest(File file) {
        if (file.toString().endsWith(ReloadManifestHandler.ARCHIVE_EXTENSION)) {
            // need to grab the manifest out of the zip file
            File tempFile = null;
            try {
                tempFile = File.createTempFile("~rld-", ".xml");
                // Make sure we delete it on exit
                tempFile.deleteOnExit();
                File man = findManifest(file, tempFile);
                if (man == null) return false;
                _manifest = new ReloadManifestHandler(man);
                return _manifest.testManifest();
            }
            catch (IOException ex) {
                System.out.println("Error: Could not create temporary file for parsing:" + ex);
                return false;
            }
            catch (ReloadException ex) {
                System.out.println("Error: An error occured while searching for the manifest inside zip entry:" + ex);
                return false;
            }
        }
        else if (file.toString().endsWith(ReloadManifestHandler.XML_EXTENSION)) {
            _manifest = new ReloadManifestHandler(file);
            return _manifest.testManifest();
        }
        else {
            return false;
        }
    }

   /**
     * Manifest Parser - used to obtain info about organizations and ascertain
     * which item reference SCOs/Assets.  Calls ReloadManifestHandler() to do this
     * @param aFile - file reference to the manifest
     */
    public static void parseManifest(File man) throws ReloadException{
        try {
            _manifest = new ReloadManifestHandler(man);
            _manifest.startParse();
        }
        catch (Exception ex) {
            System.out.println("Error parsing manifest:" + ex);
            throw new ReloadException(ex);
        }
    }

    /**
     * Method to ascertain if a file reference is a ZIP file
     * @param aFile
     * @return true or false
     */
    public static boolean isZip(File aFile) {
        if (aFile.getName().toLowerCase().endsWith(ReloadManifestHandler.
            ARCHIVE_EXTENSION))
            return true;
        else
            return false;
    }

    /**
     * Method to check if imsmanifest.xml file exists in a zip file.
     * @param zipFile
     * @returns the file if found, null if not found
     */
    public static File findManifest(File zipFile, File tempFile) throws ReloadException {
      File manifestFile = null;
       try {
           manifestFile = ZipUtils.extractZipEntry(zipFile, IMSDocumentHandler.MANIFEST_NAME, tempFile);
       }
       catch(DweezilZipException ex) {
           throw new ReloadException(ex);
       }
       return manifestFile;
    }

    /**
     * Calls ZipUtils() Unzips an archive into a specified folder
     * @param file
     * @param destinationFolder
     * @param mon
     * @return true or false if successful
     */
    public static boolean unpackZip(File file, File destinationFolder,
                                    ReloadProgressMonitor mon) {
        try {
            return ZipUtils.unpackZip(file, destinationFolder, mon);
        }
        catch (Exception ex) {
            return false;
        }
    }

    /**
     * Method to call ReloadUtils.copyFolder and copy across files & folders
     * to web server.
     * @param file
     * @param destinationFolder
     * @param mon
     * @return true or false
     */
    public static boolean copyFolder(File file, File destinationFolder,
                                     ReloadProgressMonitor mon)
                                     throws ReloadException {
        try {
            return ReloadUtils.copyFolder(file,
                                          destinationFolder, mon);
        }
        catch (IOException ex) {
            throw new ReloadException(ex);
        }
    }

    /**
     * Method to check to see if a particular project folder already exists
     * under the web server.
     * @param projectFolderName
     * @return either NULL if it exists already or a new File reference if doesnt
     */
    public static File checkProjectFolderAlreadyExists(String projectFolderName) {
        File newProjectFolder = new File(TomcatConstants.TOMCAT_PATH +
                                         TomcatConstants.SERVER_APP_FOLDER +
                                         TomcatConstants.SCORM_APP_PATH +
                                         TomcatConstants.COURSES_FOLDER +
                                         File.separatorChar +
                                         projectFolderName);
        if (newProjectFolder.exists())
            return null;
        else
            return newProjectFolder;
    }


    /**
     * Method to delete a folder and its contents
     * @param afolder -  a folder
     */
    public static void deleteFolder(File afolder) throws ReloadException {
        try {
            ReloadUtils.deleteFolder(afolder);
        }
        catch (Exception ex) {
            throw new ReloadException(ex);
        }
    }

/*
 *******************************************************************************
 * Methods for starting/stopping the web server and playing packages
 *******************************************************************************
 */

    /**
     * Method to start the web server
     */
    public static void startServer() {
        try {
            EmbeddedTomcat.sharedInstance.startTomcat();
        }
        catch (Exception ex) {
            // throw new ReloadException(ex);
            System.out.println("Cannot launch server:" + ex);
        }
    }


    /**
     * Method to ascertain if server is already running
     * @return true or false if server is running or not
     */
    public static boolean isAlive() {
        return EmbeddedTomcat.sharedInstance.isAlive();
    }


    /**
     * Method to stop web server
     */
    public static void stopServer() {
        try {
            EmbeddedTomcat.sharedInstance.stopTomcat();
        }
        catch (Exception ex) {
           // throw new ReloadException(ex);
            System.out.println("Cannot quit server:" + ex);
        }
    }


    /**
     * Method to launch a scorm package
     * @param thePackage - which package to launch
     */
    public static void startPackage(String thePackage) throws ReloadException {
        try {
            // change URL references with spaces to %20 (for valid URLs)
            thePackage = thePackage.replaceAll(" ", "%20");
            System.out.println("http://"+TomcatConstants.HOST+":8080/reload-scorm-player/ScormLaunch?scormPackageID=" +
                thePackage+"&nav="+ showNavigation + "&tree=" + showTreeWidget);
            ReloadUtils.launchFile(
                "http://"+TomcatConstants.HOST+":8080/reload-scorm-player/ScormLaunch?scormPackageID=" +
                thePackage+"&nav="+ showNavigation + "&tree=" + showTreeWidget);
        }
        catch (Exception ex) {
            throw new ReloadException(ex);
        }
    }


    public static void resetPackage(File path) {
        try {
               File man = new File (path.toString() + File.separatorChar + IMSDocumentHandler.MANIFEST_NAME);
               _manifest = new ReloadManifestHandler(man);
               _manifest.startParse();
           }
           catch (Exception ex) {
               System.out.println("Error parsing manifest:" + ex);
           }

    }


// ***************************************************************************
// TO DO - methods yet to be implemented
// ***************************************************************************



    public static void importPackage(File manifestOrArchive,
                                     String projectFolderName) {}

    public static void removePackage(File projectFolder) {}

    public static boolean doesPackageExist() {
        return true;
    }
}