/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */



package uk.ac.reload.scormplayer.client.contentpackaging;

import java.io.*;
import java.util.*;
import org.jdom.*;

import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.contentpackaging.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.prefs.*;

import uk.ac.reload.scormplayer.client.gui.*;
import uk.ac.reload.scormplayer.tomcat.*;
import uk.ac.reload.scormplayer.client.datamodels.*;
import uk.ac.reload.scormplayer.client.gui.prefs.*;

/**
 * The ScormPackageHandler Class.  A class used to parse a scorm imsmanifest.xml
 * file and build a navigation file that can then be used by the run-time system
 * Each item is examined and a decision is made about whether or not to generate
 * a sco cmi data model.
 *
 * @author Paul Sharples
 * @version 2003-07-29
 */
public class ScormPackageHandler extends ReloadDocument {

    /**
     * some constant element names that we will use in our navigation file
     */
    public static final String NAVIGATION_ROOT = "course";
    public static final String NAVIGATION_ROOT_DEFAULT = "default";
    public static final String NAVIGATION_ORG = "org";
    public static final String NAVIGATION_ORG_ID = "id";
    public static final String NAVIGATION_ORG_TITLE = "title";
    public static final String NAVIGATION_ITEM = "item";
    public static final String NAVIGATION_ITEM_ID = "id";
    public static final String NAVIGATION_ITEM_ISVISIBLE = "isvisible";
    public static final String NAVIGATION_TITLE = "title";
    public static final String NAVIGATION_LEVEL = "level";
    public static final String NAVIGATION_SEQUENCE = "sequence";
    public static final String NAVIGATION_LAUNCH = "launch";
    public static final String NAVIGATION_TYPE = "type";
    public static final String NAVIGATION_PREREQUISITES = "prerequisites";


    /**
     * var used to ascertain the order of sequence of items in an org.
     */
    private int _orderCount = 0;

    /**
     * Our instance of core scorm methods from the moonunit lib
     */
    protected SCORM1_2Core _scormCore;

    /**
     * Our unique signature
     */
    static final String[] scorm_comments = {
        "This is a ReloadScormPlayer version " + ReloadScormPlayer.VERSION + " SCORM 1.2 Content Package Navigation file",
        "Spawned from the Reload Scorm Player 1.2 - http://www.reload.ac.uk"
    };

    /**
     * Default Constructor
     */
    public ScormPackageHandler() {
        super();
        _scormCore = new SCORM1_2Core(this);
    }

    /**
     * Constructor with File & JDOM doc params
     */
    public ScormPackageHandler(File file, Document doc) {
        super(file, doc);
        _scormCore = new SCORM1_2Core(this);
    }

    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return scorm_comments;
    }

    /**
     * A class to actually do the bulk of the work. It creates an xml file
     * representing the organizations, similar to the imsmanifest, but also
     * modelling the sco/asset attributes needed by the runtime system - ie
     * order of sequence, launch URL...
     */
    public void buildSettings(){
       // first create our naviagtion document
       Document settings = createNewSettingsDoc();
       // get the root element of the manifest
       Element manifestRoot = _doc.getRootElement();
       // now get the organizations node
       Element orgs = manifestRoot.getChild(SCORM1_2Core.ORGANIZATIONS, manifestRoot.getNamespace());
       // get the identifier for the default organization
       Element defaultOrgNode = getDefaultOrganization(orgs);
       if (defaultOrgNode != null) {
           // and store the default identifier
           String defaultOrgIdentifier = defaultOrgNode.getAttributeValue(
               SCORM1_2Core.IDENTIFIER);
           // set the default organization
           settings.getRootElement().setAttribute("default",
               defaultOrgIdentifier);

           iterateThruManifest(manifestRoot, settings.getRootElement(),
                               _orderCount);
       }

       // finally, commit our navigation file to disk...
       File settingsFile = new File(_file.getParentFile().toString() +
                                      File.separatorChar + "reload-settings" +
                                      File.separatorChar + "settings.xml");
       settingsFile.getParentFile().mkdirs();
       WriteDefaultSettings (settingsFile, settings);
    }


    /**
     * A method to read through the imsmanifest and build our JDOM model in
     * memory representing our navigation file.
     * @param element
     * @param settings
     * @param indentCount
     */
    public void iterateThruManifest(Element element, Element settings,
                                    int indentCount) {
        String name = element.getName();
        // ORGANIZATION
        if (name.equals(SCORM1_2Core.ORGANIZATION) &&
            isDocumentNamespace(element)) {
            _orderCount = 0;
            String id = element.getAttributeValue(SCORM1_2Core.IDENTIFIER);

            Element org = new Element("org");
            org.setAttribute("id", id);
            settings.addContent(org);

            String title = "Organization";

            Element titleElement = element.getChild(SCORM1_2Core.TITLE,
                element.getNamespace());
            if (titleElement != null) {
                if (!titleElement.getText().equals(""))
                    title = titleElement.getText();
            }
            org.setAttribute("title", title);

            // up date the node in the settings xml doc
            settings = org;
        }

        else if (name.equals(SCORM1_2Core.ITEM) &&
                 isDocumentNamespace(element)) {
            StringBuffer sb = new StringBuffer();

            _orderCount++;

            // add the item to our settings file
            String id = element.getAttributeValue(SCORM1_2Core.IDENTIFIER);

            Element org = new Element("item");
            org.setAttribute("id", id);


            String isvisible = null;
            isvisible = element.getAttributeValue(NAVIGATION_ITEM_ISVISIBLE);
            if (isvisible == null){
               isvisible = "true";
            }

            org.setAttribute(NAVIGATION_ITEM_ISVISIBLE, isvisible);
            settings.addContent(org);

            // Add the title
            String title = "Item";
            Element titleElement = element.getChild(SCORM1_2Core.TITLE,
                element.getNamespace());
            if (titleElement != null) {
                if (!titleElement.getText().equals(""))
                    title = titleElement.getText();
            }

            // add the title to this item
            Element theTitle = new Element("title");
            theTitle.setText(title);
            org.addContent(theTitle);

            // add the indent count to this item
            Element theLevel = new Element("level");
            String indentStr = element.getParent().getAttributeValue(SCORM1_2Core.IDENTIFIER);
            theLevel.setText(indentStr);
            org.addContent(theLevel);

            if (!isvisible.equals("false")){
            // add the normal (without prerequistes) order
            Element theseq = new Element("sequence");
            String seqStr = "" + _orderCount;
            theseq.setText(seqStr);
            org.addContent(theseq);
            }

            // next see if this item references a resource
            // If it does, then look for the href
            // and also scotype...
            String thestat = "";

            String url = "";
            String scoType = "";
            Element ref_element = getReferencedElement(element);
            if (ref_element != null) {
                String ref_name = ref_element.getName();
                // A RESOURCE
                if (ref_name.equals(SCORM1_2Core.RESOURCE)) {
                    // get the sco type
                    String theScoType = ref_element.getAttributeValue(
                        "scormtype", IMSDocumentHandler.ADLCP_NAMESPACE_12);
                    if (theScoType != null) {
                        scoType = theScoType;
                        thestat = "not attempted";
                    }
                    url = getURL(element);
                    if (url.startsWith("file:///")) {
                        String tempHref;
                        if (ReloadUtils.getOS()!= ReloadUtils.MACINTOSH){
                          tempHref = url.substring(8, url.length()); // windows
                        }
                        else{
                          tempHref = url.substring(7, url.length());//mac
                        }
                        tempHref = tempHref.replaceAll("%20", " ");
                        String testHref = TomcatConstants.FULL_APP_PATH.toString().replace('\\', '/');
                        testHref = testHref.replaceAll("%20", " ");
                        if (tempHref.startsWith(testHref)) {
                            String localUrlMinusPath = tempHref.substring(
                                TomcatConstants.FULL_APP_PATH.toString().length()+1,
                                tempHref.length()
                                );
                            String correctLocalUrl = localUrlMinusPath.replace(
                                '\\', '/');
                            url = correctLocalUrl;
                        }
                    }

                }
            }

            // set our launch url
            Element theLaunchUrl = new Element("launch");
            theLaunchUrl.setText(url);
            org.addContent(theLaunchUrl);

            // set our status
            Element thestatus = new Element("status");
            thestatus.setText(thestat);
            org.addContent(thestatus);

            // and set the sco type
            Element theSco = new Element("type");
            theSco.setText(scoType);
            org.addContent(theSco);

            // next we need to find any prerequisites
            String preqText = "";
            Element prereq = element.getChild(SCORM1_2Core.PREREQUISITES,
                                              IMSDocumentHandler.
                                              ADLCP_NAMESPACE_12);
            if (prereq != null) {
                preqText = prereq.getText();
            }

            Element theprerequisites = new Element(SCORM1_2Core.PREREQUISITES);
            theprerequisites.setText(preqText);
            org.addContent(theprerequisites);

            // next we need to find any MAXTIMEALLOWED entries
            String maxTimeText = "";
            Element maxTime = element.getChild(SCORM1_2Core.MAXTIMEALLOWED,
                                               IMSDocumentHandler.
                                               ADLCP_NAMESPACE_12);
            if (maxTime != null) {
                maxTimeText = maxTime.getText();
            }

            // next find any TIMELIMITACTION entries
            String timeLimitText = "";
            Element timeLimit = element.getChild(SCORM1_2Core.
                                                 TIMELIMITACTION,
                                                 IMSDocumentHandler.
                                                 ADLCP_NAMESPACE_12);
            if (timeLimit != null) {
                timeLimitText = timeLimit.getText();
            }

            // next find any DATAFROMLMS entries
            String datafromLmsText = "";
            Element dataFromLms = element.getChild(SCORM1_2Core.DATAFROMLMS,
                IMSDocumentHandler.
                ADLCP_NAMESPACE_12);
            if (dataFromLms != null) {
                datafromLmsText = dataFromLms.getText();
            }

            // next find any MASTERYSCORE entries
            String masteryScoreText = "";
            Element masteryScore = element.getChild(SCORM1_2Core.
                MASTERYSCORE,
                IMSDocumentHandler.
                ADLCP_NAMESPACE_12);
            if (masteryScore != null) {
                masteryScoreText = masteryScore.getText();
            }

            // if the scoType is "sco", then we have to generate our CMI model
            // for it...
            if (scoType.compareTo("sco") == 0) {
                ReloadCMIDataModel scoModel = new ReloadCMIDataModel(
                    ReloadScormPlayerPrefs.getInstance().getValue(ReloadScormPlayerPrefs.DEFAULT_USERID),
                    ReloadScormPlayerPrefs.getInstance().getValue(ReloadScormPlayerPrefs.DEFAULT_USERNAME),
                     maxTimeText, timeLimitText,
                     datafromLmsText, masteryScoreText, url);

                scoModel.buildFreshModel();
                Document theModel = scoModel.getModel();
                File scoFile = new File(_file.getParentFile().toString() +
                                        File.separatorChar + "reload-settings" +
                                        File.separatorChar + id + ".xml");
                scoFile.getParentFile().mkdirs();
                WriteDefaultSettings(scoFile, theModel);
            }
        }

        Iterator it = element.getChildren().iterator();
        while (it.hasNext()) {
            Element child = (Element) it.next();
            iterateThruManifest(child, settings, indentCount);
        }
    }



    /**
     * Helper method to initially set up our new navigaton file,
     * create root node, add comments to it etc...
     * @return
     */
    public Document createNewSettingsDoc(){
        Element root = new Element ("course");
        Document doc = new Document(root);
        for(int i = 0; i < scorm_comments.length; i++) {
             Comment comment = new Comment(scorm_comments[i]);
              doc.getContent().add(0, comment);
        }
        return doc;
    }


    /**
     * A method to save our navigation file.  We havent loaded an existing
     * xml doc, instead we have created it as a JDOM doc in memory, during our
     * process of parsing the manifest, so we need a helper method here to save it
     * @param settings
     * @param doc
     */
    public void WriteDefaultSettings(File settings, Document doc) {
        try {
            XMLUtils.write2XMLFile(doc, settings);
        }
        catch (Exception ex) {
            System.out.println("Could not write file " + settings.toString());
        }
    }


    /**
     * getDefaultOrganization - wraps the same method from the moonunit lib
     * found in SCORM1_2Core.
     * @param orgs
     * @return - the JDOM element representing the default organization
     */
    public Element getDefaultOrganization(Element orgs) {
        return _scormCore.getDefaultOrganization(orgs);
    }

    /**
     * getReferencedElement - wraps the same method from the moonunit lib
     * found in SCORM1_2Core.
     * @param sourceElement
     * @return - the JDOM element representing the resource
     */
    public Element getReferencedElement(Element sourceElement) {
        return _scormCore.getReferencedElement(sourceElement);
    }

    /**
     * @return The URL string that an Element references
     */
    public String getURL(Element element) {
        return _scormCore.getURL(element);
    }
}
