/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow;

import javax.swing.*;
import java.awt.*;
import javax.swing.border.LineBorder;
import java.io.*;


import javax.swing.tree.*;


import uk.ac.reload.moonunit.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.tree.*;
import uk.ac.reload.dweezil.util.*;

import uk.ac.reload.scormplayer.*;
import uk.ac.reload.scormplayer.client.gui.*;
import uk.ac.reload.scormplayer.client.contentpackaging.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.itemwindow.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow.*;
import uk.ac.reload.scormplayer.tomcat.TomcatConstants;


/**
 * Our main GUI window. Used to show three main panels. These are:
 * (1) The package panel, which shows the packages that have currently been imported
 * into the application
 * (2) The organizations panel used to show the default organization to play
 * in the runtime environment (but also allows the user to change this to other
 * organizations as well)
 * (3) The sco/asset panel which shows the information for a sco/asset,
 * highlighted in the organization panel
 *
 * Our MainWindow also acts as a wrapper to separate graphical swing components
 * away from methods which may be used standalone.
 *
 * @author Paul Sharples
 * @version 2003-09-29
 */
public class MainWindow extends JPanel {

    /**
     * Our Tree panel for imported packages
     */
    public static TreePanel _treePanel;

    /**
     * Info panel - to show package state
     */
    public static InfoPanel _infoPanel;

    /**
     * Nav panel - to show organization
     */
    public static NavPanel _navPanel;


    public static String _projectName;

    /**
     * Default Constructor
     */
    public MainWindow() {
        setLayout(new BorderLayout());
        // projects pane
        _treePanel = new TreePanel();
        // navigation pane
        _navPanel = new NavPanel();
        // information pane
        _infoPanel = new InfoPanel();
        // set all borders to null
        _infoPanel.setBorder(null);
        _treePanel.setBorder(null);
        _navPanel.setBorder(null);
        this.setBorder(new LineBorder(Color.GRAY,1));
        JSplitPane jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        jsp.setBorder(null);
        jsp.setContinuousLayout(true);
        // allow user to colapse this divider
        jsp.setOneTouchExpandable(true);

        JSplitPane jsp2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        jsp2.setBorder(null);
        jsp2.setContinuousLayout(true);
        jsp2.setLeftComponent(_navPanel);
        jsp2.setRightComponent(_infoPanel);
        jsp2.setDividerLocation(250);
        // allow user to colapse this divider
        jsp2.setOneTouchExpandable(true);
        jsp.setLeftComponent(_treePanel);
        jsp.setRightComponent(jsp2);
        jsp.setDividerLocation(250);
        this.add(jsp);
        // let the nav panel listen to node changes on the package tree
        _treePanel.getPackageTree().addTreeSelectionListener(_navPanel.getOrgPanel());
        // let the item panel listen to node changes on the navigation & package tree
        _navPanel.getOrgPanel().getNavTree().addTreeSelectionListener(_infoPanel.getItemPanel());
        _treePanel.getPackageTree().addTreeSelectionListener(_infoPanel.getItemPanel());
        _treePanel.getPackageTree().setSelectionRow(0);
    }


    /**
     * Method to refresh the package tree in order for it to re-read the
     * the current state of the "imported packages" folder
     */
    public static void refreshScormTree() {
        // Because pretty much everything is listening to the package tree
        // we select row one before and after the actual tree update, so the
        // listeners also update and point to the first tree node.
        int[] i = _treePanel.getPackageTree().getSelectionRows();
        _treePanel.getPackageTree().setSelectionRow(0);
        _treePanel.getPackageTree()._treeModel.setTree();
        _treePanel.getPackageTree().setSelectionRow(i[0]);
    }

    /**
     * Accessor method to return which package is currently selected
     * in the package tree.
     * @return a file reference to the working folder of this package
     */
    public static File getLaunchPackage() {
        return _treePanel.getPackageTree().getSelectedPackage();
    }


    /**
     * Import Content Package into viewer
     */
    public static MainWindow openContentPackage() {
        File file = null;
        File theFolder = null;
        ReloadProgressMonitor progressMonitor = null;
        try {
            file = MainWindow.askManifestOpen();
            if (file == null)
                return null;

            // If the user has opened an archive then theres a bit of work to do
            if (ControlStation.isZip(file)) {

                // before importing the files, check the manifest for validity
                // (also checks that there is a manifest in the zip)
                if (!ControlStation.isValidManifest(file)) {
                    JOptionPane.showMessageDialog(ReloadScormPlayer.getInstance(),
                                                  "Manifest file " +
                        " does not appear to be Scorm 1.2. conformant.",
                        "Manifest Error",
                        JOptionPane.WARNING_MESSAGE);
                    return null;
                }

                // ask for project folder
                theFolder = MainWindow.askProjectFolder();
                // If the folder already contains the manifest or there
                // was some other problem then return
                if (theFolder == null) {
                    return null;
                }

                // okay up to now? ...then upzip to project folder...
                progressMonitor = new ReloadProgressMonitor(ReloadScormPlayer.
                    getInstance(), "Unzipping Content Package",
                    "Unzipping...", "Searching...", true, ReloadUIManager.getIcon(ReloadScormPlayer.ICON_APP16));

                boolean result = ControlStation.unpackZip(file, theFolder,
                    progressMonitor);

                //Pressed Cancel
                if (!result)
                    return null;

                // Get the full path to the unzipped manifest
               String fileRef = theFolder.getAbsoluteFile().toString() +
                   File.separatorChar
                   + IMSDocumentHandler.MANIFEST_NAME;
               file = new File(fileRef);

               // make sure it is there in case somthing went wrong...
               if (!file.exists())
                   return null;
            }
            else {
                // must be the imsmanifest.xml - but check to make sure its valid
                // before importing the files, check the manifest for validity
                if (!ControlStation.isValidManifest(file)) {
                    JOptionPane.showMessageDialog(ReloadScormPlayer.getInstance(),
                                                  "Manifest file" +
                        " does not appear to be Scorm 1.2. conformant.",
                        "Manifest Error",
                        JOptionPane.WARNING_MESSAGE);

                    return null;
                }

                // okay up to now, so copy the folder contents across
                // ask for project folder
                theFolder = MainWindow.askProjectFolder();
                // if user hit cancel..
                if (theFolder == null) {
                    return null;
                }


                progressMonitor = new ReloadProgressMonitor(ReloadScormPlayer.
                    getInstance(), "Copying Content Package",
                    "Copying...", "Please Wait...", true, ReloadUIManager.getIcon(ReloadScormPlayer.ICON_APP16));

                boolean result = ControlStation.copyFolder(new File(file.getParent()),
                    theFolder, progressMonitor);
                //Pressed Cancel
                if (!result)
                    return null;
            }
            File newProjectManifest = new File(TomcatConstants.TOMCAT_PATH +
                                    TomcatConstants.SERVER_APP_FOLDER +
                                    TomcatConstants.SCORM_APP_PATH +
                                    TomcatConstants.COURSES_FOLDER +
                                    File.separatorChar +
                                    _projectName +
                                    File.separatorChar +
                                    "imsmanifest.xml");

            // make sure it is there in case somthing went wrong...
                if (!newProjectManifest.exists())
                    return null;

            // start the parsing...
            ControlStation.parseManifest(newProjectManifest);
        }
        catch (Exception ex) {
            if (ReloadScormPlayer.DEBUG)
                ex.printStackTrace();
            ErrorDialogBox.showWarning("Error in file: " + file.getName(),
                                       "Could not load manifest file", ex);
        }
        finally {

            if (progressMonitor != null)
                progressMonitor.close();
        }
        return null;
    }

    /**
     * Ask for a Manifest file to open
     */
    public static File askManifestOpen() {
        // Make a Filter
        javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.
            FileFilter() {
            public boolean accept(File file) {
                if (file.isDirectory())
                    return true;
                return file.getName().equalsIgnoreCase(IMSDocumentHandler.
                    MANIFEST_NAME)||
                    file.getName().toLowerCase().endsWith(ReloadManifestHandler.
                    ARCHIVE_EXTENSION);
            }
            public String getDescription() {
                return "Manifest or Archive Files";
            }
        };
        return ReloadUtils.askFileNameOpen(ReloadScormPlayer.
            getInstance(), "Open Manifest or Archive", filter);
    }

    /**
     * Ask for a Project Folder
     * @return the Folder or null if cancelled or folder not valid
     */
    public static File askProjectFolder() {
        // read ahead loop
        String message = "Package name";
        String foldername = JOptionPane.showInputDialog(ReloadScormPlayer.
            getInstance(),
            "Please choose a name for this Package", message,
            JOptionPane.QUESTION_MESSAGE);
        while (ControlStation.checkProjectFolderAlreadyExists(foldername) == null && (foldername != null)) {
            foldername = JOptionPane.showInputDialog(ReloadScormPlayer.
                getInstance(),
                "Package name already exists, try another", message,
                JOptionPane.QUESTION_MESSAGE);
        }
        File result = ControlStation.checkProjectFolderAlreadyExists(foldername);
        if (result != null && foldername != null){
            _projectName = foldername;
            return result;
        }
        else
            return null;
    }


    /**
     * Method to allow user to remove a currently selected package from the app
     * Again, this is essentially a wrapper method - removing swing components
     * away from a file operation.
     */
    public static void removePackage() {
        File packageToDelete =  _treePanel.getPackageTree().getSelectedPackage();

        int answer = JOptionPane.showConfirmDialog(ReloadScormPlayer.getInstance(),
        "Are you sure you want to delete this package\n" +
        "You will not be able to undo this action.", "Confirm Delete",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE);
        if(answer == JOptionPane.YES_OPTION){
            _treePanel.getPackageTree().setSelectionRow(0);
            try {
                ControlStation.deleteFolder(packageToDelete);
            }
            catch (ReloadException ex) {
                System.out.println("Error. Could not delete folder: " + ex);
            }
            refreshScormTree();
        }
    }
}