/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.gui;

import java.net.*;
import java.util.*;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.dweezil.*;

/**
 * The Reload UI Manager.
 * This takes care of general application UI duties such as setting the L&F
 * or getting Icons and Colours etc.
 *
 * @author Phillip Beauvoir
 * @version 2003-09-17
 */
public class ReloadUIManager implements DweezilConstants {


    // ========================================================================
    // ============================== CURSORS =================================
    // ========================================================================
    public static Cursor TEXT_CURSOR = Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR);
    public static Cursor WAIT_CURSOR = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    public static Cursor DEFAULT_CURSOR = Cursor.getDefaultCursor();


    // ========================================================================
    // =============================== FONTS ==================================
    // ========================================================================
    public static Font plainFont11 = new Font("SansSerif", Font.PLAIN, 11);
    public static Font boldFont11 = new Font("SansSerif", Font.BOLD, 11);
    public static Font italicFont11 = new Font("SansSerif", Font.ITALIC, 11);
    public static Font bold_italicFont11 = new Font("SansSerif", Font.ITALIC + Font.BOLD, 11);
    public static Font plainFont12 = new Font("SansSerif", Font.PLAIN, 12);
    public static Font boldFont12 = new Font("SansSerif", Font.BOLD, 12);
    public static Font italicFont12 = new Font("SansSerif", Font.ITALIC, 12);
    public static Font bold_italicFont12 = new Font("SansSerif", Font.ITALIC + Font.BOLD, 12);
    public static Font plainFont13 = new Font("SansSerif", Font.PLAIN, 13);
    public static Font boldFont13 = new Font("SansSerif", Font.BOLD, 13);
    public static Font italicFont13 = new Font("SansSerif", Font.ITALIC, 13);
    public static Font bold_italicFont13 = new Font("SansSerif", Font.ITALIC + Font.BOLD, 13);
    public static Font plainFont14 = new Font("SansSerif", Font.PLAIN, 14);
    public static Font boldFont14 = new Font("SansSerif", Font.BOLD, 14);


    // =========================================================================
    // LOOK AND FEEL
    // =========================================================================

    /**
     * Do some UI Defaults stuff for the Mac
     */
    public static void setMacUIDefaults(String applicationName) {
        // APPLE MAC STUFF
        // See http://developer.apple.com/techpubs/macosx/ReleaseNotes/java141/system_properties/index.html
        // If you are using the Aqua look and feel, this puts Swing menus in the Mac OS X menu bar.
        // Note that JMenuBars in JDialogs are not moved to the Mac OS X menu bar.
        System.setProperty("apple.laf.useScreenMenuBar", "true");
        // Sets the application name that is displayed in the application menu and in the Dock.
        System.setProperty("com.apple.mrj.application.apple.menu.about.name", applicationName);
        // Most native Mac OS X windows have a resize control in the bottom right corner.
        // By default, Java application windows that use the Aqua look and feel have the functionality of this control,
        // but there is no user interface cue that it is there. This flag causes the resize control to be displayed.
        System.setProperty("apple.awt.showGrowBox", "true");
    }

    /**
     * Set The Look and Feel to a class name, these will have to be installed ones, or inbuilt ones.
     * @param className The class name for the L&F
     * @param components The components (typically JFrames) that will need to be updated
     */
    public static void setLookAndFeel(String className, Component[] components) {
        try {
            UIManager.setLookAndFeel(className);
            updateComponentTreeUI(components);
        }
        catch(Exception ex) {
            // Try again with system l&f
            try {
                String lf = UIManager.getSystemLookAndFeelClassName();
                UIManager.setLookAndFeel(lf);
                updateComponentTreeUI(components);
            }
            catch(Exception ex1) {
                if(DEBUG) System.out.println("Error loading L&F: " + ex);
            }
        }
    }

    /**
     * Update any components' UI
     */
    private static void updateComponentTreeUI(Component[] components) {
        for(int i = 0; i < components.length; i++) {
            SwingUtilities.updateComponentTreeUI(components[i]);
        }
    }

    // =========================================================================
    // ICONS
    // =========================================================================

    /**
     * A collection of Icons
     */
    private static Hashtable icons = new Hashtable();

    /**
     * Get an Icon
     * @param iconPath The full path of the Icon
     * @return The ImageIcon or null if not found
     */
    public static ImageIcon getIcon(String iconPath) {
         try {
            ImageIcon imageIcon = (ImageIcon)icons.get(iconPath);
            if(imageIcon == null) {
                URL url = icons.getClass().getResource(iconPath);
                if(url == null) return null;
                imageIcon = new ImageIcon(url);
                icons.put(iconPath, imageIcon);
            }

            return imageIcon;
         }
         catch(Exception ex) {
            return null;
         }
    }

    /**
     * Gets a resource image
     * @param imagePath The full path of the Image
     * @return The Image or null if not found
     */
    public static Image getImage(String imagePath) {
        ImageIcon imageIcon = getIcon(imagePath);
        if(imageIcon != null) return imageIcon.getImage();
        else return null;
    }
}