/**
  *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui.treetable;

import java.util.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

/**
 * An extension of the JTreeTable
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-03-12
 */
public class ReloadTreeTable
extends JTreeTable
{
    protected ReloadTreeTableModel treeTableModel;

    public ReloadTreeTable(ReloadTreeTableModel treeTableModel) {
        super(treeTableModel);
        this.treeTableModel = treeTableModel;
    }

    /**
     * Over-ride this - see comment in TreeTableModelAdapter
     * @param e the TableModelEvent
     */
    public void tableChanged(TableModelEvent e) {
        revalidate();
        super.tableChanged(e);
    }

    /**
     * Get the Treetable Model
     * @return the ReloadTreeTableModel
     */
    public ReloadTreeTableModel getTreeTableModel() {
        return treeTableModel;
    }

    /**
     * Select a node on the tree and scroll to it
     * This is some voodoo because of the way the TreeTabelModelAdapter calls
     * fireTableDataChanged() when a node is added/deleted
     * So we have to put this on a thread.
     * @param node the DefaultMutableTreeNode concerned
     */
    public synchronized void selectNode(final DefaultMutableTreeNode node) {
        if(node != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    TreePath path = new TreePath(node.getPath());
                    tree.setSelectionPath(path);
                    Rectangle bounds = getCellRect(tree.getRowForPath(path), 0, true);
                    scrollRectToVisible(bounds);
                    tree.setSelectionPath(path);  // Need to do this twice!!
                }
            });
        }
    }

    /**
    * Returns the currently selected node
    * @return the currently selected DefaultMutableTreeNode
    */
    public DefaultMutableTreeNode getSelectedNode() {
        TreePath selPath = tree.getSelectionPath();
        if(selPath != null) return (DefaultMutableTreeNode)selPath.getLastPathComponent();
        else return null;
    }

    /**
    * Update (repaint) the node that is currently selected
    */
    public void updateSelectedNode() {
        DefaultMutableTreeNode node = getSelectedNode();
        if(node != null) treeTableModel.nodeChanged(node);
    }

    /**
    * Gets a node on the tree given an Object
	* @return a node on the tree that is related to the given user object
    */
    public DefaultMutableTreeNode getNode(Object object) {
        if(object == null) return null;

        Enumeration nodes = ((DefaultMutableTreeNode)treeTableModel.getRoot()).preorderEnumeration();
        // Drill into and find it
        while(nodes.hasMoreElements()) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodes.nextElement();
            Object o = node.getUserObject();
            if(o == object) return node;
        }

        return null;
    }

}