/*
 * @(#)TreeTableModelAdapter.java	1.2 98/10/27
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package uk.ac.reload.dweezil.gui.treetable;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

/**
 * This is a wrapper class takes a TreeTableModel and implements
 * the table model interface. The implementation is trivial, with
 * all of the event dispatching support provided by the superclass:
 * the AbstractTableModel.
 *
 * @version 1.2 10/27/98
 *
 * @author Philip Milne
 * @author Scott Violet
 */
public class TreeTableModelAdapter
extends AbstractTableModel {

    JTree tree;
    TreeTableModel treeTableModel;

	/**
	 * Constructor
	 * @param treeTableModel the TreeTableModel
	 * @param tree the JTree
	 */
    public TreeTableModelAdapter(TreeTableModel treeTableModel, JTree tree) {
        this.tree = tree;
        this.treeTableModel = treeTableModel;

        tree.addTreeExpansionListener(new TreeExpansionListener() {
            // Don't use fireTableRowsInserted() here; the selection model
            // would get updated twice.
            public void treeExpanded(TreeExpansionEvent event) {
                fireTableDataChanged();
            }

            public void treeCollapsed(TreeExpansionEvent event) {
                fireTableDataChanged();
            }
        });

        // Install a TreeModelListener that can update the table when
        // tree changes. We use delayedFireTableDataChanged as we can
        // not be guaranteed the tree will have finished processing
        // the event before us.
        treeTableModel.addTreeModelListener(new TreeModelListener() {

            public void treeNodesChanged(TreeModelEvent e) {
                delayedFireTableDataChanged();
            }

            public void treeNodesInserted(TreeModelEvent e) {
                delayedFireTableDataChanged();
            }

            public void treeNodesRemoved(TreeModelEvent e) {
                delayedFireTableDataChanged();
            }

            public void treeStructureChanged(TreeModelEvent e) {
                delayedFireTableDataChanged();
            }
        });
    }

    // Wrappers, implementing TableModel interface.

	/**
	 * Returns the number of columns in the model.
	 * A JTable uses this method to determine how many columns it should create and display by default.
	 * @return the number of columns in the model
	 */
    public int getColumnCount() {
        return treeTableModel.getColumnCount();
    }

	/**
	 * Returns the name of the column at columnIndex.
	 * This is used to initialize the table's column header name.
	 * Note: this name does not need to be unique; two columns in a table can have the same name.
	 * @param column the index of the column
	 * @return the name of the column
	 */
    public String getColumnName(int column) {
        return treeTableModel.getColumnName(column);
    }

	/**
	 * Returns the most specific superclass for all the cell values in the column.
	 * This is used by the JTable to set up a default renderer and editor for the column.
	 * @param column the index of the column
	 * @return the common ancestor class of the object values in the model
	 */
    public Class getColumnClass(int column) {
        return treeTableModel.getColumnClass(column);
    }

	/**
	 * Returns the number of rows in the model.
	 * A JTable uses this method to determine how many rows it should display.
	 * This method should be quick, as it is called frequently during rendering.
	 * @return the number of rows in the model
	 */
    public int getRowCount() {
        return tree.getRowCount();
    }

	/**
	 * Get the tree node for the given row index
	 * @param row the index of the row
	 * @return the tree node at the end of the path
	 */
    protected Object nodeForRow(int row) {
        TreePath treePath = tree.getPathForRow(row);
        return treePath.getLastPathComponent();
    }

	/**
	 * Returns the value for the cell at columnIndex and rowIndex.
	 * @param row the row whose value is to be queried
	 * @param column the column whose value is to be queried
	 * @return the value Object at the specified cell
	 */
    public Object getValueAt(int row, int column) {
        return treeTableModel.getValueAt(nodeForRow(row), column);
    }

	/**
	 * Determines if a specific cell is editable or not
	 * @param row the row being queried
	 * @param column the column being queried
	 * @return true if the cell is editable, false otherwise
	 */
    public boolean isCellEditable(int row, int column) {
        return treeTableModel.isCellEditable(nodeForRow(row), column);
    }

	/**
	 * This empty implementation is provided so users don't have
	 * to implement this method if their data model is not editable.
	 * @param value value to assign to cell
	 * @param row row of cell
	 * @param column column of cell
	 */
    public void setValueAt(Object value, int row, int column) {
        treeTableModel.setValueAt(value, nodeForRow(row), column);
    }

    /**
     * Invokes fireTableRowsUpdated after all the pending events have been
     * processed. SwingUtilities.invokeLater is used to handle this.
     * Changed by PB
     * This stops deselection
     * See http://forum.java.sun.com/thread.jsp?forum=57&thread=223661
     * May have to do this:
     * "After a bit more investigation, overriding the following JTable
     * method in JTreeTable, resolves the painting issues that my last post refers to."
     * public void tableChanged(TableModelEvent e) {
     *   revalidate();
     *   super.tableChanged(e);
     * }
     */
    protected void delayedFireTableDataChanged() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                fireTableRowsUpdated(0, getRowCount() - 1);
            }
        });
    }

    /**
     * Invokes fireTableDataChanged after all the pending events have been
     * processed. SwingUtilities.invokeLater is used to handle this.
     */
/*
        protected void delayedFireTableDataChanged() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                fireTableDataChanged();
            }
        });
    }
*/
}
