/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui.widgets;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.BadLocationException;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.gui.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.metadata.formview.*;
import uk.ac.reload.editor.metadata.vdexBrowser.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.editor.vocabulary.formview.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.schema.*;


/**
 * A Combination of JTextPane and JButton.
 * This is used for providing Vdex Browsing facility from the Metadata Editor.
 *
 * @author Tina Manoharan
 * @version 2003-10-24
 */
public class ReloadButtonTextPane
extends JPanel
implements ReloadWidget
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * A JTextPane to display the value
     */
    protected JTextPane _textPane;

    /**
     * A button for browing Vdex vocabulary values
     */
    protected JButton _button;

    /**
     * The text to display on button
     */
    protected String _text;

    /**
     * The button type
     */
    protected int _type;

    /**
     * The Vdex Source Path, that is the source element that has the Vocabulary Name
     * or Identifier of the Vdex file that we have to use for browsing
     */
    protected String _vdexSourcePath;


    /**
     * The Vdex Source Value, that is the Vocabulary Name or Identifier of the Vdex
     * file that we will use for browsing
     */
    protected String _vdexSourceValue;


    /**
     * Type of button - BROWSE_TYPE
     */
    public static int VDEX_BROWSE_TYPE = 1;

    /**
     * Type of button - METADATA_TYPE
     */
    public static int VDEX_METADATA_TYPE = 2;

    /**
     * The Reload Schema Document
     */
    protected ReloadSchemaDocument _reloadSchemaDocument;

	/**
	 * The Reload Schema Element bound to the JTextPane
	 */
    protected ReloadSchemaElement _schemaElement;

	/**
	 * The document element bound to this TextPane
	 */
    protected Element _element;

    /**
     * Constructor
     * @param reloadSchemaDocument the owner document
     * @param type the button type (for example Vdex Browse Button)
     * @param text the text to display on the button
     */
    public ReloadButtonTextPane(ReloadSchemaDocument reloadSchemaDocument, int type, String text) {
        // super(new BorderLayout());
        super();
        GridBagLayout gridBag = new GridBagLayout();
        setLayout(gridBag);

		_reloadSchemaDocument = reloadSchemaDocument;
        _type = type;
        _text = text;

        // Add text field
        _textPane = new JTextPane();
        JScrollPane scrollPane = new JScrollPane(_textPane);
        setTabKeyBehaviour();
        setUI();
        // add(_textPane, BorderLayout.CENTER);
        GridBagConstraints c1 = new GridBagConstraints();
        c1.gridheight = 2;
        c1.fill = GridBagConstraints.BOTH;
        c1.weightx = 1.0;
        c1.weighty = 1.0;
        c1.insets = new Insets(0,0,0,2);
        gridBag.setConstraints(scrollPane, c1);
        add(scrollPane);

        // Add Button
        _button = new JButton(_text);
        if(type == VDEX_BROWSE_TYPE) {

			_button.setIcon(ReloadUIManager.getIcon(ReloadEditor.ICON_VXBROWSE));
		}
		else if(type == VDEX_METADATA_TYPE) {
			_button.setIcon(ReloadUIManager.getIcon(ReloadEditor.ICON_MD));
			scrollPane.setVisible(false);
		}

        GridBagConstraints c2 = new GridBagConstraints();
		c2.gridheight = 1;
		c2.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_button, c2);
		add(_button);

        setupAction();
    }


    /**
     * Make sure we can use the Tab key for traversal
     */
    protected void setTabKeyBehaviour() {
        Set forwardTraversalKeys = new HashSet();
        forwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0));
        forwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.CTRL_MASK));
        _textPane.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, forwardTraversalKeys);

        Set backwardTraversalKeys = new HashSet();
        backwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK));
        backwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK | InputEvent.CTRL_MASK));
        _textPane.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, backwardTraversalKeys);
    }

    /**
     * Set the UI
     */
    protected void setUI() {
        _textPane.setFont(ReloadUIManager.plainFont11);
    }


    /**
     * Set up the button action. If the button type is Vdex Browse, then it opens a MetadataVdexBrowser.
     */
    protected void setupAction() {
        _button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

				if(_type == VDEX_BROWSE_TYPE) {

					Thread thread = new Thread() {
		        	    public void run() {
		        	        try {

		        	            // ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

				       	        // first get the Vdex Source Value
				       	        handleVdexBrowse();

		        	            // New Vdex Browser Window
		        	            final MetadataVdexBrowser mdVdexBrowser = new MetadataVdexBrowser(ReloadButtonTextPane.this, _vdexSourceValue, "Browse Vdex Vocabulary");

		        	    	    // Show the Window later in the Event queue
		        	    	    SwingUtilities.invokeLater(new Runnable() {

									public void run() {

		        	    	        	//ReloadEditor.getInstance().addInternalFrame(mdVdexBrowser);

		        	    	        	mdVdexBrowser.show();
		        	    	        }
		        	    	    });

		        	    	 }
		        	    	 catch(Exception ex) {

								 ex.printStackTrace();
		        	    	 }
		        	    	 //finally {

								 //ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
		        	    	 //}
		        	    }
		        	};


		        	thread.start();
				}

				//ReloadEditor.getInstance().setCursor(ReloadUIManager.TEXT_CURSOR);

				else if(_type == VDEX_METADATA_TYPE) {

					// Launch the Metadata Editor

					if(_element == null) _element = _reloadSchemaDocument.addElementUnique(this, _schemaElement, true);

					if(_element != null) {

						final VocabularyMetadata vocabularyMetadata = new VocabularyMetadata(_element, (Vocabulary)_reloadSchemaDocument);

						if(vocabularyMetadata != null) {

							// Loading of Document on Thread
			        		Thread thread = new Thread() {
			        		    MetadataEditorDialog_Vocabulary mdDialog;
			        		    ReloadProgressMonitor progressMonitor = null;

			        			    public void run() {
			        			        try {
			        		            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

			        		            // Tina Manoharan, 2003-12-15

							            progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(),
							            "Launching Metadata Editor", "Opening File", "Loading external vocabularies, Please wait...", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

			        		            mdDialog = new MetadataEditorDialog_Vocabulary(vocabularyMetadata);
			        		            // Show the Window later in the Event queue
			        		            SwingUtilities.invokeLater(new Runnable() {
			        		                public void run() {
			        		                    mdDialog.show();
			        		                }
			        		            });
			        		        }
			        		        catch(Exception ex) {
			        		            if(ReloadEditor.DEBUG) ex.printStackTrace();
			        		            if(mdDialog != null) mdDialog.dispose();
			        		            ErrorDialogBox.showWarning("Error:", "Could not open Metadata Editor", ex);
			        		        }
			        		        finally {
										if(progressMonitor != null) progressMonitor.close();
			        		            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
			        		        }
			        		    }
			        		};

			        		thread.start();
						}
					}
				}

			}
		});
    }


    /**
     * Get the TextPane
     * @return the TextPane
     */
    public JTextPane getTextPane() {
        return _textPane;
    }

    /**
     * Set the Value on the TextPane
     * @param value the value to be set on the TextPane
     */
    public void setTextValue(String value) {
        _textPane.setText(value);
        _textPane.setCaretPosition(0);
    }

    /**
     * Insert some text into the TextPane
     * @param text the text to be inserted into the TextPane
     */
    public void insertTextValue(String text) {
        try {
            _textPane.getDocument().insertString(_textPane.getCaretPosition(), text, null);
        }
        catch(BadLocationException ex) {
        }
    }

    /**
     * Get the Value on the TextPane
     * @return the value on the TextPane
     */
    public String getTextValue() {
        return _textPane.getText();
    }

    /**
     * Set the maximum allowed number of characters
     * @param maxLength the maximum allowed number of characters
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * Determine whether this widget only needs a single line to display itself
     * @return true if this widget only needs a single line to display itself, false otherwise
     */
    public boolean isSingleLine() {
        return false;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl the DocumentListener to add
     */
    public void addDocumentListener(DocumentListener dl) {
        _textPane.getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl the DocumentListener to remove
     */
    public void removeDocumentListener(DocumentListener dl) {
        _textPane.getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the components enabled/disabled
     * @param enabled if true enables the component, and if false disables it
     */
    public void setEnabled(boolean enabled) {
        _textPane.setEnabled(enabled);
        _button.setEnabled(enabled);
    }


    /**
     * Update the UI
     */
    public void updateUI() {
        super.updateUI();
    }


    /**
     * Set the Vdex Source Path
     * @param vdexSourcePath the vdex source path
     */
    public void setVdexSourcePath(String vdexSourcePath) {

		_vdexSourcePath = vdexSourcePath;
	}


    /**
     * Get the Vdex Source Path
     * @return the vdex source path
     */
    public String getVdexSourcePath() {

		return _vdexSourcePath;
	}

    /**
     * Set the Vdex Source Value
     * @param vdexSourceValue
     */
    public void setVdexSourceValue(String vdexSourceValue) {

		_vdexSourceValue = vdexSourceValue;
	}

    /**
     * Get the Vdex Source Value
     * @return the vdex source value
     */
    public String getVdexSourceValue() {

		return _vdexSourceValue;
	}

	/**
	 * Set the schema element that is bound to this TextPane
	 * @param schemaElement the schema element that is bound to this TextPane
	 */
	public void setSchemaElement(ReloadSchemaElement schemaElement) {

		_schemaElement = schemaElement;
	}

	/**
	 * Get the schema element that is bound to this TextPane
	 * @return the schema element that is bound to this TextPane
	 */
	public ReloadSchemaElement getSchemaElement() {

		return _schemaElement;
	}

	/**
	 * Set the element that is bound to this TextPane
	 * @param element the element that is bound to this TextPane
	 */
	public void setElement(Element element) {

		_element = element;
	}


    /**
     * Handle Vdex Browsing. Gets the Vdex Source Value using the Vdex Source Path information.
     */
    public void handleVdexBrowse() {

		if(_element == null) _element = _reloadSchemaDocument.addElementUnique(this, _schemaElement, true);

		if(_element != null && _vdexSourcePath != null) {

			DomPath domPath = new DomPath(_vdexSourcePath);

			Element[] elements = _reloadSchemaDocument.getAllElements(domPath);

			if(elements == null) showWarningMessage("Choose a Vdex source file");
			else this.elementWithSameAncestor(_element.getParent(), elements);
		}
	}

	private void elementWithSameAncestor(Element ancestor, Element[] elements) {

		boolean found = false;

		if(ancestor != null) {

			for(int i = 0; i < elements.length; i++) {

				if(elements[i].isAncestor(ancestor)) {

					if(elements[i].getText() != null) {

						System.out.println("Ancestor found: " + elements[i].getText());

						this.setVdexSourceValue(elements[i].getText());
					}
					else System.out.println("Ancestor found: element text NULL");

					found = true;

					break;
				}
			}

			if(!found) elementWithSameAncestor(ancestor.getParent(), elements);
		}
		else System.out.println("Ancestor NULL");
	}


    /**
     * Warn the user that a source vdex file was not choosen. You need a Vdex Source Path to get the
     * Vdex Source Value.
     * @param msg the warning message to display in the dialog
     */
    protected void showWarningMessage(String msg) {
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg,
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Set Vdex Vocabulary Values selected on the Browser to ReloadDocument
     * @param vals an array of string values
     */
	public void setVdexValuesToDocument(String[] vals) {

		StringTokenizer st;

		String val1 = null, val2 = null;

		Element element, parentElement = null, newParentElement = null;

		int index;

		ReloadSchemaElement parentSchemaElement = null, newSchemaElement = null, secondSchemaElement = null;


		for(int i = 0; i < vals.length; i++) {

			val1 = null; val2 = null;

			if(vals[i].indexOf(':') != -1) {

				st = new StringTokenizer(vals[i], ":");

				while(st.hasMoreTokens()) {

					val1 = st.nextToken().trim();

					val2 = st.nextToken().trim();
				}
			}
			else {

				val1 = vals[i];
			}

			// For the element bound to this component

			if(i == 0) {

				// Case2: Second part is for the Entry element
				// Here there is no ID element part

				if(val2 != null) {

					element = _reloadSchemaDocument.getElement(_schemaElement);

					if(element != null) {

						element.setText(val2);

						_reloadSchemaDocument.changedElement(this, element);
					}
					else {

						element = _reloadSchemaDocument.addElementUnique(this, _schemaElement, true);

						if(element != null) {

							element.setText(val2);

							_reloadSchemaDocument.changedElement(this, element);

						}
					}

					// setting the parentElement
					if(_schemaElement.getName().equals("langstring")) {

						parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent().getParent();

						parentElement = element.getParent().getParent();
					}
					else {

						parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent();

						parentElement = element.getParent();
					}

				}

			}
			else { // if (i > 0)

				// Note: All elements (e.g. Taxons) are in a hierarchy, this is according to Reload IMS Metadata Schema

				// Case2: Second part is for the Entry element
				// Here there is no ID element part

				if(val2 != null && parentElement != null && parentSchemaElement != null) {

					if(parentSchemaElement.hasChild(parentSchemaElement.getName())) {

						// hierarchy
						// TaxonPath -> taxon -> taxon

						newSchemaElement = parentSchemaElement.getChild(parentSchemaElement.getName(), parentSchemaElement.getNamespace());
					}
					else {

						// same level
						// TaxonPath -> taxon
						// 			 -> taxon

						newSchemaElement = parentSchemaElement;

						parentElement = parentElement.getParent();
					}

					if(_schemaElement.getName().equals("langstring")) {

						newParentElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);


						element = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, (ReloadSchemaElement)_schemaElement.getParent(), true);


						element = element.getChild("langstring", element.getNamespace());
					}
					else {

						newParentElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);


						element = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, _schemaElement, true);

					}

					if(element != null) {

						element.setText(val2);

						_reloadSchemaDocument.changedElement(this, element);
					}

				}

				// setting parentElement
				parentElement = newParentElement;

				// setting parentSchemaElement
				parentSchemaElement = newSchemaElement;
			}
		}
	}


}
