/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.util;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.text.*;
import java.util.*;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.*;
import javax.swing.filechooser.*;

/**
 * Some useful General Utilities
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-10-10
 */
public final class ReloadUtils implements DweezilConstants {

    private ReloadUtils() {}

	/**
     * Macintosh OS
     */
    public static final int MACINTOSH = 0;

    /**
     * Windows XP
     */
	public static final int WINDOWS_XP = 1;

    /**
     * Windows 98
     */
    public static final int WINDOWS_9x = 2;

    /**
     * Windows NT
     */
    public static final int WINDOWS_NT = 3;

    /**
     * Windows 2000
     */
    public static final int WINDOWS_2000 = 4;

    /**
     * Unix OS
     */
    public static final int UNIX = 99;


    /**
     * Determine the OS we are running on.
     * @return The OS we are running on:-<br>
     * 0 - MACINTOSH<br>
     * 1 - WINDOWS<br>
     * 2 - UNIX<br>
     * 3 - NT<br>
     */
    public static int getOS() {
		String osName = System.getProperty("os.name");
        osName = osName.toLowerCase();
        if(osName.equalsIgnoreCase("windows xp")) return WINDOWS_XP;
        if(osName.equalsIgnoreCase("windows 2000")) return WINDOWS_2000;
        if(osName.equalsIgnoreCase("windows nt")) return WINDOWS_NT;
        if(osName.startsWith("mac os")) return MACINTOSH;
		if(osName.startsWith("windows")) return WINDOWS_9x;
        else return UNIX;
    }

    /**
     * Get the version of Java we are on.
     * @return The version
     */
    public static String getJavaVersion() {
        return System.getProperties().getProperty("java.version");
    }

    /**
     * Make sure we've got java version 1.4.x.
     * @return true if we are running on Java 1.4.x
     */
    public static boolean isJava14() {
        return getJavaVersion().startsWith("1.4");
    }

    /**
     * Parse a date to a short date string.
     * @param date The date to parse
     * @return A String representation of the date such as "25-12-2002"
     */
    public static String parseDate(Date date) {
        if(date == null) return "";
        DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
        return df.format(date);
    }

    /**
     * Get the time as it is now.
     * @return The Time Now
     */
    public static Date getNow() {
        Calendar c = Calendar.getInstance();
        return c.getTime();
    }

    /**
     * Get the extension portion of a filename.
     * @param file The File in question
     * @return The extension part of the filename excluding the "."
     */
    public static String getFileExtension(File file) {
        String fileName = file.getName();
        int i = fileName.lastIndexOf('.');
        if(i > 0 && i < fileName.length() - 1) {
            return fileName.substring(i + 1).toLowerCase();
        }
	    return "";
    }

    /**
     * Get the name portion of a filename not including the extension.
     * @param file The File in question
     * @return The name part of a file name excluding the extension
     */
    public static String getFileNameWithoutExtension(File file) {
        String fileName = file.getName();
        int i = fileName.lastIndexOf('.');
	    if(i > 0 && i < fileName.length() - 1) return fileName.substring(0, i);
	    else return fileName;
    }


    /**
     * Copy a Folder and all its files and sub-folder to target Folder
     * Both Source Folder and Target Folder must exist
     * @throws IOException
     */
    public static boolean copyFolder(File srcFolder, File destFolder, ReloadProgressMonitor progressMonitor) throws IOException {
        if(srcFolder != null || destFolder != null && srcFolder.exists() && destFolder.exists()) {
            destFolder.mkdirs();
            File[] srcFiles = srcFolder.listFiles();
            for(int i = 0; i < srcFiles.length; i++) {
                File srcFile = srcFiles[i];
                // If we have a Progress Monitor...
                if(progressMonitor != null) {
                    progressMonitor.setNote(srcFile.getName());
                    if(progressMonitor.isCanceled()) return false;
                }
                if(srcFile.isDirectory()) copyFolder(srcFile, new File(destFolder, srcFile.getName()), progressMonitor);
                else copyFile(srcFile, new File(destFolder, srcFile.getName()));
            }
            return true;
        }
        return false;
    }

    /**
     * Copy a File.  The Source file must exist.  The target file will be created.
     */
    public static void copyFile(File srcFile, File destFile) throws IOException {
        if(srcFile != null || destFile != null && srcFile.exists()) {
            int bufSize = 512;
            byte[] buf = new byte[bufSize];
            BufferedInputStream bis = new BufferedInputStream(new FileInputStream(srcFile), bufSize);
            BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(destFile), bufSize);
            int size;
            while((size = bis.read(buf)) != -1) bos.write(buf, 0, size);
            bos.flush();
            bos.close();
            bis.close();
        }
    }

    /**
     * Move a File
     */
    public static void moveFile(File srcFile, File destFile) throws IOException {
        if(srcFile == null || destFile == null) return;
        if(!srcFile.exists() || destFile.exists()) return;
        copyFile(srcFile, destFile);
        srcFile.delete();
    }

    /**
     * Delete a folder and its contents
     * @param afolder -  a folder
     */
    public static void deleteFolder(File afolder) throws IOException {
        if(afolder != null) {
            // Not root directories
            File parent = afolder.getParentFile();
            if(parent == null) throw new IOException("Cannot delete root folder");
            FileSystemView view = FileSystemView.getFileSystemView();
            if(view.isRoot(afolder)) throw new IOException("Cannot delete root folder");

            if(afolder.exists() && afolder.isDirectory()) {
                //delete content of directory:
                File[] files = afolder.listFiles();
                int count = files.length;
                for(int i = 0; i < count; i++) {
                    File f = files[i];
                    if(f.isFile()) {
                        f.delete();
                    }
                    else if(f.isDirectory()) {
                        deleteFolder(f);
                    }
                }
                afolder.delete();
            }
        }
    }

    /**
     * Counts the total files & folders in a given folder
     * @param folder The folder parent to count in
     * @return The number of files & folders in folder
     */
    public static int countFiles(File folder) {
        int count = 1;
        File[] files = folder.listFiles();
        for(int i = 0; i < files.length; i++) {
            File f = files[i];
        	if(files[i].isDirectory()) count += countFiles(files[i]);
            else count++;
        }
        return count;
    }

    /**
     * Centres a window on the screen given the proportional size of window.
     * It does not show the window.
     * @param window The Window to centre
     * @param width The width of the Window as a percentage of the Screen Width
     * @param height The height of the Window as a percentage of the Screen Height
     */
    public static void centreWindowProportional(Window window, double width, double height) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int screenWidth = screenSize.width / 2;
        int screenHeight = screenSize.height / 2;
        int newWidth = (int)((double)screenSize.width * width);
        int newHeight = (int)((double)screenSize.height * height);
        window.setSize(newWidth, newHeight);
        window.setLocation(screenWidth - newWidth / 2, screenHeight - newHeight / 2);
    }

    /**
     * Centres a window on the screen given the size of window.
     * It does not show the window.
     * @param window The Window to centre
     * @param width The width of the Window
     * @param height The height of the Window
     */
    public static void centreWindow(Window window, int width, int height) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int screenWidth = screenSize.width / 2;
        int screenHeight = screenSize.height / 2;
        window.setSize(width, height);
        window.setLocation(screenWidth - width / 2, screenHeight - height / 2);
    }

    /**
     * Get the Screen Width
     * @return The width of the screen
     */
    public static int getScreenWidth() {
        return Toolkit.getDefaultToolkit().getScreenSize().width;
    }

    /**
     * Get the Screen Height
     * @return The height of the screen
     */
    public static int getScreenHeight() {
        return Toolkit.getDefaultToolkit().getScreenSize().height;
    }

    /**
     * Gets a local resource URL.<br>
     * This may be for example a help file or image that exists actually in the
     * shipped jar.
     * @param fileName The filename of the Resource
     * @return The URL of the Resource
     */
    public static URL getResourceURL(String fileName) {
         return fileName == null ? null : fileName.getClass().getResource(fileName);
    }

    /**
     * Returns a class from a jar file that exists in the given folder
     * @param className The name of the class required
     * @param jarFile The file of the jar
     * @return The class or null if could not be created
     */
	public static Class getClassFromJar(String className, File jarFile) {
        try {
            URL url = jarFile.toURL();
            ClassLoader loader = new URLClassLoader(new URL[] { url });
            return loader.loadClass(className);
        }
        catch(Exception ex) {
            if(DEBUG) System.out.println("Could not create class: "  + className + " " + ex);
            return null;
        }
    }

    /**
     * Returns a class New Instance from a jar file that exists in the given jar
     * @param className The name of the class required
     * @param jarFile The file of the jar
     * @return An instance of the class or null if it could not be created
     */
    public static Object getClassInstanceFromJar(String className, File jarFile) {
        Object o = null;

        try {
	    	Class c = getClassFromJar(className, jarFile);
        	if(c != null) o = c.newInstance();
        }
        catch(Exception ex) {
            if(DEBUG) System.out.println("Could not create class instance: "  + className + " " + ex);
            return null;
        }
        return o;
    }

    /**
     * Get a relative path for a file given its relationship to rootFolder
     */
    public static String getRelativePath(File rootFolder, File file) {
        URI uriRoot = rootFolder.toURI();
        URI uriFile = file.toURI();

        URI result = uriRoot.relativize(uriFile);
        if(result == null) return file.getName();

        String str = result.getPath();
        return str == null ? file.getName() : str;
    }

    /**
     * @return the first JInternalFrame ancestor of c, or null if component is not contained inside a JInternalFrame
     * or will return it if it actually is a frame
     */
    public static JInternalFrame getInternalFrameAncestor(Component c) {
        if(c == null) return null;
        if(c instanceof JInternalFrame) return (JInternalFrame)c;
        for(Container p = c.getParent(); p != null; p = p.getParent()) {
            if (p instanceof JInternalFrame) {
                return (JInternalFrame)p;
            }
        }
        return null;
    }

    /**
     * @return whether Component c belongs directly to the Reload Main Frame
     */
    public static boolean isComponentInMainFrame(Component c) {
        // If it's in a JInternalFrame then no
        return !isComponentInInternalFrame(c);
    }

    /**
     * @return whether Component c belongs in a JInternalFrame
     */
    public static boolean isComponentInInternalFrame(Component c) {
        // If it's in a JInternalFrame then no
        JInternalFrame frame = getInternalFrameAncestor(c);
        return frame != null;
    }


    /**
     * Mac and Linux absolute file paths begin with "/"
     * i.e /Users/fredbloggs/packages/index.htm
     * so simply prepending "file:///" to obtain a valid URL will not work because
     * we will end up with four forward slashes, which is an illegal URL
     * This method examines the file reference and if it starts with a
     * forward slash prepends "file://" instead of "file:///" and returns
     * the String result
     * @return  a valid local URL reference as a string
     */
    public static String getLocalFileURL(File file) {
        String path = file.getAbsolutePath();

        path = path.replaceAll(" ", "%20");

        if(File.separatorChar != '/')
            path = path.replace(File.separatorChar, '/');

        if(!path.endsWith("/") && file.isDirectory())
            path = path + "/";

        if(path.startsWith("/"))
            return "file://" + path;
        else
            return "file:///" + path;
    }

   /**
     * Launch a File in the OS
     */
    public static void launchFile(File file) {
        if(file == null) return;
        String path = file.getAbsolutePath();
        launchFile(getLocalFileURL(file));
    }

    /**
     * Launch a File by Path in the OS
     */
    public static void launchFile(String path) {

        int OS = getOS();
        switch(OS) {
            case WINDOWS_XP:
            case WINDOWS_2000:
            case WINDOWS_NT:
                try {
                    Process process = Runtime.getRuntime().exec(new String[] { "cmd.exe", "/c", "start", "\"\"", '"' + path + '"' });
                    process.waitFor();
                    process.exitValue();
                }
                catch(InterruptedException ex) {
                }
                catch(IOException ex) {
                }
                break;

            case WINDOWS_9x:
                try {
                    Process process = Runtime.getRuntime().exec("rundll32.exe url.dll,FileProtocolHandler " + path);
                    process.waitFor();
                    process.exitValue();
                }
                catch(InterruptedException ex) {
                }
                catch(IOException ex) {
                }
                break;

            case MACINTOSH:
                try {
                    Class targetClass = Class.forName("com.apple.mrj.MRJFileUtils");
                    // Let MRJ do all the work for us.
                    Method openURL = targetClass.getDeclaredMethod("openURL", new Class[] {String.class});
                    openURL.invoke(null, new Object[] {path});
                }
                catch(Exception ex) {
                   if(DEBUG) System.out.println("Error opening native browser on Mac");
                }
                break;

            default:
                JOptionPane.showMessageDialog(null,
                                          "Unknown Operating System",
                                          "View File",
                                          JOptionPane.INFORMATION_MESSAGE);

        }
    }

    /**
     * Ask the user for a File name to open
     * @return File or null if cancelled
     */
    public static File askFileNameOpen(Component parent, String title, javax.swing.filechooser.FileFilter filter) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(parent);

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        return chooser.getSelectedFileAndStore();
    }


    /**
     * Ask the user for a File name to open, the name of the current dir is given
     * @param parent the parent component
     * @param title the open dialog title
     * @param filter the File Filter
     * @param currentDir the current directory to open
     * @return File or null if cancelled
     */
    public static File askFileNameOpenUsingDirName(Component parent, String title, javax.swing.filechooser.FileFilter filter, File currentDir) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        chooser.setCurrentDirectory(currentDir);
        int returnVal = chooser.showOpenDialog(parent);

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        return chooser.getSelectedFileAndStore();
    }


    /**
     * Ask the user for a File name to save a Document as
     * @param parent the parent component of the dialog
     * @param title the title to display on the dialog
     * @param filter the file filter
     * @param extension the default extension to use in case the user doesn't provide one (no dot)
     * @return the File or null if cancelled
     */
    public static File askFileNameSave(Component parent, String title, javax.swing.filechooser.FileFilter filter, String extension) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showSaveDialog(parent);

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        File file = chooser.getSelectedFileAndStore();

        // Ensure we have an extension if given
        if(extension != null) {
            String fileName = file.getPath();
            if(fileName.indexOf(".") == -1) file = new File(fileName + "." + extension);
        }

        return file;
    }

}

