/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor;

import java.io.*;
import java.util.*;

import java.awt.datatransfer.*;
import java.awt.dnd.*;
import javax.swing.*;

import uk.ac.reload.dweezil.util.*;

/**
 * The Reload Editor Frame Desktop Panel.
 * This can handle Native Drag and Drop
 *
 * @author Phillip Beauvoir
 * @version 2003-10-08
 */
public class ReloadEditorDesktopPane
extends JDesktopPane
implements DropTargetListener
{

	/**
	 * Default constructor
	 */
    public ReloadEditorDesktopPane() {
        // Drag Target
        DropTarget dropTarget = new DropTarget(this, this);
    }


    /**
     * Attempt to open the files in the Editor
     * @param files the files concerned
     */
    protected void openFiles(File[] files) {
        for(int i = 0; i < files.length; i++) {
            String name = files[i].getName().toLowerCase();
            if(name.endsWith(".zip") || name.endsWith(".xml")) {
                ReloadEditorDocumentHandler.openFile(ReloadEditor.getInstance(), files[i]);
            }
        }
    }

    /**
     * We just Dropped some files
     * @param event the DropTargetDropEvent
     */
    public void drop(DropTargetDropEvent event) {
        // Get what we dropped
        Transferable transferable = event.getTransferable();

        // If we are the correct drag object
        if(transferable.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
            // Accept the Drop
            event.acceptDrop(getCorrectDropContext(event));

            // Get the User Object and do something
            try {
                Object userObject = transferable.getTransferData(DataFlavor.javaFileListFlavor);
                if(userObject instanceof AbstractList) {
                    AbstractList list = (AbstractList)userObject;
                    Object[] files = list.toArray();
                    openFiles((File[])files);
                }
            }
            catch(Exception ex) {
                if(ReloadEditor.DEBUG) ex.printStackTrace();
            }
            finally {

            }

            event.getDropTargetContext().dropComplete(true);
        }

        // Else we are not the right object
        else event.rejectDrop();
    }

    /**
     * We accept Native Drops
     * @param event the DropTargetDragEvent
     * @return true if the Drop was OK, false otherwise
     */
    public boolean isDropOK(DropTargetDragEvent event) {
        // Only support javaFileListFlavor
        DataFlavor[] flavors = event.getCurrentDataFlavors();
        for(int i = 0; i < flavors.length; i++) {
            if(flavors[i].equals(DataFlavor.javaFileListFlavor)) {
                return true;
            }
        }

        return false;
    }

	/**
	 * Called while a drag operation is ongoing, when the mouse pointer has exited the operable
	 * part of the drop site for the DropTarget registered with this listener.
	 * Over-this function
	 * @param event the DropTargetEvent
	 */
    public void dragExit(DropTargetEvent event) {}

	/**
	 * Called while a drag operation is ongoing, when the mouse pointer enters the operable
	 * part of the drop site for the DropTarget registered with this listener.
	 * Over-ride this function
	 * @param event the DropTargetDragEvent
	 */
    public void dragEnter(DropTargetDragEvent event) {
        if(isDropOK(event)) event.acceptDrag(getCorrectDropContext(event));
        else event.rejectDrag();
    }

    /**
    * We are being dragged over - select the Node if OK
    * Over-ride and call super if need be
    * @param event the DropTargetDragEvent
    */
    public void dragOver(DropTargetDragEvent event) {
        if(isDropOK(event)) event.acceptDrag(getCorrectDropContext(event));
        else event.rejectDrag();
    }

	/**
	 * Called if the user has modified the current drop gesture.
	 * Over-ride this function
	 * @param event the DropTargetDragEvent
	 */
    public void dropActionChanged(DropTargetDragEvent event) {
        if(isDropOK(event)) event.acceptDrag(getCorrectDropContext(event));
        else event.rejectDrag();
    }

    /**
     * <h3>Amendment History</h3>
     * <p>Due to Mac specific issues, this method has been added to ensure
     * that the correct drop target event is recorded.  On MacOS X
     * "event.getDropAction()" does not return 1 for move and 2 for copy as
     * it should do (like it does on PC).  This method checks to see
     * if the OS is Mac and then makes sure the correct int value is returned</p>
     *
     * @param event a DropTargetDropEvent
     * @return int representing the type of action - move or copy
     */
    public int getCorrectDropContext(DropTargetDropEvent event){
       int OS = ReloadUtils.getOS();
       if(OS == ReloadUtils.MACINTOSH){
           int result = event.getDropAction();
           switch(result) {
               case 0:
                   return 1;
               case 1:
                   return 2;
               default:
                   return 1;
           }
       }
       return event.getDropAction();
    }

    /**
     * <h3>Amendment History</h3>
     * <p>Due to Mac specific issues, this method has been added to ensure
     * that the correct drop target event is recorded.  On MacOS X
     * "event.getDropAction()" does not return 1 for move and 2 for copy as
     * it should do (like it does on PC).  This method checks to see
     * if the OS is Mac and then makes sure the correct int value is returned</p>
     *
     * @param event a DropTargetDropEvent
     * @return int representing the type of action - move or copy
     */
    public int getCorrectDropContext(DropTargetDragEvent event){
       int OS = ReloadUtils.getOS();
       if(OS == ReloadUtils.MACINTOSH){
           int result = event.getDropAction();
           switch(result) {
               case 0:
                   return 1;
               case 1:
                   return 2;
               default:
                   return 1;
           }
       }
       return event.getDropAction();
    }
}