/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.editor;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;


/**
 * The Profile for a given editing View - this contains the schemahelper and the
 * vocab file to use for a given type of IMS document
 *
 * @author Phillip Beauvoir
 * @version 2003-10-01
 */
public abstract class ReloadHelperProfile {
    /**
     * The JDOM Document of the XML Helper File
     */
    protected Document _docProfile;

    /**
     * The File of the Profile File
     */
    protected File _fileProfile;

    /**
     * Default Constructor
     */
    public ReloadHelperProfile() { }

    /**
     * Constructor
     * @param fileProfile the file of the helper profile
     * @throws ReloadEditorException
     */
    public ReloadHelperProfile(File fileProfile) throws ReloadEditorException {
        if(fileProfile == null) throw new ReloadEditorException("Profile file cannot be null");
        _fileProfile = fileProfile;

        // Get the helper JDOM Document
        try {
            _docProfile = loadProfile(_fileProfile);
        }
        catch(DweezilXMLException ex) {
            throw new ReloadEditorException(ex);
        }
    }

    /**
     * Load the XML Profile File into a JDOM Document
     *
     * @param fileProfile The XML File to read in
     * @return The JDOM Document
     * @throws DweezilXMLException
     */
    protected Document loadProfile(File fileProfile) throws DweezilXMLException {
        return XMLUtils.readXMLFile(fileProfile);
    }

    /**
     * Get hte Vocabulary Helper file
     * @return the Vocabulary helper File
     */
    public abstract File getVocabFile();

    /**
     * Get the Schema helper file
     * @return the Schema Helper File
     */
    public abstract File getSchemaHelperFile();

    /**
     * Get name of the Profile.  This is stored
     * in the XML Profile file against the attribute "name"
     * @return The name of the Profile
     */
    public String getProfileName() {
        return getRootAttributeValue("name");
    }

    /**
     * Get the profile XMl file
     * @return the actual File
     */
    public File getFile() {
        return _fileProfile;
    }

    /**
     * Get the file name of the associated Vocabulary file.  This is stored
     * in the XML Profile file against the attribute "vocabfile"
     * @return The name of the Vocabulary file
     */
    public String getVocabularyFileName() {
        return getRootAttributeValue("vocabfile");
    }

    /**
     * Get the file name of the associated Schema Helper file.  This is stored
     * in the XML Profile file against the attribute "schemahelperfile"
     * @return The name of the Vocabulary file
     */
    public String getSchemaHelperFileName() {
        return getRootAttributeValue("schemahelperfile");
    }

    /**
     * Get the value of a root attribute
     * @return the value of a root attribute or null
     */
    protected String getRootAttributeValue(String attName) {
        if(_docProfile != null && _docProfile.hasRootElement()) {
            Element root = _docProfile.getRootElement();
            Attribute att = root.getAttribute(attName);
            return att == null ? null : att.getValue();
        }
        return null;
    }

    /**
     * String representation of the Metadata Profile file
     * @return the profile name
     */
    public String toString() {
        return getProfileName();
    }
}