/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.htmlparser.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.ReloadSchemaDocument.*;
import uk.ac.reload.moonunit.dom.contentpackaging.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The Content Package Class
 *
 * @author Phillip Beauvoir
 * @version 2003-10-13
 */
public class ContentPackage
extends ReloadSchemaDocument
implements AbstractContentPackage
{

    /**
     * Delegate class for common CP functionality
     */
    protected ContentPackageCore _cpCore = new ContentPackageCore(this);

    /**
     * Our unique signature
     */
    static final String[] cp_comments = {
        "This is a Reload version " + ReloadEditor.VERSION + " Content Package document",
        "Spawned from the Reload Content Package Generator - http://www.reload.ac.uk"
    };

    /**
     * Default Constructor
     */
    public ContentPackage() {
        super();
    }

    /**
     * Constructor
     */
    public ContentPackage(File file, Document doc, ContentPackageSchemaController schemaController) {
        super(file, doc, schemaController);
    }

    /**
     * Set up New CP with new project folder, imsmanifest.xml amd Schema files
     */
    public void newContentPackage(File projectFolder, ContentPackageSchemaController cpController, MetadataSchemaController mdController) throws ReloadEditorException {
        // New Default Document
        newDocument(projectFolder, cpController, mdController);

        // Save it
        try {
            saveDocument();
        }
        catch(ReloadDomException ex) {
            throw new ReloadEditorException(ex);
        }

        // Clear the dirty flag
        setDirty(false);

        // Copy the Schema Files to the Project Folder
        cpController.copySchemaFilesToFolder(projectFolder);
    }

    /**
     * Setup the JDOM Document with Namespaces set and root element set and some
     * default attributes.
     */
    protected void newDocument(File projectFolder, ContentPackageSchemaController cpController, MetadataSchemaController mdController) throws ReloadEditorException {
        setReloadSchemaController(cpController);

        _doc = new Document();

        // Add our signature
        addComments(_doc);

        // Content Packaging Schema
        ReloadSchema cpSchema = cpController.getReloadSchema();

        // Set Root Element
        Element root = addElementBySchema(this, null, cpSchema.getRootElement(), false);
        _doc.setRootElement(root);

        // Metadata Schema
        ReloadSchema mdSchema = mdController.getReloadSchema();

        // Add Metadata Namespace
        Namespace mdNamespace = Namespace.getNamespace(IMSDocumentHandler.IMSMD_NAMESPACE_PREFIX, mdSchema.getTargetNamespace());
        root.addNamespaceDeclaration(mdNamespace);

        // Add XSI Schema Instance Namespace
        root.addNamespaceDeclaration(IMSDocumentHandler.XSI_Namespace);

        // Identifier attribute
        // Note - the JDOM XMLOutputter.printElement() method prints attributes AFTER namespaces
        root.setAttribute(ContentPackageCore.IDENTIFIER,  RandomGUID.getUniqueID("MANIFEST-"));

        // Add Schema Location Attribute which is constructed from Target Namespace
        // and file name of Schema, and also with Metadata Schema location
        StringBuffer schemaLocationURI = new StringBuffer();
        schemaLocationURI.append(cpSchema.getTargetNamespace());
        schemaLocationURI.append(" ");
        schemaLocationURI.append(cpSchema.getSchemaName());
        schemaLocationURI.append(" ");
        schemaLocationURI.append(mdSchema.getTargetNamespace());
        schemaLocationURI.append(" ");
        schemaLocationURI.append(mdSchema.getSchemaName());
        root.setAttribute(IMSDocumentHandler.XSI_SchemaLocation, schemaLocationURI.toString(), IMSDocumentHandler.XSI_Namespace);

        // Set as File
        _file = new File(projectFolder, IMSDocumentHandler.MANIFEST_NAME);
    }


    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return cp_comments;
    }

    /**
     * Import (aggregate) another Package into this Content Package at manifestElement
     */
    public Element importManifest(File manifestFile, Element manifestElement, String base) throws ReloadEditorException {
        Element importedRootElement = null;

        // Read in imsmanifest.xml
        Document importedDoc = null;
        try {
            importedDoc = XMLUtils.readXMLFile(manifestFile);
        }
        catch(DweezilXMLException ex) {
            throw new ReloadEditorException(ex);
        }

        if(importedDoc != null) {
            // Root Element detach
            importedRootElement = (Element)importedDoc.getRootElement().detach();

            // Check we got the right kind of file
            String rootSource = importedRootElement.getName();
            String rootTarget = getTargetRootElementName();
            if(!rootSource.equals(rootTarget)) throw new ReloadEditorException("Not a valid Manifest file");

            // Set Namespace to ours
            Namespace ns = importedRootElement.getNamespace();
            if(ns != null) IMSDocumentHandler.replaceNamespaces(importedRootElement, ns, getSchemaNamespace());

            // Remove stuff
            importedRootElement.removeAttribute(IMSDocumentHandler.XSI_SchemaLocation, IMSDocumentHandler.XSI_Namespace);
            importedRootElement.removeAttribute(IMSDocumentHandler.XSI_SchemaLocation, IMSDocumentHandler.XSI_NamespaceOLD);
            importedRootElement.removeNamespaceDeclaration(IMSDocumentHandler.XSI_Namespace);
            importedRootElement.removeNamespaceDeclaration(IMSDocumentHandler.XSI_NamespaceOLD);

            // Remove Namespaces
            Vector v = new Vector(); // Use a copy of the list so we don't get a concurrent modification exception
            Iterator it = importedRootElement.getAdditionalNamespaces().listIterator();
            while(it.hasNext()) {
                ns = (Namespace) it.next();
                v.add(ns);
            }
            it = v.iterator();
            while(it.hasNext()) {
                ns = (Namespace) it.next();
                importedRootElement.removeNamespaceDeclaration(ns);
                // Add to root
                Namespace ns2 = getRootElement().getNamespace(ns.getPrefix());
                if(ns2 == null) getRootElement().addNamespaceDeclaration(ns);
            }

            // Create new IDENTIFIERs and map to old IDENTIFIERs
            Hashtable idmap = createNewIDs(importedRootElement);
            // Change each IDENTIFIERREF to new IDENTIFIERs
            createNewIDRefs(importedRootElement, idmap);
            // SCORM
            createNewPrerequisiteRefs(importedRootElement, idmap);

            // Set Base attribute to target folder
            importedRootElement.setAttribute(ContentPackageCore.BASE, base + "/", Namespace.XML_NAMESPACE);

            // Add it
            addElementUndoable(this, manifestElement, importedRootElement, false);
        }

        return importedRootElement;
    }

    /**
     * Export (disagggregate) manifestElement to new Content Package
     */
    public boolean exportManifest(Element manifestElement, File targetFolder) throws ReloadEditorException {
        // If sub-Manifest has valid xml:base property, find sub-Folder of that name in current package
        // If this sub-Folder exists and and imsmanifest.xml exists, copy contents to new folder. Finish.
        String base = getElementBase(manifestElement);
        if(base != null) {
            File sourceFolder = new File(getProjectFolder(), base);
            if(sourceFolder.exists() && targetFolder.exists()) {
                try {
                    // Do we have an imsmanifest.xml file?
                    File imsmanifestFile = new File(sourceFolder, IMSDocumentHandler.MANIFEST_NAME);
                    if(imsmanifestFile.exists()) {
                        ReloadUtils.copyFolder(sourceFolder, targetFolder, null);
                        return true;
                    }
                }
                catch(IOException ex) {
                    throw new ReloadEditorException(ex);
                }
            }
        }

        // Nope - so try this:-

        // Walk the tree to get all referenced files, and copy over to new Folder
        File[] files = getResourceFiles(manifestElement);
        for(int i = 0; i < files.length; i++) {
            try {
                String path = ReloadUtils.getRelativePath(getProjectFolder(), files[i]);
                File newFile = new File(targetFolder, path);
                newFile.getParentFile().mkdirs();
                ReloadUtils.copyFile(files[i], newFile);
            }
            catch(IOException ex) {
                throw new ReloadEditorException(ex);
            }
        }

        // Create new imsmanifest.xml file from sub-Manifest node
        Document newDoc = new Document();

        // Add comments
        addComments(newDoc);

        // Clone the root
        Element newRoot = (Element)manifestElement.clone();
        newDoc.setRootElement(newRoot);

        // Save it
        File newFile = new File(targetFolder, IMSDocumentHandler.MANIFEST_NAME);

        try {
            XMLUtils.write2XMLFile(newDoc, newFile);
        }
        catch(DweezilXMLException ex1) {
            throw new ReloadEditorException(ex1);
        }

        // Copy the Default Schema Files to the target Folder
        // NOTE - Should we do this?  Suppose they don't match the instance
        ((ContentPackageSchemaController)getReloadSchemaController()).copySchemaFilesToFolder(targetFolder);

        return true;
    }

    /**
     * @return the Target Root element name
     */
    public String getTargetRootElementName() {
        return ContentPackageCore.MANIFEST;
    }

    /**
     * @return the project name - the project folder
     */
    public String getProjectName() {
        return getProjectFolder() == null ? null : getProjectFolder().getName();
    }

    /**
     * @return the CP Project Folder which is the containing folder of imsmanifest.xml
     */
    public File getProjectFolder() {
        return _file == null ? null : _file.getParentFile();
    }


// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * @return whether we can copy an element
     */
    public boolean canCopyElement(Element element) {
        return false;
        //return canDragElement(element, ReloadDocumentClipboard.ACTION_COPY);
    }

    /**
     * @return whether we can cut an element
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return false;
        //return canCopyElement(element) && canDeleteElement(element, schemaElement);
    }

    /**
     * @return whether we can paste an element from the Clipboard to element
     */
    public boolean canPasteFromClipboard(Element element) {
        //Element clipboardElement = ReloadDocumentClipboard.getElement();

        //if(clipboardElement != null && element != null) {
            // Translate a Cut Action into a Move Action??
            //return isAllowedChild(element, clipboardElement);
        //}

        return false;
    }

    /**
     * @return whether we can drag an element around
     */
    public boolean canDragElement(Element element, int action) {
        if(element != null) {
            // Has to be our namespace
            if(!isDocumentNamespace(element)) return false;

            String name = element.getName();
            if(name.equals(ContentPackageCore.RESOURCE) ||
               name.equals(ContentPackageCore.METADATA) ||
               name.equals(ContentPackageCore.ITEM) ||
               name.equals(ContentPackageCore.ORGANIZATION) ||
               name.equals(ContentPackageCore.MANIFEST)) return true;
        }
        return false;
    }

    /**
     * @return whether we can copy/move sourceElement to targetElement
     */
    public boolean acceptElement(Element sourceElement, Element targetElement, int actionCopyMove) {
        if(sourceElement != null && targetElement != null) {
            // Must be our Namespace
            if(!sourceElement.getNamespace().equals(targetElement.getNamespace())) return false;

            // Can't copy/move onto self
            if(sourceElement.equals(targetElement)) return false;

            // General Rules for Moving an Element
            if((actionCopyMove & ReloadDocumentClipboard.ACTION_MOVE) != 0) {
                // Can't move if already there
                if(targetElement.getContent().contains(sourceElement)) return false;
                // Can't move to a child
                if(targetElement.isAncestor(sourceElement)) return false;
            }

            // Get Target & Source Parent Manifests
            Element targetManifest = getParentManifestElement(targetElement);
            Element sourceManifest = getParentManifestElement(sourceElement);

            if(targetManifest == null || sourceManifest == null) return false;

            String sourceName = sourceElement.getName();
            String targetName = targetElement.getName();

            // Different Document?
            boolean differentDoc = !isMemberOfSameDocument(sourceElement, targetElement);
            if(differentDoc) {
                // Only Manifests
                //return sourceName.equals(MANIFEST) && targetName.equals(MANIFEST);
                return false;
            }

            // Source == RESOURCE
            if(sourceName.equals(ContentPackageCore.RESOURCE)) {
                if(targetName.equals(ContentPackageCore.ORGANIZATION) || targetName.equals(ContentPackageCore.ITEM)) {
                    // Same manifest
                    if(targetManifest == sourceManifest) return true;
                    // Target must be higher than resource
                    return sourceManifest.isAncestor(targetManifest);
                }
                // Dragging into RESOURCES Element
                else if(targetName.equals(ContentPackageCore.RESOURCES)) {
                    return !containsResource(sourceElement, targetElement);
                }
            }

            // Source == ITEM
            if(sourceName.equals(ContentPackageCore.ITEM)) {
                if(targetName.equals(ContentPackageCore.ORGANIZATION) || targetName.equals(ContentPackageCore.ITEM)) {
                    // Same manifest
                    if(targetManifest == sourceManifest) return true;
                    // If there is an identifierref, the referenced object's (parent) manifest must be below or same as the target's Parent Manifest
                    Element refElement = getReferencedElement(sourceElement);
                    if(refElement != null) {
                        Element ref_targetManifest = getParentManifestElement(refElement);
                        return ref_targetManifest.isAncestor(targetManifest) || (ref_targetManifest == targetManifest);
                    }
                    else return true;
                }
            }

            // Source == MANIFEST
            else if(sourceName.equals(ContentPackageCore.MANIFEST)) {
                if(targetName.equals(ContentPackageCore.ORGANIZATION) || targetName.equals(ContentPackageCore.ITEM)) {
                    // Target must be higher than manifest
                    return sourceManifest.isAncestor(targetManifest);
                }
                // Dragging into MANIFEST Element
                else if(targetName.equals(ContentPackageCore.MANIFEST)) {
                    return false;
                }
            }

            // Default
            else return isAllowedChild(targetElement, sourceElement);

        }

        return false;
    }

    /**
     * Copy/Move sourceElement to targetElement
     * action is either ACTION_COPY or ACTION MOVE
     */
    public Element shiftElement(Element sourceElement, Element targetElement, int action) {
        if(sourceElement != null && targetElement != null) {

            // Get Target & Source Parent Manifests
            Element targetManifest = getParentManifestElement(targetElement);
            Element sourceManifest = getParentManifestElement(sourceElement);

            if(targetManifest == null || sourceManifest == null) return null;

            boolean isSameDocument = isMemberOfSameDocument(sourceElement, targetElement);

            // If Moving from another Manifest make sure we only copy it
            if(targetManifest != sourceManifest) action = ReloadDocumentClipboard.ACTION_COPY;

            String sourceName = sourceElement.getName();
            String targetName = targetElement.getName();

            // Copying/Moving a RESOURCE
            if(sourceName.equals(ContentPackageCore.RESOURCE)) {
                if(targetName.equals(ContentPackageCore.ORGANIZATION) || targetName.equals(ContentPackageCore.ITEM)) {
                    // Add ITEM that references this RESOURCE
                    return addItemResource(targetElement, sourceElement, true);
                }
                else if(targetName.equals(ContentPackageCore.RESOURCES)) {
                    return shiftElement(sourceElement, targetElement, action, true);
                }
            }

            // Copying a MANIFEST
            else if(sourceName.equals(ContentPackageCore.MANIFEST)) {
                if(targetName.equals(ContentPackageCore.ORGANIZATION) || targetName.equals(ContentPackageCore.ITEM)) {
                    // Add ITEM that references this MANIFEST
                    return addItemManifest(sourceElement, targetElement, true);
                }
                if(targetName.equals(ContentPackageCore.MANIFEST)) {
                    return shiftElement(sourceElement, targetElement, action, true);
                }
            }

            // Default
            else if(isAllowedChild(targetElement, sourceElement))
                return shiftElement(sourceElement, targetElement, action, true);
        }

        return null;
    }

    protected Element shiftElement(Element sourceElement, Element targetElement, int action, boolean doSelect) {
        // Copy
        if((action & ReloadDocumentClipboard.ACTION_COPY) != 0) {
            return copyElementUndoable(this, sourceElement, targetElement, doSelect);
        }
        // Move
        if((action & ReloadDocumentClipboard.ACTION_MOVE) != 0) {
            return moveElementUndoable(this, sourceElement, targetElement, doSelect);
        }
        return null;
    }


    /**
     * Over-ride this so we can intercept it to do stuff
     */
    public Element addElementBySchema(Object source, Element parentElement, ReloadSchemaElement newSchemaElement, boolean doSelect) {
        Element newElement = super.addElementBySchema(source, parentElement, newSchemaElement, doSelect);

        if(newElement != null) {
            // Here we add a unique identifier if it needs one
            if(newSchemaElement.hasAttribute(ContentPackageCore.IDENTIFIER)) {
                String id = newElement.getAttributeValue(ContentPackageCore.IDENTIFIER);
                if(id == null) {
                    ReloadSchemaAttribute schemaAtt = newSchemaElement.getAttribute(ContentPackageCore.IDENTIFIER);
                    Attribute att = addAttribute(source, newElement, schemaAtt);
                    String uniqueID = generateUniqueID(newElement);
                    att.setValue(uniqueID);
                    fireElementChanged(new ReloadDocumentListenerEvent(source, this, newElement, false));
                }
            }

            String name = newElement.getName();

            if(name.equals(ContentPackageCore.ORGANIZATION)) {
                // If user added first Organization, set it as default
                setDefaultOrganization(newElement);
                // Add default Title
                addTitle(newElement, "Organization");
            }

            else if(name.equals(ContentPackageCore.ITEM)) {
                // Add default Title
                addTitle(newElement, "Item");
            }
        }

        return newElement;
    }

    /**
     * Over-ride this so we can intercept it to do stuff
     */
    public Element copyElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = super.copyElementUndoable(source, element, newParent, doSelect);
        copypasteElement(newElement);
        return newElement;
    }

    /**
     * Over-ride this so we can intercept it to do stuff
     */
    public Element pasteElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = super.pasteElementUndoable(source, element, newParent, doSelect);
        copypasteElement(newElement);
        return newElement;
    }

    /**
     * When an Element is copied over or pasted we need to do a bunch of stuff
     */
    protected void copypasteElement(Element newElement) {
        if(newElement != null) {
           String name = newElement.getName();

           if(name.equals(ContentPackageCore.ITEM) || name.equals(ContentPackageCore.RESOURCE)) {
               // Create new IDs
               createNewIDs(newElement);
           }

           else if(name.equals(ContentPackageCore.ORGANIZATION)) {
               // Create new IDs
               createNewIDs(newElement);
               // If user added first Organization, set it as default
               setDefaultOrganization(newElement);
               // Check valid IDENTIFIERREFs
               checkIDRefs(newElement);
           }

           else if(name.equals(ContentPackageCore.MANIFEST)) {
               // Create new IDENTIFIERs and map to old IDENTIFIERs
               Hashtable map = createNewIDs(newElement);
               // Change each IDENTIFIERREF to new IDENTIFIERs
               createNewIDRefs(newElement, map);
           }
       }
    }

    /**
     * Add an Item to Item or Organization parent - this references a sub-Manifest
     */
    protected Element addItemManifest(Element manifestElement, Element targetElement, boolean doSelect) {
        // Add it
        DomPath domPath = DomPath.getDomPath(targetElement);
        domPath.appendElementName(ContentPackageCore.ITEM);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        Element itemElement = addElementBySchemaUndoable(this, targetElement, schemaElement, doSelect);

        // Set IDENTIFIERREF to MANIFEST's IDENTIFIER if specified
        if(itemElement != null) {
            String manifestID = manifestElement.getAttributeValue(ContentPackageCore.IDENTIFIER);
            if(manifestID != null) itemElement.setAttribute(ContentPackageCore.IDENTIFIERREF, manifestID);
        }

        return itemElement;
    }

    /**
     * Add an Item to Item or Organization parent - this references a Resource
     * If resourcesElement is not null set the Item's identifierref to it
     */
    protected Element addItemResource(Element parentElement, Element resourceElement, boolean doSelect) {
        // Add it
        DomPath domPath = DomPath.getDomPath(parentElement);
        domPath.appendElementName(ContentPackageCore.ITEM);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        Element itemElement = addElementBySchemaUndoable(this, parentElement, schemaElement, doSelect);

        if(itemElement != null && resourceElement != null) {
            // Set IDENTIFIERREF to resource's IDENTIFIER if specified
            String resourceID = resourceElement.getAttributeValue(ContentPackageCore.IDENTIFIER);
            if(resourceID != null) itemElement.setAttribute(ContentPackageCore.IDENTIFIERREF, resourceID);
            // Add Title
            addItemTitle(itemElement, resourceElement);
        }

        return itemElement;
    }

    /**
     * Add an Organization
     */
    protected Element addOrganization(Element parentElement, boolean doSelect) {
        DomPath domPath = DomPath.getDomPath(parentElement);
        domPath.appendElementName(ContentPackageCore.ORGANIZATION);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        return addElementBySchemaUndoable(this, parentElement, schemaElement, doSelect);
    }

    /**
     * Add a Title element (only one allowed).
     * If a Title Element already exists the text will be replaced by new text
     */
    protected Element addTitle(Element parentElement, String text) {
        Element titleElement = parentElement.getChild(ContentPackageCore.TITLE, parentElement.getNamespace());

        if(titleElement != null) {
            titleElement.setText(text);
        }

        else {
            titleElement = new Element(ContentPackageCore.TITLE, parentElement.getNamespace());
            titleElement.setText(text);
            addElement(this, parentElement, titleElement, false);
        }

        return titleElement;
    }

    /**
     * Add a Title to an Item based on the Resource it References.
     * If it's an html file use the <title> element from that or the file name
     */
    protected Element addItemTitle(Element itemElement, Element resourceElement) {
        Element titleElement = null;

        if(itemElement != null && resourceElement != null) {
            String title = null;
            File file = getResourceFile(resourceElement);
            if(file != null) {
                String fileName = file.getName().toLowerCase();
                if(fileName.endsWith(".html") || fileName.endsWith(".htm")) {
                    HTMLParser parser = new HTMLParser(file);
                    title = parser.getTagText("title");
                }
                else title = ReloadUtils.getFileNameWithoutExtension(file);
            }
            titleElement = addTitle(itemElement, title);
        }

        return titleElement;
    }

    /**
     * @return a nice name to display for an Element depending on element type
     */
    public String getElementDisplayName(Element element) {
        String fname = null;

        if(element != null) {
            String name = element.getName();

            // ITEM & ORGANIZATION - display Title Element
            if(name.equals(ContentPackageCore.ITEM) || name.equals(ContentPackageCore.ORGANIZATION)) {
                Element titleElement = element.getChild(ContentPackageCore.TITLE, element.getNamespace());
                if(titleElement != null) fname = titleElement.getText();
            }

            // DEPENDENCY - display path of referenced Resource
            else if(name.equals(ContentPackageCore.DEPENDENCY)) {
                fname = getElementHREF(element);
            }

            // MANIFEST - display path of referenced Resource
            else if(name.equals(ContentPackageCore.MANIFEST)) {
                fname = element.getAttributeValue(ContentPackageCore.IDENTIFIER);
            }

            // FILE & RESOURCE - display href
            else if(name.equals(ContentPackageCore.FILE) || name.equals(ContentPackageCore.RESOURCE)) {
                fname = element.getAttributeValue(ContentPackageCore.HREF);
            }

            // MD SCHEMA & SCHEMAVERSION
            else if(name.equals(ContentPackageCore.SCHEMA) || name.equals(ContentPackageCore.SCHEMAVERSION)) {
                fname = element.getText();
            }

            // Get Friendly name if possible, or qualified name if not
            if(fname == null || fname.equals("")) {
                fname = getReloadSchemaController().getElementFriendlyName(DomPath.getDomPath(element));
                if(fname == null) fname = element.getQualifiedName();
            }
        }

        return fname;
    }

// =============================================================================
// Delegate Methods to ContentPackageCore class
// =============================================================================

    /**
     * @return The Element referenced by identifierref in sourceElement.
     * The search digs downward from sourceElement - this is important.
     * This will be either a Resource or a sub-Manifest.
     */
    public Element getReferencedElement(Element sourceElement) {
        return _cpCore.getReferencedElement(sourceElement);
    }

    /**
     * @return true if this element has an identifierref attribute
     */
    public boolean isReferencingElement(Element element) {
        return _cpCore.isReferencingElement(element);
    }

    /**
     * @return all available Identifiers from Resources and sub-Manifests
     * that an element can legally reference.  For an Item this will be Resources and sub-manifests
     * and for a Resource Dependency this will be another Resource
     */
    public String[] getReferencedIdentifersAllowed(Element element) {
        return _cpCore.getReferencedIdentifersAllowed(element);
    }

    /**
     * @return all available Elements (Resources and sub-Manifests)
     * that an element can legally reference.  For an Item this will be Resources and sub-manifests
     * and for a Resource Dependency this will be another Resource
     */
    public Element[] getReferencedElementsAllowed(Element element) {
        return _cpCore.getReferencedElementsAllowed(element);
    }

    /**
     * @return all ORGANIZATION elements in an ORGANIZATIONS element
     */
    public Element[] getOrganizations(Element orgsElement) {
        return _cpCore.getOrganizations(orgsElement);
    }

    /**
     * @return all ORGANIZATION elements in an ORGANIZATIONS element that can be referenced
     */
    public Element[] getOrganizationsAllowed(Element orgsElement) {
        return _cpCore.getOrganizationsAllowed(orgsElement);
    }

    /**
     * @return the default Organization for an ORGANIZATIONS element
     * If there is a DEFAULT ref in the ORGANIZATIONS element will return that
     * otherwise return first ORGANIZATION Element if found
     */
    public Element getDefaultOrganization(Element orgsElement) {
        return _cpCore.getDefaultOrganization(orgsElement);
    }

    /**
     * Find the local parent "manifest" Element for a given Element
     * If element is the manifest, will return that
     */
    public Element getParentManifestElement(Element element) {
        return _cpCore.getParentManifestElement(element);
    }

    /**
     * @return all sub-Elements of type elementName from a given parent Manifest
     * This does a deep recursive search
     */
    public Element[] getElementsInManifest(Element manifestElement, String elementName, Namespace ns) {
        return _cpCore.getElementsInManifest(manifestElement, elementName, ns);
    }

    /**
     * @return an Element by its IDENTIFIER attribute starting at Parent element
     * This will only search one-level deep from parent
     */
    public Element getElementByIdentifier(Element parent, String identifier) {
        return _cpCore.getElementByIdentifier(parent, identifier);
    }

    /**
     * @return the MD Namespace for a CP Document or null if none declared
     */
    public Namespace getMetadataNamespace() {
        return _cpCore.getMetadataNamespace();
    }

    /**
     * Return whether we have the MD root name of any embedded Metadata "lom"
     */
    public boolean isMetadataRoot(String name) {
        return _cpCore.isMetadataRoot(name);
    }

    /**
     * Return whether we have the "metadata" element
     */
    public boolean isMetadataElement(String name) {
        return _cpCore.isMetadataElement(name);
    }

    /**
     * @return The URL string that an Element references
     */
    public String getURL(Element element) {
        return _cpCore.getURL(element);
    }

    /**
     * @return An actual HREF reference from an Element's HREF taking into account xml:base
     */
    public String getElementHREF(Element element) {
        return _cpCore.getElementHREF(element);
    }

    /**
     * @return An actual BASE reference for an Element taking into account xml:base
     */
    public String getElementBase(Element element) {
        return _cpCore.getElementBase(element);
    }

    /**
     * @return the parameters attribute of an element
     */
    public String getParameters(Element element) {
        return _cpCore.getParameters(element);
    }

    /**
     * @return An actual File reference that an Element references.
     * If it's not a local file or does not exist, return null
     */
    public File getResourceFile(Element element) {
        return _cpCore.getResourceFile(element);
    }

    /**
     * @return All File HREFs for a given Manifest Element taking into account xml:base
     */
    public File[] getResourceFiles(Element manifestElement) {
        return _cpCore.getResourceFiles(manifestElement);
    }

    /**
     * Return a Resource Element given its HREF
     */
    public Element getResource(String href, Element resourcesElement) {
        return _cpCore.getResource(href, resourcesElement);
    }

    /**
     * Return a File Element given its HREF
     */
    public Element getFile(String href, Element resourceElement) {
        return _cpCore.getFile(href, resourceElement);
    }

    /**
     * Find the local "resources" Element for a given Element position.  This is because there may
     * be more than one with sub-manifests
     */
    public Element getResourcesElement(Element element) {
        return _cpCore.getResourcesElement(element);
    }

//==============================================================================
// UTILS
//==============================================================================

    /**
     * Create new IDENTIFIERREFs and DEFAULT attributes
     */
    protected void createNewIDRefs(Element element, Hashtable id_map) {
        if(element != null && isDocumentNamespace(element)) {
            String name = element.getName();

            String attrib = null;
            if(name.equals(ContentPackageCore.ITEM)) attrib = ContentPackageCore.IDENTIFIERREF;
            else if(name.equals(ContentPackageCore.ORGANIZATIONS)) attrib = ContentPackageCore.DEFAULT;

            if(attrib != null) {
                String idref = element.getAttributeValue(attrib);
                if(idref != null && !idref.equals("")) {
                    Element ref_element = (Element) id_map.get(idref);
                    if(ref_element != null) {
                        String newID = ref_element.getAttributeValue(ContentPackageCore.IDENTIFIER);
                        if(newID != null) {
                            element.setAttribute(attrib, newID);
                        }
                    }
                }
            }

            // Children
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                createNewIDRefs(child, id_map);
            }
        }
    }

    /**
     * Create new SCORM Prerequisite Refs
     * Really this method should be in the SCORM1_2Package class
     */
    protected void createNewPrerequisiteRefs(Element element, Hashtable id_map) {
        if(element != null && isDocumentNamespace(element)) {
            // SCORM <prerequisite> elements
            Element prereq = element.getChild(SCORM1_2Core.PREREQUISITES, IMSDocumentHandler.ADLCP_NAMESPACE_12);
            if(prereq != null) {
                String script = prereq.getText();
                if(script != null && !script.equals("")) {
                    // Parse script String for this
                    Enumeration e = id_map.keys();
                    while(e.hasMoreElements()) {
                        String oldID = (String)e.nextElement();
                        int index = script.indexOf(oldID);
                        if(index != -1) {
                            Element ref_element = (Element) id_map.get(oldID);
                            if(ref_element != null) {
                                String newID = ref_element.getAttributeValue(ContentPackageCore.IDENTIFIER);
                                if(newID != null) script = script.replaceAll(oldID, newID);
                            }
                        }
                    }
                    prereq.setText(script);
                }
            }

            // Children
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                createNewPrerequisiteRefs(child, id_map);
            }
        }
    }

    /**
     * Create a new ID for an Element that has an IDENTIFIER attribute and also for its children
     * Only works if the Element already has an IDENTIFIER attribute
     * @return a Hashtable of old IDs mapped to Elements
     */
    protected Hashtable createNewIDs(Element element) {
       Hashtable idmap = new Hashtable();
       createNewIDs(element, idmap);
       return idmap;
    }

    protected void createNewIDs(Element element, Hashtable idmap) {
        if(element != null && isDocumentNamespace(element)) {
            String oldID = element.getAttributeValue(ContentPackageCore.IDENTIFIER);
            if(oldID != null) {
                String newID = generateUniqueID(element);
                idmap.put(oldID, element);
                element.setAttribute(ContentPackageCore.IDENTIFIER, newID);
            }
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                createNewIDs(child, idmap);
            }
        }
    }

    /**
     * If we move or copy an Element that contains an IDENTIFIERREF to another Element
     * we check that the Element referenced is still below us. If it is then OK, if not the
     * reference is invalid so we remove it - also for its children
     */
    protected void checkIDRefs(Element element) {
        if(element != null && isDocumentNamespace(element)) {
            Element ref_element = getReferencedElement(element);
            // Null means it's not found and therefore below us
            if(ref_element == null) {
                Attribute att = element.getAttribute(ContentPackageCore.IDENTIFIERREF);
                if(att != null) element.removeAttribute(att);
            }
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                checkIDRefs(child);
            }
        }
    }

    /**
     * Generate a Unique Identifier for an Element
     */
    protected String generateUniqueID(Element element){
        String prefix;
        String elementName = element.getName();
        if(elementName.equals(ContentPackageCore.MANIFEST)) prefix = "MANIFEST-";
        else if(elementName.equals(ContentPackageCore.ORGANIZATION)) prefix = "ORG-";
        else if(elementName.equals(ContentPackageCore.ITEM)) prefix = "ITEM-";
        else if(elementName.equals(ContentPackageCore.RESOURCE)) prefix = "RES-";
        else prefix = "RLD-";
        return RandomGUID.getUniqueID(prefix);
    }

    /**
     * When an Organization element is added/copied to an Organizations Element
     * and it is the only one, set the Organizations "default" attribute to it
     */
    protected void setDefaultOrganization(Element organization) {
        Element orgs = organization.getParent();
        if(orgs.getChildren(ContentPackageCore.ORGANIZATION, orgs.getNamespace()).size() == 1) {
            String id = organization.getAttributeValue(ContentPackageCore.IDENTIFIER);
            if(id != null) orgs.setAttribute(ContentPackageCore.DEFAULT, id);
        }
    }



// =============================================================================
// RESOURCE HANDLING
// =============================================================================

    /**
     * Add a bunch of ReloadResources to the Manifest
     * @return the last added Element
     */
    public Element addReloadResources(ReloadResource[] reloadResources, Element parentElement) {
        Element element = null;

        for(int i = 0; i < reloadResources.length; i++) {
            element = addReloadResource(reloadResources[i], parentElement);
        }

        return element;
    }

    /**
     * Add a ReloadResource to the Manifest
     * @return the last added Element
     */
    public Element addReloadResource(ReloadResource reloadResource, Element parentElement) {
        // We are adding a Folder of Resources
        if(reloadResource.isDirectory()) {

            String name = parentElement.getName();

            // A folder added to ORGANIZATIONS Element so create new Organization
            if(name.equals(ContentPackageCore.ORGANIZATIONS)) {
                // Add new Organization
                parentElement = addOrganization(parentElement, false);
                // Add Title Element - the name of the folder - only here
                addTitle(parentElement, reloadResource.getName());
            }

            // A folder added to ORGANIZATION or ITEM
            else if(name.equals(ContentPackageCore.ORGANIZATION) || name.equals(ContentPackageCore.ITEM)) {
                // Add new empty Item
                parentElement = addItemResource(parentElement, null, false);
                // Add Title Element - the name of the folder - only here
                addTitle(parentElement, reloadResource.getName());
            }

            // Recurse for all children
            ReloadResource[] resources = reloadResource.getChildren();
            if(resources != null) {
                for(int i = 0; i < resources.length; i++) {
                    addReloadResource(resources[i], parentElement);
                }
            }

            return parentElement;
        }

        // A Single Resource
        else return addReloadResource(reloadResource, parentElement, false);
    }

    /**
     * Add a ReloadResource to a given Element
     */
    protected Element addReloadResource(ReloadResource reloadResource, Element parentElement, boolean doSelect) {
        String name = parentElement.getName();

        // If we are adding the ReloadResource to a <resource> element then add it as a <file> element
        if(name.equals(ContentPackageCore.RESOURCE)) {
            // Do we already have the File in our Manifest?
            Element fileElement = getFileByReloadResouce(reloadResource, parentElement);
            if(fileElement == null) {
                fileElement = addFileElement(reloadResource, parentElement, doSelect);
            }
            return fileElement;
        }

        // ----------  Add to <resources> element -------------------

        // Do we already have the Resource in our Manifest?
        Element resourcesElement = getResourcesElement(parentElement);
        Element resourceElement = getResourceByReloadResouce(reloadResource, resourcesElement);

        // Nope - so add to local Resources Element
        if(resourceElement == null) {
            resourceElement = addReloadResourceToResourcesElement(reloadResource, resourcesElement, doSelect);
        }

        // Once added to <resources> element we can add it to the <organization> or <item>
        // as a reference to the <resource>

        // We have added it to Organization or Item Element
        if(name.equals(ContentPackageCore.ORGANIZATION) || name.equals(ContentPackageCore.ITEM)) {
            // Add Item that references the Resource
            return addItemResource(parentElement, resourceElement, doSelect);
        }

        return resourceElement;
    }

    /**
     * Add a Resource to the "resources" element
     */
    protected Element addReloadResourceToResourcesElement(ReloadResource reloadResource, Element resourcesElement, boolean doSelect) {
        DomPath domPath = DomPath.getDomPath(resourcesElement);
        domPath.appendElementName(ContentPackageCore.RESOURCE);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);

        Element resourceElement = addElementBySchema(this, resourcesElement, schemaElement, doSelect);

        if(resourceElement != null) {
            // HREF
            String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
            resourceElement.setAttribute(ContentPackageCore.HREF, href);

            // Add sub-Files
            addFileElements(reloadResource, resourceElement, doSelect);

            // Undo Event
            if(_undoHandler != null) {
                UndoableAddAction addAction = new UndoableAddAction(resourcesElement, resourceElement);
                _undoHandler.addUndoableAction(addAction);
            }

        }

        return resourceElement;
    }

    /**
     * @return true or false as to whether this CP contains a ReloadResource
     */
    protected boolean containsReloadResource(ReloadResource reloadResource, Element resourcesElement) {
        // Get a Relative Path
        Element element = getResourceByReloadResouce(reloadResource, resourcesElement);
        return element != null;
    }

    /**
     * @return a Resource in reference to a ReloadResource
     */
    protected Element getResourceByReloadResouce(ReloadResource reloadResource, Element resourcesElement) {
        // Get a Relative Path
        String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
        return getResource(href, resourcesElement);
    }

    /**
     * @return a File in reference to a ReloadResource
     */
    protected Element getFileByReloadResouce(ReloadResource reloadResource, Element resourceElement) {
        // Get a Relative Path
        String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
        return getFile(href, resourceElement);
    }

    /**
     * Return whether resourcesElement already contains sourceElement as a Resource
     * This searches by the HREF attribute
     */
    public boolean containsResource(Element sourceElement, Element resourcesElement) {
        if(sourceElement != null && resourcesElement != null) {
            if(sourceElement.getName().equals(ContentPackageCore.RESOURCE) && resourcesElement.getName().equals(ContentPackageCore.RESOURCES)) {
                String href = sourceElement.getAttributeValue(ContentPackageCore.HREF);
                //String href = getElementHREF(sourceElement);
                if(href != null) return containsResource(href, resourcesElement);
            }
        }
        return false;
    }

    /**
     * Return whether a Resources Element contains a Resource by HREF
     */
    protected boolean containsResource(String href, Element resourcesElement) {
        Element element = getResource(href, resourcesElement);
        return element != null;
    }

    /**
     * Return whether a Resource Element contains a File Element by HREF
     */
    protected boolean containsFile(String href, Element resourceElement) {
        Element element = getFile(href, resourceElement);
        return element != null;
    }

    /**
     * Return whether targetElement can accept ReloadResources
     */
    public boolean acceptsReloadResources(Element targetElement) {
        // Correct Namespace?
        if(!targetElement.getNamespace().equals(getTargetNamespace())) return false;

        String name = targetElement.getName();

        if(name.equals(ContentPackageCore.MANIFEST) ||
           name.equals(ContentPackageCore.RESOURCES) ||
           name.equals(ContentPackageCore.RESOURCE) ||
           name.equals(ContentPackageCore.ORGANIZATION) ||
           name.equals(ContentPackageCore.ITEM) ||
           name.equals(ContentPackageCore.ORGANIZATIONS)) {
            return true;
        }

        return false;
    }

    /**
     * Add File dependencies for a ReloadResource. Note - this is NOT an UNDOABLE EVENT
     */
    protected void addFileElements(ReloadResource reloadResource, Element resourceElement, boolean doSelect) {
        // Add it
        DomPath domPath = DomPath.getDomPath(resourceElement);
        domPath.appendElementName(ContentPackageCore.FILE);
        ReloadSchemaElement fileSchemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        Element fileElement = addElementBySchema(this, resourceElement, fileSchemaElement, doSelect);

        // Now add the href
        if(fileElement != null) {
            String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
            fileElement.setAttribute(ContentPackageCore.HREF, href);
        }

        // Add the other <file> elements if this is a Parseable resource file
        if(ReloadFileType.isParseableFile(reloadResource)) {
            File[] files = reloadResource.getDependencies(getProjectFolder());
            for(int i = 0; i < files.length; i++) {
                fileElement = addElementBySchema(this, resourceElement, fileSchemaElement, doSelect);
                String href = ReloadUtils.getRelativePath(getProjectFolder(), files[i]);
                fileElement.setAttribute(ContentPackageCore.HREF, href);
            }
        }
    }

    /**
     * Add a <file> element to a <resource> element from a ReloadResource. Note - this is an UNDOABLE EVENT
     * @return the added <file> element
     */
    protected Element addFileElement(File reloadResource, Element resourceElement, boolean doSelect) {
        // Add it
        DomPath domPath = DomPath.getDomPath(resourceElement);
        domPath.appendElementName(ContentPackageCore.FILE);
        ReloadSchemaElement fileSchemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        Element fileElement = addElementBySchemaUndoable(this, resourceElement, fileSchemaElement, doSelect);

        // Now add the href
        if(fileElement != null) {
            String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
            fileElement.setAttribute(ContentPackageCore.HREF, href);
            // Seems as if we need to notify to repaint the tree
            fireElementChanged(new ReloadDocumentListenerEvent(this, this, fileElement, false));
        }

        return fileElement;
    }
}
