/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.layout.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.gui.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.contentpackaging.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The SCORM Editor Dialog used in the CP Editor to edit SCORM Elements
 * It edits SCORM Elements from an Item node of a CP
 *
 * @author Phillip Beauvoir
 * @version 2003-09-26
 */
public class SCORM1_2EditorDialog extends JDialog {

    /**
     * The Content Package to which this Item Element belongs
     */
    protected ContentPackage _contentPackage;

    /**
     * The parent "item" element containing the SCORM Elements
     */
    protected Element _itemElement;

    /**
     * The schemaElement
     */
    protected ReloadSchemaElement _schema_itemElement;

    /**
     * A list box to display the list of Items
     */
    protected JList _itemListBox;

    /**
     * Prerequisites Editor
     */
    protected ReloadElementAttributePanel _prereqEditor;

    /**
     * Dialog Width
     */
    protected int WIDTH = 780;

    /**
     * Dialog Height
     */
    protected int HEIGHT = 555;

    /**
     * Constructor
     */
    public SCORM1_2EditorDialog(ContentPackage contentPackage, Element itemElement, ReloadSchemaElement schema_itemElement) {
        super(ReloadEditor.getInstance(), "SCORM Properties", true);

        _itemElement = itemElement;
        _contentPackage = contentPackage;
        _schema_itemElement = schema_itemElement;

        setResizable(false);

        // Trap window closing event for our Window listener
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // Add a listener to Close our window down on exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });

        // Main Panel
        JPanel mainPanel = createMainPanel();
        getContentPane().add(mainPanel, BorderLayout.CENTER);

        // Button Panel
        JPanel buttonPanel = createButtonPanel();
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);

        // Dialog Size
        setSize(WIDTH, HEIGHT);
        setLocationRelativeTo(ReloadEditor.getInstance());
    }


    /**
     * @return the Main Panel
     */
    protected JPanel createMainPanel() {
        JPanel mainPanel = new JPanel(new XYLayout());

        int x = 5, y = 5;

        // Prerequisites Panel
        JPanel prereqPanel = createPrerequisitesPanel();
        mainPanel.add(prereqPanel, new XYConstraints(x, y, WIDTH - 15, 250));

        y += 265;

        // Other Panel
        JPanel otherPanel = createSCORMElementsPanel();
        mainPanel.add(otherPanel, new XYConstraints(x, y, WIDTH - 15, 220));

        return mainPanel;
    }

    /**
     * @return the Prerequisites element editing Panel
     */
    protected JPanel createPrerequisitesPanel() {
        JPanel panel = new JPanel(new XYLayout());
        panel.setBorder(BorderFactory.createEtchedBorder());

        ReloadSchemaController schemaController = _contentPackage.getReloadSchemaController();

        ReloadSchemaElement prereqSchemaElement = _schema_itemElement.getChild(SCORM1_2Core.PREREQUISITES,
            IMSDocumentHandler.ADLCP_NAMESPACE_12);

        if(prereqSchemaElement != null) {
            int x = 5, y = 0;

            // Friendly name
            String name = schemaController.getElementFriendlyName(prereqSchemaElement.getDomPath());
            JLabel nameLabel = new JLabel();
            nameLabel.setForeground(Color.BLUE);
            nameLabel.setFont(ReloadUIManager.boldFont12);
            nameLabel.setText(name);
            panel.add(nameLabel, new XYConstraints(x, y, 100, 25));

            y += 35;

            // Element binding
            Element element = _itemElement.getChild(prereqSchemaElement.getName(), prereqSchemaElement.getNamespace());
            ReloadElementBinding eb = new ReloadElementBinding(element, prereqSchemaElement, _contentPackage);
            // No element yet so set parent instead
            if(element == null) eb.setParentElement(_itemElement);

            // Element Editor
            _prereqEditor = new ReloadElementAttributePanel();
            _prereqEditor.setElement(eb);
            panel.add(_prereqEditor, new XYConstraints(x, y, 340, 200));

            // Add Button
            JButton addButton = new JButton(ReloadUIManager.getIcon(ReloadEditor.ICON_LEFT));
            panel.add(addButton, new XYConstraints(345, 120, 0, 0));
            addButton.addActionListener(new AddClick(_prereqEditor));

            // Add Item Listbox
            _itemListBox = new JList(new ItemListModel());
            _itemListBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            _itemListBox.addMouseListener(new AddClick(_prereqEditor));
            panel.add(new JScrollPane(_itemListBox), new XYConstraints(405, y, 340, 200));
        }

        return panel;
    }

    /**
     * @return the editing Panel for the other SCORM Elements
     */
    protected JPanel createSCORMElementsPanel() {
        JPanel panel = new JPanel(new XYLayout());
        panel.setBorder(BorderFactory.createEtchedBorder());

        ReloadSchemaController schemaController = _contentPackage.getReloadSchemaController();

        int x = 5, y = 0;

        // Max time allowed
        ReloadSchemaElement maxtimeallowedSchemaElement = _schema_itemElement.getChild(SCORM1_2Core.MAXTIMEALLOWED,
            IMSDocumentHandler.ADLCP_NAMESPACE_12);
        addElement(maxtimeallowedSchemaElement, panel, x, y, 120, 25);

        y += 60;

        // Time Limit Action
        ReloadSchemaElement schemaElement = _schema_itemElement.getChild(SCORM1_2Core.TIMELIMITACTION,
            IMSDocumentHandler.ADLCP_NAMESPACE_12);
        addElement(schemaElement, panel, x, y, 320, 100);

        x = 360; y = 0;

        // Data from LMS
        schemaElement = _schema_itemElement.getChild(SCORM1_2Core.DATAFROMLMS,
            IMSDocumentHandler.ADLCP_NAMESPACE_12);
        addElement(schemaElement, panel, x, y, 200, 25);

        y += 60;

        // Mastery Score
        schemaElement = _schema_itemElement.getChild(SCORM1_2Core.MASTERYSCORE,
            IMSDocumentHandler.ADLCP_NAMESPACE_12);
        addElement(schemaElement, panel, x, y, 200, 25);

        return panel;
    }


    /**
     * Add an Element to the Panel
     */
    protected void addElement(ReloadSchemaElement schemaElement, JPanel panel, int x, int y, int width, int height) {
        if(schemaElement != null) {
            ReloadSchemaController schemaController = _contentPackage.getReloadSchemaController();

            // Friendly name
            String name = schemaController.getElementFriendlyName(schemaElement.getDomPath());

            // Name label
            JLabel nameLabel = new JLabel();
            nameLabel.setForeground(Color.BLUE);
            nameLabel.setFont(ReloadUIManager.boldFont12);
            nameLabel.setText(name);
            panel.add(nameLabel, new XYConstraints(x, y, 100, 25));

            y += 30;

            // Element binding
            Element element = _itemElement.getChild(schemaElement.getName(), schemaElement.getNamespace());
            ReloadElementBinding eb = new ReloadElementBinding(element, schemaElement, _contentPackage);
            // No element yet so set parent instead
            if(element == null) eb.setParentElement(_itemElement);

            // Element Editor
            ReloadElementEditor elementEditor = new ReloadElementEditor();
            elementEditor.setElement(eb);
            panel.add(elementEditor, new XYConstraints(x, y, width, height));
        }
    }

    /**
     * @return the Button Panel
     */
    protected JPanel createButtonPanel() {
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));

        // OK Button
        JButton btnOK = new JButton("Done");
        btnOK.addActionListener(new OKClick());
        getRootPane().setDefaultButton(btnOK);
        buttonPanel.add(btnOK);

        return buttonPanel;
    }

    /**
     * Show the Dialog
     */
    public void show() {
        // Do this later because a modal dialog will block
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                _prereqEditor.getSplitpane().setDividerLocation(0.7);
            }
        });
        super.show();
    }

    /**
     * User pressed OK
     */
    protected void finish() {

        // Tell listeners
        //_contentPackage.changedElement(this, _itemElement);

        // Finished
        dispose();
    }

    /**
     * Clean up
     */
    public void dispose() {
        try {
            // Do cleanup stuff here
            super.dispose();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }
    }

    /**
     * OK handler
     */
    private class OKClick extends AbstractAction {

        public void actionPerformed(ActionEvent e) {
            finish();
        }
    }

    /**
     * Add handler
     */
    private class AddClick extends AbstractAction implements MouseListener {
        ReloadElementAttributePanel _elementEditor;

        public AddClick(ReloadElementAttributePanel elementEditor) {
            _elementEditor = elementEditor;
        }

        public void actionPerformed(ActionEvent e) {
            addText();
        }

        private void addText() {
            // Insert Identifier Text
            Object object = _itemListBox.getSelectedValue();
            if(object instanceof ID_Mapping) {
                String id = ((ID_Mapping)object).getIdentifier();
                if(id != null) {
                    _elementEditor.getElementEditor().insertText(id);
                    _elementEditor.getElementEditor().setFocus();
                }
            }
        }

        /**
         * Double-clicks
         */
        public void mouseClicked(MouseEvent e) {
            if(e.getClickCount() == 2) addText();
        }

        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
    }

    /**
     * Convenience mapping class
     */
    class ID_Mapping {
        Element _element;

        ID_Mapping(Element element) {
            _element = element;
        }

        public String getIdentifier() {
            return _element.getAttributeValue(ContentPackageCore.IDENTIFIER);
        }

        public String toString() {
            return _contentPackage.getElementDisplayName(_element) + "   (" + getIdentifier() + ")";
        }
    }

    /**
     * List Model
     */
    class ItemListModel extends AbstractListModel {
        Vector v = new Vector();

        public ItemListModel() {
            // Fill the list with Items that can be Pre-requisites
            Element manifestElement = _contentPackage.getParentManifestElement(_itemElement);
            if(manifestElement != null) {
                Element[] items = _contentPackage.getElementsInManifest(manifestElement, ContentPackageCore.ITEM, manifestElement.getNamespace());
                for(int i = 0; i < items.length; i++) {
                    // Don't include the Item that we are
                    if(!items[i].equals(_itemElement)) {
                        ID_Mapping mapping = new ID_Mapping(items[i]);
                        v.add(mapping);
                    }
                }
            }
        }

        public int getSize() {
            return v.size();
        }

        public Object getElementAt(int index) {
            return v.elementAt(index);
        }
    }

}