/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging.resourceview;

import java.io.*;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.*;
import uk.ac.reload.editor.menu.*;


/**
 * A Content Package Resources Panel for the CP UI
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-10-07
 */
public class CPResourcesPanel
extends JPanel
{
    /**
     * The Resources Tree
     */
    protected CPResourcesTree _resourcesTree;

    /**
     * The Root Folder
     */
    protected File _rootFolder;

    /**
     * Our import handler from the main menu and panel
     */
    protected ProxyImportHandler _proxyImportHandler, _proxyImportHandlerLocal;

    /**
     * Our refresh handler from the panel
     */
    protected ProxyRefreshHandler _proxyRefreshHandlerLocal;

    /**
     * Our resource remover handler from the panel
     */
    protected ProxyRemoveResourceHandler _proxyRemoveResourceHandlerLocal;

    /**
     * Our view file handler from the panel
     */
    protected ProxyViewFileHandler _proxyViewFileHandlerLocal;

    /**
     * Our new folder handler from the panel
     */
    protected ProxyNewFolderHandler _proxyNewFolderHandler;


    /**
     * Constructor
     */
    public CPResourcesPanel() {
        setLayout(new BorderLayout());
        setMinimumSize(new Dimension(0, 0));

        // Import Files
        MenuAction_ImportResources _importMenuAction = new MenuAction_ImportResources();
        _importMenuAction.setEnabled(true);
        _proxyImportHandlerLocal = new ProxyImportHandler(_importMenuAction);

        // Proxy Import handler from Main menu
        _proxyImportHandler = new ProxyImportHandler(MainMenu.sharedInstance.actionImport);

        // Refresh
        _proxyRefreshHandlerLocal = new ProxyRefreshHandler();

        // Delete
        _proxyRemoveResourceHandlerLocal = new ProxyRemoveResourceHandler();

        // View File handler from Panel
        _proxyViewFileHandlerLocal = new ProxyViewFileHandler();

        // New Folder Proxy
        _proxyNewFolderHandler = new ProxyNewFolderHandler();

        // ToolBar
        ReloadToolBar _toolBar = new ReloadToolBar();
        _toolBar.add(_proxyImportHandlerLocal.getMenuAction());
        _toolBar.add(_proxyNewFolderHandler.getMenuAction());
        _toolBar.add(_proxyRemoveResourceHandlerLocal.getMenuAction());
        _toolBar.add(_proxyRefreshHandlerLocal.getMenuAction());
        _toolBar.add(_proxyViewFileHandlerLocal.getMenuAction());

        add(_toolBar, BorderLayout.NORTH);

        // Tree
        _resourcesTree = new CPResourcesTree(this);

        add(new JScrollPane(_resourcesTree), BorderLayout.CENTER);
    }

    /**
     * Set the File View
     */
    public void setFileView(File rootFolder) {
        _rootFolder = rootFolder;
        _resourcesTree.setFileView(rootFolder);
    }

    /**
     * Destroy this Panel
     */
    public void destroy() {
        _proxyImportHandler.removeListener();
        _proxyImportHandlerLocal.removeListener();
        _proxyNewFolderHandler.removeListener();
        _proxyRefreshHandlerLocal.removeListener();
        _proxyRemoveResourceHandlerLocal.removeListener();
        _proxyViewFileHandlerLocal.removeListener();
    }

// =============================================================================
// Focus Handler
// =============================================================================

    /**
     * We got the focus
     */
    public void setFocusGained() {
        _resourcesTree.setFocusGained();
        _proxyImportHandler.addListener();
    }

    /**
     * We lost the focus
     */
    public void setFocusLost() {
        _resourcesTree.setFocusLost();

        _proxyImportHandler.setEnabled(false);
        _proxyImportHandler.removeListener();
    }


    /**
     * Update our buttons and menus depending on resource selected
     */
    public void updateMenus(ReloadResource resource) {
        // Make sure the delete resource button is disabled if root selected
        _proxyRemoveResourceHandlerLocal.setEnabled(!resource.equals(_rootFolder));

        // Launch File
        _proxyViewFileHandlerLocal.update(resource);

        // New Folder
        _proxyNewFolderHandler.update(resource);

        // Import Files
        _proxyImportHandler.update(resource);
        _proxyImportHandlerLocal.update(resource);
    }


// =============================================================================


    /**
     * Dialogue routine to allow user to import a new resource or folder
     * Checks to see if the user wants to import dependent files and
     * then instantiates ResourceImporter to bring in the files
     * Finally, it refreshes the resource tree to show the new listing
     */
    protected void importResources(ReloadResource folder){
        ReloadResourceChooser fileChooserOption = new ReloadResourceChooser();

        int returnVal = fileChooserOption.showOpenDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return;

        // Get the chosen File(s)
        final File[] files = fileChooserOption.getSelectedFilesAndStore();

        // Set the checkbox to remember if it was selected previously
        //fileChooserOption.setCheckbox(fileChooserOption.isCheckboxChecked());

        ReloadProgressMonitor progressMonitor = null;

        try {
            // Disable buttons
            _proxyImportHandler.setEnabled(false);
            _proxyImportHandlerLocal.setEnabled(false);

            progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Import Resources",
                "Importing...", "", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

            // Call routine
            ResourceImporter importRes = new ResourceImporter(files, folder,
                    fileChooserOption.isCheckboxChecked(), progressMonitor);

            // Only refresh the tree if the user imported a file
            if(importRes.getHasUserImportedFiles()) refreshResourceTree();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Resources", "Error importing", ex);
        }
        finally {
            if(progressMonitor != null) progressMonitor.close();
            // Enable buttons
            _proxyImportHandler.setEnabled(true);
            _proxyImportHandlerLocal.setEnabled(true);
        }
    }

    /**
     * Remove the resources the user wanted to delete from the resources tree
     */
    private void removeTheResources(){
        Thread thread = new Thread() {
            public void run() {
                try {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            ResourceRemover removeRes = new ResourceRemover();
                            removeRes.removeFiles(_resourcesTree);
                        }
                    });
                    // We dont need to refresh the tree here as each branch/leaf
                    // would have been deleted as it was encountered within
                    // ResourceRemover() class.  However, just in case there is
                    // an exception of some sort, its safer to refresh
                    // at the end of the following catch block
                }
                catch(Exception ex) {
                    if(ReloadEditor.DEBUG) ex.printStackTrace();
                    ErrorDialogBox.showWarning("Remove Resources", "Error deleting", ex);
                    refreshResourceTree();
                }
            }
        };
        thread.start();
    }

    /**
     * Refresh the Resource tree after changes
     */
    public void refreshResourceTree(){
        _resourcesTree.refresh();
    }

//==============================================================================
// ============================== PROXY MENU HANDLERS ==========================
//==============================================================================

    /**
     * Deals with Import Menu Events
     */
    class ProxyImportHandler extends ProxyAction {
        ReloadResource _resource;

        public ProxyImportHandler(MenuAction menuAction) {
            super(menuAction);
        }

        public void update(ReloadResource resource) {
            setEnabled(resource.isDirectory());
            _resource = resource;
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            Thread thread = new Thread() {
                public void run() {
                    if(_resource != null) importResources(_resource);
                }
            };

            thread.start();
        }
    }

    /**
     * Deals with remove resource Events
     */
    class ProxyRemoveResourceHandler extends ProxyAction {
        public ProxyRemoveResourceHandler() {
            super(new MenuAction_RemoveResources());
        }

        // when user clicks 'remove resource' from the resources panel...
        public void menuActionPerformed(ReloadMenuEvent event) {
            removeTheResources();
        }
    }

    /**
     * Deals with refresh Menu Events
     */
    class ProxyRefreshHandler extends ProxyAction {
        public ProxyRefreshHandler() {
            super(new MenuAction_Refresh());
            setEnabled(true);
        }

        // when user clicks 'import' from the main menu...
        public void menuActionPerformed(ReloadMenuEvent event) {
            Thread thread = new Thread() {
                 public void run() {
                     setCursor(ReloadUIManager.WAIT_CURSOR);
                     _proxyRefreshHandlerLocal.setEnabled(false);
                     refreshResourceTree();
                     _proxyRefreshHandlerLocal.setEnabled(true);
                     setCursor(ReloadUIManager.DEFAULT_CURSOR);
                 }
             };

            thread.start();
        }
    }


    /**
     * View File Proxy Action
     */
    class ProxyViewFileHandler extends ProxyAction {
        ReloadResource _resource;

        public ProxyViewFileHandler() {
            super(new MenuAction_ViewFile());
        }

        public void update(ReloadResource resource) {
            setEnabled(resource.isFile());
            _resource = resource;
        }

        /**
         * Launch the file
         * @param event the ReloadMenuEvent
         */
        public void menuActionPerformed(ReloadMenuEvent event) {
            if(_resource != null) {
                // Start the loader thread
                Thread thread = new Thread() {
                    public void run() {
                        ReloadUtils.launchFile(_resource);
                    }
                };

                thread.start();
            }
        }
    }

    /**
     * New Folder Action
     */
    class ProxyNewFolderHandler extends ProxyAction {
        ReloadResource _resource;

        public ProxyNewFolderHandler() {
            super(new MenuAction_NewFolder());
        }

        public void update(ReloadResource resource) {
            setEnabled(resource.isDirectory());
            _resource = resource;
        }

        /**
         * Make a New Folder
         * @param event the ReloadMenuEvent
         */
        public void menuActionPerformed(ReloadMenuEvent event) {
            if(_resource != null) {
                String s = JOptionPane.showInputDialog(ReloadEditor.getInstance(), "Folder Name", "New Folder", JOptionPane.PLAIN_MESSAGE);
                if(s != null) {
                    File file = new File(_resource, s);
                    if(!file.exists()) {
                        file.mkdir();
                        refreshResourceTree();
                    }
                }
            }
        }
    }
}