/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging.resourceview;

import java.io.*;
import java.util.*;

import javax.swing.*;

import uk.ac.reload.dweezil.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.contentpackaging.htmlparser.*;

/**
 * ResourceImporter class.  Class is used to decide what type of resource
 * the user is trying to import into the resources pool.  It will decide on
 * which it is and then either call the HTMLParser to find dependencies or
 * simply copy over the files (or folder contents) into the project folder
 * (resource pool)
 *
 * @author Paul Sharples
 * @author Phillip Beauvoir
 * @version 2003-10-07
 */
public class ResourceImporter
implements DweezilConstants
{

    protected boolean _yesToAll;

    /**
     * Because we are using nested and recursive function calls
     * it is neccessary to hold a implementation of when a user
     * clicks on the cancel button of a dialogue box.
     */
    protected boolean _cancelOperation;

    /**
     * During the process of importing resources a user can import a file, then
     * subsequently click cancel when prompted to overwrite a file that is
     * already in the resource pool.  This variable will tell us if a user
     * has actually copied somthing into the project folder, so we can
     * notify the resource tree to update.
     */
    private boolean _userHasImportedFile;

    /**
     * ReloadProgressMonitor
     */
    private ReloadProgressMonitor _progressMonitor;


    /**
     * Constructor - Is passed a file reference
     * @param aFile - The file we want to import
     * @param destination - The destination folder to copy it to
     * @param obtainDependencies - Does the user want to pull all dependencies across
     */
    public ResourceImporter(File aFile, File destination, boolean obtainDependencies, ReloadProgressMonitor progressMonitor){
        _progressMonitor = progressMonitor;
        _yesToAll = false;
        decideImportType (aFile, destination, obtainDependencies);
    }

    /**
     * Constructor - Is passed an array of file references
     * @param files - An array of files we wish to import
     * @param destination - The destination folder to copy it to
     * @param obtainDependencies - Does the user want to pull all dependencies across
     */
    public ResourceImporter(File[] files, File destination, boolean obtainDependencies, ReloadProgressMonitor progressMonitor){
        _progressMonitor = progressMonitor;
        _yesToAll = false;
        for(int i = 0; i < files.length; i++){
            decideImportType (files[i], destination, obtainDependencies);
        }
    }

    /**
     * Method to decide what exactly we are trying to import
     * @param aFile - A file to import
     * @param destination - Folder to copy it to
     * @param obtainDependencies - Does the user want to pull all dependencies across
     */
    private void decideImportType(File aFile, File destination, boolean obtainDependencies){
        // we need to check the file extension to see if it is one of the files to pass to the HTMLParser
        boolean pageToParseFound = ReloadFileType.isParseableFile(aFile);

        // If its a page to be parsed then pass it to HTMLParser via importHtmlFile()
        if(pageToParseFound && obtainDependencies) {
            importHtmlFile(aFile, destination);
        }
        else{
            if(aFile.isDirectory()){
                importFolder(aFile, destination);
            }
            else if(aFile.isFile()){
                importSingleResource(aFile, destination);
            }
        }
    }


    /**
     * Accessor method to return the above variable
     * @return a boolean indicating if the user actually imported anything
     */
    public boolean getHasUserImportedFiles(){
        return _userHasImportedFile;
    }

    /**
     * Method used to copy a folders contents into the resource pool
     * @param theFolder - Top level folder the user wishes to copy across
     * @param destinationPath - Where the user wants to copy it to
     */
    private void importFolder(File theFolder, File destinationPath) {
        // Sanity check
        if(!theFolder.isDirectory()) return;

        // Check for project folder and return (we don't want to import the project folder)
        if(theFolder.equals(destinationPath)) return;

        File destinationFolder = new File(destinationPath, theFolder.getName());

        if(!destinationFolder.exists()) destinationFolder.mkdirs();

        copyTheFilesAndFolders(theFolder.listFiles(), destinationFolder);
    }

    /**
     * Method to find all of the files and folders contained within a
     * folder and then to copy them over
     * @param allTheFiles
     * @param destinationPath
     */
    private void copyTheFilesAndFolders(File[] allTheFiles, File destinationPath){
        if(isCancelled()) return;

        // Do this in two passes so we can do the folders first
        for(int i = 0; i < allTheFiles.length; i++) {
            File file = allTheFiles[i];
            if(file.isDirectory()) {
                // copy it across
                File destinationFolder = new File(destinationPath, file.getName());
                if(!destinationFolder.exists()) destinationFolder.mkdir();
                copyTheFilesAndFolders(file.listFiles(), destinationFolder);
            }
       }

       for(int i = 0; i < allTheFiles.length; i++) {
           File file = allTheFiles[i];
           if(!file.isDirectory()) {
               importSingleResource(file, destinationPath);
           }
       }
    }


    /**
     * Method to copy over a single resource into the project folder.  It needs
     * to flag up when the user has selected the cancel operation on a dialogue
     * window, so it returns a boolean value to tell the calling code.
     * @param originalFile
     * @param destination
     * @return boolean - true if the user selected cancel on the dialogue
     */
    private void importSingleResource(File originalFile, File destination) {
        if(isCancelled()) return;

        File destinationFile = new File(destination, originalFile.getName());
        try {
            if(destinationFile.exists() && (!_yesToAll)) {
                int answer = getUserResponse(destinationFile);
                if (answer == ReloadYesAllNoDialog.YES_TO_ALL)
                     _yesToAll = true;
                if(answer == ReloadYesAllNoDialog.YES || answer == ReloadYesAllNoDialog.YES_TO_ALL) {
                    ResourceImporter.copyFile(originalFile, destinationFile);
                    _userHasImportedFile = true;
                }
                else if(answer == ReloadYesAllNoDialog.CANCEL || answer == ReloadYesAllNoDialog.CLOSE)
                    _cancelOperation = true;
            }
            else {
                copyFile(originalFile, destinationFile);
                _userHasImportedFile = true;
            }
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Resources", "Error importing", ex);
        }

    }


    /**
     * Import the HTML page by first parsing the page using HTMLParser
     * which will return a Hashtable of all the original links within each
     * page and the destination to copy them.
     * @param theFile
     * @param destFolder
     */

    private void importHtmlFile(File theFile, File destFolder)  {
        // pass the start page and target folder to HTMLParser
        HTMLParser htmlParser = new HTMLParser(theFile, destFolder, true);

        if(htmlParser.isHigherLocalLinkFound()){
            showMessageDialog("One of the pages you requested to import has links that"
                              + " are higher in the folder structure than the project folder."
                              + CR
                              + "Please see the Status Window for more information."
                              , "Import Warning");
        }
        if (htmlParser.isAbsoluteLocalLinkFound()){
            showMessageDialog("One of the pages you requested to import has absolute links "
                              + " to pages on your hard drive." + CR
                              + "Please see the Status Window for more information."
                              , "Import Warning");
        }

        // Retrieve all the local links
        Hashtable allLinks = htmlParser.getLinks();
        // Enumerate thru all elements copying keys(source path) to values (destination path)
        Enumeration keys = allLinks.keys();
        while(keys.hasMoreElements()) {
            if(isCancelled()) return;

            File originalFile = (File)keys.nextElement();
            File destinationFile = (File)allLinks.get(originalFile);
            destinationFile.getParentFile().mkdirs();
            // check for any ../. (parent folder)links and ignore them...
            if (!destinationFile.isDirectory()){
                try {
                    // check if file exists?
                    if(destinationFile.exists() && (!_yesToAll)) {
                        int answer = getUserResponse(destinationFile);
                        if (answer == ReloadYesAllNoDialog.YES_TO_ALL)
                            _yesToAll = true;
                        if(answer == ReloadYesAllNoDialog.YES || answer == ReloadYesAllNoDialog.YES_TO_ALL){
                            copyFile(originalFile, destinationFile);
                            _userHasImportedFile = true;
                        }
                         else if(answer == ReloadYesAllNoDialog.CANCEL || answer == ReloadYesAllNoDialog.CLOSE) {
                            _cancelOperation = true;
                            return;
                        }
                    }
                    else {
                        copyFile(originalFile, destinationFile);
                        _userHasImportedFile = true;
                    }
                }
                catch(Exception ex) {
                    if(ReloadEditor.DEBUG) ex.printStackTrace();
                    ErrorDialogBox.showWarning("Import Resources", "Error importing", ex);
                }
            }
        }
    }


    /**
       * Method to build message string, pass it to the YesAllNo dialog
       * and return the users choice
       * @param res - a ReloadResource object
       * @return users choice for yes. yes to all, no, cancel or close
       */
      private int getUserResponse(File res){
          ReloadYesAllNoDialog dialog = new ReloadYesAllNoDialog();
          String message, title, fileOrFolder;
          fileOrFolder = res.isDirectory() ? "Folder " : "File ";
          message = fileOrFolder + res.getName() + " already exists. "
              + CR +
              "Do you wish to overwrite?";
          title = "Import Resources";
          return dialog.getUserResponse(ReloadEditor.getInstance(), message, title);
      }


    protected void showMessageDialog(String msg, String title){
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg, title, JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Copy a file to a different file.
     * @param from
     * @param to
     * @throws IOException
     */
    public static void copyFile(File from, File to) {
        try {
            byte[] buffer = new byte[256];
            FileInputStream is = new FileInputStream(from);
            FileOutputStream os = new FileOutputStream(to);
            int i;
            while((i = is.read(buffer)) >= 0) {
                os.write(buffer, 0, i);
            }
            is.close();
            os.close();
        }
        catch(IOException e) {
            if(ReloadEditor.DEBUG) System.out.println("Error: " + e);
        }
    }

    /**
     * Find out if user cancelled
     * @return
     */
    private boolean isCancelled() {
        boolean progressCancelled = false;
        if(_progressMonitor != null) progressCancelled = _progressMonitor.isCanceled();
        return _cancelOperation || progressCancelled;
    }

}
