/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.widgets.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.schema.*;


/**
 * A Panel to display and edit a JDOM Element
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public class ReloadElementEditor
extends JPanel
implements ReloadDocumentListener, DocumentListener
{
    /**
     * The ElementBinding we are editing
     */
    protected ReloadElementBinding _elementBinding;

    /**
     * The widget
     */
    protected ReloadWidget _reloadWidget;

    /**
     * Padding
     */
    protected JPanel padPanel;

    /**
     * Whether we notify or not
     */
    protected boolean allowNotification = true;

    /**
     * Default constructor
     */
    public ReloadElementEditor() {
        setLayout(new BorderLayout());
        setBackground(Color.lightGray);
        padPanel = new JPanel();
    }

    /**
     * Set The Panel to display the data for the Element
     * @param elementBinding the element binding we are editing
     */
    public void setElement(ReloadElementBinding elementBinding) {
        // Remove old Listener
        if(_elementBinding != null) _elementBinding.getReloadDocument().removeReloadDocumentListener(this);

        _elementBinding = elementBinding;

        // Add Listener
        _elementBinding.getReloadDocument().addReloadDocumentListener(this);

        // Set it up
        setupWidget(_elementBinding.getReloadDocument(), _elementBinding.getElement(), _elementBinding.getSchemaElement());
    }

    /**
     * Set The Widget to display the data for the Element
     * @param doc the owner document
     * @param element the element who's data we want to display
     * @param schemaElement the schema element bound to this element
     */
    protected void setupWidget(ReloadSchemaDocument doc, Element element, ReloadSchemaElement schemaElement) {
        if(_reloadWidget != null) _reloadWidget.removeDocumentListener(this);

        // Remove all components from the panel
        removeAll();

        // If this Element is merely a container do not show
        boolean enabled = schemaElement == null ? true : schemaElement.isValue();

		// Tina Manoharan, 2003-11-02
		// This is a added for the Vdex Metadata Element, so that a Metadata Editor can be launched

		if(schemaElement != null) {

			if(doc.getRootElement().getName().equals("vdex") && schemaElement.getName().equals("metadata")) {

				enabled = true;

				/*
				enabled = false;

				JButton button = new JButton("Launch Metadata Editor", ReloadUIManager.getIcon(ReloadEditor.ICON_MD));

				add(button, BorderLayout.NORTH);
				add(padPanel, BorderLayout.CENTER);

				if(doc instanceof Vocabulary) {

					Vocabulary vocabulary = (Vocabulary) doc;

					VocabularyMetadata vocabMetadata = new VocabularyMetadata(element, vocabulary);

					button.addActionListener(new Action_LaunchMetadataEditor(vocabMetadata));
				}
				*/
			}
		}


        if(enabled) {
            // Get and display the Widget
            // _reloadWidget = ReloadWidgetFactory.getWidget(doc.getReloadSchemaController(), schemaElement);

			// Tina Manoharan 2003-12-10

            _reloadWidget = ReloadWidgetFactory.getWidget(doc, doc.getReloadSchemaController(), schemaElement, element);

            if(_reloadWidget.isSingleLine()) {
                add((Component)_reloadWidget, BorderLayout.NORTH);
                add(padPanel, BorderLayout.CENTER); // Need to do this to stop bleed in SCORM Editor
            }
            else if(_reloadWidget instanceof ReloadTextPane) {
                JScrollPane scrollPane = new JScrollPane((Component) _reloadWidget);
                add(scrollPane, BorderLayout.CENTER);
            }
            else {
                add((Component) _reloadWidget, BorderLayout.CENTER);
            }

            // Set the text value
            setText(element);

            // Listener
            _reloadWidget.addDocumentListener(this);
        }
    }

    /**
     * Set the Text in the widget to that in the Element
     * @param element the element concerned
     */
    protected void setText(Element element) {
        allowNotification = false;
        if(_reloadWidget != null) _reloadWidget.setTextValue(element == null ? "" : element.getText());
        allowNotification = true;
    }

    /**
     * Insert text in the widget
     * @param text the text to insert in the widget
     */
    public void insertText(String text) {
        if(_reloadWidget != null) _reloadWidget.insertTextValue(text);
    }

    /**
     * Set the focus
     */
    public void setFocus() {
        if(_reloadWidget != null) ((Component)_reloadWidget).requestFocus();
    }

    /**
     * Clean up
     */
    public void destroy() {
        if(_elementBinding != null) _elementBinding.getReloadDocument().removeReloadDocumentListener(this);
        if(_reloadWidget != null) _reloadWidget.removeDocumentListener(this);
    }

    /**
     * Update the Model
     * @param text the text to set for the element
     */
    protected synchronized void fireElementChanged(String text) {
        Element element = _elementBinding.getElement();

        // Element was null so create new one
        if(element == null) element = _elementBinding.createElement(this);

        if(element != null) {
            element.setText(text);
            _elementBinding.getReloadDocument().changedElement(this, element);
        }
    }

// =============================================================================
// ===================== These events are received FROM the ReloadDocument =====
// =============================================================================

	/**
	 * Handles the element changed event from the ReloadDocument
	 * @param event the ReloadDocumentListenerEvent
	 */
    public void elementChanged(ReloadDocumentListenerEvent event) {
        // Don't listen to events that we have fired
        if(event.getSource() == this) return;

        // If it's us update
        Element element = _elementBinding.getElement();
        if(element != null) {
            Element changed_element = event.getElement();
            if(element == changed_element) setText(element);
        }
    }

	/**
	 * Handles the element removed event from the ReloadDocument
	 * @param event the ReloadDocumentListenerEvent
	 */
    public void elementRemoved(ReloadDocumentListenerEvent event) {}

    /**
     * Handles the element added event from the ReloadDocument
     * @param event the ReloadDocumentListenerEvent
     */
    public void elementAdded(ReloadDocumentListenerEvent event) {}

    /**
     * Handles the document saved event from the ReloadDocument
     * @param doc the ReloadDocument
     */
    public void documentSaved(ReloadSchemaDocument doc) {}


//==============================================================================
// DocumentListener for changes in the Widget
//==============================================================================

	/**
	 * Notify listeners that the reload widget's value is removed
	 */
    public void removeUpdate(DocumentEvent evt) {
        if(allowNotification && _reloadWidget != null) fireElementChanged(_reloadWidget.getTextValue());
    }

	/**
	 * Notify listeners that the reload widget's value is changed
	 */
    public void changedUpdate(DocumentEvent evt) {
        if(allowNotification && _reloadWidget != null) fireElementChanged(_reloadWidget.getTextValue());
    }

	/**
	 * Notify listeners that text is inserted into the reload widget
	 */
    public void insertUpdate(DocumentEvent evt) {
        if(allowNotification && _reloadWidget != null) fireElementChanged(_reloadWidget.getTextValue());
    }


// Tina Manoharan, 2003-11-02


//==============================================================================
//       ACTION HANDLERS
//==============================================================================

    /**
     * An Action Class that allows us to Launch the Metadata Editor
     */
    class Action_LaunchMetadataEditor extends AbstractAction {

        /**
         * The Vocabulary Metadata
         */
        VocabularyMetadata _vocabularyMetadata;

		/**
		 * Constructor
		 * @param vocabularyMetadata the VocabularyMetadata
		 * @see VocabularyMetadata
		 */
        public Action_LaunchMetadataEditor(VocabularyMetadata vocabularyMetadata) {
            super("Edit Metadata...", ReloadUIManager.getIcon(ReloadEditor.ICON_MD));
            _vocabularyMetadata = vocabularyMetadata;
        }

        public void actionPerformed(ActionEvent e) {
            editMetadata(_vocabularyMetadata);
        }
    }

    /**
     * Edit the Metadata in the Metadata dialog Editor
     * @param vocabularyMetadata the vocabularyMetadata
     */
    protected void editMetadata(final VocabularyMetadata vocabularyMetadata) {
        // Loading of Document on Thread
        Thread thread = new Thread() {
            MetadataEditorDialog_Vocabulary mdDialog;
            ReloadProgressMonitor progressMonitor = null;

            public void run() {
                try {
                    ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

                    // Tina Manoharan, 2003-12-15

		            progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(),
		            "Launching Metadata Editor", "Opening File", "Loading external vocabularies, Please wait...", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                    mdDialog = new MetadataEditorDialog_Vocabulary( vocabularyMetadata);
                    // Show the Window later in the Event queue
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            mdDialog.show();
                        }
                    });
                }
                catch(Exception ex) {
                    if(ReloadEditor.DEBUG) ex.printStackTrace();
                    if(mdDialog != null) mdDialog.dispose();
                    ErrorDialogBox.showWarning("Error:", "Could not open Metadata Editor", ex);
                }
                finally {
					if(progressMonitor != null) progressMonitor.close();
                    ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                }
            }
        };

        thread.start();
    }

}
