/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.moonunit.*;

/**
 * The Metadata Class
 *
 * @author Phillip Beauvoir
 * @version 2003-09-26
 */
public class Metadata
extends ReloadSchemaDocument
implements AbstractMetadata
{
    /**
     * Our unique signature
     */
    static final String[] md_comments = {
        "This is a Reload version " + ReloadEditor.VERSION + " Metadata document",
        "Spawned from the Reload Metadata Generator - http://www.reload.ac.uk"
    };

    /**
     * Whether we are a standalone file or attached
     */
    protected boolean _isStandalone;

    /**
     * Constructor for blank MD
     * @param isStandalone whether we are standalone file or attached
     */
    public Metadata(boolean isStandalone) {
        super();
        _isStandalone = isStandalone;
    }

    /**
     * Constructor for a Standalone MD File that exists
     * @param file 				the XML file
     *		  doc 				the JDOM document of the XML file
     *		  schemaController 	the ReloadSchemaController corresponding to the schema of the XML file
     */
    public Metadata(File file, Document doc, MetadataSchemaController schemaController) {
        this(true);
        _file = file;
        _doc = doc;
        _controller = schemaController;
    }

    /**
     * Constructor for a non-standalone MD that has existing MD Element
     * @param mdElement the metadata element
     * @param schemaController the ReloadSchemaController corresponding to the schema of the XML file
     */
    public Metadata(Element mdElement, MetadataSchemaController schemaController) {
        this(false);
        _doc = new Document();
        _doc.setRootElement(mdElement);
        _controller = schemaController;
    }

    /**
     * Setup the JDOM Document with Namespaces and root element
     * @param controller the MetadataSchemaController used for obtaining the ReloadSchema
     */
    public void newDocument(MetadataSchemaController controller) {
        _doc = new Document();

        _controller = controller;

        // Add our signature
        if(_isStandalone) addComments(_doc);

        // Add Root Element according to Schema
        ReloadSchema mdSchema = _controller.getReloadSchema();
        Element root = addElementBySchema(this, null, mdSchema.getRootElement(), true);
        _doc.setRootElement(root);

        // Add Root Declarations
        if(_isStandalone) addRootDeclarations(root);

        // Clear the dirty flag
        setDirty(false);
    }

    /**
     * @return the Root element name
     */
    public String getTargetRootElementName() {
        return MetadataCore.ROOT_NAME;
    }

    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return md_comments;
    }

    /**
     * Add namespace and other attribute decalarations to the root element
     * @param root the root element
     */
    private void addRootDeclarations(Element root) {
        ReloadSchema mdSchema = getReloadSchemaController().getReloadSchema();

        // Add XSI Schema Instance Namespace
        root.addNamespaceDeclaration(IMSDocumentHandler.XSI_Namespace);

        // Add Schema Location Attribute which is constructed from Target Namespace
        // and file name of Schema
        String schemaLocationURI = mdSchema.getTargetNamespace() + " " + mdSchema.getSchemaName();
        root.setAttribute(IMSDocumentHandler.XSI_SchemaLocation, schemaLocationURI, IMSDocumentHandler.XSI_Namespace);
    }

    /**
     * Set the Metadata to an Imported Metadata Document
     * @param doc the imported metadata document
     *   	  controller the MetadataSchemaController
     * @throws ReloadEditorException
     */
    public void importMetadata(Document doc, MetadataSchemaController controller) throws ReloadEditorException {
        // Get the Root Element
        Element root = (Element)doc.getRootElement();

        // Clear any Attributes
        root.setAttributes(null);

        // Add Namespace prefixes to all Elements
        Namespace ns = root.getNamespace();
        if(ns != null) IMSDocumentHandler.replaceNamespaces(root, ns, getSchemaNamespaceEmbedded());

        _doc = doc;
        _controller = controller;
    }

    /**
     * Export the MD from a CP or Vocabulary node to a standalone file
     * @param file the metadata XML file
     * @throws ReloadEditorException
     */
    public void exportMetadata(File file) throws ReloadEditorException {
        if(!isStandalone() && _doc != null) {
            Document newDoc = new Document();
            // Add comments
            addComments(newDoc);

            // Clone
            Element root = (Element)_doc.getRootElement().clone();

            // Remove Namespace prefixes from all Elements
            Namespace ns = root.getNamespace();
            if(ns != null) IMSDocumentHandler.replaceNamespaces(root, ns, Namespace.getNamespace(ns.getURI()));

            // Set to root
            newDoc.setRootElement(root);

            // Add root declarations
            addRootDeclarations(root);

            try {
                XMLUtils.write2XMLFile(newDoc, file);
            }
            catch(DweezilXMLException ex) {
                throw new ReloadEditorException(ex);
            }

            newDoc = null;
        }
    }

    /**
     * Return whether we are a standalone file
     * @return true if we are standalone, false otherwise
     */
    public boolean isStandalone() {
        return _isStandalone;
    }

    /**
     * Get the appropriate Namespace
     * @return the appropriate schema namespace
     */
    public Namespace getSchemaNamespace() {
        if(_isStandalone) return super.getSchemaNamespace();
        else return getSchemaNamespaceEmbedded();
    }

    /**
     * Get the standalone Namespace
     * @return the standalone schema namespace
     */
    public Namespace getSchemaNamespaceStandalone() {
        return super.getSchemaNamespace();
    }

    /**
     * Get the embedded Namespace
     * @return the embedded schema namespace
     */
    public Namespace getSchemaNamespaceEmbedded() {
        return Namespace.getNamespace(IMSDocumentHandler.IMSMD_NAMESPACE_PREFIX, getSchemaNamespaceStandalone().getURI());
    }

// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * Whether we can copy and element or not
     * @param element the element concerned
     * @return true if we can copy the given element, false otherwise
     */
    public boolean canCopyElement(Element element) {
        return !element.isRootElement();
    }

    /**
     * Whether we can cut an element or not
     * @param element the element concerned
     * @param schemaElement the ReloadSchemaElement
     * @return true if we can cut the given element, false otherwise
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return canDeleteElement(element, schemaElement);
    }

    /**
     * Whether we can paste an element or not
     * @param element the element concerned
     * @return true if we can paste the given element, false otherwise
     */
    public boolean canPasteFromClipboard(Element element) {
        Element clipboardElement = ReloadDocumentClipboard.getElement();
        return clipboardElement != null && isAllowedChild(element, clipboardElement);
    }

}