/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.editor.metadata.tabbedformview.*;
import uk.ac.reload.editor.metadata.tableview.*;
import uk.ac.reload.editor.metadata.formview.*;

/**
 * The Metadata Editor
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-10-24
 */
public class MetadataEditor
extends JPanel
{

    /**
     * The Metadata Document
     */
    protected Metadata _metadata;

    /**
     * The Prefs panel
     */
    protected MetadataPrefsPanel _prefsPanel;

    /**
     * The Form View
     */
    // protected MetadataTabbedFormPanel _formPanel;
    protected MetadataFormPanel _formPanel;

    /**
     * The Table View
     */
    protected MetadataTablePanel _tablePanel;

    /**
     * The Edit Menu
     */
    protected Menu_Edit _editMenu;

    /**
     * The Undo Menu Manager for this Window
     */
    protected ReloadUndoMenuManager _undoMenuManager;

    /**
     * Constructor
     * @param editMenu the Edit Menu
     */
    public MetadataEditor(Menu_Edit editMenu) {
        _editMenu = editMenu;

        setLayout(new BorderLayout());

        // Add a Prefs panel
        _prefsPanel = new MetadataPrefsPanel();
        add(_prefsPanel, BorderLayout.NORTH);

        // Get a new Table View
        _tablePanel = new MetadataTablePanel(editMenu);

        // Get a new Form View
        // _formPanel = new MetadataTabbedFormPanel(_tablePanel);
        _formPanel = new MetadataFormPanel();

        // Tab Pane
        JTabbedPane tabPane = new JTabbedPane();
        tabPane.addTab("Form View", null, new JScrollPane(_formPanel), "Form View");
        tabPane.addTab("Tree View", null, _tablePanel, "Tree View");
        add(tabPane, BorderLayout.CENTER);

        // Add MetadataTabbedFormPanel
        // add(_formPanel, BorderLayout.CENTER);
    }

    /**
     * Get the Metadata JDOM Document
     * @return the Metadata JDOM document
     */
    public Metadata getMetadata() {
        return _metadata;
    }

    /**
     * Load a new Metadata JDOM Document
     * @param standalone whether the Metadata document is a standalone file or attached
     *	  	  schemeController the MetadataSchemaController
     * @throws ReloadEditorException
     */
    public void newDocument(boolean standalone, MetadataSchemaController schemaController) throws ReloadEditorException {
        _metadata = new Metadata(standalone);
        _metadata.newDocument(schemaController);
        init(_metadata);
    }

    /**
     * Set a Metadata JDOM Document - this will be standalone
     *
     * @param file the Metadata XML file
     *		  doc the Metadata JDOM document
     *		  schemaController the MetadataSchemaController
     * @throws ReloadEditorException
     */
    public void setDocument(File file, Document doc, MetadataSchemaController schemaController) throws ReloadEditorException {
        _metadata = new Metadata(file, doc, schemaController);
        init(_metadata);
    }

    /**
     * Set the Metadata to a MD Element from a CP
     * @param mdElement the metadata Element from a content package
     *		  schemaController the MetadataSchemaController
     */
    public void setMetadataElement(Element mdElement, MetadataSchemaController schemaController) {
        _metadata = new Metadata(mdElement, schemaController);
        init(_metadata);
    }

    /**
     * Set the Metadata to an Imported MD Doc for a CP
     * @param doc the imported metadata document for a content package
     *		  schemaController the MetadataSchemaController
     */
    public void importMetadata(Document doc, MetadataSchemaController schemaController) throws ReloadEditorException {
        //_metadata = new Metadata(false);
        _metadata.importMetadata(doc, schemaController);
        _metadata.setDirty(true);
        init(_metadata);
    }

    /**
     * Export the Metadata to a File
     * @param file the file to which the metadata is to be exported
     * @throws ReloadEditorException
     */
    public void exportMetadata(File file) throws ReloadEditorException {
        if(_metadata != null) _metadata.exportMetadata(file);
    }

    /**
     * Initialise some stuff
     * @param metadata the metadata JDOM document
     */
    private void init(Metadata metadata) {
        // Tell Prefs Panel
        _prefsPanel.init(metadata);

        _formPanel.setDocument(metadata);
        _tablePanel.setDocument(metadata);

        // New Undo Menu Manager
        if(_undoMenuManager != null) _undoMenuManager.destroy();
        _undoMenuManager = new ReloadUndoMenuManager(_editMenu.actionUndo, _editMenu.actionRedo);
        metadata.setUndoHandler(_undoMenuManager.getUndoHandler());
    }

    /**
     * Over-ride this so we can set view stuff
     */
    public void initView() {
        _tablePanel.initView();
    }

    /**
     * Get the ReloadUndoMenuManager
     * @return the ReloadUndoMenuManager
     */
    public ReloadUndoMenuManager getUndoManager() {
        return _undoMenuManager;
    }

    /**
     * Dynamically load a Metadata Helper Profile
     * @param mdProfile the metadata helper profile to be loaded
     */
    public void loadProfile(MetadataHelperProfile mdProfile) {
        // New Schema Controller for this Profile
        ((MetadataSchemaController)getMetadata().getReloadSchemaController()).setHelperProfile(mdProfile);

        // Tell Panels
        if(_metadata != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    _formPanel.setDocument(_metadata);
                    _tablePanel.refresh();
                }
            });
        }
    }

    /**
     * Dispose of this and clean up
     */
    public void destroy() {
        try {
            if(_undoMenuManager != null) _undoMenuManager.destroy();
            _formPanel.destroy();
            _tablePanel.destroy();
            _metadata.destroy();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }
    }

    /**
     * The Preferences Panel
     */
    class MetadataPrefsPanel extends JPanel {

        /**
         * Profile Combo box
         */
        ReloadComboBox profileCombobox;

        /**
         * Default constructor
         */
        MetadataPrefsPanel() {
            setLayout(new FlowLayout(FlowLayout.LEFT));

            // PROFILE COMBO BOX
            add(new JLabel("Profile: "));
            profileCombobox = new ReloadComboBox(false);
            add(profileCombobox);

            // Populate it
            MetadataHelperProfile[] profiles = MetadataHelperProfile.getMetadataHelperProfiles();
            profileCombobox.setItems(profiles);
        }

        /**
         * Initialise
         * @param metadata the metadata JDOM document
         */
        public void init(Metadata metadata) {
            MetadataHelperProfile mdProfile = (MetadataHelperProfile)((MetadataSchemaController)metadata.getReloadSchemaController()).getReloadHelperProfile();
            profileCombobox.setSelectedItem(mdProfile);

            // Listen to changes on the Profile Combo
            profileCombobox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    MetadataHelperProfile profile = (MetadataHelperProfile)profileCombobox.getSelectedItem();
                    loadProfile(profile);
                }
            });
        }
    }
}