/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;
import uk.ac.reload.dweezil.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.editor.prefs.*;

/**
 * The Metadata Editor Frame
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-09-23
 */
public class MetadataEditorFrame
extends ReloadInternalFrame
implements ApplicationListener, ReloadDocumentListener
{
    /**
     * The Metadata Editor
     */
    protected MetadataEditor _mdEditor;

    /**
     * Handles Save and SaveAs Events
     */
    protected ProxySaveHandler _saveHandler, _saveAsHandler;

    /**
     * Default constructor
     */
    public MetadataEditorFrame() {
        super("Metadata", ReloadEditor.ICON_MD);

        // Ensure we handle closing
        setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);

        // Tell the application that we want to know if it's closing
        ReloadEditor.getInstance().addApplicationListener(this);

        // We listen for what this frame is doing
        addInternalFrameListener(new InternalFrameAdapter() {
            // The Frame is closing
            public void internalFrameClosing(InternalFrameEvent e) {
                // Make sure button stuff is completed
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        checkNeedsSaving();
                    }
                });
            }

            // The Frame is activated
            public void internalFrameActivated(InternalFrameEvent e) {
                setFocusGained();
            }

            // The Frame is deactivated
            public void internalFrameDeactivated(InternalFrameEvent e) {
                setFocusLost();
            }
        });

        // New MetadataEditor Panel
        _mdEditor = new MetadataEditor(MainMenu.sharedInstance.editMenu);
        getContentPane().add(_mdEditor);
    }

    /**
     * Create a new Metadata JDOM Document
     * @param schemaController the MetadataSchemaController
     * @throws ReloadEditorException
     */
    public void newDocument(MetadataSchemaController schemaController) throws ReloadEditorException {
        setTitle("Metadata - untitled");
        _mdEditor.newDocument(true, schemaController);
        init(_mdEditor.getMetadata());
    }

    /**
     * Set a Metadata JDOM Document - this will be standalone
     * @param file the Metadata XML file
     * 		  doc the Metadata JDOM document
     *		  schemaController the MetadataSchemaController
     * @throws ReloadEditorException
     */
    public void setDocument(File file, Document doc, MetadataSchemaController schemaController) throws ReloadEditorException {
        setTitle("Metadata - " + file.getPath());
        // Load
        _mdEditor.setDocument(file, doc, schemaController);
        // Add to file->frame map
        ReloadInternalFrameManager.addReloadInternalFrameToWindowMap(file, this);
        // Init
        init(_mdEditor.getMetadata());
    }

    /**
     * Initialise some stuff
     */
    private void init(Metadata metadata) {
        // Document listener
        metadata.addReloadDocumentListener(this);

        // Proxy Save Handlers
        _saveHandler = new ProxySaveHandler(MainMenu.sharedInstance.actionSave);
        _saveAsHandler = new ProxySaveHandler(MainMenu.sharedInstance.actionSaveAs);
    }

    /**
     * Over-ride this so we can set view stuff
     */
    public void show() {
        super.show();
        _mdEditor.initView();
    }

    // Tina Manoharan 2003-12-18

    public MetadataEditor getMetadataEditor() {

		return _mdEditor;
	}


// =============================================================================
// ReloadDocument Listener Events so we know when to set Save Menu
// =============================================================================

	/**
	 * Handling the element added event from the ReloadDocument
	 */
    public void elementAdded(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_mdEditor.getMetadata().isDirty());
    }

	/**
	 * Handling the element changed event from the ReloadDocument
	 */
    public void elementChanged(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_mdEditor.getMetadata().isDirty());
    }

	/**
	 * Handling the element removed event from the ReloadDocument
	 */
    public void elementRemoved(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_mdEditor.getMetadata().isDirty());
    }

	/**
	 * Handling the document saved event from the ReloadDocument
	 */
    public void documentSaved(ReloadSchemaDocument doc) {
        if(isSelected()) _saveHandler.setEnabled(_mdEditor.getMetadata().isDirty());
    }

// =============================================================================
// Focus Handler
// =============================================================================

    /**
     * We got the focus
     */
    protected void setFocusGained() {
        if(_mdEditor.getMetadata() != null) {
            _mdEditor.getUndoManager().setFocusGained();

            _saveHandler.setEnabled(_mdEditor.getMetadata().isDirty());
            _saveHandler.addListener();

            _saveAsHandler.setEnabled(true);
            _saveAsHandler.addListener();
        }
    }

    /**
     * We lost the focus
     */
    protected void setFocusLost() {
        _mdEditor.getUndoManager().setFocusLost();

        _saveHandler.setEnabled(false);
        _saveHandler.removeListener();

        _saveAsHandler.setEnabled(false);
        _saveAsHandler.removeListener();
    }

// =============================================================================
// Window Closing, Cleanup and Saving Handlers
// =============================================================================

    /**
     * We have been told by the Application that it is closing.
     * Here we can save if need be.
     * @return true if all is OK, false if not.
     */
    public boolean applicationClosing() {
        return checkNeedsSaving();
    }

    /**
     * We can check if the File needs saving
     * @return true if all is OK, false if not
     */
    protected boolean checkNeedsSaving() {
        boolean isOK = true;

        if(_mdEditor.getMetadata() != null && _mdEditor.getMetadata().isDirty()) {
            int doSave = JOptionPane.showConfirmDialog(this,
                "File has been modified. Do you wish to save changes?",
                "Save Metadata File",
                JOptionPane.YES_NO_CANCEL_OPTION);

            if(doSave == JOptionPane.YES_OPTION) isOK = saveDocument();
            else if(doSave == JOptionPane.CANCEL_OPTION) isOK = false;
        }

        if(isOK) dispose();

        return isOK;
    }

    /**
     * Dispose of this Window and clean up
     */
    public void dispose() {
        try {
            ReloadEditor.getInstance().removeApplicationListener(this);

            _saveHandler.removeListener();
            _saveAsHandler.removeListener();

            _mdEditor.getMetadata().removeReloadDocumentListener(this);
            _mdEditor.destroy();

            super.dispose();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }
    }

    /**
     * If the Metadata file has been edited, ask whether we should save it
     * @return true if OK, false if not
     */
    protected boolean saveDocument() {
        // If we don't have a file name, ask for one
        if(_mdEditor.getMetadata().getFile() == null) return saveAsDocument();

        try {
            _mdEditor.getMetadata().saveDocument();
        }
        catch(ReloadDomException ex) {
            ErrorDialogBox.showWarning(this, "Could not save file: ", "Save Metadata", ex);
            return false;
        }
        return true;
    }

    /**
     * Save As...
     * @return true if OK, false if not
     */
    protected boolean saveAsDocument() {
        // Ask for name
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "xml files");
        File file = ReloadUtils.askFileNameSave(this, "Save Metadata", filter, "xml");
        // User cancelled
        if(file == null)  return false;

        // Set Title bar
        setTitle("Metadata - " + file.getPath());

        try {
            _mdEditor.getMetadata().saveAsDocument(file);

            // Put it in the opened list
            ReloadInternalFrameManager.addReloadInternalFrameToWindowMap(file, this);

            // Add to Recent File List
            ReloadEditorPrefs.getInstance().addFileToHistory(file);
        }
        catch(ReloadDomException ex) {
            ErrorDialogBox.showWarning(this, "Could not save file: ", "Save Metadata", ex);
            return false;
        }
        return true;
    }


    /**
     * Deals with Save Menu Events
     */
    class ProxySaveHandler extends ProxyAction {

		/**
		 * Constructor
		 * @param menuAction the Menu Action
		 */
        public ProxySaveHandler(MenuAction menuAction) {
            super(menuAction);
        }

		/**
		 * Invoked when the Save or SaveAs menu action occurs
		 */
        public void menuActionPerformed(ReloadMenuEvent event) {
            if(isSelected() && event.getSource() == MainMenu.sharedInstance.actionSave) saveDocument();
            if(isSelected() && event.getSource() == MainMenu.sharedInstance.actionSaveAs) saveAsDocument();
        }
    }
}