/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.io.*;
import java.util.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.prefs.*;


/**
 * The Metadata Helper Profile
 * This reads in a MD User Profile XML File for helper stuff
 *
 * @author Phillip Beauvoir
 * @version 2003-10-01
 */
public class MetadataHelperProfile
extends ReloadHelperProfile
{
    /**
     * Static Profiles - we only want to create these just once and re-use them
     */
    protected static Hashtable _mdProfilesTable = new Hashtable();

    /**
     * The Root MetadataProfileElement
     */
    protected MetadataProfileElement _rootElement;

    /**
     * Constructor
     * @param fileProfile the file of the MetadataHelperProfile file
     * @throws ReloadEditorException
     */
    public MetadataHelperProfile(File fileProfile) throws ReloadEditorException {
        super(fileProfile);

        // Get a Root Element
        _rootElement = new MetadataProfileElement(_docProfile.getRootElement());
    }

    /**
     * Factory method for getting a static re-usable MetadataProfile
     * @param fileProfile the file of the MetadataProfile file
     * @throws ReloadEditorException
     */
    public static MetadataHelperProfile getMetadataProfile(File fileProfile) throws ReloadEditorException {
        if(fileProfile == null) return null;
        // Do we have it already?
        MetadataHelperProfile mdProfile = (MetadataHelperProfile)_mdProfilesTable.get(fileProfile);
        // No, first time creation, create and store in table
        if(mdProfile == null) {
            // Does it exist?
            if(fileProfile.exists()) {
                mdProfile = new MetadataHelperProfile(fileProfile);
                _mdProfilesTable.put(fileProfile, mdProfile);
            }
        }
        return mdProfile;
    }

    /**
     * Get all available MetadataHelperProfiles
     * @return all available MetadataHelperProfiles
     */
    public static MetadataHelperProfile[] getMetadataHelperProfiles() {
        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        Vector v = new Vector();

        // Get the Files
        File[] files = ReloadEditorDocumentHandler.folderMDProfile.listFiles(filter);
        if(files != null) {
            for(int i = 0; i < files.length; i++) {
                try {
                    MetadataHelperProfile profile = getMetadataProfile(files[i]);
                    v.addElement(profile);
                }
                catch(ReloadEditorException ex) {
                }
            }
        }

        MetadataHelperProfile[] profiles = new MetadataHelperProfile[v.size()];
        v.copyInto(profiles);

        return profiles;
    }

    /**
     * Get the Vocabulary Helper File
     * @return the Vocabulary Helper File
     */
    public File getVocabFile() {
        return new File(ReloadEditorDocumentHandler.folderMDVocab, getVocabularyFileName());
    }

    /**
     * Get the Schema Helper File
     * @return the Schema Helper File
     */
    public File getSchemaHelperFile() {
        return new File(ReloadEditorDocumentHandler.folderMDSchemahelper, getSchemaHelperFileName());
    }

    /**
     * Get the default MetadataHelperProfile
     * @return the default MetadataHelperProfile
     * @throws ReloadEditorException
     */
    public static MetadataHelperProfile getDefaultMetadataHelperProfile() throws ReloadEditorException {
        // Try User Prefs
        String defProfileName = ReloadEditorPrefs.getInstance().getValue(ReloadEditorPrefs.MD_DEFAULT_PROFILE);

        // Nope, so use default
        if(defProfileName == null) return getMetadataProfile(ReloadEditorDocumentHandler.fileDefaultMDProfile);

        // Try this
        File file = new File(ReloadEditorDocumentHandler.folderMDProfile, defProfileName);
        if(file.exists()) return getMetadataProfile(file);

        // Default
        return getMetadataProfile(ReloadEditorDocumentHandler.fileDefaultMDProfile);
    }

    /**
     * Get the root MetadataProfileElement
     * @return the root MetadataProfileElement
     */
    public MetadataProfileElement getRootElement() {
        return _rootElement;
    }

}