/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.metadata.vdexBrowser;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.widgets.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.vocabulary.*;


/**
 * The Metadata Vdex Browser used to display the contents of a Vdex Vocabulary file.
 * The browser is used for the 'classification' tree of the LOM metadata. When a user selects
 * a particular 'term id and caption' on the browser, the values are inserted in the 'taxon id and
 * entry' elements of the LOM metadata respectively.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class MetadataVdexBrowser extends JFrame {

// public class MetadataVdexBrowser extends ReloadInternalFrame {
// Had to change from ReloadInternalFrame to JFrame because it would
// display with the MetadataEditorDialog of the CP.

	/**
	 * The Vdex Vocabulary Panel that displays the vocabulary name, id, default language
	 */
	private VdexVocabPanel vocabPanel;

	/**
	 * The Vdex Term Panel that displays a tree, with term id and caption on its node, and other term information on a JEditorPane
	 */
	private VdexTermPanel termPanel;

	/**
	 * The Vdex Metadata Panel that displays the vocabulary metadata on a JEditorPane
	 */
	private VdexMetadataPanel mdPanel;

	/**
	 * The JTabbedPane that provides two tabs, one for 'Term' other for 'Vocabulary Metadata'
	 */
	private JTabbedPane tabPanel;

	/**
	 * Current tab selected
	 */
	private int selectedTab = 0;

	/**
	 * The Vdex Tree Model for all the terms in the vocabulary
	 */
	private VdexTreeModel treeModel;

	/**
	 * The Vdex Tree that displays term ids and caption values on its node
	 */
	private VdexTree tree;

	/**
	 * The Vdex Vocabulary file
	 */
	public static File _file;


	/**
	 * Constructor
	 * @param source the ReloadWidget that is bound to this browser
	 *		  vdexSourceValue the vocabulary name or id that helps to determine the vdex XML file
	 *		  title the title of the browser frame
	 */
	public MetadataVdexBrowser(ReloadWidget source, String vdexSourceValue, String title) {

		super(title);
		setIconImage(ReloadUIManager.getIcon(ReloadEditor.ICON_VXBROWSE).getImage());

		try {
			_file = this.getVdexFile(vdexSourceValue);
		}
		catch(ReloadEditorException ex) {}

		if(_file != null) {

			ReloadDocument doc = ReloadEditorDocumentHandler.openFile(_file);

			if(doc != null) {

				if(doc instanceof Vocabulary) {

					Vocabulary vocabulary = (Vocabulary) doc;

					// create new vdex tree model
					treeModel = new VdexTreeModel(vocabulary);

					// create new tree
					tree = new VdexTree();

					// Add the treeModel to the tree
	    	    	tree.setModel(treeModel);


					// NORTH
					vocabPanel = new VdexVocabPanel(treeModel.getVdexVocabNode());

					// CENTER
					termPanel = new VdexTermPanel(tree, source, this);
					tree.setTermPanel(termPanel);
					mdPanel = new VdexMetadataPanel(treeModel.getVdexVocabNode());

					tabPanel = new JTabbedPane();
					tabPanel.addTab(" Term ", null, termPanel, " Term ");
					tabPanel.addTab(" Metadata ", null, mdPanel, " Vocab Metadata ");
					tabPanel.addChangeListener(new tabListener());

					// ADD
					getContentPane().add(vocabPanel, BorderLayout.NORTH);
					getContentPane().add(tabPanel, BorderLayout.CENTER);


			        // Dialog Size
			        setSize(ReloadEditor.getInstance().getWidth() * 3 / 4, ReloadEditor.getInstance().getHeight() * 3 / 4);
			        setLocationRelativeTo(ReloadEditor.getInstance());

				}
			}
		}
		else {

	        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      "The choosen source is not a Vdex vocabulary",
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);


			MetadataVdexBrowser.this.dispose();
		}


        // Add a listener to Close our window down on exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });
	}


	/**
	 * Get the Vdex XML file's using the vocabulary name or id
	 * @param vdexSourceValue the vdex vocabulary name or id
	 * @return the vdex file if found, null otherwise
	 */
	public File getVdexFile(String vdexSourceValue) throws ReloadEditorException {

		DomPath domPath;

		boolean found = false;

        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

		// Get the Files
		File[] files = ReloadEditorDocumentHandler.folderMDVdexVocab.listFiles(filter);

        for(int i = 0; i < files.length; i++) {

			domPath = new DomPath(VocabularyCore.VOCABULARY_IDENTIFIER_PATH);

			try {

				found = XMLUtils.getElementByNameValuePair(VocabularyCore.ROOT_NAME, domPath, files[i], vdexSourceValue);
			}
			catch(DweezilXMLException ex) {
				throw new ReloadEditorException(ex);
			}

			if(!found) {

				domPath = new DomPath(VocabularyCore.VOCABULARY_NAME_PATH);

				try {

					found = XMLUtils.getElementByNameValuePair(VocabularyCore.ROOT_NAME, domPath, files[i], vdexSourceValue);
				}
				catch(DweezilXMLException ex) {
					throw new ReloadEditorException(ex);
				}
			}

			if(found) return files[i];
        }

		return null;
	}

	/**
	 * Listener class that listeners to the tab select action
	 */
	private class tabListener implements ChangeListener {

		/**
		 * Action to be performed when a tab is selected (term tab or vocabulary metadata tab)
		 */
		public void stateChanged(ChangeEvent ce) {

			selectedTab = tabPanel.getSelectedIndex();

			tabPanel.setIconAt(selectedTab,null);
		}
	}
}