/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.editor.vocabulary.formview.*;
import uk.ac.reload.editor.vocabulary.tableview.*;


/**
 * The Vocabulary Editor
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
public class VocabularyEditor
extends JPanel
{

    /**
     * The Vocabulary Document
     */
    protected Vocabulary _vocabulary;

    /**
     * The Prefs panel
     */
    protected VocabularyPrefsPanel _prefsPanel;

    /**
     * The Form View
     */
    protected VocabularyFormPanel _formPanel;

    /**
     * The Table View
     */
    protected VocabularyTablePanel _tablePanel;


    /**
     * The Edit Menu
     */
    protected Menu_Edit _editMenu;

    /**
     * The Undo Menu Manager for this Window
     */
    protected ReloadUndoMenuManager _undoMenuManager;



    /**
     * Constructor
     * @param editMenu the Edit Menu
     */
    public VocabularyEditor(Menu_Edit editMenu) {
        _editMenu = editMenu;

        setLayout(new BorderLayout());

        // Add a Prefs panel
        _prefsPanel = new VocabularyPrefsPanel();
        add(_prefsPanel, BorderLayout.NORTH);

        // Get a new Table View
        _tablePanel = new VocabularyTablePanel(editMenu);

        // Get a new Form View
        _formPanel = new VocabularyFormPanel();

        // Tab Pane
        JTabbedPane tabPane = new JTabbedPane();
        tabPane.addTab("Form View", null, new JScrollPane(_formPanel), "Form View");
        tabPane.addTab("Tree View", null, _tablePanel, "Tree View");
        add(tabPane, BorderLayout.CENTER);
    }

    /**
     * Get the Vocabulary JDOM Document
     * @return the Vocabulary JDOM document
     */
    public Vocabulary getVocabulary() {
        return _vocabulary;
    }

	/**
	 * Set the Vocabulary Profile Type.
	 * Vdex profile type names - lax, heirarchicalTokenTerms, flatTokenTerms, thesaurus, glossaryOrDictionary.
	 * @param profileType the name of the profile type
	 */
    public void setProfileType(String profileType) {

		_prefsPanel.setProfileType(profileType);
	}

	/**
	 * Get the Vocabulary profile type name
	 * @return the vocabulary profile type name
	 */
	public String getProfileType() {

		return _prefsPanel.getProfileType();
	}

    /**
     * Load a new Vocabulary JDOM Document
     * @param standalone whether the Vocabulary document is a standalone file or attached
     *	  	  schemeController the VocabularySchemaController
     * @throws ReloadEditorException
     */
    public void newDocument(boolean standalone, VocabularySchemaController schemaController) throws ReloadEditorException {

        _vocabulary = new Vocabulary(standalone);
        _vocabulary.newDocument(schemaController);

        init(_vocabulary);
    }

    /**
     * Set a Vocabulary JDOM Document - this will be standalone
     *
     * @param file the Vocabulary XML file
     *		  doc the Vocabulary JDOM document
     *		  schemaController the VocabularySchemaController
     * @throws ReloadEditorException
     */
    public void setDocument(File file, Document doc, VocabularySchemaController schemaController) throws ReloadEditorException {
        _vocabulary = new Vocabulary(file, doc, schemaController);
        init(_vocabulary);
    }


    /**
     * Initialise some stuff
     * @param vocabulary the Vocabulary JDOM document
     */
    private void init(Vocabulary vocabulary) {

        // Tell Prefs Panel
        _prefsPanel.init(vocabulary);

        _formPanel.setDocument(vocabulary);
        _tablePanel.setDocument(vocabulary);

        // New Undo Menu Manager
        if(_undoMenuManager != null) _undoMenuManager.destroy();
        _undoMenuManager = new ReloadUndoMenuManager(_editMenu.actionUndo, _editMenu.actionRedo);
        vocabulary.setUndoHandler(_undoMenuManager.getUndoHandler());
    }


    /**
     * Over-ride this so we can set view stuff
     */
    public void initView() {
        _tablePanel.initView();
    }

    /**
     * Get the ReloadUndoMenuManager
     * @return the ReloadUndoMenuManager
     */
    public ReloadUndoMenuManager getUndoManager() {
        return _undoMenuManager;
    }

    /**
     * Dynamically load a Vocabulary Schema
     * @param vxProfile the Vocabulary helper profile used for getting the vocabulary schema
     */
    public void loadSchema(VocabularyHelperProfile vxProfile) {
        // Tell Schema Controller
        VocabularySchemaController vocabSchemaController = ((VocabularySchemaController)getVocabulary().getReloadSchemaController()).setVocabularySchema(vxProfile);

        _vocabulary.updateDocument(vocabSchemaController);
    }


    /**
     * Dynamically load a Vocabulary Helper Profile
     * @param vxProfile the vocabulary helper profile to be loaded
     */
    public void loadProfile(VocabularyHelperProfile vxProfile) {
        // Tell Schema Controller
        ((VocabularySchemaController)getVocabulary().getReloadSchemaController()).setHelperProfile(vxProfile);

        // Tell Panels
        if(_vocabulary != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {

                    _formPanel.setDocument(_vocabulary);

                    _tablePanel.setDocument(_vocabulary);

					// Repaint the table panel when the profile type is changed
                    _tablePanel.refresh();
                }
            });
        }
    }

    /**
     * Dispose of this and clean up
     */
    public void destroy() {
        try {
            if(_undoMenuManager != null) _undoMenuManager.destroy();
            _formPanel.destroy();
            _tablePanel.destroy();
            _vocabulary.destroy();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
        }
    }

    /**
     * The Preferences Panel
     */
    class VocabularyPrefsPanel extends JPanel {

        /**
         * Profile Combo box
         */
        ReloadComboBox profileCombobox;

        String profileType;

        /**
         * Default constructor
         */
        VocabularyPrefsPanel() {
            setLayout(new FlowLayout(FlowLayout.LEFT));

            // PROFILE COMBO BOX
            add(new JLabel("Profile: "));
            profileCombobox = new ReloadComboBox(false);
            add(profileCombobox);

            // Populate it
            VocabularyHelperProfile[] profiles = VocabularyHelperProfile.getVocabularyHelperProfiles();
            profileCombobox.setItems(profiles);
        }

        /**
         * Initialise
         * @param vocabulary the vocabulary JDOM document
         */
        public void init(Vocabulary vocabulary) {

            VocabularyHelperProfile vxProfile = ((VocabularySchemaController)vocabulary.getReloadSchemaController()).getVocabularyHelperProfile();

            profileCombobox.setSelectedItem(vxProfile);
            profileType = vxProfile.getProfileName();

            // Listen to changes on the Profile Combo
            profileCombobox.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {

               	    VocabularyHelperProfile profile = (VocabularyHelperProfile)profileCombobox.getSelectedItem();

					if(!profile.getProfileName().equals(profileType)) {

			        	JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                    	                  "Changing the profile type will delete or add elements according to what is permitted and required for that profile type",
                    	                  "Reload Editor",
                    	                  JOptionPane.WARNING_MESSAGE);

                	    profileType = profile.getProfileName();

                	    loadSchema(profile);
                	    loadProfile(profile);
					}
                }
            });
        }

        /**
         * Set the vocabulary profile type.
         * Vdex profile type names - lax, heirarchicalTokenTerms, flatTokenTerms, thesaurus, glossaryOrDictionary.
         * @param pType the vocabulary profile type name
         */
		public void setProfileType(String pType) {

			for(int i = 0; i < profileCombobox.getItemCount(); i++) {

				VocabularyHelperProfile vp = (VocabularyHelperProfile) profileCombobox.getItemAt(i);

				if(vp.getProfileName().equals(pType)) {

					profileCombobox.setSelectedItem(vp);

					break;
				}
			}
		}


        /**
         * get profile type
         * @return the vocabulary profile type name
         */
		public String getProfileType() {

			return profileType;
		}

    }

}