/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary;

import java.io.*;
import java.util.*;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.prefs.*;


/**
 * The Vocabulary Profile
 * This reads in a VX User Profile XML File
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
public class VocabularyHelperProfile
extends ReloadHelperProfile
{

    /**
     * Static Profiles - we only want to create these just once and re-use them
     */
     private static Hashtable _vxProfilesTable = new Hashtable();


    /**
     * The Root VocabularyProfileElement
     */
    protected VocabularyProfileElement _rootElement;


    /**
     * Constructor
     * @param fileProfile the file of the VocabularyHelperProfile file
     * @throws ReloadEditorException
     */
    private VocabularyHelperProfile(File fileProfile) throws ReloadEditorException {
        super(fileProfile);

        // Get a Root Element
        _rootElement = new VocabularyProfileElement(_docProfile.getRootElement());
    }

    /**
     * Factory method for getting a static re-usable VocabularyHelperProfile
     * @param fileProfile the file of the VocabularyHelperProfile file
     * @throws ReloadEditorException
     */
    public static VocabularyHelperProfile getVocabularyProfile(File fileProfile) throws ReloadEditorException {
        if(fileProfile == null) return null;
        // Do we have it already?
        VocabularyHelperProfile vxProfile = (VocabularyHelperProfile)_vxProfilesTable.get(fileProfile);
        // No, first time creation, create and store in table
        if(vxProfile == null) {
            // Does it exist?
            if(fileProfile.exists()) {
                vxProfile = new VocabularyHelperProfile(fileProfile);
                _vxProfilesTable.put(fileProfile, vxProfile);
            }
        }
        return vxProfile;
    }

    /**
     * Get all available VocabularyHelperProfiles
     * @return all available VocabularyHelperProfiles
     */
    public static VocabularyHelperProfile[] getVocabularyHelperProfiles() {
        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        Vector v = new Vector();

        // Get the Files
        File[] files = ReloadEditorDocumentHandler.folderVXProfile.listFiles(filter);
        for(int i = 0; i < files.length; i++) {
            try {

                VocabularyHelperProfile profile = getVocabularyProfile(files[i]);
                v.addElement(profile);
            }
            catch(ReloadEditorException ex) {
            }
        }

        VocabularyHelperProfile[] profiles = new VocabularyHelperProfile[v.size()];
        v.copyInto(profiles);

        return profiles;
    }


    /**
     * Get the Vocabulary Helper File
     * @return the Vocabulary Helper File
     */
    public File getVocabFile() {
        return new File(ReloadEditorDocumentHandler.folderVXVocab, getVocabularyFileName());
    }

    /**
     * Get the Schema Helper File
     * @return the Schema Helper File
     */
    public File getSchemaHelperFile() {
        return new File(ReloadEditorDocumentHandler.folderVXSchemahelper, getSchemaHelperFileName());
    }

    /**
     * Get the default VocabularyHelperProfile
     * @return the default VocabularyHelperProfile
     */
    public static VocabularyHelperProfile getDefaultVocabularyHelperProfile() throws ReloadEditorException {
        // Try User Prefs
        String defProfileName = ReloadEditorPrefs.getInstance().getValue(ReloadEditorPrefs.VX_DEFAULT_PROFILE);

        // Nope, so use default
        if(defProfileName == null) return getVocabularyProfile(ReloadEditorDocumentHandler.fileDefaultVXProfile);

        // Try this
        File file = new File(ReloadEditorDocumentHandler.folderVXProfile, defProfileName);
        if(file.exists()) return getVocabularyProfile(file);

        // Default
        return getVocabularyProfile(ReloadEditorDocumentHandler.fileDefaultVXProfile);
    }


	// Tina Manoharan, 2003-12-05

	// This function can be added to the ReloadHelperProfile or it can remain
	// here if it is required only for the VocabularyHelperProfile

    /**
     * Get the file name of the associated Schema file.  This is stored
     * in the XML Profile file against the attribute "schemafile"
     * @return The name of the Schema file
     */
    public String getSchemaFileName() {
        return getRootAttributeValue("schemafile");
    }

    /**
     * Get the VocabularyProfileField root field
     * @return the root VocabularyProfileElement
     */
    public VocabularyProfileElement getRootElement() {
        return _rootElement;
    }

}