/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary;

import java.io.*;
import java.util.*;
import javax.swing.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.vocabulary.*;
import uk.ac.reload.moonunit.schema.*;

 /**
  * The Vocabulary Schema Controller - this interfaces from the various helpers
  *
  * @author Tina Manoharan
  * @version 2003-08-11
 */
public class VocabularySchemaController
extends ReloadEditorSchemaController
{

    /**
     * Constructor with Default Profile
     * @param fileSchema the schema file
     * 		  version the schema version
     */
    public VocabularySchemaController(File fileSchema, String version) {
        try {
            setVersion(version);

            loadSchema(fileSchema);

            // Get the Default Profile
            VocabularyHelperProfile defProfile = VocabularyHelperProfile.getDefaultVocabularyHelperProfile();
            setHelperProfile(defProfile);
        }
        catch(ReloadEditorException ex) {
            ErrorDialogBox.showWarning("Could not load Default Vocabulary Profile", "Load Profile", ex);
        }
    }


    /**
     * Constructor
     * @param vxProfile the vocabulary helper profile
     */
    public VocabularySchemaController(VocabularyHelperProfile vxProfile) {
        setHelperProfile(vxProfile);
    }


    /**
     * Get the default VocabularySchemaController
     * @return the default VocabularySchemaController
     */
    public static VocabularySchemaController getDefaultSchemaController() {
        return (VocabularySchemaController)ReloadEditorSchemaController.getReloadSchemaController(ReloadEditorDocumentHandler.IMSVX_LAX_100);
    }


    /**
     * Set the Vocabulary Schema and return the approrpiate VocabularySchemaController
     * @param vxProfile the Vocabulary helper profile that is used for getting the appropriate vocabulary schema
     * @return the VocabualarySchemaController
     */
    public VocabularySchemaController setVocabularySchema(VocabularyHelperProfile vxProfile) {

		setHelperProfile(vxProfile);

        // Set Schema
        File schemaFile = new File(new File(ReloadEditorDocumentHandler.SCHEMAMODELFOLDER, IMSDocumentHandler.IMSVX_100), vxProfile.getSchemaFileName());

        loadSchema(schemaFile);

		return VocabularySchemaController.this;
    }

    /**
     * Get the Reload Vocabulary Helper Profile
     * @return the vocabulary helper profile
     */
    public VocabularyHelperProfile getVocabularyHelperProfile() {

		try {

            setHelperProfile(VocabularyHelperProfile.getVocabularyProfile(ReloadEditorDocumentHandler.getVXSchemaProfileFile(_schemaFile.getName())));
		}
		catch(ReloadEditorException ex) {
			ErrorDialogBox.showWarning("Could not load Vocabulary Helper Profile", "Load Profile", ex);
        }

        return (VocabularyHelperProfile)_helperProfile;
    }


    /**
     * Return the Vocabulary Root Element name
     * @return the root element name
     */
    public String getRootElementName() {
        return VocabularyCore.ROOT_NAME;
    }


    /**
     * Get the top level VDEX Element Categories (4 in this case)
     * @return An Enumeration of ReloadSchemaElement objects
     */
    public ReloadSchemaElement[] getTopLevelElements() {
        return _reloadSchema.getRootElement().getChildren();
    }

    /**
     * Find out if the given Element is a top level one below "vdex"
     * @param schemaElement The Schema Element to query
     * @return true if the parent of schemaElement is the root Element
     */
    public boolean isTopLevelElement(ReloadSchemaElement schemaElement) {
        return schemaElement.getParent() == _reloadSchema.getRootElement();
    }

    /**
     * Get The Leaf Icon (for the Full View Tree)
     * @param element the leaf element
     * @return the leaf icon
     */
    public Icon getLeafIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(VocabularyCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_VXLEAF);
    }

    /**
     * Get The Leaf Icon (for the Full View Tree)
     * @param schemaElement the leaf schema element
     * @return the leaf icon
     */
    public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(VocabularyCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_VXLEAF);
    }

    /**
     * Get The Open Icon (for the Full View Tree)
     * @param element the open element
     * @return the open icon
     */
    public Icon getOpenIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(VocabularyCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_VXFOLDER);
    }

    /**
     * Get The Open Icon (for the Full View Tree)
     * @param schemaElement the open schema element
     * @return the open icon
     */
    public Icon getOpenIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(VocabularyCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_VXFOLDER);
    }

    /**
     * Get The Closed Icon (for the Full View Tree)
     * @param element the closed element
     * @return the close icon
     */
    public Icon getClosedIcon(Element element) {
        return getOpenIcon(element);
    }

    /**
     * Get The Closed Icon (for the Full View Tree)
     * @param schemaElement the closed schemaElement
     * @return the close icon
     */
    public Icon getClosedIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }
}