/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.ldeditor;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.dweezil.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.ldeditor.menu.*;
import uk.ac.reload.ldeditor.prefs.*;


/**
 * The Learning Design Editor Frame
 *
 * @author Phillip Beauvoir
 * @version 2003-10-15
 */
public class LDEditor
extends JFrame
implements InternalFrameListener
{
    /**
     * Set this to false for a Release Build, keep true for debugging
     */
    public static boolean DEBUG = true;

    // Information Constants
    public static String APP_NAME =           "Reload Learning Design Editor";
    public static String VERSION =            "0.1";
    public static String BUILD_DATE =         "15 October 2003";
    public static String COPYRIGHT =          "Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir";
    public static String AUTHORS =            "Authors - Phillip Beauvoir & Paul Sharples";
    public static String PROJMANAGER =        "Project Management - Oleg Liber";
    public static String BILL =               "Adviser on Specifications and Architecture - Bill Olivier";
    public static String DOCDISS =            "Documentation and Dissemination - Colin Milligan";
    public static String CONTRIBUTORS =       "Advisers & Contributors - Fred Bloggs, " +
                                              "Nigel Bloggs, " +
                                              "Wilbert Kraan, " +
                                              "Sharon Perry, " +
                                              "Scott Wilson";
    public static String EMAIL_CONTACT =      "p.beauvoir@bolton.ac.uk";
    public static String WEB_PAGE =           "http://www.reload.ac.uk";

    // ========================================================================
    // ============================== ICONS ===================================
    // ========================================================================
    public static String ICON_PATH = "/resources/images/";

    public static String ICON_APP16 = ICON_PATH + "app16.gif";
    public static String ICON_APP32 = ICON_PATH + "app32.gif";
    public static String ICON_COPY = ICON_PATH + "copy.gif";
    public static String ICON_CUT = ICON_PATH + "cut.gif";
    public static String ICON_DELETE = ICON_PATH + "delete.gif";
    public static String ICON_DOWN = ICON_PATH + "down.gif";
    public static String ICON_FILE = ICON_PATH + "file.gif";
    public static String ICON_LEFT = ICON_PATH + "left.gif";
    public static String ICON_NEW = ICON_PATH + "new.gif";
    public static String ICON_OPEN = ICON_PATH + "open.gif";
    public static String ICON_PASTE = ICON_PATH + "paste.gif";
    public static String ICON_REDO = ICON_PATH + "redo.gif";
    public static String ICON_REFRESH = ICON_PATH + "refresh.gif";
    public static String ICON_SAVE = ICON_PATH + "save.gif";
    public static String ICON_UNDO = ICON_PATH + "undo.gif";
    public static String ICON_UP = ICON_PATH + "up.gif";

    public static String IMAGE_SPLASH = ICON_PATH + "splash.gif";
    public static String IMAGE_EASTER_EGG = ICON_PATH + "easter_egg.jpg";

    /**
     * The Status Window
     */
    private ReloadStatusWindow _statusWindow;

    /**
     * The ReloadInternalFrameManager
     */
    private ReloadInternalFrameManager _internalFrameManager;

    /**
     * The singleton instance of the app
     */
    private static LDEditor _instance;

    /**
     * The desktop pane.
     */
    private JDesktopPane _desktop;


    /**
     * Constructor for new Reload Editor
     */
    public LDEditor() {
        _instance = this;

        // UI Defaults
        ReloadUIManager.setMacUIDefaults(APP_NAME);

        // SplashScreen
        SplashScreen splash = new SplashScreen(ReloadUIManager.getIcon(IMAGE_SPLASH));

        // Get the Status Window going
        _statusWindow = new ReloadStatusWindow(APP_NAME, VERSION, BUILD_DATE, ReloadUIManager.getIcon(ICON_APP32), !DEBUG);

        // Set Look And Feel from User Prefs
        String lf = LDEditorPrefs.getInstance().getValue(LDEditorPrefs.GENERAL_PREFS_LF);
        if(lf == null) lf = UIManager.getSystemLookAndFeelClassName();
        ReloadUIManager.setLookAndFeel(lf, new Component[] {getInstance(), getStatusWindow()});

        // Set default folder from User Prefs
        String defFolder = LDEditorPrefs.getInstance().getValue(LDEditorPrefs.GENERAL_PREFS_DEFAULT_FOLDER);
        if(defFolder != null) ReloadFileChooser.setDefaultFolder(new File(defFolder));

        // Set application title in title bar
        super.setTitle(APP_NAME);

        // Set application Icon
        ImageIcon icon = ReloadUIManager.getIcon(ICON_APP32);
        if(icon != null) setIconImage(icon.getImage());

        // Trap window closing event for our Window listener
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // Add a listener to Close our window down on exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	closeApp();
            }
        });

        // Set up desktop and internal frame manager
        JPanel panel = new JPanel(new BorderLayout());
        _desktop = new JDesktopPane();
        panel.add(_desktop, BorderLayout.CENTER);
        setContentPane(panel);

        _internalFrameManager = new ReloadInternalFrameManager(_desktop);

        // Create Menu Bar (after setting up the desktop, please!)
        setJMenuBar(MainMenu.sharedInstance);

        // Create Toolbar
        getContentPane().add(MainMenu.sharedInstance.getReloadToolBar(), BorderLayout.NORTH);

        splash.close();

        // Show the main Frame Window, centred
        ReloadUtils.centreWindowProportional(this, 0.9, 0.9);
        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Main program entry point
     * @param args Command line arguments
     */
    public static void main(String[] args) {
        new LDEditor();
    }

    /**
     * Gets the Singleton Instance of the Editor
     * @return The Singleton Instance of the Editor
     */
    public static LDEditor getInstance() {
        return _instance;
    }

    /**
     * @return The Status Window
     */
    public ReloadStatusWindow getStatusWindow() {
        return _statusWindow;
    }

    /**
     * @return The ReloadInternalFrameManager
     */
    public ReloadInternalFrameManager getInternalFrameManager() {
        return _internalFrameManager;
    }

    /**
     * Get the Instance of the Help File
     * @return the Help File
     */
    public File getHelpFile() {
        return new File(System.getProperty("user.dir"), "/help/index.html");
    }

    /**
     * Quit the application
     * Put any cleanup code here.
     */
    public void closeApp() {
    	// Tell any registered listeners
        boolean result = fireApplicationClosing();
        if(result) {
            // Save some Prefs
            String lf = UIManager.getLookAndFeel().getClass().getName();
            LDEditorPrefs.getInstance().putValue(LDEditorPrefs.GENERAL_PREFS_LF, lf);

            try {
                LDEditorPrefs.getInstance().save();
            }
            catch(DweezilXMLException ex) {
                ErrorDialogBox.showWarning(this, "Could not save Preferences: ", "Preferences", ex);
            }

            // Quit
            System.exit(0);
        }
    }

    /**
     * Set the Title Bar String
     * @param title The Title Bar String
     */
    public void setTitle(String title) {
    	super.setTitle(APP_NAME + " - " + title);
    }


    ////////////////////////////////////////////////////////////////////////////
    //   INTERNAL FRAME MANAGEMENT   ///////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    public JDesktopPane getDesktop() {
        return _desktop;
    }

    /**
     * Add a new ReloadInternalFrame
     * @param frame The new Frame
     */
    public void addInternalFrame(ReloadInternalFrame frame) {
        // Add us as a frame listener
        frame.addInternalFrameListener(this);
        // Add to Internal Frame Manager
        _internalFrameManager.addInternalFrame(frame);
        // Add to main menu
        MainMenu.sharedInstance.addInternalFrame(frame);
    }

    /**
     * Remove a new ReloadInternalFrame
     * @param frame The Frame
     */
    public void removeInternalFrame(ReloadInternalFrame frame) {
        // Remove us as a frame listener
        frame.removeInternalFrameListener(this);
        // Remove from Internal Frame Manager
        _internalFrameManager.removeInternalFrame(frame);
        // Remove from main menu
        MainMenu.sharedInstance.removeInternalFrame(frame);
    }

    ////////////////////////////////////////////////////////////////////////////
    // Listeners to InternalFrameListener
    ////////////////////////////////////////////////////////////////////////////

    public void internalFrameOpened(InternalFrameEvent e) {
    }

    public void internalFrameClosing(InternalFrameEvent e) {
    }

    /**
     * The Internal frame is closed so we need to remove it.
     * @param e
     */
    public void internalFrameClosed(InternalFrameEvent e) {
        Object o = e.getSource();
        if(o instanceof ReloadInternalFrame) removeInternalFrame((ReloadInternalFrame)o);
    }

    public void internalFrameIconified(InternalFrameEvent e) {
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {
    }

    public void internalFrameActivated(InternalFrameEvent e) {
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
    }

    ////////////////////////////////////////////////////////////////////////////
    //   APPLICATION LISTENERS  ////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    /**
     * List of Listeners
     */
    private Vector listeners = new Vector();

    /**
     * Add an Application Listener
     * @param listener The Application Listener
     */
    public synchronized void addApplicationListener(ApplicationListener listener) {
        if(!listeners.contains(listener)) listeners.addElement(listener);
    }

    /**
     * Remove an Application Listener
     * @param listener The Application Listener
     */
    public synchronized void removeApplicationListener(ApplicationListener listener) {
        listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we are closing the App
     * They must reply by answering with true for OK, or false for not OK
     */
    public boolean fireApplicationClosing() {
        boolean ok = true;
        // Go from last to first in case anyone removes themselves
        for(int i = listeners.size() - 1; i >= 0; i--) {
            ApplicationListener listener = (ApplicationListener)listeners.elementAt(i);
            boolean result = listener.applicationClosing();
            if(result == false) ok = false;
        }
        return ok;
    }

}