/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.ldeditor.menu;

import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.ldeditor.*;
import uk.ac.reload.ldeditor.prefs.*;
import uk.ac.reload.moonunit.*;

/**
 * The Main Menu for the Application.  It extends JMenuBar.  It also takes care
 * of the Toolbar and any main Popup menus.  It can listen to any refresh requests
 * so that for example, a change will set enabled and disabled menu items and buttons.
 * This class is tied in with the MainMenuAction and MenuAction classes.
 *
 * @author Phillip Beauvoir
 * @version 2003-10-15
 */
public class MainMenu
extends JMenuBar
{
    /**
     * The Singleton instance
     */
    public static MainMenu sharedInstance = new MainMenu();

    /**
     * The main application toolbar
     */
    private ReloadToolBar toolBar;

    /**
     * The Key Mask will be 'Meta' for Mac and 'Ctrl' for PC/Unix
     */
    private int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

    /**
     * Constructor
     */
    private MainMenu() {
        // Attach Actions to Menu Items
        createMenuBar();

        // Create the ToolBar
        toolBar = createToolBar();
    }

    /**
     * Add an Internal Frame Window to the "Window" Menu Item.
     * We get the Menu Item from the Frame itself.
     * @param frame The frame to add to the menu
     */
    public void addInternalFrame(ReloadInternalFrame frame) {
        windowMenu.add(frame.getMenuItem());
    }

    /**
     * Remove an Internal Frame Window from the "Window" Menu Item.
     * We get the Menu Item from the Frame itself.
     * @param frame The frame to remove from the menu
     */
    public void removeInternalFrame(ReloadInternalFrame frame) {
        windowMenu.remove(frame.getMenuItem());
    }

    // =========================== MENU BAR ==================================
    /**
     * The File Menu
     */
    public JMenu fileMenu;

    /**
     * The Debug Menu
     */
    public JMenu debugMenu;

    /**
     * The Options Menu
     */
    public JMenu optionsMenu;

    /**
     * The Window Menu
     */
    public JMenu windowMenu;

    /**
     * The Help Menu
     */
    public JMenu helpMenu;

    /**
     * Create the Menu Bar
     */
    private void createMenuBar() {
        // File Menu
        fileMenu = createFileMenu();
        // Debug menu
        if(LDEditor.DEBUG) debugMenu = createDebugMenu();
        // Options menu
        optionsMenu = createOptionsMenu();
        // Window Menu
        windowMenu = createWindowMenu();
        // Help Menu
        helpMenu = createHelpMenu();
    }

    /**
     * Create the File Menu
     * @return The JMenu
     */
    private JMenu createFileMenu() {
        JMenu jmenu = add(new JMenu("File"));
        jmenu.setMnemonic('f');

        //jmenu.addSeparator();

        // Recent File List
        final JMenu recentMenu = new JMenu("Recent Files");
        recentMenu.setMnemonic('r');
        jmenu.add(recentMenu);

        // Listen to this menu selection so we can populate it
        recentMenu.addMenuListener(new MenuListener() {
            public void menuSelected(MenuEvent e) {
                recentMenu.removeAll();
                String[] s = LDEditorPrefs.getInstance().getFileHistory();
                for(int i = 0; i < s.length; i++) {
                    File file = new File(s[i]);
                    recentMenu.add(new MenuAction_RecentOpen(i + 1, file));
                }
            }

            public void menuDeselected(MenuEvent e) {}

            public void menuCanceled(MenuEvent e) {}
        });

        jmenu.addSeparator();

        // Exit
        JMenuItem item = jmenu.add(new MenuAction_Exit());
        item.setMnemonic('x');

        return jmenu;
    }

    /**
     * Create the Debug Menu
     * @return The JMenu
     */
    private JMenu createDebugMenu() {
        JMenu jmenu = add(new JMenu("Debug"));
        jmenu.setMnemonic('d');

        // LD Schemas
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(IMSDocumentHandler.IMSLD_100A, "Learning Design A")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(IMSDocumentHandler.IMSLD_100B, "Learning Design B")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(IMSDocumentHandler.IMSLD_100C, "Learning Design C")));

        return jmenu;
    }

    /**
     * Create the Options Menu
     * @return The JMenu
     */
    private JMenu createOptionsMenu() {
        JMenu jmenu = add(new JMenu("Options"));
        jmenu.setMnemonic('o');

        JMenuItem item = jmenu.add(new MenuAction_Prefs());

        // Look & Feels
        JMenu subMenu = new JMenu("Look & Feel");
        addLookAndFeels(subMenu);
        jmenu.add(subMenu);

        return jmenu;
    }

    /**
     * Add installed Look & Feels to the menu
     * @param menu The menu to add the L&Fs to.
     */
    protected void addLookAndFeels(JMenu menu) {
        // Add some Menu Items for each installed L&F
        ButtonGroup bg = new ButtonGroup();

        // Get the look and feels
        UIManager.LookAndFeelInfo[] lfs = UIManager.getInstalledLookAndFeels();

        for(int i = 0; i < lfs.length; i++) {
            UIManager.LookAndFeelInfo lfInfo = lfs[i];

            // Add menu item
            JCheckBoxMenuItem jcb = new JCheckBoxMenuItem(new MenuAction_LookAndFeel(lfInfo));
            JMenuItem item = menu.add(jcb);
            bg.add(item);

            // Select currently set L&F as ticked
            if(UIManager.getLookAndFeel().getName().equals(lfInfo.getName())) jcb.setSelected(true);
        }
    }

    /**
     * Create the Window Menu
     * @return The JMenu
     */
    private JMenu createWindowMenu() {
        JMenu jmenu = add(new JMenu("Window"));
        jmenu.setMnemonic('w');

        // Cascade
        JMenuItem item = jmenu.add(new MenuAction_CascadeWindow());
        jmenu.setMnemonic('c');
        // Tile H
        item = jmenu.add(new MenuAction_TileHorizontalWindow());
        jmenu.setMnemonic('t');
        // Tile V
        item = jmenu.add(new MenuAction_TileVerticalWindow());
        jmenu.setMnemonic('v');

        jmenu.addSeparator();

        return jmenu;
    }

    /**
     * Create the Help Menu
     * @return The JMenu
     */
    private JMenu createHelpMenu() {
        JMenu jmenu = add(new JMenu("Help"));
        jmenu.setMnemonic('h');

        // Help
        JMenuItem item = jmenu.add(new MenuAction_Help());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));

        jmenu.addSeparator();

        // About
        item = jmenu.add(new MenuAction_About());
        item.setMnemonic('a');

        return jmenu;
    }

    // =========================== TOOLBAR ==================================

    /**
     * Create the Toolbar.
     * @return The Toolbar
     */
    private ReloadToolBar createToolBar() {
        ReloadToolBar tBar = new ReloadToolBar();

        // New
        MenuAction_Exit actionExit = new MenuAction_Exit();
        tBar.add(actionExit);

        return tBar;
    }

    /**
     * Get the ReloadToolBar.
     * @return The main toolbar
     */
    public ReloadToolBar getReloadToolBar() {
        return toolBar;
    }

}

