/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.dom;

import java.util.*;

import org.jdom.*;

import uk.ac.reload.moonunit.*;

/**
 * This encapsulates the JDOM Path of a given Element and Attribute
 *
 * @author Phillip Beauvoir
 * @version 2003-07-28
 */
public class DomPath implements MoonUnitConstants {
    /**
     * The Path separator used for xml elements
     */
    public static String XMLPATHSEPARATOR = "/";

    /**
     * The Path separator used for xml attributes
     */
    public static String XMLATTSEPARATOR = "@";

    /**
     * The actual Path String
     */
    private String path;

	/**
	 * Constructor that takes in a path string.
	 *
	 * @param path the actual path string
	 */
    public DomPath(String path) {
        this.path = path;
    }

    /**
     * Get the Path String
     * @return the path string
     */
    public String getPath() {
        return path;
    }

    /**
     * Append an Element name to the Path
     * @param elementName to be appended to the path
     * @return the full path string
     */
    public String appendElementName(String elementName) {
        return path += XMLPATHSEPARATOR + elementName;
    }

    /**
     * Prepend an Element name to the Path
     * @param elementName to be prepended to the path
     * @return the full path string
     */
    public String prependElementName(String elementName) {
        return path = (elementName + XMLPATHSEPARATOR + path);
    }

    /**
     * Append an Attribute name to the Path
     * @param attributeName to be appended to the path
     * @return the full path string
     */
    public String appendAttributeName(String attributeName) {
        return path += XMLATTSEPARATOR + attributeName;
    }

    /**
     * Return an Enumeration of the Elements as a StringTokenizer
     * @return the StringTokenizer
     */
    public StringTokenizer getElements() {
        return new StringTokenizer(path, XMLPATHSEPARATOR);
    }

    /**
     * Return true if this path describes an Attribute
     * @return true if this path describes an attribute, false otherwise
     */
    public boolean isAttribute() {
        return path.indexOf(XMLATTSEPARATOR) != -1;
    }

    /**
     * Return the Attribute part of this path if it contains one, or null
     * @return the attribute part of this path if it contains one, null otherwise
     */
    public String getAttributePart() {
        int index = path.indexOf(XMLATTSEPARATOR);
        if(index != -1) return path.substring(index + 1);
        else return null;
    }

    /**
     * Return the Elements part of this path
     * @return the elements part of this path
     */
    public String getElementsPart() {
        int index = path.indexOf(XMLATTSEPARATOR);
        if(index != -1) return path.substring(0, index);
        else return path;
    }

    /**
     * Get the Root name of this Path
     * @return the root name of this path
     */
    public String getRootPart() {
        int index = path.indexOf(XMLPATHSEPARATOR);
        if(index != -1) return path.substring(0, index);
        else return path;
    }

    /**
     * Return true if the path part equals
     * @param domPath to check for equality with the current path
     * @return true if the path part equals to the given path, false otherwise
     */
    public boolean equals(DomPath domPath) {
        return path.equals(domPath.getPath());
    }

    /**
     * Return true if the path part endsWith
     * @param domPath to check for endsWith with the current path
     * @return true if the path part endsWith the given path, false otherwise
     */
    public boolean endsWith(DomPath domPath) {
        return path.endsWith(domPath.getPath());
    }

    /**
     * Get the path string
     * @return the path string
     */
    public String toString() {
        return path;
    }

    /**
     * Gets the DOM Path of the given element
     * @param element the element for which you want the DOM Path
     * @return a DOM Path for a JDOM Element
     */
    public static DomPath getDomPath(Element element) {
        String s = element.getName();
        Element parent;
        while((parent = element.getParent()) != null) {
            // If parent has a different namespace, add our prefix namespace
            if(!parent.getNamespace().equals(element.getNamespace())) s = element.getNamespacePrefix() + ":" + s;
            s = parent.getName() + XMLPATHSEPARATOR + s;
            element = parent;
        }

        DomPath domPath = new DomPath(s);
        return domPath;

        //DomPath domPath = new DomPath(element.getQualifiedName());
        //while((element = element.getParent()) != null) domPath.prependElementName(element.getQualifiedName());
        //if(DEBUG) System.out.println(domPath);
        //return domPath;
    }

    /**
     * Gets the DOM Path of the given JDOM attribute
     * @param attribute the given JDOM attribute
     * @return a DOM Path for a JDOM Attribute
     */
    public static DomPath getDomPath(Attribute attribute) {
        DomPath domPath = getDomPath(attribute.getParent());
        domPath.appendAttributeName(attribute.getQualifiedName());
        return domPath;
    }
}