/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.dom;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;



/**
 * This encapsulates the JDOM XML Document file
 * with it plus helper methods.
 *
 * @author Phillip Beauvoir
 * @version 2003-09-29
 */
public abstract class ReloadDocument {
    /**
     * The JDOM Document
     */
    protected Document _doc;

    /**
     * A flag to set if this Document is dirty (edited)
     */
    protected boolean _dirty;

    /**
     * The File for this Document - this may be null if not saved to disk
     */
    protected File _file;


    /**
     * Default Constructor
     */
    public ReloadDocument() { }

    /**
     * Constructor that takes in a an XML filename, JDOM document and a ReloadSchemaController
     *
     * @param file 				the XML file, for example an IMS metadata or an IMS VDEX vocabulary file
     * 		  doc 				the JDOM document of the XML file
     */
    public ReloadDocument(File file, Document doc) {
        _file = file;
        _doc = doc;
    }

    /**
     * Get the JDOM Metadata or Vocabulary Document.
     * @return The JDOM Document
     */
    public Document getDocument() {
        return _doc;
    }


    /**
     * Get the File of this Document.  This might be null.
     * @return the file of this document, null otherwise
     */
    public File getFile() {
        return _file;
    }

    /**
     * Load the JDOM Document File
     * @param file The XML File to read in
     * @throws DweezilXMLException
     */
    public void loadDocument(File file) throws DweezilXMLException {
        _file = file;
        _doc = XMLUtils.readXMLFile(file);
        _dirty = false;
    }

    /**
     * This will save the XML IMS file with the existing File ref.
     * @throws ReloadDomException if an error occurs
     */
    public void saveDocument() throws ReloadDomException {
        if(_doc != null) {
            try {
                XMLUtils.write2XMLFile(_doc, _file);
            }
            catch(DweezilXMLException ex) {
                throw new ReloadDomException(ex);
            }
            _dirty = false;
        }
    }


    /**
     * This will save the XML IMS file with a new File ref
     * @param file The File to save as
     * @throws ReloadDomException if an error occurs
     */
    public void saveAsDocument(File file) throws ReloadDomException {
        _file = file;
        saveDocument();
    }


    /**
     * Whether this Document has been changed in some way
     * @return true if the document is changed, false otherwise
     */
    public boolean isDirty() {
        return _dirty;
    }

    /**
     * Set whether this Document is dirty or not
     * @param isDirty true if the document is changed, false otherwise
     */
    public void setDirty(boolean isDirty) {
        _dirty = isDirty;
    }


    /**
     * Get the Root Element of the JDOM Document
     * @return the Root Element of the JDOM Document, or null if none
     */
    public Element getRootElement() {
        if(_doc != null && _doc.hasRootElement()) return _doc.getRootElement();
        return null;
    }

    /**
     * Get the Schema Namespace of the Document if there is one
     * @return the Schema Namespace of the Document if there is one, or null
     */
    public Namespace getSchemaNamespace() {
        Element root = getRootElement();
        if(root != null) return root.getNamespace();
        else return null;
    }

    /**
     * Checks if element belongs to the namespace of this document
     * @return true if element belongs to the Namespace of this Document, false otherwise
     */
    public boolean isDocumentNamespace(Element element) {
        Namespace ns = element.getNamespace();
        if(ns == null) return false;
        return ns == getSchemaNamespace();
    }

    //==========================================================================
    //===================== STATIC UTILITY METHODS =============================
    //==========================================================================

    /**
     * Checks of element1 and element2 belongs to the same document
     * @return true if element1 and element2 belong to the same Document, false otherwise
     */
    public static boolean isMemberOfSameDocument(Element element1, Element element2) {
        Document doc1 = element1.getDocument();
        Document doc2 = element2.getDocument();
        if(doc1 == null || doc2 == null) return false;
        return doc1.equals(doc2);
    }
}
