/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.dom;

import java.io.*;
import java.util.*;
import javax.swing.*;

import org.jdom.*;

import uk.ac.reload.dweezil.undo.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.gui.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.metadata.formview.*;
import uk.ac.reload.editor.metadata.tableview.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.vocabulary.*;
import uk.ac.reload.moonunit.dom.metadata.*;

/**
 * This encapsulates the JDOM XML Document file and the ReloadSchema associated
 * with it plus helper methods.
 *
 * @author Phillip Beauvoir
 * @version 2003-10-25
 */
public abstract class ReloadSchemaDocument
extends ReloadDocument
{
    /**
     * The Schema that will be our template
     */
    protected ReloadSchemaController _controller;

    /**
     * The Undo Handler
     */
    protected UndoHandler _undoHandler;

    /**
     * Default Constructor
     */
    public ReloadSchemaDocument() {
        super();
    }

    /**
     * Constructor that takes in a an XML filename, JDOM document and a ReloadSchemaController
     *
     * @param file 				the XML file, for example an IMS metadata or an IMS VDEX vocabulary file
     * 		  doc 				the JDOM document of the XML file
     *		  schemaController 	the ReloadSchemaController corresponding to the schema of the XML file
     */
    public ReloadSchemaDocument(File file, Document doc, ReloadSchemaController schemaController) {
        _file = file;
        _doc = doc;
        _controller = schemaController;
    }

    /**
     * Set the ReloadSchemaController
     * @param controller the ReloadSchemaController
     */
    public void setReloadSchemaController(ReloadSchemaController controller) {
        _controller = controller;
    }

    /**
     * Get the ReloadSchemaController for this Document
     * @return the ReloadSchemaController for this document
     */
    public ReloadSchemaController getReloadSchemaController() {
        return _controller;
    }

    /**
     * This will save the XML IMS file with the existing File ref and notify any listeners
     * @throws ReloadDomException if an error occurs
     */
    public void saveDocument() throws ReloadDomException {
        if(_doc != null) {
            super.saveDocument();
            fireDocumentSaved();
        }
    }

    /**
     * Get the Target Root element name
     * @return the Target Root Element name
     */
    public abstract String getTargetRootElementName();

    /**
     * Get the comments to add to the XML document
     * @return the Comments to add to the XML Document
     */
    public abstract String[] getComments();

    /**
     * Get the schema namespace of the document, or the target namespace of there isn't one
     * @return the Schema Namespace of the Document, or the Target Namespace if there isn't one
     */
    public Namespace getSchemaNamespace() {
        // Use Document's if we have one
        Namespace ns = super.getSchemaNamespace();
        if(ns != null) return ns;
        // Otherwise, from Target Schema
        else return getTargetNamespace();
    }

    /**
     * Get the target schema namespace of the document
     * @return the Target Schema Namespace of the Document
     */
    public Namespace getTargetNamespace() {
        ReloadSchema reloadSchema = _controller.getReloadSchema();
        return Namespace.getNamespace(reloadSchema.getTargetNamespace());
    }

    /**
     * Return the 'first' found Element for the given DOM Path or null if not found
     * @return the first found element for the given DOM Path or null if not found
     */
    public Element getElement(ReloadSchemaElement schemaElement) {
        return getElement(schemaElement.getDomPath());
    }

    /**
     * Return the 'first' found Element for the given DOM Path or null if not found
     * return the first found element for the given DOM Path or null if not found
     */
    public Element getElement(DomPath domPath) {
        Element element = _doc.getRootElement();

        String rootName = domPath.getRootPart();

        // Now work thru the path so we can drill into the Schema to find the correct point in the Schema
        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // Ignore Root Element
            if(name.equals(rootName)) continue;

            element = element.getChild(name, element.getNamespace());

            // Not found
            if(element == null) break;
        }

        return element;
    }

    /**
     * Return all Elements for the given ReloadSchemaElement Path or an empty array
     */
    public Element[] getElements(ReloadSchemaElement schemaElement) {
        return getElements(schemaElement.getDomPath());
    }

    /**
     * Return all Elements for the given DOM Path or null
     */
    public Element[] getElements(DomPath domPath) {

        Element element = _doc.getRootElement();

        Element[] elements = null;

        String rootName = domPath.getRootPart();

        // Now work thru the path so we can drill into the Schema to find the correct point in the Schema
        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // Ignore Root Element
            if(name.equals(rootName)) continue;

            // Last in the chain
            if(!t.hasMoreElements()) {

                List list = element.getChildren(name, element.getNamespace());
                elements = new Element[list.size()];
                if(!list.isEmpty()) list.toArray(elements);
                break;
            }

            element = element.getChild(name, element.getNamespace());

            // Not found
            if(element == null) break;
        }

        return elements;
    }

    // Tina Manoharan, 2004-01-13

    /**
     * Return all the elements for the given ReloadSchemaElement Path or an empty string
     */
    public Element[] getAllElements(ReloadSchemaElement schemaElement) {

		return getAllElements(schemaElement.getDomPath());
	}

    /**
     * Return all Elements for the given DOM Path or null
     */
    public Element[] getAllElements(DomPath domPath) {

        Element element = _doc.getRootElement();

        String rootName = domPath.getRootPart();

        ArrayList temp = new ArrayList();
        temp.add(element);

        ArrayList result = new ArrayList();

        // Now work thru the path so we can drill into the Schema to find the correct point in the Schema
        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // Ignore Root Element
            if(name.equals(rootName)) continue;

            for(int i = 0; i < temp.size(); i++) {

				Element elem = (Element) temp.get(i);

				List list = elem.getChildren(name, element.getNamespace());
            	if(!list.isEmpty()) result.addAll(list);
			}

			if(t.hasMoreElements()) {

				temp.clear();
				temp.addAll(result);

				result.clear();
			}
         }

        return (Element[]) result.toArray(new Element[result.size()]);
    }


    /**
     * Add our signature comments to the XML Document
     * @param doc the document to which the signature comments has to be added
     */
    protected void addComments(Document doc) {
        String[] comments = getComments();
        if(comments != null) {
            for(int i = 0; i < comments.length; i++) {
                Comment comment = new Comment(comments[i]);
                doc.addContent(comment);
            }
        }
    }

    /**
     * Cleanup
     */
    public void destroy() {
       _listeners.clear();
       _doc = null;
    }


//========================== ELEMENT OPERATIONS ================================

    /**
     * Add a new Element given the Path in the ReloadSchemaElement.
     * If the Element already exists, it won't be added
     * @param source Who is adding it
     *        newSchemaElement the new schema element type we want to add
     * 		  doSelect true or false depending on whether the new added element should be selected or not on the user interface
     * @return the added Element
     */
    public Element addElementUnique(Object source, ReloadSchemaElement newSchemaElement, boolean doSelect) {
        if(newSchemaElement == null) return null;

        // Get to the Root
        Element element = _doc.getRootElement();
        ReloadSchemaElement schemaElement = newSchemaElement.getReloadSchema().getRootElement();

        // Create a dom Path
        DomPath domPath = newSchemaElement.getDomPath();

        // Make a note of the Root name
        String rootName = domPath.getRootPart();

        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // Ignore root
            if(name.equals(rootName)) continue;

            Namespace ns;

            // Check for Namespace prefix
            int idx = name.indexOf(':');
            if(idx >= 0) {
                String prefix = name.substring(0, idx);
                name = name.substring(idx + 1);
                ns = element.getNamespace(prefix);
            }
            else ns = element.getNamespace();

            // Get corresponding Child ReloadSchemaElement
            schemaElement = schemaElement.getChild(name, ns);
            if(schemaElement == null) break; // Oops

            // Get Child Element
            Element childElement = element.getChild(name, ns);

            // There is no child, so create one
            if(childElement == null) childElement = addElementBySchemaUndoable(source, element, schemaElement, doSelect);

            element = childElement;
        }

        return element;
    }

    /**
     * Add a new Element to the parent Element based on RELOADSchemaElement and register an Undo event.
     * Create new JDOM Element based on childSchemaElement.
     * Insert into the correct position!.
     * Set any defaults of the Element.
     * Add any default child Elements if needed.
     * @param source who is telling us this
     * @param parentElement the parent Element to add to
     * @param newSchemaElement the Type of Element we want to add
     * @param doSelect true or false depending on whether the new added element should be selected or not on the user interface
     */
    public Element addElementBySchemaUndoable(Object source, Element parentElement, ReloadSchemaElement newSchemaElement, boolean doSelect) {
        Element newElement = addElementBySchema(source, parentElement, newSchemaElement, doSelect);

        // Get Undoable Action after adding
        if(_undoHandler != null && newElement != null) {
            UndoableAddAction addAction = new UndoableAddAction(parentElement, newElement);
            _undoHandler.addUndoableAction(addAction);
        }

        return newElement;
    }


    /**
     * Add a new Element to the parent Element based on ReloadSchemaElement
     * Create new JDOM Element based on childSchemaElement
     * Insert into the correct position!
     * Set any defaults of the Element
     * Add any default child Elements if needed
     * @param source who is telling us this
     * @param parentElement the parent Element to add to
     * @param newSchemaElement the Type of Element we want to add
     * @param doSelect true or false depending on whether the new added element should be selected or not on the user interface
     */
    public Element addElementBySchema(Object source, Element parentElement, ReloadSchemaElement newSchemaElement, boolean doSelect) {
        if(newSchemaElement == null) return null;

        // Get the Namespace
        Namespace ns;
        // If no parent use Document Namespace or given Namespace
        if(parentElement == null) ns = getSchemaNamespace();
        // Else determine parent or schema
        else {
            // If the prefix is missing it won't work properly, so use parent's NS
            String nsSchemaPrefix = newSchemaElement.getNamespacePrefix();
            if(nsSchemaPrefix.equals("")) ns = parentElement.getNamespace();
            else ns = newSchemaElement.getNamespace();
        }

        // Create Element
        Element newElement = new Element(newSchemaElement.getName(), ns);

        // If we have a default value, add it
        String defaultValue = _controller.getDefaultValue(newSchemaElement);
        if(defaultValue != null)  newElement.setText(defaultValue);

        // Add any default Attributes, if any - they will be added in order as per Schema
        if(newSchemaElement.hasAttributes()) {
            ReloadSchemaAttribute[] atts = newSchemaElement.getAttributes();
            for(int i = 0; i < atts.length; i++) {
                defaultValue = _controller.getDefaultValue(atts[i]);
                if(defaultValue != null) {
                    Attribute att = atts[i].createAttribute();
                    att.setValue(defaultValue);
                    newElement.setAttribute(att);
                }
            }
        }

        // Add to parent at correct position
        // If parent is null, that's OK it's probably the Document
        if(parentElement != null) {
            int index = getInsertPositionOfElement(parentElement, newSchemaElement);
            addElementAtIndex(source, parentElement, newElement, index, doSelect);
        }

        // Add any children that have a minimum of one (i.e. mandatory)
        ReloadSchemaElement[] schemaChildren = newSchemaElement.getChildren();
        for(int i = 0; i < schemaChildren.length; i++) {
            ReloadSchemaElement child = schemaChildren[i];
            if(child.getMinOccurs() == 1) addElementBySchema(source, newElement, child, doSelect);
        }

        return newElement;
    }

    /**
     * Just add a new Element.  Listeners will only be notified once. Undo Event will be fired.
     * @param source 		tho is telling us this
     *		  parentElement	the parent Element to add to
     *		  childElement	the child element to be added to the parent element
     *		  doSelect 		true or false depending on whether the new added element should be selected or not on the user interface
     */
    public Element addElementUndoable(Object source, Element parentElement, Element childElement, boolean doSelect) {
        Element newElement = addElement(source, parentElement, childElement, doSelect);

        // Get Undoable Action after adding
        if(_undoHandler != null && newElement != null) {
            UndoableAddAction addAction = new UndoableAddAction(parentElement, newElement);
            _undoHandler.addUndoableAction(addAction);
        }

        return newElement;
    }

    /**
     * Just add a new Element.  Listeners will only be notified once.
     * If the new Element has children it's up to the listener to handle them
     *
     * @param source 			who is telling us this
     *		  parentElement		the parent Element to add to
     *		  childElement		the child element to be added to the parent element
     *		  doSelect 			true or false depending on whether the new added element should be selected or not on the user interface
     */
    public Element addElement(Object source, Element parentElement, Element childElement, boolean doSelect) {
        DomPath domPath = DomPath.getDomPath(parentElement);
        domPath.appendElementName(childElement.getName());
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)_controller.getReloadSchemaNode(domPath);

        // Ascertain correct position
        int index = getInsertPositionOfElement(parentElement, schemaElement);
        return addElementAtIndex(source, parentElement, childElement, index, doSelect);
    }

    /**
     * Add an Element at an index Position and Notify listeners
     *
     * @param source 		who is telling us this
     *		  parentElement	the parent Element to add to
     *		  newElement	the new element to be added to the parent element
     *		  index			the index position at which the new element has to be added
     *		  doSelect 		true or false depending on whether the new added element should be selected or not on the user interface
     */
    public synchronized Element addElementAtIndex(Object source, Element parentElement, Element newElement, int index, boolean doSelect) {
        List children = parentElement.getChildren();
        if(index < 0 || index > children.size()) index = 0;
        children.add(index, newElement);

        // Notify
        ReloadDocumentListenerEvent event = new ReloadDocumentListenerEvent(source, this, newElement, doSelect);
        _dirty = true;  // this first
        fireElementAdded(event);

        return newElement;
    }

    /**
     * Delete a child Element from its parent and notify any listeners and Undo
     * @param source who is telling us this
     * @param element the Element to remove
     */
    public synchronized Element deleteElementUndoable(Object source, Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            // Get Undoable Action First so we can get full dom path
            UndoableAction action = null;
            if(_undoHandler != null) {
                action = new UndoableDeleteAction(parent, element);
            }

            // Do it
            removeElement(source, element);

            if(_undoHandler != null && action != null) {
                _undoHandler.addUndoableAction(action);
            }
        }
        return element;
    }

    /**
     * Cut a child Element from its parent and notify any listeners and Undo
     * @param source who is telling us this
     * @param element the Element to remove
     */
    public synchronized Element cutElementUndoable(Object source, Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            // Get Undoable Action First so we can get full dom path
            UndoableAction action = null;
            if(_undoHandler != null) {
                action = new UndoableCutAction(parent, element);
            }

            // Do it
            removeElement(source, element);

            if(_undoHandler != null && action != null) {
                _undoHandler.addUndoableAction(action);
            }
        }
        return element;
    }

    /**
     * Remove a child Element from its parent and notify any listeners
     * @param source who is telling us this
     * @param element the Element to remove
     */
    public synchronized Element removeElement(Object source, Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            parent.removeContent(element);
            ReloadDocumentListenerEvent event = new ReloadDocumentListenerEvent(source, this, element, false);
            _dirty = true; // this first
            fireElementRemoved(event);
        }
        return element;
    }

    /**
     * Move an Element to a new Parent
     * @param source	who is telling us this
     *		  element	the element to be moved
     *		  newParent	the new parent element
     *		  doSelect 	true or false depending on whether the new added element should be selected or not on the user interface
     * @return the Element moved
     */
    public Element moveElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element oldParent = element.getParent();
        if(oldParent != null) {
            // Get Undoable Action First so we can get index position
            UndoableAction action = null;
            if(_undoHandler != null) {
                action = new UndoableMoveAction(oldParent, newParent, element);
            }

            // Do it
            removeElement(source, element);
            addElement(this, newParent, element, doSelect);

            if(_undoHandler != null && action != null) {
                _undoHandler.addUndoableAction(action);
            }
        }
        return element;
    }

    /**
     * Copy an Element to a new Parent
     * @param source	who is telling us this
     *		  element	the element to be copied
     *		  newParent	the new parent element
     *		  doSelect 	true or false depending on whether the new added element should be selected or not on the user interface
     * @return the Element copied
     */
    public Element copyElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = (Element)element.clone();
        addElement(this, newParent, newElement, doSelect);
        if(_undoHandler != null) {
            UndoableAction action = new UndoableCopyAction(newParent, newElement);
            _undoHandler.addUndoableAction(action);
        }
        return newElement;
    }


    /**
     * Paste an Element to a new Parent
     * @param source		who is telling us this
     *		  element		the element to be pasted
     *		  newParent		the new parent element
     *		  doSelect 		true or false depending on whether the new added element should be selected or not on the user interface
     * @return the Element pasted
     */
    public Element pasteElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = (Element)element.clone();
        addElement(this, newParent, newElement, doSelect);
        if(_undoHandler != null) {
            UndoableAction action = new UndoablePasteAction(newParent, newElement);
            _undoHandler.addUndoableAction(action);
        }
        return newElement;
    }

    /**
     * Move an Element up one place
     * @param source		who is telling us this
     *		  element		the element to be moved up
     *		  doSelect 		true or false depending on whether the new added element should be selected or not on the user interface
     * @return the Element moved
     */
    public Element moveElementUp(Object source, Element element, boolean doSelect) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index > 0) {
                    removeElement(source, element);
                    addElementAtIndex(source, parent, element, index - 1, doSelect);
                }
            }
        }
        return element;
    }

    /**
     * Move an Element down one place
     * @param source		who is telling us this
     *		  element		the element to be moved down
     *		  doSelect 		true or false depending on whether the new added element should be selected or not on the user interface
     * @return the Element moved
     */
    public Element moveElementDown(Object source, Element element, boolean doSelect) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index < children.size() - 1) {
                    removeElement(source, element);
                    addElementAtIndex(source, parent, element, index + 1, doSelect);
                }
            }
        }
        return element;
    }

    /**
     * Somebody has notified us that they have changed an Element in some way.
     * We will notify all listeners.
     * @param source who is telling us this
     * @param element the Element that changed
     */
    public void changedElement(Object source, Element element) {
        ReloadDocumentListenerEvent event = new ReloadDocumentListenerEvent(source, this, element, false);
        _dirty = true; // this first
        fireElementChanged(event);
    }


// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * Decide whether a child Element can be added to a parent Element by max amount
     * @param parentElement the parent Element to which we wish to add
     * @param childSchemaElement the SchemaElement of the child to add
     * @return true if we can add it, or false if not
     */
    public boolean canAddElement(Element parentElement, ReloadSchemaElement childSchemaElement) {
        if(childSchemaElement == null) return true;

        // Find out how many we can add
        int max = childSchemaElement.getMaxOccurs();
        // Unlimited
        if(max == -1) return true;
        // Compare with how many we have - we must use the namespace!
        List list = parentElement.getChildren(childSchemaElement.getName(), childSchemaElement.getNamespace());
        return list.size() < max;
    }

    /**
     * Decide whether a child is allowed or not depending on the schema rules
     * @param parentElement the parent element to which we wish to add
     *		  childElement	the child element to add
     * @return true if we are allowed (per Schema rules) to add a child, false otherwise
     */
    public boolean isAllowedChild(Element parentElement, Element childElement) {
        // For now, don't allow different namespaces
        if(!parentElement.getNamespace().equals(childElement.getNamespace())) return false;

        DomPath domPath = DomPath.getDomPath(parentElement);
        ReloadSchemaElement parentSchemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        if(parentSchemaElement != null) {
            ReloadSchemaElement childSchemaElement = parentSchemaElement.getChild(childElement.getName());
            if(childSchemaElement == null) return false;
            return canAddElement(parentElement, childSchemaElement);
        }

        return false;
    }

    /**
     * Decide whether a Element can be deleted
     * @param element the Element which we wish to delete
     * @param schemaElement the SchemaElement of the Element
     * @return true if we can delete it, or false if not
     */
    public boolean canDeleteElement(Element element, ReloadSchemaElement schemaElement) {
        if(schemaElement == null) return true;

        // Root
        Element parentElement = element.getParent();
        if(parentElement == null) return false;
        // If we have to have a minimum of one
        List list = parentElement.getChildren(schemaElement.getName(), schemaElement.getNamespace());
        return list.size() > schemaElement.getMinOccurs();
    }

    /**
     * Decides whether an element can be copied or not - will need to override this
     * @param element the element which we are testing
     * @return true if we can copy the element, false otherwise
     */
    public boolean canCopyElement(Element element) {
        return false;
    }

    /**
     * Decides whether we can cut an element - will need to override this
     * @param element the element which we are testing
     *		  schemaElement the schemaElement against which we are testing
     * @return true if we can cut an element, false otherwise
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return false;
    }

    /**
     * Decides whether we can paste an element - will need to override this
     * @param element the element which we are testing
     * @return true if we can paste an element, false otherwise
     */
    public boolean canPasteFromClipboard(Element element) {
        return false;
    }

    /**
     * Decides whether we can move the element up by one place
     * @param element the element which we are testing
     * @return true if we can move an element, false otherwise
     */
    public boolean canMoveElementUp(Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index > 0) {
                    // Only if same type and namesace
                    Element previousSibling = (Element)children.get(index - 1);
                    if(previousSibling.getName().equals(element.getName()) &&
                       previousSibling.getNamespace().equals(element.getNamespace())) return true;
                }
            }
        }
        return false;
    }

    /**
     * Decides whether we can move the element down by one place
     * @param element the element which we are testing
     * @return true if we can move the element, false otherwise
     */
    public boolean canMoveElementDown(Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index < children.size() - 1) {
                    // Only if same type and namesace
                    Element nextSibling = (Element)children.get(index + 1);
                    if(nextSibling.getName().equals(element.getName()) &&
                       nextSibling.getNamespace().equals(element.getNamespace())) return true;
                }
            }
        }
        return false;
    }

//========================== ATTRIBUTE OPERATIONS ==============================

    /**
     * Create and Add a new Attribute to a given Element as per the Schema
     * @param source				who is telling us this
     * 		  parentElement			the parent element to which we wish to add
     *		  newSchemaAttribute 	the new schema attribute we wish to add to the parent element
     * @return The created attribute or null if it cannot be done
     */
    public Attribute addAttribute(Object source, Element parentElement, ReloadSchemaAttribute newSchemaAttribute) {
        if(parentElement == null || newSchemaAttribute == null) return null;

        Attribute att = newSchemaAttribute.createAttribute();
        if(att != null) {
            // Add it at the correct position
            java.util.List list = parentElement.getAttributes();
            int index = getInsertPositionOfAttribute(parentElement, newSchemaAttribute);
            if(index < 0 || index > list.size()) index = 0;
            list.add(index, att);
        }

        return att;
    }

//============================== SOME USEFUL ROUTINES  ==========================


    /**
     * Get the index position of an element in relation to its parent
     * @param element the element for which we require the index position
     * @return the index position of an Element in relation to its parent
     */
    public static int getIndexOfElement(Element element) {
        int index = 0;
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            index = children.indexOf(element);
            if(index == -1) index = 0;
        }
        return index;
    }

    /**
     * Get the correct position to insert an element according to the Schema
     * @param parentElement		the parent element to which the new element is to be added
     *		  newSchemaElement	the type of Element we want to add
     * @return the correct position to insert the element in the parent element
     */
    public static int getInsertPositionOfElement(Element parentElement, ReloadSchemaElement newSchemaElement) {
        if(newSchemaElement == null) return parentElement.getChildren().size();

        // No children
        if(parentElement.getChildren().size() == 0) return 0;

        int index = 0;

        // New Element's name
        String name = newSchemaElement.getName();

        // Get the Parent SchemaElement
        ReloadSchemaElement parentSchemaElement = (ReloadSchemaElement)newSchemaElement.getParent();

        // If it's null we must be at the root
        if(parentSchemaElement == null) parentSchemaElement = newSchemaElement;

        // Get the Parent SchemaElement's children
        ReloadSchemaElement[] schemaChildren = parentSchemaElement.getChildren();

        // Iterate thru the Parent Element's children
        List children = parentElement.getChildren();
        for(int i = 0; i < children.size(); i++) {
            Element element = (Element)children.get(i);

            // Get Position of this Element as per Schema
            String elementName = element.getName();
            int pos = parentSchemaElement.indexofChild(elementName);

            // Reached an element with a different namespace
            //if(!element.getNamespaceURI().equals(newSchemaElement.getNamespaceURI())) pos = -1;

            // Check for -1 - this means an element not in the Schema
            if(pos != -1) {
                for(int j = pos; j < schemaChildren.length; j++) {
                    if(name.equals(schemaChildren[j].getName())) index = i + 1;
                }
            }
        }

        return index;
    }

    /**
     * Get the correct position to insert an Attribute in an Element according to the Schema
     * @param parentElement the parent element to which the new attribute is to be added
     *		  reloadSchemaAttribute the type of attribute we want to add
     * @return the correct position to insert the Attribute in the parent Element
     */
    public static int getInsertPositionOfAttribute(Element parentElement, ReloadSchemaAttribute reloadSchemaAttribute) {
        int index = 0;

        // Get Element's Attributes
        java.util.List elementAtts = parentElement.getAttributes();

        // None here
        if(elementAtts.size() == 0) return 0;

        // Attributes's name
        String name = reloadSchemaAttribute.getName();

        // Get the Parent SchemaElement
        ReloadSchemaElement parentSchemaElement = (ReloadSchemaElement)reloadSchemaAttribute.getParent();

        // Get the Parent SchemaElement's schema attributes
        ReloadSchemaAttribute[] schemaAtts = parentSchemaElement.getAttributes();

        // Iterate thru the Parent Element's attributes
        for(int i = 0; i < elementAtts.size(); i++) {
            Attribute att = (Attribute)elementAtts.get(i);

            // Get Position of this Attribute as per Schema
            String attName = att.getName();
            int pos = parentSchemaElement.indexofAttribute(attName);

            // Check for -1 - this means an Attribute not in the Schema
            if(pos != -1) {
                for(int j = pos; j < schemaAtts.length; j++) {
                    if(name.equals(schemaAtts[j].getName())) index = i + 1;
                }
            }
            else {
                index = i + 1;
            }
        }

        return index;
    }

    //============================== LISTENER EVENTS  ==========================

    /**
     * List of Listeners
     */
    private Vector _listeners = new Vector();

    /**
     * Add a ReloadDocumentListener
     * @param listener The ReloadDocumentListener
     */
    public synchronized void addReloadDocumentListener(ReloadDocumentListener listener) {
        if(!_listeners.contains(listener)) _listeners.addElement(listener);
    }

    /**
     * Remove a ReloadDocumentListener
     * @param listener The ReloadDocumentListener
     */
    public synchronized void removeReloadDocumentListener(ReloadDocumentListener listener) {
        _listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we have added a new Element
     * @param event the ReloadDocumentListenerEvent
     */
    public void fireElementAdded(ReloadDocumentListenerEvent event) {
        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.elementAdded(event);
        }
    }

    /**
     * Tell our listeners that we have deleted an Element
     * @param event the ReloadDocumentListenerEvent
     */
    public void fireElementRemoved(ReloadDocumentListenerEvent event) {
        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.elementRemoved(event);
        }
    }

    /**
     * Tell our listeners that we have changed an Element
     * @param event the ReloadDocumentListenerEvent
     */
    public void fireElementChanged(ReloadDocumentListenerEvent event) {
        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.elementChanged(event);
        }


		// Tina Manoharan, 2003-11-10

       	Element sourceElement = event.getElement();

        // -- This part is only for the Vocabulary Editor when Vdex ProfileType attribute is changed from the ReloadAttributeEditor ---

		if(event.getSource() instanceof ReloadAttributeEditor && sourceElement.getName().equals(VocabularyCore.ROOT_NAME)) {

			this.setVocabularyProfileType(sourceElement);
		}

        // --- This part is only for the Metadata Editor when using Vdex vocabulary files ---

        if(event.getSource() instanceof MetadataEditorFrame || event.getSource() instanceof MetadataComboBox
           || event.getSource() instanceof MetadataTextField || event.getSource() instanceof MetadataTreeTable
           || event.getSource() instanceof ReloadAttributeEditor || event.getSource() instanceof ReloadElementEditor) {

        	if(sourceElement.getName().equals(MetadataCore.LANGSTRING)) {

				if(sourceElement.getParent().getName().equals(MetadataCore.SOURCE)) {

					// Check if the element that changed is the TaxonPath Source element
	        		// lom/classification/taxonpath/source/langstring

					if(sourceElement.getParent().getParent().getName().equals(MetadataCore.TAXON_PATH) &&
					   sourceElement.getParent().getParent().getParent().getName().equals(MetadataCore.CLASSIFICATION)) {

						System.out.println("taxonpath/source/langstring -> delete taxon tree with warning");

						this.deleteTaxonTreeWithWarning(sourceElement);
					}
					else {

						System.out.println("source/langstring -> update value based on source");

						this.updateValueElementBasedOnSourceElement(event.getSource(), sourceElement);
					}
				}
				else if(sourceElement.getParent().getName().equals(MetadataCore.VALUE)) {

					// Check if the element that changed is the Classification Purpose Value element
		        	// lom/classification/purpose/value/langstring
		        	if(sourceElement.getParent().getParent().getName().equals(MetadataCore.PURPOSE) &&
		        	   sourceElement.getParent().getParent().getParent().getName().equals(MetadataCore.CLASSIFICATION)) {

						System.out.println("purpose/value/langstring -> update taxonpath/source/langstring");

						this.updateTaxonPathSourceBasedOnPurposeValue(event.getSource(), sourceElement);
					}
				}
			}
		}
    }


    /**
     * Tell our listeners that we have saved
     */
    public void fireDocumentSaved() {
        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.documentSaved(this);
        }
    }


    //============================== HELPERS for FIRE ELEMENT CHANGED ==========================


	// Tina Manoharan, 2003-11-10
	/**
	 * Sets the vocabulary profile type for the vocabulary editor.
	 * The profile type is changed from the ReloadAttributeEditor.
	 * This change is communicated to the VocabularyEditor.
	 *
	 * @param sourceElement the element concerned
	 */
    private void setVocabularyProfileType(Element sourceElement) {

		ReloadInternalFrame focussedInternalFrame = ReloadEditor.getInstance().getFocussedInternalFrame();

		if(focussedInternalFrame instanceof VocabularyEditorFrame) {

			VocabularyEditorFrame vxFrame = (VocabularyEditorFrame)focussedInternalFrame;

			VocabularyEditor vxEditor = vxFrame.getVocabularyEditor();

			Attribute att = sourceElement.getAttribute(VocabularyCore.PROFILE_TYPE);

		    if(att != null) vxEditor.setProfileType(att.getValue());
		}
	}

	// Tina Manoharan, 2003-11-10
	/**
	 * Deletes the classification Taxon tree when the Taxon Path source is changed.
	 * Before doing that a warning is given to the user.
	 *
	 * @param sourceElement the element concerned
	 */
	private void deleteTaxonTreeWithWarning(Element sourceElement) {

		// if so then delete the Taxon tree below it after giving user warning
		// if user presses OK then delete, if CANCEL is pressed then dont delete!
		// lom/classification/taxonpath/taxon


		// getChildren instead of getChild is done considering the case where the Schema has many Taxon children
		// rather than a hierarchy of taxon elements
		java.util.List taxonElements = sourceElement.getParent().getParent().getChildren(MetadataCore.TAXON, sourceElement.getNamespace());

		if(!taxonElements.isEmpty()) {

			int value = JOptionPane.showConfirmDialog(ReloadEditor.getInstance(),"Changing the source will delete any classifications in this taxon path", "Reload Editor",JOptionPane.OK_CANCEL_OPTION);

	    	if(value == JOptionPane.YES_OPTION) {

				for(int i = 0; i < taxonElements.size(); i++) {

					System.out.println("Delete taxon of source: " + sourceElement.getText());

					removeElement(ReloadSchemaDocument.this, (Element) taxonElements.get(i));
				}
			}
		}
	}

	// Tina Manoharan, 2003-11-10
	/**
	 * Updates the value element vocabulary list based on the value of the source element
	 *
	 * @param 	source the who is telling us this
	 * 			sourceElement the element concerned
	 */
	private void updateValueElementBasedOnSourceElement(Object source, Element sourceElement) {

		// for all other 'source' elements get the next 'value' element
		// get the schemaElement corresponding to this 'value' element
		// get the vocabularyList corresponding to this schemaElement and update it
		// add ReloadListBox and MetadataComboBox as listeners to the VocabularyList
		// so when list updates these Boxes should also update their displayed lists!

		// (1) get element DomPath
		// (2) get rvList by calling ReloadVocabulary getVocabularyList(DomPath domPath)
		// (4) rvList.getList(vocabSource);

		String vocabSource = sourceElement.getText();

		Element valueElement = sourceElement.getParent().getParent().getChild(MetadataCore.VALUE, sourceElement.getNamespace());

		valueElement = valueElement.getChild(MetadataCore.LANGSTRING, sourceElement.getNamespace());

		if(valueElement != null) {

			DomPath domPath = DomPath.getDomPath(valueElement);

			System.out.println("Update value: " + domPath.getPath());

			ReloadVocabularyList rvList = getReloadSchemaController().getVocabularyList(domPath);

			if(rvList != null) rvList.setListUsingVocabSource(source, vocabSource);
		}

	}

	// Tina Manoharan, 2003-11-10
	/**
	 * Updates the Taxon Path source vocabulary list based on the value of the purpose value element
	 *
	 * @param 	source who is telling us this
	 *			sourceElement the element concerned
	 */
	private void updateTaxonPathSourceBasedOnPurposeValue(Object source, Element sourceElement) {

		// update the vocabulary list of the
		// lom/classification/taxonpath/source/langstring
		// lom/classification/purpose/value/langstring

		String purposeValue = sourceElement.getText();

		if(purposeValue != null && !purposeValue.equals("")) {

			// DomPath domPath = new DomPath(MetadataCore.CLASSIFICATION_TAXONPATH_SOURCE_PATH);

			Element classificationElement = sourceElement.getParent().getParent().getParent();

			if(classificationElement != null) {

				java.util.List taxonPathElements = classificationElement.getChildren(MetadataCore.TAXON_PATH, sourceElement.getNamespace());

				System.out.println("No of taxonpaths for this classification: " + taxonPathElements.size());

				for(int i = 0; i < taxonPathElements.size(); i++) {

					Element taxonPathElement = (Element) taxonPathElements.get(i);

					if(taxonPathElement != null) {

						System.out.println("update taxonpath: " + i + " " + taxonPathElement.getName());

						Element taxonPathSourceElement = taxonPathElement.getChild(MetadataCore.SOURCE, sourceElement.getNamespace());

						if(taxonPathSourceElement != null)  {

							Element taxonPathSourceLangElement = taxonPathSourceElement.getChild(MetadataCore.LANGSTRING, sourceElement.getNamespace());

							if(taxonPathSourceLangElement != null) {

								DomPath domPath = DomPath.getDomPath(taxonPathSourceLangElement);

								ReloadVocabularyList rvList = getReloadSchemaController().getVocabularyList(domPath);

								if(rvList != null) rvList.updateTaxonPathSourceListUsingPurposeValue(source, purposeValue);
							}
						}
					}
				}

				if(taxonPathElements.isEmpty()) {

					System.out.println("No taxonpath elements");

					// create lom/classification/taxonpath/source/langstring element

					DomPath domPath = new DomPath(MetadataCore.CLASSIFICATION_TAXONPATH_SOURCE_PATH);

					ReloadVocabularyList rvList = getReloadSchemaController().getVocabularyList(domPath);

					if(rvList != null) rvList.updateTaxonPathSourceListUsingPurposeValue(source, purposeValue);
				}
			}

		}
	}



// =============================================================================
// ============================ UNDO HANDLING ==================================
// =============================================================================

	/**
	 * Set the undo handler for this document
	 */
    public void setUndoHandler(UndoHandler undoHandler) {
        _undoHandler = undoHandler;
    }

	/**
	 * Get the undo handler for this document
	 */
    public UndoHandler getUndoHandler() {
        return _undoHandler;
    }

    /**
     * Handles Undo/Redo for Deleting Elements
     */
    protected class UndoableDeleteAction implements UndoableAction {
        Element parent;
        Element element;
        int index;
        String name;

		/**
		 * Constructor
		 * @param parent the parent element
		 *		  element the element to delete
		 */
        public UndoableDeleteAction(Element parent, Element element) {
            this.parent = parent;
            this.element = element;
            index = getIndexOfElement(element);
            // Friendly Name
            name = _controller.getElementFriendlyName(DomPath.getDomPath(element));
            if(name == null) name = element.getName();
        }

		/**
		 * Undo's remove by adding the element at the given index
		 */
        public void undo() {
            addElementAtIndex(null, parent, element, index, true);
        }

		/**
		 * Redo's add by removing the element
		 */
        public void redo() {
            index = getIndexOfElement(element);
            removeElement(null, element);
        }

		/**
		 * Gets the name of the delete element
		 */
        public String getName() {
            return "Delete " + name;
        }
    }

    /**
     * Handles Undo/Redo for Cutting Elements
     */
    protected class UndoableCutAction extends UndoableDeleteAction {

		/**
		 * Constructor
		 * @param parent the parent element
		 * 		  element	the element to cut
		 */
        public UndoableCutAction(Element parent, Element element) {
            super(parent, element);
        }

		/**
		 * Gets the name of the cut element
		 */
        public String getName() {
            return "Cut " + name;
        }
    }

    /**
     * Handles Undo/Redo for Adding Elements
     */
    protected class UndoableAddAction implements UndoableAction {
        Element parent;
        Element element;
        int index;
        String name;

		/**
		 * Constructor
		 * @param parent the parent element
		 *		  element the element to add
		 */
        public UndoableAddAction(Element parent, Element element) {
            this.parent = parent;
            this.element = element;
            index = getIndexOfElement(element);
            // Friendly Name
            name = _controller.getElementFriendlyName(DomPath.getDomPath(element));
            if(name == null) name = element.getName();
        }

		/**
		 * Undo's the add by removing the element
		 */
        public void undo() {
            index = getIndexOfElement(element);
            removeElement(null, element);
        }

		/**
		 * Redo's the add by adding the element at the given index
		 */
        public void redo() {
            addElementAtIndex(null, parent, element, index, true);
        }

		/**
		 * Gets the name of the add element
		 */
        public String getName() {
            return "Add " + name;
        }
    }

    /**
     * Handles Undo/Redo for Copying Elements
     */
    protected class UndoableCopyAction extends UndoableAddAction {

		/**
		 * Constructor
		 * @param parent the parent element
		 *		  element the element to copy
		 */
        public UndoableCopyAction(Element parent, Element element) {
            super(parent, element);
        }

		/**
		 * Gets the name of the copy element
		 */
        public String getName() {
            return "Copy " + name;
        }
    }

    /**
     * Handles Undo/Redo for Pasting Elements
     */
    protected class UndoablePasteAction extends UndoableAddAction {

		/**
		 * Sole constructor
		 * @param parent the parent element
		 *		  element the element to paste
		 */
        public UndoablePasteAction(Element parent, Element element) {
            super(parent, element);
        }

		/**
		 * Gets the name of the paste element
		 */
        public String getName() {
            return "Paste " + name;
        }
    }

    /**
     * Handles Undo/Redo for Moving Elements
     */
    protected class UndoableMoveAction implements UndoableAction {
        Element oldParent;
        Element newParent;
        Element element;
        int oldIndex, newIndex;
        String name;

		/**
		 * Constructor
		 * @param oldParent the old parent element from which the given element is moved
		 *		  newParent the new parent element to which the given element is moved
		 *		  element the element to move
		 */
        public UndoableMoveAction(Element oldParent, Element newParent, Element element) {
            this.oldParent = oldParent;
            this.newParent = newParent;
            this.element = element;
            oldIndex = getIndexOfElement(element);
            // Friendly Name
            name = _controller.getElementFriendlyName(DomPath.getDomPath(element));
            if(name == null) name = element.getName();
        }

		/**
		 * Undo's a move by removing the element from the new parent and adding to the old parent at the given index
		 */
        public void undo() {
            newIndex = getIndexOfElement(element);
            removeElement(null, element);
            addElementAtIndex(null, oldParent, element, oldIndex, true);
        }

		/**
		 * Redo's a move by removing an element from the old parent and adding it to the new parent at the given index
		 */
        public void redo() {
            oldIndex = getIndexOfElement(element);
            removeElement(null, element);
            addElementAtIndex(null, newParent, element, newIndex, true);
        }

		/**
		 * Gets the name of the move element
		 */
        public String getName() {
            return "Move " + name;
        }
    }
}