/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.dom.contentpackaging;

import java.io.*;
import java.net.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.metadata.*;


/**
 * Core IMS Content Package Methods and Functionality
 *
 * @author Phillip Beauvoir
 * @version 2003-10-21
 */
public class ContentPackageCore {

    // Element and Attribute Names
    public static final String MANIFEST = "manifest";
    public static final String ORGANIZATIONS = "organizations";
    public static final String RESOURCES = "resources";
    public static final String DEFAULT = "default";
    public static final String ORGANIZATION = "organization";
    public static final String ITEM = "item";
    public static final String PARAMETERS = "parameters";
    public static final String RESOURCE = "resource";
    public static final String BASE = "base";
    public static final String FILE = "file";
    public static final String TYPE = "type";
    public static final String HREF = "href";
    public static final String METADATA = "metadata";
    public static final String IDENTIFIER = "identifier";
    public static final String IDENTIFIERREF = "identifierref";
    public static final String TITLE = "title";
    public static final String DEPENDENCY = "dependency";
    public static final String VERSION = "version";
    public static final String SCHEMA = "schema";
    public static final String SCHEMAVERSION = "schemaversion";

    /**
     * The JDOM Document that forms the Content Package that we shall be working on
     */
    protected ReloadDocument _doc;

    /**
     * Constructor
     */
    public ContentPackageCore(ReloadDocument doc) {
        _doc = doc;
    }

    /**
     * @return the CP Project Folder which is the containing folder of imsmanifest.xml
     */
    public File getProjectFolder() {
        File file = _doc.getFile();
        return file == null ? null : file.getParentFile();
    }

// =============================================================================
// ELEMENT FINDING/NAVIGATION HANDLING
// =============================================================================

    /**
     * @return The Element referenced by identifierref in sourceElement.
     * The search digs downward from sourceElement - this is important.
     * This will be either a Resource or a sub-Manifest.
     */
    public Element getReferencedElement(Element sourceElement) {
        String idref = sourceElement.getAttributeValue(IDENTIFIERREF);

        if(idref != null) {
            Element manifest = getParentManifestElement(sourceElement);
            if(manifest != null) {
                // Search the Resources first
                Element[] resources = getElementsInManifest(manifest, RESOURCE, _doc.getSchemaNamespace());
                for(int i = 0; i < resources.length; i++) {
                    String id = resources[i].getAttributeValue(IDENTIFIER);
                    if(id != null) {
                        if(idref.equals(id)) return resources[i];
                    }
                }

                // Search for a sub-Manifest
                Element[] submanifests = getElementsInManifest(manifest, MANIFEST, _doc.getSchemaNamespace());
                for(int i = 0; i < submanifests.length; i++) {
                    String id = submanifests[i].getAttributeValue(IDENTIFIER);
                    if(id != null && idref.equals(id)) return submanifests[i];
                }
            }
        }

        return null;
    }

    /**
     * @return true if element has an identifierref attribute
     */
    public boolean isReferencingElement(Element element) {
        String idref = element.getAttributeValue(IDENTIFIERREF);
        return idref != null && !idref.equals("");
    }

    /**
     * @return all available Identifiers from Resources and sub-Manifests
     * that an element can legally reference.  For an Item this will be Resources and sub-manifests
     * and for a Resource Dependency this will be another Resource
     */
    public String[] getReferencedIdentifersAllowed(Element element) {
        Vector v = new Vector();

        Element[] elements = getReferencedElementsAllowed(element);
        for(int i = 0; i < elements.length; i++) {
            String id = elements[i].getAttributeValue(IDENTIFIER);
            if(id != null && !id.equals("")) v.add(id);
        }

        String[] ids = new String[v.size()];
        v.copyInto(ids);
        return ids;
    }

    /**
     * @return all available Elements (Resources and sub-Manifests)
     * that an element can legally reference.  For an Item this will be Resources and sub-manifests
     * and for a Resource Dependency this will be another Resource
     */
    public Element[] getReferencedElementsAllowed(Element element) {
        Vector v = new Vector();

        Element manifest = getParentManifestElement(element);
        if(manifest != null) {
            String elementName = element.getName();

            // Search the Resources first
            Element[] resources = getElementsInManifest(manifest, RESOURCE, _doc.getSchemaNamespace());
            for(int i = 0; i < resources.length; i++) {
                String id = resources[i].getAttributeValue(IDENTIFIER);
                if(id != null && !id.equals("")) v.add(resources[i]);
            }

            // Search for sub-Manifests (if an Item)
            if(elementName.equals(ContentPackageCore.ITEM)) {
                Element[] submanifests = getElementsInManifest(manifest, MANIFEST, _doc.getSchemaNamespace());
                for(int i = 0; i < submanifests.length; i++) {
                    String id = submanifests[i].getAttributeValue(IDENTIFIER);
                    if(id != null && !id.equals("")) v.add(submanifests[i]);
                }
            }
        }

        Element[] elements = new Element[v.size()];
        v.copyInto(elements);
        return elements;
    }

    /**
     * @return all sub-Elements of type elementName from a given parent Manifest
     * This does a deep recursive search
     */
    public Element[] getElementsInManifest(Element manifestElement, String elementName, Namespace ns) {
        Vector v = new Vector();
        _getElementsInManifest(manifestElement, v, elementName, ns);
        Element[] elements = new Element[v.size()];
        v.copyInto(elements);
        return elements;
    }

    protected void _getElementsInManifest(Element parent, Vector v, String elementName, Namespace ns) {
        Iterator it = parent.getChildren().iterator();
        while(it.hasNext()) {
            Element child = (Element)it.next();
            if(child.getName().equals(elementName) && child.getNamespace().equals(ns)) {
                v.add(child);
            }
            _getElementsInManifest(child, v, elementName, ns);
        }
    }

    /**
     * Find the local parent "manifest" Element for a given Element
     * If element is the manifest, will return that
     */
    public Element getParentManifestElement(Element element) {
        while(!element.getName().equals(MANIFEST)) {
            element = element.getParent();
            if(element == null) return null;
        }
        return element;
    }

    /**
     * @return an Element by its IDENTIFIER attribute starting at Parent element
     * This will only search one-level deep from parent
     */
    public Element getElementByIdentifier(Element parent, String identifier) {
        Iterator it = parent.getChildren().iterator();
        while(it.hasNext()) {
            Element child = (Element)it.next();
            if(_doc.isDocumentNamespace(child)) {
                String id = child.getAttributeValue(IDENTIFIER);
                if(id != null) {
                    if(id.equals(identifier)) return child;
                }
            }
        }

        return null;
    }

    /**
     * @return all ORGANIZATION elements in an ORGANIZATIONS element
     */
    public Element[] getOrganizations(Element orgsElement) {
        Vector v = new Vector();

        Iterator it = orgsElement.getChildren(ORGANIZATION, orgsElement.getNamespace()).iterator();
        while(it.hasNext()) {
            Element orgElement = (Element)it.next();
            v.add(orgElement);
        }

        Element[] orgs = new Element[v.size()];
        v.copyInto(orgs);
        return orgs;
    }

    /**
     * @return all ORGANIZATION elements in an ORGANIZATIONS element that can be referenced
     */
    public Element[] getOrganizationsAllowed(Element orgsElement) {
        Vector v = new Vector();

        Element[] orgs = getOrganizations(orgsElement);
        for(int i = 0; i < orgs.length; i++) {
            String id = orgs[i].getAttributeValue(IDENTIFIER);
            if(id != null && !id.equals("")) v.add(orgs[i]);
        }

        Element[] new_orgs = new Element[v.size()];
        v.copyInto(new_orgs);
        return new_orgs;
    }

    /**
     * @return the default Organization for an ORGANIZATIONS element
     * If there is a DEFAULT ref in the ORGANIZATIONS element will return that
     * otherwise retunr first ORGANIZATION Element if found
     */
    public Element getDefaultOrganization(Element orgsElement) {
        Element element = null;

        if(orgsElement != null && orgsElement.getName().equals(ContentPackageCore.ORGANIZATIONS)) {
            // Get ORGANIZATION by default attribute ref
            String defOrg = orgsElement.getAttributeValue(ContentPackageCore.DEFAULT);
            if(defOrg != null) element = getElementByIdentifier(orgsElement, defOrg);
            // Not found so get first ORGANIZATION Element
            if(element == null) element = orgsElement.getChild(ContentPackageCore.ORGANIZATION, orgsElement.getNamespace());
        }

        return element;
    }

    /**
     * @return the MD Namespace for a CP Document or null if none declared
     */
    public Namespace getMetadataNamespace() {
        Namespace ns = null;

        // Do we have one set in the root element?
        Element root = _doc.getRootElement();
        if(root != null) {
            // Find it according to prefix
            ns = root.getNamespace(IMSDocumentHandler.IMSMD_NAMESPACE_PREFIX);

            // No luck, so iterate thru Namespaces and search for one
            if(ns == null) {
                Iterator it = root.getAdditionalNamespaces().iterator();
                while(it.hasNext()) {
                    Namespace nsTemp = (Namespace) it.next();
                    String nsURI = nsTemp.getURI();
                    if(IMSDocumentHandler.isAcceptedMetadataNamespaceURI(nsURI)) return nsTemp;
                }
            }
        }

        return ns;
    }

    /**
     * Return whether we have the MD root name of any embedded Metadata "lom"
     */
    public boolean isMetadataRoot(String name) {
        return name.equals(MetadataCore.ROOT_NAME);
    }

    /**
     * Return whether we have the "metadata" element
     */
    public boolean isMetadataElement(String name) {
        return name.equals(METADATA);
    }

    /**
     * @return The URL string that an Element references
     */
    public String getURL(Element element) {
        String href = getElementHREF(element);
        if(href != null) {

            // None of this please
            href = href.replaceAll("%20", " ");

            // Save any parameters
            String params = getParameters(element);

            try {
                // Is it a local file for local people?
                File file = new File(getProjectFolder(), href);
                if(file.exists()) {
                    href = ReloadUtils.getLocalFileURL(file);
                    if(params != null) href = href + params;
                    return href;
                }
                // Else see if it's a URL
                else {
                    URL url = new URL(href + (params == null ? "" : params));
                    return url.toString();
                }
            }
            catch(MalformedURLException ex) {
                try {
                    URL url = new URL("http://" + href + (params == null ? "" : params));
                    return url.toString();
                }
                catch(MalformedURLException ex1) {
                    return null;
                }
            }
        }

        return null;
    }

    /**
     * @return An actual HREF reference from an Element's HREF taking into account xml:base
     */
    public String getElementHREF(Element element) {
        String name = element.getName();

        // If this is an ITEM OR DEPENDENCY, change element to referenced element
        if(name.equals(ITEM) || name.equals(DEPENDENCY)) {
            // Get referenced Element
            element = getReferencedElement(element);
            if(element == null) return null;
        }

        // Get the HREF
        String href = element.getAttributeValue(HREF);
        if(href != null) {
            // Do we have a "base" attribute?
            String base = getElementBase(element);
            if(base != null) href = base + href;
        }

        return href;
    }

    /**
     * @return An actual BASE reference for an Element taking into account xml:base
     */
    public String getElementBase(Element element) {
        String totalBase = "";
        do {
            String base = element.getAttributeValue(BASE, Namespace.XML_NAMESPACE);
            if(base != null) {
                totalBase = base + totalBase;
                if(base.startsWith("/") || base.startsWith("http:")) break; // Stop at absolute URL
            }
            //if(element.getName().equals(MANIFEST)) break; // Stop at first parent Manifest
            element = element.getParent();
        }
        while(element != null);

        return totalBase.equals("") ? null : totalBase;
    }

    /**
     * @return the parameters attribute of an element
     */
    public String getParameters(Element element) {
        String params = element.getAttributeValue(PARAMETERS);

        if(params != null && !params.equals("")) {
            char c = params.charAt(0);
            if((c != '?') && (c != '#')) params = "?" + params;
        }

        return params;
    }

    /**
     * @return An actual File reference that an Element references.
     * If it's not a local file or does not exist, return null
     */
    public File getResourceFile(Element element) {
        String href = getElementHREF(element);
        if(href != null) {
            File file = new File(getProjectFolder(), href);
            if(file.exists()) return file;
        }
        return null;
    }


    /**
     * @return All File HREFs for a given Manifest Element taking into account xml:base
     */
    public File[] getResourceFiles(Element manifestElement) {
        Vector v = new Vector();
        _getResourceFiles(manifestElement, v);
        File[] files = new File[v.size()];
        v.copyInto(files);
        return files;
    }

    protected void _getResourceFiles(Element manifestElement, Vector v) {
        // Resources Element
        Element resourcesElement = manifestElement.getChild(RESOURCES, manifestElement.getNamespace());
        if(resourcesElement != null) {
            // Resource Elements
            Iterator it = resourcesElement.getChildren(RESOURCE, resourcesElement.getNamespace()).iterator();
            while(it.hasNext()) {
                Element resourceElement = (Element)it.next();
                File file = getResourceFile(resourceElement);
                if(file != null && !v.contains(file)) v.add(file);
                // File Elements
                Iterator it2 = resourceElement.getChildren(FILE, resourceElement.getNamespace()).iterator();
                while(it2.hasNext()) {
                    Element fileElement = (Element)it2.next();
                    File file2 = getResourceFile(fileElement);
                    if(file2 != null && !v.contains(file2)) v.add(file2);
                }
            }
        }

        // Recurse Sub-Manifests
        Element submanifestElement = manifestElement.getChild(MANIFEST, manifestElement.getNamespace());
        if(submanifestElement != null) {
            _getResourceFiles(submanifestElement, v);
        }
    }

    /**
     * @return a Resource Element given its HREF - the first one found
     */
    public Element getResource(String href, Element resourcesElement) {
        Iterator it = resourcesElement.getChildren(RESOURCE, resourcesElement.getNamespace()).iterator();
        while(it.hasNext()) {
            Element resource = (Element) it.next();
            String tmp = resource.getAttributeValue(HREF);
            if(tmp != null) {
                if(tmp.equalsIgnoreCase(href)) return resource;
            }
        }
        return null;
    }

    /**
     * @return a File Element given its HREF - the first one found
     */
    public Element getFile(String href, Element resourceElement) {
        Iterator it = resourceElement.getChildren(FILE, resourceElement.getNamespace()).iterator();
        while(it.hasNext()) {
            Element file = (Element) it.next();
            String tmp = file.getAttributeValue(HREF);
            if(tmp != null) {
                if(tmp.equalsIgnoreCase(href)) return file;
            }
        }
        return null;
    }

    /**
     * Find the local "resources" Element for a given Element position.  This is because there may
     * be more than one with sub-manifests
     */
    public Element getResourcesElement(Element element) {
        if(element.getName().equals(RESOURCES)) return element;
        Element manifest = getParentManifestElement(element);
        return manifest == null ? null : manifest.getChild(RESOURCES, element.getNamespace());
    }

}