/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import java.util.*;

import org.exolab.castor.xml.schema.*;
import org.jdom.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.*;


/**
 * An Element node that represents an Element in the Schema.
 * Essentially - this is a wrapper for an ElementDecl type
 *
 * @author Phillip Beauvoir
 * @version 2003-10-15
 */
public class ReloadSchemaElement
extends ReloadSchemaNode implements MoonUnitConstants
{
    /**
     * The wrapped Castor ElementDecl that we will interrogate
     */
    protected ElementDecl _elementDecl;

    /**
     * Any child ReloadSchemaElement objects
     */
    protected Vector _children;

    /**
     * Any Attributes
     */
    protected Vector _attributes;

    /**
     * Constructor to wrap ElementDecl.
     * @param elementDecl The wrapped Castor ElementDecl
     */
    public ReloadSchemaElement(ElementDecl elementDecl) {
        _elementDecl = elementDecl;

        // Add the Attributes if we have any
        _attributes = new Vector();
        addAttributes();
    }

    /**
     * Add any ReloadSchemaAttribute objects
     */
    protected void addAttributes() {
        XMLType xmlType = getXMLType();

        if(xmlType instanceof ComplexType) {

	        // Tina Manoharan, 2003-09-10

	        // Added this to cope with complexContent within complexType, for example
	        //	<xs:element name="vdex" block="#all">
			//		<xs:complexType>
			//			<xs:complexContent>
			//				<xs:extension base="vdexType"/>
			//			</xs:complexContent>
			//		</xs:complexType>
			//	</xs:element>
			ComplexType complexType = (ComplexType) xmlType;
			xmlType = complexType.getBaseType();

			while(xmlType != null) {
				if(xmlType instanceof ComplexType) {
					complexType = (ComplexType) xmlType;
					xmlType = complexType.getBaseType();
				}
				else break;
			}

            // Enumeration e = ((ComplexType)xmlType).getAttributeDecls();
            Enumeration e = complexType.getAttributeDecls();
            while(e.hasMoreElements()) {
                AttributeDecl attDecl = (AttributeDecl) e.nextElement();
                ReloadSchemaAttribute att = new ReloadSchemaAttribute(attDecl);
                att.setParent(this);
                _attributes.add(att);
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    ///// PUBLIC METHODS
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Set the parent owning ReloadSchema
     * @param reloadSchema The parent owning ReloadSchema
     * @return This Element
     */
    public ReloadSchemaElement setReloadSchema(ReloadSchema reloadSchema) {
        _parent = reloadSchema;
        return this;
    }

    /**
     * Get the name of the ElementDecl
     * @return The Element name
     */
    public String getName() {
        return _elementDecl.getName();
    }

    /**
     * Get the Prefix of this Element, or "" if no prefix
     * @return the Prefix of this Element, or "" if no prefix
     */
    public String getNamespacePrefix() {
        String refname = _elementDecl.getReferenceName();
        if(refname != null) {
            int idx = refname.indexOf(':');
            if(idx >= 0) {
                String prefix = refname.substring(0, idx);
                return prefix;
            }
        }
        return "";
    }

    /**
     * Get the Namespace of this Schema Element
     * @return the Namespace of this Schema Element
     */
    public Namespace getNamespace() {
        String nsPrefix = getNamespacePrefix();
        String nsURI = _elementDecl.getSchema().getNamespace(nsPrefix);
        return Namespace.getNamespace(nsPrefix, nsURI);
    }

    /**
     * Return whether we are the Root Element, i.e our parent is null
     * @return true if we are the root element, false otherwise
     */
    public boolean isRoot() {
        return getParent() == null;
    }

    /**
     * Get the path to this Element.  This will be the full path of all parent
     * elements plus this Element's name.  For example: manifest/resources/resource
     * This DomPath is allocated dynamically, otherwise it doesn't work!
     * @return The path for this Element
     */
    public DomPath getDomPath() {
        if(_parent instanceof ReloadSchemaElement) {
            DomPath domPath = ((ReloadSchemaElement)_parent).getDomPath();
            String name = getName();
            // Add external namespace prefix
            if(isExternalNamespace()) name = getNamespacePrefix() + ":" + name;
            domPath.appendElementName(name);
            return domPath;
        }
        return new DomPath(getName());
    }

    /**
     * Get the inbuilt vocabulary list for this Element.  If it's a Simple type of type
     * BOOLEAN we can return "true, false" as the list, otherwise we will look
     * in the Vocab Helper file.
     * This method should only be called from a SchemaController.
     * @return vocabulary list for this Element or null if there isn't one
     */
    public ReloadVocabularyList getVocabularyList() {
        ReloadVocabularyList rvList = null;

        XMLType xmlType = getXMLType();

        // If simple type
        if(xmlType.isSimpleType()) {
            SimpleType simpleType = (SimpleType)xmlType;
            int type = simpleType.getTypeCode();
            switch(type) {
                // If boolean
                case SimpleTypesFactory.BOOLEAN_TYPE:
                    rvList = new ReloadVocabularyList(getDomPath(), ReloadVocabularyList.TRUEFALSE_LISTNAME);
                    break;

                // Look at the enumeration facets in the Schema
                default:
                    Vector v = null;
                    Enumeration e = simpleType.getFacets(Facet.ENUMERATION);
                    while(e.hasMoreElements()) {
                        if(v == null) v = new Vector();
                        Facet facet = (Facet)e.nextElement();
                        v.addElement(facet.getValue());
                    }
                    if(v != null) {
                        String[] s = new String[v.size()];
                        v.copyInto(s);
                        rvList = new ReloadVocabularyList(getDomPath(), ReloadVocabularyList.INTERNAL_LISTNAME);
                        rvList.setList(s);
                    }
                    break;
            }

        }

        return rvList;
    }

    /**
     * Get the type name of this Element.
     * @return The type name of this Element
     */
    public String getTypeName() {
        return getXMLType().getName();
    }

    /**
     * Get the Base type name of this Element.
     * @return The base type name of this Element or the type name if there is none
     */
    public String getBaseTypeName() {
        XMLType xmlType = getXMLType();
        XMLType baseType = xmlType.getBaseType();
        return baseType == null ?  xmlType.getName() : baseType.getName();
    }

    /**
     * Get the Atomic Base type name of this Element.
     * @return The base type name of this Element or the type name if there is none
     */
    public String getAtomicBaseTypeName() {
        String name = null;

        XMLType xmlType = getXMLType();
        if(xmlType != null) {
            XMLType baseType = xmlType.getBaseType();

            while(baseType != null) {
                name = baseType.getName();
                baseType = baseType.getBaseType();
            }

            return name == null ? xmlType.getName() : name;
        }

        return "Unknown BaseType";
    }

    /**
     * @return whether this can hold an actual text/number value rather than just being a container for
     * sub-elements
     */
    public boolean isValue() {
        String baseTypeName = getAtomicBaseTypeName();
        if(baseTypeName == null) return false;
        //if(baseTypeName.equals("string") || baseTypeName.equals("int") || baseTypeName.equals("integer")
        //   || baseTypeName.equals("decimal")) return true;


		// Tina Manoharan, 2003-09-02

        if(baseTypeName.equals("string") || baseTypeName.equals("int") || baseTypeName.equals("integer")
           || baseTypeName.equals("decimal") || baseTypeName.equals("anyURI")) return true;

        return false;
    }

    /**
     * Determine whether this element's base type name is a String type
     * @return true if this element's base type name is a String type, false otherwise
     */
    public boolean isStringType() {
        String baseTypeName = getAtomicBaseTypeName();
        return baseTypeName != null && baseTypeName.equals("string");
    }

    /**
     * Get the minimum amount of times this element can occur - 0 means optional
     * @return the minimum amount of times this element can occur - 0 means optional
     */
    public int getMinOccurs() {
        return _elementDecl.getMinOccurs();
    }

    /**
     * Get the maximum amount of times this element can occur - -1 means unbounded.
     * @return the maximum amount of times this element can occur - -1 means unbounded
     */
    public int getMaxOccurs() {
        return _elementDecl.getMaxOccurs();
    }

    /**
     * Returns the default value of this element definition if it is an ElementDecl.
     * This method should only be called from a SchemaController - look into the vocabulary list first,
     * if that is null, look here.
     * @return the default value of this element definition or null if no default was specified
     */
    public String getDefaultValue() {
        return _elementDecl.getDefaultValue();
    }

    /**
     * Get the xml type of the ElementDecl - this is either ComplexType or SimpleType.
     * This only applies to ElementDecl.
     * @return the xml type of the ElementDecl or null if this Particle is not an ElementDecl
     */
    protected XMLType getXMLType() {
        return _elementDecl.getType();
    }

    /**
     * Return whether this Element is made up of other Elements.
     * @return true if this Element is a Complex Type, or false if not
     */
    public boolean isComplexType() {
        return getXMLType().isComplexType();
    }

    /**
     * Get a facet value
     * @param facetName the facet name
     * @return a Facet Value or null
     */
    public String getFacetValue(String facetName) {
        String val = null;

        // If simple type
        XMLType xmlType = getXMLType();
        if(xmlType.isSimpleType()) {
            SimpleType simpleType = (SimpleType)xmlType;
            Facet facet = simpleType.getFacet(facetName);
            if(facet != null) val = facet.getValue();
        }

        return val;
    }

    /**
     * Get an array of ReloadSchemaAttribute objects
     * @return an array of ReloadSchemaAttribute objects or null if there aren't any
     */
    public ReloadSchemaAttribute[] getAttributes() {
        ReloadSchemaAttribute[] atts = new ReloadSchemaAttribute[_attributes.size()];
        _attributes.copyInto(atts);
        return atts;
    }

    /**
     * Get a ReloadSchemaAttribute by its name
     * @param attName the attribute name
     * @return a ReloadSchemaAttribute by its name
     */
    public ReloadSchemaAttribute getAttribute(String attName) {
        Namespace ns;

        // Check for Namespace prefix
        int idx = attName.indexOf(':');
        if(idx >= 0) {
            String prefix = attName.substring(0, idx);
            attName = attName.substring(idx + 1);
            String nsURI = _elementDecl.getSchema().getNamespace(prefix);
            // ns = Namespace.getNamespace(prefix, nsURI);

			// Tina Manoharan, 2003-09-02

			if(nsURI != null) ns = Namespace.getNamespace(prefix, nsURI);
			else ns = getNamespace();
        }
        else ns = getNamespace();

        return getAttribute(attName, ns);
    }

    /**
     * Get a ReloadSchemaAttribute by its name and Namespace
     * @param attName the attribute name
     * @param ns the attribute namespace
     * @return a ReloadSchemaAttribute by its name and Namespace
     */
    public ReloadSchemaAttribute getAttribute(String attName, Namespace ns) {
        for(int i = 0; i < _attributes.size(); i++) {
            ReloadSchemaAttribute att = (ReloadSchemaAttribute)_attributes.get(i);
            if(att.getName().equals(attName) && att.getNamespace().equals(ns)) return att;
        }
        return null;
    }

    /**
     * Return a ReloadSchemaAttribute by its index position
     * @param index the index position of the Attribute
     * @return the ReloadSchemaAttribute or null if there isn't one
     */
    public ReloadSchemaAttribute getAttributeAtIndex(int index) {
        if(index >= _attributes.size()) return null;
        return (ReloadSchemaAttribute)_attributes.elementAt(index);
    }

    /**
     * Return whether this Element has Attributes
     * @return true if this element has attributes, false otherwise
     */
    public boolean hasAttributes() {
        return !_attributes.isEmpty();
    }

    /**
     * Return whether this Element has an Attribute by Name
     * @param attName the attribute name
     * @return true if this element has the attribute, false otherwise
     */
    public boolean hasAttribute(String attName) {
        if(hasAttributes()) {
            return indexofAttribute(attName) != -1;
        }
        return false;
    }

    /**
     * Return the index of an Attribute
     * @param att the ReloadSchemaAttribute concerned
     * @return the index of the Attribute or -1 if not found
     */
    public int indexofAttribute(ReloadSchemaAttribute att) {
        if(att == null) return -1;
        return indexofAttribute(att.getName());
    }

    /**
     * Return the index of an Attribute by name
     * @param attName the attribute name
     * @return the index of the Attribute or -1 if not found
     */
    public int indexofAttribute(String attName) {
        if(attName == null) return -1;

        for(int i = 0; i < _attributes.size(); i++) {
            ReloadSchemaAttribute att = (ReloadSchemaAttribute)_attributes.get(i);
            if(att.getName().equals(attName)) return i;
        }

        return -1;
    }

    /**
     * A String representation of this Element.
     * @return the name of the Element
     */
    public String toString() {
        return getName();
    }

///////////////////////// CHILDREN //////////////////////////////////////////////

    /**
     * Get the Children Elements of this Element.
     * @return an array of ReloadSchemaElement child objects. It will never be null
     */
    public ReloadSchemaElement[] getChildren() {
        Vector v = buildChildren();
        ReloadSchemaElement[] elements = new ReloadSchemaElement[v.size()];
        v.copyInto(elements);
        return elements;
    }

    /**
     * Get the Children Elements of this Element with a given Namespace.
     * @param ns the given namespace
     * @return an array of ReloadSchemaElement child objects. It will never be null
     */
    public ReloadSchemaElement[] getChildren(Namespace ns) {
        Vector v = new Vector();
        ReloadSchemaElement[] children = getChildren();
        for(int i = 0; i < children.length; i++) {
            if(children[i].getNamespace() == ns) v.add(children[i]);
        }
        ReloadSchemaElement[] elements = new ReloadSchemaElement[v.size()];
        v.copyInto(elements);
        return elements;
    }

    /**
     * Get the child count of ReloadSchemaElement child objects.
     * @return the Child count
     */
    public int getChildCount() {
        Vector v = buildChildren();
        return v.size();
    }

    /**
     * Return whether this Element has Child Elements.
     * @return true if this Element has Children, or false if not
     */
    public boolean hasChildren() {
        return getChildCount() > 0;
    }

    /**
     * Return whether this ReloadSchemaElement has a child Element by name.
     * @param childName the name of the child element
     * @return true if it has the child, false otherwise
     */
    public boolean hasChild(String childName) {
        return getChild(childName) == null ? false : true;
    }

    /**
     * Get a child schema element by name
     * @param childName the name of the child element
     * @return a child ReloadSchemaElement by name or null
     */
    public ReloadSchemaElement getChild(String childName) {
        Namespace ns;

        // Check for Namespace prefix
        int idx = childName.indexOf(':');
        if(idx >= 0) {
            String prefix = childName.substring(0, idx);
            childName = childName.substring(idx + 1);
            String nsURI = _elementDecl.getSchema().getNamespace(prefix);
            ns = Namespace.getNamespace(prefix, nsURI);
        }
        else ns = getNamespace();

        return getChild(childName, ns);
    }

    /**
     * Get a child schema element by name with given namespace
     * @param childName the name of the child element
     * @param ns the namespace of the child element
     * @return a child ReloadSchemaElement by name or null
     */
    public ReloadSchemaElement getChild(String childName, Namespace ns) {
        Iterator i = buildChildren().iterator();
        while(i.hasNext()) {
            ReloadSchemaElement child = (ReloadSchemaElement)i.next();
            if(child.getName().equals(childName) && child.getNamespace().equals(ns)) return child;
        }
        return null;
    }

    /**
     * Get the child ReloadSchemaElement at index position
     * @param index the index position of the child element
     * @return the child ReloadSchemaElement at index position or null if no children
     */
    public ReloadSchemaElement getChildAt(int index) {
        if(index >= getChildCount() || index < 0) return null;
        return (ReloadSchemaElement)buildChildren().get(index);
    }

    /**
     * Get the index of the child element
     * @return the index of the child or -1 if not found
     */
    public int indexofChild(ReloadSchemaElement child) {
        if(child == null) return -1;
        return buildChildren().indexOf(child);
    }

    /**
     * Return the index of a child by name
     * @param childName the name of the child element
     * @return the index of the child or -1 if not found
     */
    public int indexofChild(String childName) {
        if(childName == null) return -1;

        Vector v = buildChildren();
        for(int i = 0; i < v.size(); i++) {
            ReloadSchemaElement element = (ReloadSchemaElement)v.elementAt(i);
            if(element.getName().equals(childName)) return i;
        }

        return -1;
    }

    /**
     * Return a String array of child names
     * @return a String array of child names
     */
    public String[] getChildNames() {
        Vector v = buildChildren();
        String s[] = new String[v.size()];
        for(int i = 0; i < v.size(); i++) {
            ReloadSchemaElement element = (ReloadSchemaElement)v.elementAt(i);
            s[i] = element.getName();
        }
        return s;
    }


    ////////////////////////////////////////////////////////////////////////////
    // Building methods
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Recurse and build up all child Elements and Attributes of ElementDecl
     * @return a Vector of all child elements and attributes of the ElementDecl
     */
    protected Vector buildChildren() {
        if(_children == null) {
            _children = new Vector();

            XMLType xmlType = getXMLType();
            // If Complex Type build children
            if(xmlType instanceof ComplexType) {
                ComplexType complexType = (ComplexType) xmlType;

				// Tina Manoharan, 2003-09-05

				// Had to do this to be able to read nested VDEX Schema such as:
        		//	<xs:element name="vdex" block="#all">
				//		<xs:complexType>
				//			<xs:complexContent>
				//				<xs:extension base="vdexType"/>
				//			</xs:complexContent>
				//		</xs:complexType>
				//	</xs:element>

				xmlType = complexType.getBaseType();

				while(xmlType != null) {
					if(xmlType instanceof ComplexType) {
						complexType = (ComplexType) xmlType;
						xmlType = complexType.getBaseType();
					}
					else break;
				}

                // Build/Add sub-group child structures
                Enumeration cTypeBits = complexType.enumerate();
                while(cTypeBits.hasMoreElements()) {
                    Structure structure = (Structure) cTypeBits.nextElement();

                    switch(structure.getStructureType()) {
                        case Structure.GROUP:
                            buildChildGroup((Group) structure);
                            break;

                        case Structure.MODELGROUP:
                            buildChildGroup((ModelGroup) structure);
                            break;

                        default:
                            if(DEBUG) System.out.println("Other ComplexType Structure: " + structure.toString());
                    }
                }
            }
        }
        return _children;
    }

    /**
     * Build a Group type - this can contain more Elements and Groups
     * @param group
     */
    protected void buildChildGroup(Group group) {
        Enumeration groupMembers = group.enumerate();
        while(groupMembers.hasMoreElements()) {
            Particle particle = (Particle)groupMembers.nextElement();

            switch(particle.getStructureType()) {
                // Another Element Decl
                case Structure.ELEMENT:
                    addChildElementDecl((ElementDecl)particle);
                    break;

                // A Model Group
                case Structure.MODELGROUP:
                    //addChildModelGroup((ModelGroup)particle);
                    buildChildGroup((ModelGroup)particle);
                    break;

                // Another sub-group
                case Structure.GROUP:
                    //addChildGroup((Group)particle);
                    buildChildGroup((Group)particle);
                    break;

                case Structure.WILDCARD:
                    //addChildWildCard((Wildcard)particle);
                    break;

                default:
                    if(DEBUG) System.out.println("Group Particle: " + particle.toString());
            } // switch
        } // while
    }

    /**
     * Add another ElementDecl child
     * @param childElementDecl the child ElementDecl to add
     */
    protected void addChildElementDecl(ElementDecl childElementDecl) {
        ReloadSchemaElement child = new ReloadSchemaElement(childElementDecl);
        child.setParent(this);
        _children.add(child);
    }

    /**
     * Determine if a schemaElement has an ancestor of the same name
     * @return true if a schemaElement has an ancestor of the same name, false otherwise
     */
    public boolean hasAncestor(ReloadSchemaElement schemaElement) {
        if(_parent instanceof ReloadSchemaElement)
            return ((ReloadSchemaElement)_parent).getName().equals(schemaElement.getName()) ? true : ((ReloadSchemaElement)_parent).hasAncestor(schemaElement);
        return false;
    }
}