/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.schema;

import java.util.*;
import java.io.*;
import javax.swing.*;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.moonunit.dom.vocabulary.*;

/**
 * A Reload Vocabulary List - A String list and default value
 *
 * @author Phillip Beauvoir
 * @version 2003-05-14
 */
public class ReloadVocabularyList {
    protected static String INTERNAL_LISTNAME = "internal";
    protected static String LANG_LISTNAME = "lang";
    protected static String YESNO_LISTNAME = "yesno";
    protected static String TRUEFALSE_LISTNAME = "truefalse";

    /**
     * Yes / No list
     */
    protected static String[] YESNO_LIST = new String[] { "yes", "no" };

    /**
     * True / False vocabulary list
     */
    protected static String[] TF_LIST = new String[] { "true", "false" };


	// Tina Manoharan, 2003-09-09

	/**
	 * Vdex vocabulary file without a default list name
	 */
	protected static String VDEX_VOCAB = "vdexVocab";

	/**
	 * Vdex vocabulary file with a default list name
	 */
	protected static String DEFAULT_VDEX_VOCAB = "default_vdexVocab";

    /**
     * The path mapping
     */
    protected DomPath _domPath;

    /**
     * The list name.  If the list comes from the Schema, this should be set to "internal"
     * or can be set to one of the special list names like "lang" or "yesno"
     */
    protected String listName;


	// Tina Manoharan, 2003-09-09

	/**
	 * The default vocabulary list name if we are using Vdex files
	 */
    protected String defaultListName;


    /**
     * The Actual String list, initialised to an Empty list
     */
    protected String[] list = new String[] { "" };


	// Tina Manoharan, 2003-09-09

    /**
     * The Default String list, initialised to an Empty list
     */
    protected String[] defaultList = new String[] { "" };


    /**
     * The default value
     */
    protected String defaultValue;


	// Tina Manoharan, 2003-11-10

	/**
	 * The LOM Element lom/classification/purpose/value/langstring Value
	 */
    protected String purposeValue = null;



	/**
	 * Default constructor
	 */
    public ReloadVocabularyList() {

    }

	/**
	 * Constructor
	 * @param domPath the node path bound to this vocabulary list
	 * @param listName the list name
	 */
    public ReloadVocabularyList(DomPath domPath, String listName) {
        _domPath = domPath;
        this.listName = listName;
    }

	/**
	 * Get the DomPath
	 * @return the node path bound to this vocabulary list
	 */
    public DomPath getDomPath() {
        return _domPath;
    }

    /**
     * Set the Vocabulary List
     * @param list the vocabulary list
     */
    public void setList(String[] list) {
        this.list = list;
    }

	// Tina Manoharan, 2003-09-09

    /**
     * Set the Default Vocabulary List
     * @param list the default vocabulary list
     */
    public void setDefaultList(String[] list) {
        this.defaultList = list;
    }


    /**
     * Get the Vocab list.  It will at least be an array of one empty String
     * @return the vocabulary list
     */
    public String[] getList() {
        // Special case of lang list
        if(listName.equals(LANG_LISTNAME)) return getLanguageList();
        // Special case of yesno list
        if(listName.equals(YESNO_LISTNAME)) return YESNO_LIST;
        // Special case of truefalse list
        if(listName.equals(TRUEFALSE_LISTNAME)) return TF_LIST;

		// Tina Manoharan, 2003-09-09

		// Special case of Vdex Vocab list
		if(listName.equals(VDEX_VOCAB) || listName.equals(DEFAULT_VDEX_VOCAB)) return getVdexVocabList();

        return list;
    }

    /**
     * Get the Name of the List
     * @return the name of the list
     */
    public String getListName() {
        return listName;
    }

    // Tina Manoharan, 2003-09-09

    /**
     * Set the default listName for this vocab list
     * @param defaultListName the default list name to be given for this vocab list
     */
    public void setDefaultListName(String defaultListName) {

        this.defaultListName = defaultListName;

        // Tina Manoharan, 2003-09-09

		if(listName.equals(DEFAULT_VDEX_VOCAB) && this.defaultListName != null) this.defaultValue = this.defaultListName;
    }


    /**
     * Determine if this list is a language list.
     * @return true if this is a language list, false otherwise
     */
    public boolean isLangList() {
        return listName.equals(LANG_LISTNAME);
    }

    /**
     * Set the default value for this vocab list
     * @param defaultValue the default value to be set for this vocab list
     */
    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    /**
     * Get the default value of this vocab list
     * @return the default value of this vocab list
     */
    public String getDefaultValue() {
        return defaultValue;
    }

    /**
     * Get a Vocabulary List that is full of the Language Strings
     * @return a String array of ISO language codes
     */
    private String[] getLanguageList() {
        return Locale.getISOLanguages();
    }


	// Tina Manoharan, 2003-09-09


    //============================== VDEX VOCABULARY  ==========================


    /**
     * Get Vdex Vocab List from saved files in the folder helpers/md/vdexVocab
     * @return an array of Vdex vocabulary names or identifiers
     */
    private String[] getVdexVocabList() {

		DomPath domPath;

        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        // Get the Files
        File[] files = ReloadEditorDocumentHandler.folderMDVdexVocab.listFiles(filter);
        String s = "";
        Vector v = new Vector();

        if(listName.equals(DEFAULT_VDEX_VOCAB)) v.addElement(defaultListName);

        files = this.getVdexFilesBasedOnVocabularyMetadata(files);

        for(int i = 0; i < files.length; i++) {

			domPath = new DomPath(VocabularyCore.VOCABULARY_IDENTIFIER_PATH);

			try {
				s = XMLUtils.getElementValueByName(VocabularyCore.ROOT_NAME, domPath, files[i]);
			}
			catch(DweezilXMLException ex) {}

			if(s == null || s.equals("")) {

				domPath = new DomPath(VocabularyCore.VOCABULARY_NAME_PATH);

				try {
					s = XMLUtils.getElementValueByName(VocabularyCore.ROOT_NAME, domPath, files[i]);
				}
				catch(DweezilXMLException ex) {}
			}

			if(s != null && !s.equals("")) {

				v.addElement(s);
			}
        }

        String[] vdexVocabList = new String[v.size()];
        v.copyInto(vdexVocabList);

        return vdexVocabList;
    }


    /**
     * Identify a set of Vdex files, from the folder helpers/md/vdexVocab, that are appropriate for
     * providing vocabulary for the currently open metadata file elements. The appropriateness is
     * determined based on the information provided in the vocabulary metadata element of the Vdex file.
     *
     * @param files all the Vdex files in the folder helpers/md/vdexVocab
     * @return a set of Vdex files, from the folder helpers/md/vdexVocab, that are appropriate for
     * providing vocabulary for the currently open metadata file elements
     */
    private File[] getVdexFilesBasedOnVocabularyMetadata(File[] files) {

		ArrayList result = new ArrayList(Arrays.asList(files));

		ArrayList test = new ArrayList(Arrays.asList(files));

		ArrayList entries = new ArrayList();

		ArrayList keywords = new ArrayList();

		File file, stylesheet = null;


		if(_domPath != null) {

			String path = _domPath.getPath();

			// first check the vocabulary domPath
			if(path.equals(MetadataCore.CLASSIFICATION_TAXONPATH_SOURCE_PATH)) {

				if(this.purposeValue == null) {

					// lom/classification/taxonpath/source/langstring
					// lom/classification/purpose/value/langstring

					ReloadInternalFrame focussedInternalFrame = ReloadEditor.getInstance().getFocussedInternalFrame();

					if(focussedInternalFrame instanceof MetadataEditorFrame) {

						MetadataEditorFrame mdFrame = (MetadataEditorFrame)focussedInternalFrame;

						MetadataEditor mdEditor = mdFrame.getMetadataEditor();

						Metadata md = mdEditor.getMetadata();

						Element taxonpathSourceElement = md.getElement(_domPath);

						if(taxonpathSourceElement != null) {

							Element purposeElement = taxonpathSourceElement.getParent().getParent().getParent().getChild(MetadataCore.PURPOSE, taxonpathSourceElement.getNamespace());

							if(purposeElement != null)
							{
								Element purposeValueElement = purposeElement.getChild(MetadataCore.VALUE, taxonpathSourceElement.getNamespace());

								if(purposeValueElement != null) this.purposeValue = purposeValueElement.getChildTextTrim(MetadataCore.LANGSTRING, taxonpathSourceElement.getNamespace());
							}
						}
					}
				}

				if(this.purposeValue != null) {

					stylesheet = ReloadEditorDocumentHandler.VDEX_TO_ENTRIES_STYLESHEET;

					for(int i = 0; i < test.size(); i++) {

						file = (File) test.get(i);

						try {
							entries = XMLUtils.getElementValuesFromFileByName(file, stylesheet);
						}
						catch(DweezilXMLException ex) {}

						// check if the domPath of this VocabularyList is in keywords

						if(!entries.contains(this.purposeValue)) {

							try {

								result.remove(file);
							}
							catch(UnsupportedOperationException ex) {System.out.println("Unsupported Operation Exception");}
						}
					}

					this.purposeValue = null;
				}
			}
			else {

				// for each file check if it has a VDEX_METADATA, then if it has a LOM_GENERAL_KEYWORD
				// if the keyword is the same as the vocabulary domPath then select this file

				stylesheet = ReloadEditorDocumentHandler.VDEX_TO_KEYWORDS_STYLESHEET;

				for(int i = 0; i < test.size(); i++) {

					file = (File) test.get(i);

					try {
						keywords = XMLUtils.getElementValuesFromFileByName(file, stylesheet);
					}
					catch(DweezilXMLException ex) {}

					// check if the domPath of this VocabularyList is in keywords

					if(!keywords.contains(path)) {

						try {

							result.remove(file);
						}
						catch(UnsupportedOperationException ex) {System.out.println("Unsupported Operation Exception");}
					}
				}
			}
		}

		return (File[]) result.toArray(new File[result.size()]);
	}



    /**
     * Set the vocabulary list depending on the vocabulary source value. If the source is a Vdex file, then
     * get the list items from this file.
     *
     * @param source who is telling us this
     * @param vocabSource the vocabulary source value
     */
    public void setListUsingVocabSource(Object source, String vocabSource) {

		if(!vocabSource.equals("") && !vocabSource.equals(defaultListName) && vocabSource != null) {

			// get list items from vocabSource vdex file
			this.list = getListFromVdexSource(vocabSource, VocabularyCore.TERM_CAPTION_PATH);
		}
		else {

			this.list = this.defaultList;
		}

		// inform listeners that are bound to this particular valueElement to update their vocabulary list
        ReloadVocabularyListenerEvent event = new ReloadVocabularyListenerEvent(source, ReloadVocabularyList.this, this.list);
        fireElementChanged(event);
    }




	/**
	 * Update the LOM lom/classification/taxonpath/source/langstring element's vocabulary list depending
	 * on the value of the Purpose element lom/classification/purpose/value/langstring.
	 *
	 * @param source who is telling us this
	 * @param purposeValue the value of the Purpose element
	 */
    public void updateTaxonPathSourceListUsingPurposeValue(Object source, String purposeValue) {

		// use the purposeValue to filter all the files in the vdexVocab folder
		// update the list

		this.purposeValue = purposeValue;

		this.list = this.getVdexVocabList();

		// inform listeners that are bound to this particular source to update their list
        ReloadVocabularyListenerEvent event = new ReloadVocabularyListenerEvent(source, ReloadVocabularyList.this, this.list);
        fireElementChanged(event);
    }



	/**
	 * Get the vocabulary list items from a Vdex file.
	 *
	 * @param vocabSource the Vdex vocabulary name or identifier
	 * @param elementName the Vdex vocabulary element who's value becomes an item of this vocabulary list
	 * @return a String array of items of the vocabulary list
	 */
    public String[] getListFromVdexSource(String vocabSource, String elementName) {

		ArrayList result = new ArrayList();

		DomPath domPath;

		File stylesheet = null;

        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        // Get the File with the given vocabName/ID
        File[] files = ReloadEditorDocumentHandler.folderMDVdexVocab.listFiles(filter);
		boolean found = false;

        for(int i = 0; i < files.length; i++) {

			domPath = new DomPath(VocabularyCore.VOCABULARY_IDENTIFIER);

			try {
				found = XMLUtils.getFileByNameValuePair(VocabularyCore.ROOT_NAME, domPath, files[i], vocabSource);
			}
			catch(DweezilXMLException ex) {}

			if(!found) {

				domPath = new DomPath(VocabularyCore.VOCABULARY_NAME_PATH);

				try {
					found = XMLUtils.getFileByNameValuePair(VocabularyCore.ROOT_NAME, domPath, files[i], vocabSource);
				}
				catch(DweezilXMLException ex) {}
			}

			if(found) {

				if(elementName.equals(VocabularyCore.TERM_CAPTION_PATH)) {

					stylesheet = ReloadEditorDocumentHandler.VDEX_TO_TEMP_STYLESHEET;
				}

				// Get all the term captions

				try {
					result = XMLUtils.getElementValuesFromFileByName(files[i], stylesheet);
				}
				catch(DweezilXMLException ex) {}

				break;
			}
        }

		return (String[]) result.toArray(new String[result.size()]);
	}


	// Tina Manoharan, 2003-11-10


    //============================== LISTENER EVENTS  ==========================

    /**
     * List of Listeners
     */
    private Vector _listeners = new Vector();

    /**
     * Add a ReloadVocabularyListener
     * @param listener The ReloadVocabularyListener
     */
    public synchronized void addReloadVocabularyListener(ReloadVocabularyListener listener) {
        if(!_listeners.contains(listener)) _listeners.addElement(listener);
    }

    /**
     * Remove a ReloadVocabularyListener
     * @param listener The ReloadVocabularyListener
     */
    public synchronized void removeReloadVocabularyListener(ReloadVocabularyListener listener) {
        _listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we have changed the vocabulary list
     * @param event the ReloadVocabularyListenerEvent
     */
    public void fireElementChanged(ReloadVocabularyListenerEvent event) {

        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadVocabularyListener listener = (ReloadVocabularyListener)_listeners.elementAt(i);

            listener.vocabularyListChanged(event);
        }
	}

}